/* Generated by Cython 3.0.11 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "depends": [],
        "name": "CyEulerBallistic.trajectory_calc",
        "sources": [
            "CyEulerBallistic/trajectory_calc.pyx"
        ]
    },
    "module_name": "CyEulerBallistic.trajectory_calc"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
#if defined(CYTHON_LIMITED_API) && 0
  #ifndef Py_LIMITED_API
    #if CYTHON_LIMITED_API+0 > 0x03030000
      #define Py_LIMITED_API CYTHON_LIMITED_API
    #else
      #define Py_LIMITED_API 0x03030000
    #endif
  #endif
#endif

#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02070000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.7+ or Python 3.3+.
#else
#if defined(CYTHON_LIMITED_API) && CYTHON_LIMITED_API
#define __PYX_EXTRA_ABI_MODULE_NAME "limited"
#else
#define __PYX_EXTRA_ABI_MODULE_NAME ""
#endif
#define CYTHON_ABI "3_0_11" __PYX_EXTRA_ABI_MODULE_NAME
#define __PYX_ABI_MODULE_NAME "_cython_" CYTHON_ABI
#define __PYX_TYPE_MODULE_PREFIX __PYX_ABI_MODULE_NAME "."
#define CYTHON_HEX_VERSION 0x03000BF0
#define CYTHON_FUTURE_DIVISION 1
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(_WIN32) && !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #define HAVE_LONG_LONG
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#define __PYX_LIMITED_VERSION_HEX PY_VERSION_HEX
#if defined(GRAALVM_PYTHON)
  /* For very preliminary testing purposes. Most variables are set the same as PyPy.
     The existence of this section does not imply that anything works or is even tested */
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 1
  #define CYTHON_COMPILING_IN_NOGIL 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_TYPE_SPECS
  #define CYTHON_USE_TYPE_SPECS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS (PY_MAJOR_VERSION >= 3)
  #endif
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
  #undef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 0
#elif defined(PYPY_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_NOGIL 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS (PY_MAJOR_VERSION >= 3)
  #endif
  #if PY_VERSION_HEX < 0x03090000
    #undef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #elif !defined(CYTHON_PEP489_MULTI_PHASE_INIT)
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1 && PYPY_VERSION_NUM >= 0x07030C00)
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
  #undef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 0
#elif defined(CYTHON_LIMITED_API)
  #ifdef Py_LIMITED_API
    #undef __PYX_LIMITED_VERSION_HEX
    #define __PYX_LIMITED_VERSION_HEX Py_LIMITED_API
  #endif
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 1
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_NOGIL 0
  #undef CYTHON_CLINE_IN_TRACEBACK
  #define CYTHON_CLINE_IN_TRACEBACK 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_TYPE_SPECS
  #define CYTHON_USE_TYPE_SPECS 1
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #endif
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 1
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 0
  #endif
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
  #undef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 0
#elif defined(Py_GIL_DISABLED) || defined(Py_NOGIL)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_NOGIL 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #ifndef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #ifndef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #ifndef CYTHON_METH_FASTCALL
    #define CYTHON_METH_FASTCALL 1
  #endif
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_MODULE_STATE
    #define CYTHON_USE_MODULE_STATE 0
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 1
  #endif
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 1
  #endif
  #ifndef CYTHON_USE_FREELISTS
    #define CYTHON_USE_FREELISTS 0
  #endif
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_NOGIL 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #ifndef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #ifndef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0 || PY_VERSION_HEX >= 0x030B00A2
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_GIL
    #define CYTHON_FAST_GIL (PY_MAJOR_VERSION < 3 || PY_VERSION_HEX >= 0x03060000 && PY_VERSION_HEX < 0x030C00A6)
  #endif
  #ifndef CYTHON_METH_FASTCALL
    #define CYTHON_METH_FASTCALL (PY_VERSION_HEX >= 0x030700A1)
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #elif !defined(CYTHON_PEP489_MULTI_PHASE_INIT)
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_MODULE_STATE
    #define CYTHON_USE_MODULE_STATE 0
  #endif
  #if PY_VERSION_HEX < 0x030400a1
    #undef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 0
  #elif !defined(CYTHON_USE_TP_FINALIZE)
    #define CYTHON_USE_TP_FINALIZE 1
  #endif
  #if PY_VERSION_HEX < 0x030600B1
    #undef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS 0
  #elif !defined(CYTHON_USE_DICT_VERSIONS)
    #define CYTHON_USE_DICT_VERSIONS  (PY_VERSION_HEX < 0x030C00A5)
  #endif
  #if PY_VERSION_HEX < 0x030700A3
    #undef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK 0
  #elif !defined(CYTHON_USE_EXC_INFO_STACK)
    #define CYTHON_USE_EXC_INFO_STACK 1
  #endif
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 1
  #endif
  #ifndef CYTHON_USE_FREELISTS
    #define CYTHON_USE_FREELISTS 1
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if !defined(CYTHON_VECTORCALL)
#define CYTHON_VECTORCALL  (CYTHON_FAST_PYCCALL && PY_VERSION_HEX >= 0x030800B1)
#endif
#define CYTHON_BACKPORT_VECTORCALL (CYTHON_METH_FASTCALL && PY_VERSION_HEX < 0x030800B1)
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_MAJOR_VERSION < 3
    #include "longintrepr.h"
  #endif
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
  #if defined(__cplusplus)
    /* for clang __has_cpp_attribute(maybe_unused) is true even before C++17
     * but leads to warnings with -pedantic, since it is a C++17 feature */
    #if ((defined(_MSVC_LANG) && _MSVC_LANG >= 201703L) || __cplusplus >= 201703L)
      #if __has_cpp_attribute(maybe_unused)
        #define CYTHON_UNUSED [[maybe_unused]]
      #endif
    #endif
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
  #define CYTHON_MAYBE_UNUSED_VAR(x) CYTHON_UNUSED_VAR(x)
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_USE_CPP_STD_MOVE
  #if defined(__cplusplus) && (\
    __cplusplus >= 201103L || (defined(_MSC_VER) && _MSC_VER >= 1600))
    #define CYTHON_USE_CPP_STD_MOVE 1
  #else
    #define CYTHON_USE_CPP_STD_MOVE 0
  #endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
            typedef unsigned char     uint8_t;
            typedef unsigned short    uint16_t;
            typedef unsigned int      uint32_t;
        #else
            typedef unsigned __int8   uint8_t;
            typedef unsigned __int16  uint16_t;
            typedef unsigned __int32  uint32_t;
        #endif
    #endif
    #if _MSC_VER < 1300
        #ifdef _WIN64
            typedef unsigned long long  __pyx_uintptr_t;
        #else
            typedef unsigned int        __pyx_uintptr_t;
        #endif
    #else
        #ifdef _WIN64
            typedef unsigned __int64    __pyx_uintptr_t;
        #else
            typedef unsigned __int32    __pyx_uintptr_t;
        #endif
    #endif
#else
    #include <stdint.h>
    typedef uintptr_t  __pyx_uintptr_t;
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus)
    /* for clang __has_cpp_attribute(fallthrough) is true even before C++17
     * but leads to warnings with -pedantic, since it is a C++17 feature */
    #if ((defined(_MSVC_LANG) && _MSVC_LANG >= 201703L) || __cplusplus >= 201703L)
      #if __has_cpp_attribute(fallthrough)
        #define CYTHON_FALLTHROUGH [[fallthrough]]
      #endif
    #endif
    #ifndef CYTHON_FALLTHROUGH
      #if __has_cpp_attribute(clang::fallthrough)
        #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
      #elif __has_cpp_attribute(gnu::fallthrough)
        #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
      #endif
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif
#ifdef __cplusplus
  template <typename T>
  struct __PYX_IS_UNSIGNED_IMPL {static const bool value = T(0) < T(-1);};
  #define __PYX_IS_UNSIGNED(type) (__PYX_IS_UNSIGNED_IMPL<type>::value)
#else
  #define __PYX_IS_UNSIGNED(type) (((type)-1) > 0)
#endif
#if CYTHON_COMPILING_IN_PYPY == 1
  #define __PYX_NEED_TP_PRINT_SLOT  (PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x030A0000)
#else
  #define __PYX_NEED_TP_PRINT_SLOT  (PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000)
#endif
#define __PYX_REINTERPRET_FUNCION(func_pointer, other_pointer) ((func_pointer)(void(*)(void))(other_pointer))

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_DefaultClassType PyClass_Type
  #define __Pyx_PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_DefaultClassType PyType_Type
#if CYTHON_COMPILING_IN_LIMITED_API
    static CYTHON_INLINE PyObject* __Pyx_PyCode_New(int a, int p, int k, int l, int s, int f,
                                                    PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                                    PyObject *fv, PyObject *cell, PyObject* fn,
                                                    PyObject *name, int fline, PyObject *lnos) {
        PyObject *exception_table = NULL;
        PyObject *types_module=NULL, *code_type=NULL, *result=NULL;
        #if __PYX_LIMITED_VERSION_HEX < 0x030B0000
        PyObject *version_info;
        PyObject *py_minor_version = NULL;
        #endif
        long minor_version = 0;
        PyObject *type, *value, *traceback;
        PyErr_Fetch(&type, &value, &traceback);
        #if __PYX_LIMITED_VERSION_HEX >= 0x030B0000
        minor_version = 11;
        #else
        if (!(version_info = PySys_GetObject("version_info"))) goto end;
        if (!(py_minor_version = PySequence_GetItem(version_info, 1))) goto end;
        minor_version = PyLong_AsLong(py_minor_version);
        Py_DECREF(py_minor_version);
        if (minor_version == -1 && PyErr_Occurred()) goto end;
        #endif
        if (!(types_module = PyImport_ImportModule("types"))) goto end;
        if (!(code_type = PyObject_GetAttrString(types_module, "CodeType"))) goto end;
        if (minor_version <= 7) {
            (void)p;
            result = PyObject_CallFunction(code_type, "iiiiiOOOOOOiOO", a, k, l, s, f, code,
                          c, n, v, fn, name, fline, lnos, fv, cell);
        } else if (minor_version <= 10) {
            result = PyObject_CallFunction(code_type, "iiiiiiOOOOOOiOO", a,p, k, l, s, f, code,
                          c, n, v, fn, name, fline, lnos, fv, cell);
        } else {
            if (!(exception_table = PyBytes_FromStringAndSize(NULL, 0))) goto end;
            result = PyObject_CallFunction(code_type, "iiiiiiOOOOOOOiOO", a,p, k, l, s, f, code,
                          c, n, v, fn, name, name, fline, lnos, exception_table, fv, cell);
        }
    end:
        Py_XDECREF(code_type);
        Py_XDECREF(exception_table);
        Py_XDECREF(types_module);
        if (type) {
            PyErr_Restore(type, value, traceback);
        }
        return result;
    }
    #ifndef CO_OPTIMIZED
    #define CO_OPTIMIZED 0x0001
    #endif
    #ifndef CO_NEWLOCALS
    #define CO_NEWLOCALS 0x0002
    #endif
    #ifndef CO_VARARGS
    #define CO_VARARGS 0x0004
    #endif
    #ifndef CO_VARKEYWORDS
    #define CO_VARKEYWORDS 0x0008
    #endif
    #ifndef CO_ASYNC_GENERATOR
    #define CO_ASYNC_GENERATOR 0x0200
    #endif
    #ifndef CO_GENERATOR
    #define CO_GENERATOR 0x0020
    #endif
    #ifndef CO_COROUTINE
    #define CO_COROUTINE 0x0080
    #endif
#elif PY_VERSION_HEX >= 0x030B0000
  static CYTHON_INLINE PyCodeObject* __Pyx_PyCode_New(int a, int p, int k, int l, int s, int f,
                                                    PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                                    PyObject *fv, PyObject *cell, PyObject* fn,
                                                    PyObject *name, int fline, PyObject *lnos) {
    PyCodeObject *result;
    PyObject *empty_bytes = PyBytes_FromStringAndSize("", 0);
    if (!empty_bytes) return NULL;
    result =
      #if PY_VERSION_HEX >= 0x030C0000
        PyUnstable_Code_NewWithPosOnlyArgs
      #else
        PyCode_NewWithPosOnlyArgs
      #endif
        (a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, name, fline, lnos, empty_bytes);
    Py_DECREF(empty_bytes);
    return result;
  }
#elif PY_VERSION_HEX >= 0x030800B2 && !CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_NewWithPosOnlyArgs(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx_PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
#endif
#if PY_VERSION_HEX >= 0x030900A4 || defined(Py_IS_TYPE)
  #define __Pyx_IS_TYPE(ob, type) Py_IS_TYPE(ob, type)
#else
  #define __Pyx_IS_TYPE(ob, type) (((const PyObject*)ob)->ob_type == (type))
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_Is)
  #define __Pyx_Py_Is(x, y)  Py_Is(x, y)
#else
  #define __Pyx_Py_Is(x, y) ((x) == (y))
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsNone)
  #define __Pyx_Py_IsNone(ob) Py_IsNone(ob)
#else
  #define __Pyx_Py_IsNone(ob) __Pyx_Py_Is((ob), Py_None)
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsTrue)
  #define __Pyx_Py_IsTrue(ob) Py_IsTrue(ob)
#else
  #define __Pyx_Py_IsTrue(ob) __Pyx_Py_Is((ob), Py_True)
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsFalse)
  #define __Pyx_Py_IsFalse(ob) Py_IsFalse(ob)
#else
  #define __Pyx_Py_IsFalse(ob) __Pyx_Py_Is((ob), Py_False)
#endif
#define __Pyx_NoneAsNull(obj)  (__Pyx_Py_IsNone(obj) ? NULL : (obj))
#if PY_VERSION_HEX >= 0x030900F0 && !CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyObject_GC_IsFinalized(o) PyObject_GC_IsFinalized(o)
#else
  #define __Pyx_PyObject_GC_IsFinalized(o) _PyGC_FINALIZED(o)
#endif
#ifndef CO_COROUTINE
  #define CO_COROUTINE 0x80
#endif
#ifndef CO_ASYNC_GENERATOR
  #define CO_ASYNC_GENERATOR 0x200
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef Py_TPFLAGS_SEQUENCE
  #define Py_TPFLAGS_SEQUENCE 0
#endif
#ifndef Py_TPFLAGS_MAPPING
  #define Py_TPFLAGS_MAPPING 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #if PY_VERSION_HEX >= 0x030d00A4
  #  define __Pyx_PyCFunctionFast PyCFunctionFast
  #  define __Pyx_PyCFunctionFastWithKeywords PyCFunctionFastWithKeywords
  #else
  #  define __Pyx_PyCFunctionFast _PyCFunctionFast
  #  define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
  #endif
#endif
#if CYTHON_METH_FASTCALL
  #define __Pyx_METH_FASTCALL METH_FASTCALL
  #define __Pyx_PyCFunction_FastCall __Pyx_PyCFunctionFast
  #define __Pyx_PyCFunction_FastCallWithKeywords __Pyx_PyCFunctionFastWithKeywords
#else
  #define __Pyx_METH_FASTCALL METH_VARARGS
  #define __Pyx_PyCFunction_FastCall PyCFunction
  #define __Pyx_PyCFunction_FastCallWithKeywords PyCFunctionWithKeywords
#endif
#if CYTHON_VECTORCALL
  #define __pyx_vectorcallfunc vectorcallfunc
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  PY_VECTORCALL_ARGUMENTS_OFFSET
  #define __Pyx_PyVectorcall_NARGS(n)  PyVectorcall_NARGS((size_t)(n))
#elif CYTHON_BACKPORT_VECTORCALL
  typedef PyObject *(*__pyx_vectorcallfunc)(PyObject *callable, PyObject *const *args,
                                            size_t nargsf, PyObject *kwnames);
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  ((size_t)1 << (8 * sizeof(size_t) - 1))
  #define __Pyx_PyVectorcall_NARGS(n)  ((Py_ssize_t)(((size_t)(n)) & ~__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET))
#else
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  0
  #define __Pyx_PyVectorcall_NARGS(n)  ((Py_ssize_t)(n))
#endif
#if PY_MAJOR_VERSION >= 0x030900B1
#define __Pyx_PyCFunction_CheckExact(func)  PyCFunction_CheckExact(func)
#else
#define __Pyx_PyCFunction_CheckExact(func)  PyCFunction_Check(func)
#endif
#define __Pyx_CyOrPyCFunction_Check(func)  PyCFunction_Check(func)
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CyOrPyCFunction_GET_FUNCTION(func)  (((PyCFunctionObject*)(func))->m_ml->ml_meth)
#elif !CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyOrPyCFunction_GET_FUNCTION(func)  PyCFunction_GET_FUNCTION(func)
#endif
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CyOrPyCFunction_GET_FLAGS(func)  (((PyCFunctionObject*)(func))->m_ml->ml_flags)
static CYTHON_INLINE PyObject* __Pyx_CyOrPyCFunction_GET_SELF(PyObject *func) {
    return (__Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_STATIC) ? NULL : ((PyCFunctionObject*)func)->m_self;
}
#endif
static CYTHON_INLINE int __Pyx__IsSameCFunction(PyObject *func, void *cfunc) {
#if CYTHON_COMPILING_IN_LIMITED_API
    return PyCFunction_Check(func) && PyCFunction_GetFunction(func) == (PyCFunction) cfunc;
#else
    return PyCFunction_Check(func) && PyCFunction_GET_FUNCTION(func) == (PyCFunction) cfunc;
#endif
}
#define __Pyx_IsSameCFunction(func, cfunc)   __Pyx__IsSameCFunction(func, cfunc)
#if __PYX_LIMITED_VERSION_HEX < 0x030900B1
  #define __Pyx_PyType_FromModuleAndSpec(m, s, b)  ((void)m, PyType_FromSpecWithBases(s, b))
  typedef PyObject *(*__Pyx_PyCMethod)(PyObject *, PyTypeObject *, PyObject *const *, size_t, PyObject *);
#else
  #define __Pyx_PyType_FromModuleAndSpec(m, s, b)  PyType_FromModuleAndSpec(m, s, b)
  #define __Pyx_PyCMethod  PyCMethod
#endif
#ifndef METH_METHOD
  #define METH_METHOD 0x200
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyThreadState_Current PyThreadState_Get()
#elif !CYTHON_FAST_THREAD_STATE
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x030d00A1
  #define __Pyx_PyThreadState_Current PyThreadState_GetUnchecked()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE void *__Pyx_PyModule_GetState(PyObject *op)
{
    void *result;
    result = PyModule_GetState(op);
    if (!result)
        Py_FatalError("Couldn't find the module state");
    return result;
}
#endif
#define __Pyx_PyObject_GetSlot(obj, name, func_ctype)  __Pyx_PyType_GetSlot(Py_TYPE(obj), name, func_ctype)
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyType_GetSlot(type, name, func_ctype)  ((func_ctype) PyType_GetSlot((type), Py_##name))
#else
  #define __Pyx_PyType_GetSlot(type, name, func_ctype)  ((type)->name)
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if PY_MAJOR_VERSION < 3
    #if CYTHON_COMPILING_IN_PYPY
        #if PYPY_VERSION_NUM < 0x07030600
            #if defined(__cplusplus) && __cplusplus >= 201402L
                [[deprecated("`with nogil:` inside a nogil function will not release the GIL in PyPy2 < 7.3.6")]]
            #elif defined(__GNUC__) || defined(__clang__)
                __attribute__ ((__deprecated__("`with nogil:` inside a nogil function will not release the GIL in PyPy2 < 7.3.6")))
            #elif defined(_MSC_VER)
                __declspec(deprecated("`with nogil:` inside a nogil function will not release the GIL in PyPy2 < 7.3.6"))
            #endif
            static CYTHON_INLINE int PyGILState_Check(void) {
                return 0;
            }
        #else  // PYPY_VERSION_NUM < 0x07030600
        #endif  // PYPY_VERSION_NUM < 0x07030600
    #else
        static CYTHON_INLINE int PyGILState_Check(void) {
            PyThreadState * tstate = _PyThreadState_Current;
            return tstate && (tstate == PyGILState_GetThisThreadState());
        }
    #endif
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030d0000 || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX > 0x030600B4 && PY_VERSION_HEX < 0x030d0000 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStrWithError(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
static CYTHON_INLINE PyObject * __Pyx_PyDict_GetItemStr(PyObject *dict, PyObject *name) {
    PyObject *res = __Pyx_PyDict_GetItemStrWithError(dict, name);
    if (res == NULL) PyErr_Clear();
    return res;
}
#elif PY_MAJOR_VERSION >= 3 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07020000)
#define __Pyx_PyDict_GetItemStrWithError  PyDict_GetItemWithError
#define __Pyx_PyDict_GetItemStr           PyDict_GetItem
#else
static CYTHON_INLINE PyObject * __Pyx_PyDict_GetItemStrWithError(PyObject *dict, PyObject *name) {
#if CYTHON_COMPILING_IN_PYPY
    return PyDict_GetItem(dict, name);
#else
    PyDictEntry *ep;
    PyDictObject *mp = (PyDictObject*) dict;
    long hash = ((PyStringObject *) name)->ob_shash;
    assert(hash != -1);
    ep = (mp->ma_lookup)(mp, name, hash);
    if (ep == NULL) {
        return NULL;
    }
    return ep->me_value;
#endif
}
#define __Pyx_PyDict_GetItemStr           PyDict_GetItem
#endif
#if CYTHON_USE_TYPE_SLOTS
  #define __Pyx_PyType_GetFlags(tp)   (((PyTypeObject *)tp)->tp_flags)
  #define __Pyx_PyType_HasFeature(type, feature)  ((__Pyx_PyType_GetFlags(type) & (feature)) != 0)
  #define __Pyx_PyObject_GetIterNextFunc(obj)  (Py_TYPE(obj)->tp_iternext)
#else
  #define __Pyx_PyType_GetFlags(tp)   (PyType_GetFlags((PyTypeObject *)tp))
  #define __Pyx_PyType_HasFeature(type, feature)  PyType_HasFeature(type, feature)
  #define __Pyx_PyObject_GetIterNextFunc(obj)  PyIter_Next
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_SetItemOnTypeDict(tp, k, v) PyObject_GenericSetAttr((PyObject*)tp, k, v)
#else
  #define __Pyx_SetItemOnTypeDict(tp, k, v) PyDict_SetItem(tp->tp_dict, k, v)
#endif
#if CYTHON_USE_TYPE_SPECS && PY_VERSION_HEX >= 0x03080000
#define __Pyx_PyHeapTypeObject_GC_Del(obj)  {\
    PyTypeObject *type = Py_TYPE((PyObject*)obj);\
    assert(__Pyx_PyType_HasFeature(type, Py_TPFLAGS_HEAPTYPE));\
    PyObject_GC_Del(obj);\
    Py_DECREF(type);\
}
#else
#define __Pyx_PyHeapTypeObject_GC_Del(obj)  PyObject_GC_Del(obj)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GetLength(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_ReadChar(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((void)u, 1114111U)
  #define __Pyx_PyUnicode_KIND(u)         ((void)u, (0))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)u)
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)k, PyUnicode_ReadChar((PyObject*)(d), i))
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GetLength(u))
#elif PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_READY(op)       (0)
  #else
    #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                                0 : _PyUnicode_Ready((PyObject *)(op)))
  #endif
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         ((int)PyUnicode_KIND(u))
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, (Py_UCS4) ch)
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #else
    #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
    #else
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
    #endif
  #endif
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535U : 1114111U)
  #define __Pyx_PyUnicode_KIND(u)         ((int)sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = (Py_UNICODE) ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #if !defined(PyUnicode_DecodeUnicodeEscape)
    #define PyUnicode_DecodeUnicodeEscape(s, size, errors)  PyUnicode_Decode(s, size, "unicode_escape", errors)
  #endif
  #if !defined(PyUnicode_Contains) || (PY_MAJOR_VERSION == 2 && PYPY_VERSION_NUM < 0x07030500)
    #undef PyUnicode_Contains
    #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
  #endif
  #if !defined(PyByteArray_Check)
    #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
  #endif
  #if !defined(PyObject_Format)
    #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
  #endif
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#if CYTHON_COMPILING_IN_CPYTHON
  #define __Pyx_PySequence_ListKeepNew(obj)\
    (likely(PyList_CheckExact(obj) && Py_REFCNT(obj) == 1) ? __Pyx_NewRef(obj) : PySequence_List(obj))
#else
  #define __Pyx_PySequence_ListKeepNew(obj)  PySequence_List(obj)
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        __Pyx_IS_TYPE(obj, &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_ITEM(o, i) PySequence_ITEM(o, i)
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
  #define __Pyx_PyTuple_SET_ITEM(o, i, v) (PyTuple_SET_ITEM(o, i, v), (0))
  #define __Pyx_PyList_SET_ITEM(o, i, v) (PyList_SET_ITEM(o, i, v), (0))
  #define __Pyx_PyTuple_GET_SIZE(o) PyTuple_GET_SIZE(o)
  #define __Pyx_PyList_GET_SIZE(o) PyList_GET_SIZE(o)
  #define __Pyx_PySet_GET_SIZE(o) PySet_GET_SIZE(o)
  #define __Pyx_PyBytes_GET_SIZE(o) PyBytes_GET_SIZE(o)
  #define __Pyx_PyByteArray_GET_SIZE(o) PyByteArray_GET_SIZE(o)
#else
  #define __Pyx_PySequence_ITEM(o, i) PySequence_GetItem(o, i)
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
  #define __Pyx_PyTuple_SET_ITEM(o, i, v) PyTuple_SetItem(o, i, v)
  #define __Pyx_PyList_SET_ITEM(o, i, v) PyList_SetItem(o, i, v)
  #define __Pyx_PyTuple_GET_SIZE(o) PyTuple_Size(o)
  #define __Pyx_PyList_GET_SIZE(o) PyList_Size(o)
  #define __Pyx_PySet_GET_SIZE(o) PySet_Size(o)
  #define __Pyx_PyBytes_GET_SIZE(o) PyBytes_Size(o)
  #define __Pyx_PyByteArray_GET_SIZE(o) PyByteArray_Size(o)
#endif
#if __PYX_LIMITED_VERSION_HEX >= 0x030d00A1
  #define __Pyx_PyImport_AddModuleRef(name) PyImport_AddModuleRef(name)
#else
  static CYTHON_INLINE PyObject *__Pyx_PyImport_AddModuleRef(const char *name) {
      PyObject *module = PyImport_AddModule(name);
      Py_XINCREF(module);
      return module;
  }
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define __Pyx_Py3Int_Check(op)       PyLong_Check(op)
  #define __Pyx_Py3Int_CheckExact(op)  PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#else
  #define __Pyx_Py3Int_Check(op)       (PyLong_Check(op) || PyInt_Check(op))
  #define __Pyx_Py3Int_CheckExact(op)  (PyLong_CheckExact(op) || PyInt_CheckExact(op))
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsHash_t
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsSsize_t
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)
  #if !defined(_USE_MATH_DEFINES)
    #define _USE_MATH_DEFINES
  #endif
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__;  (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifdef CYTHON_EXTERN_C
    #undef __PYX_EXTERN_C
    #define __PYX_EXTERN_C CYTHON_EXTERN_C
#elif defined(__PYX_EXTERN_C)
    #ifdef _MSC_VER
    #pragma message ("Please do not define the '__PYX_EXTERN_C' macro externally. Use 'CYTHON_EXTERN_C' instead.")
    #else
    #warning Please do not define the '__PYX_EXTERN_C' macro externally. Use 'CYTHON_EXTERN_C' instead.
    #endif
#else
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__CyEulerBallistic__trajectory_calc
#define __PYX_HAVE_API__CyEulerBallistic__trajectory_calc
/* Early includes */
#include <math.h>
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE Py_ssize_t __Pyx_ssize_strlen(const char *s);
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
static CYTHON_INLINE PyObject* __Pyx_PyByteArray_FromString(const char*);
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
#define __Pyx_PyUnicode_FromOrdinal(o)       PyUnicode_FromOrdinal((int)o)
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject*);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_VERSION_HEX >= 0x030C00A7
  #ifndef _PyLong_SIGN_MASK
    #define _PyLong_SIGN_MASK 3
  #endif
  #ifndef _PyLong_NON_SIZE_BITS
    #define _PyLong_NON_SIZE_BITS 3
  #endif
  #define __Pyx_PyLong_Sign(x)  (((PyLongObject*)x)->long_value.lv_tag & _PyLong_SIGN_MASK)
  #define __Pyx_PyLong_IsNeg(x)  ((__Pyx_PyLong_Sign(x) & 2) != 0)
  #define __Pyx_PyLong_IsNonNeg(x)  (!__Pyx_PyLong_IsNeg(x))
  #define __Pyx_PyLong_IsZero(x)  (__Pyx_PyLong_Sign(x) & 1)
  #define __Pyx_PyLong_IsPos(x)  (__Pyx_PyLong_Sign(x) == 0)
  #define __Pyx_PyLong_CompactValueUnsigned(x)  (__Pyx_PyLong_Digits(x)[0])
  #define __Pyx_PyLong_DigitCount(x)  ((Py_ssize_t) (((PyLongObject*)x)->long_value.lv_tag >> _PyLong_NON_SIZE_BITS))
  #define __Pyx_PyLong_SignedDigitCount(x)\
        ((1 - (Py_ssize_t) __Pyx_PyLong_Sign(x)) * __Pyx_PyLong_DigitCount(x))
  #if defined(PyUnstable_Long_IsCompact) && defined(PyUnstable_Long_CompactValue)
    #define __Pyx_PyLong_IsCompact(x)     PyUnstable_Long_IsCompact((PyLongObject*) x)
    #define __Pyx_PyLong_CompactValue(x)  PyUnstable_Long_CompactValue((PyLongObject*) x)
  #else
    #define __Pyx_PyLong_IsCompact(x)     (((PyLongObject*)x)->long_value.lv_tag < (2 << _PyLong_NON_SIZE_BITS))
    #define __Pyx_PyLong_CompactValue(x)  ((1 - (Py_ssize_t) __Pyx_PyLong_Sign(x)) * (Py_ssize_t) __Pyx_PyLong_Digits(x)[0])
  #endif
  typedef Py_ssize_t  __Pyx_compact_pylong;
  typedef size_t  __Pyx_compact_upylong;
  #else
  #define __Pyx_PyLong_IsNeg(x)  (Py_SIZE(x) < 0)
  #define __Pyx_PyLong_IsNonNeg(x)  (Py_SIZE(x) >= 0)
  #define __Pyx_PyLong_IsZero(x)  (Py_SIZE(x) == 0)
  #define __Pyx_PyLong_IsPos(x)  (Py_SIZE(x) > 0)
  #define __Pyx_PyLong_CompactValueUnsigned(x)  ((Py_SIZE(x) == 0) ? 0 : __Pyx_PyLong_Digits(x)[0])
  #define __Pyx_PyLong_DigitCount(x)  __Pyx_sst_abs(Py_SIZE(x))
  #define __Pyx_PyLong_SignedDigitCount(x)  Py_SIZE(x)
  #define __Pyx_PyLong_IsCompact(x)  (Py_SIZE(x) == 0 || Py_SIZE(x) == 1 || Py_SIZE(x) == -1)
  #define __Pyx_PyLong_CompactValue(x)\
        ((Py_SIZE(x) == 0) ? (sdigit) 0 : ((Py_SIZE(x) < 0) ? -(sdigit)__Pyx_PyLong_Digits(x)[0] : (sdigit)__Pyx_PyLong_Digits(x)[0]))
  typedef sdigit  __Pyx_compact_pylong;
  typedef digit  __Pyx_compact_upylong;
  #endif
  #if PY_VERSION_HEX >= 0x030C00A5
  #define __Pyx_PyLong_Digits(x)  (((PyLongObject*)x)->long_value.ob_digit)
  #else
  #define __Pyx_PyLong_Digits(x)  (((PyLongObject*)x)->ob_digit)
  #endif
#endif
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
#include <string.h>
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = (char) c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#include <string.h>
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

#if !CYTHON_USE_MODULE_STATE
static PyObject *__pyx_m = NULL;
#endif
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm = __FILE__;
static const char *__pyx_filename;

/* #### Code section: filename_table ### */

static const char *__pyx_f[] = {
  "<stringsource>",
  "CyEulerBallistic/trajectory_calc.pyx",
  "CyEulerBallistic/vector.pxd",
  "CyEulerBallistic/trajectory_data.pxd",
};
/* #### Code section: utility_code_proto_before_types ### */
/* ForceInitThreads.proto */
#ifndef __PYX_FORCE_INIT_THREADS
  #define __PYX_FORCE_INIT_THREADS 0
#endif

/* #### Code section: numeric_typedefs ### */
/* #### Code section: complex_type_declarations ### */
/* #### Code section: type_declarations ### */

/*--- Type declarations ---*/
struct __pyx_obj_16CyEulerBallistic_6vector_Vector;
struct __pyx_obj_16CyEulerBallistic_15trajectory_data_TrajectoryData;
struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter;
struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock;
struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc;
struct __pyx_t_16CyEulerBallistic_6vector_CVector;

/* "CyEulerBallistic/vector.pxd":1
 * cdef struct CVector:             # <<<<<<<<<<<<<<
 *     double x, y, z
 * 
 */
struct __pyx_t_16CyEulerBallistic_6vector_CVector {
  double x;
  double y;
  double z;
};

/* "CyEulerBallistic/trajectory_data.pxd":1
 * cdef enum CTrajFlag:             # <<<<<<<<<<<<<<
 *     NONE = 0
 *     ZERO_UP = 1
 */
enum __pyx_t_16CyEulerBallistic_15trajectory_data_CTrajFlag {

  /* "CyEulerBallistic/trajectory_data.pxd":9
 *     DANGER = 16
 *     ZERO = ZERO_UP | ZERO_DOWN
 *     ALL = RANGE | ZERO_UP | ZERO_DOWN | MACH | DANGER             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_e_16CyEulerBallistic_15trajectory_data_NONE = 0,
  __pyx_e_16CyEulerBallistic_15trajectory_data_ZERO_UP = 1,
  __pyx_e_16CyEulerBallistic_15trajectory_data_ZERO_DOWN = 2,
  __pyx_e_16CyEulerBallistic_15trajectory_data_MACH = 4,
  __pyx_e_16CyEulerBallistic_15trajectory_data_RANGE = 8,
  __pyx_e_16CyEulerBallistic_15trajectory_data_DANGER = 16,
  __pyx_e_16CyEulerBallistic_15trajectory_data_ZERO = (__pyx_e_16CyEulerBallistic_15trajectory_data_ZERO_UP | __pyx_e_16CyEulerBallistic_15trajectory_data_ZERO_DOWN),
  __pyx_e_16CyEulerBallistic_15trajectory_data_ALL = ((((__pyx_e_16CyEulerBallistic_15trajectory_data_RANGE | __pyx_e_16CyEulerBallistic_15trajectory_data_ZERO_UP) | __pyx_e_16CyEulerBallistic_15trajectory_data_ZERO_DOWN) | __pyx_e_16CyEulerBallistic_15trajectory_data_MACH) | __pyx_e_16CyEulerBallistic_15trajectory_data_DANGER)
};
struct __pyx_t_16CyEulerBallistic_8cy_euler_Config_t;
struct __pyx_t_16CyEulerBallistic_8cy_euler_CurvePoint_t;
struct __pyx_t_16CyEulerBallistic_8cy_euler_Curve_t;
struct __pyx_t_16CyEulerBallistic_8cy_euler_MachList_t;
struct __pyx_t_16CyEulerBallistic_8cy_euler_Atmosphere_t;
struct __pyx_t_16CyEulerBallistic_8cy_euler_ShotData_t;
struct __pyx_opt_args_16CyEulerBallistic_8cy_euler_cy_get_calc_step;
struct __pyx_t_16CyEulerBallistic_8cy_euler_Wind_t;

/* "CyEulerBallistic/cy_euler.pxd":3
 * from CyEulerBallistic.vector cimport CVector
 * 
 * cdef struct Config_t:             # <<<<<<<<<<<<<<
 *     double max_calc_step_size_feet
 *     double chart_resolution
 */
struct __pyx_t_16CyEulerBallistic_8cy_euler_Config_t {
  double max_calc_step_size_feet;
  double chart_resolution;
  double cZeroFindingAccuracy;
  double cMinimumVelocity;
  double cMaximumDrop;
  int cMaxIterations;
  double cGravityConstant;
  double cMinimumAltitude;
};

/* "CyEulerBallistic/cy_euler.pxd":15
 * cdef Config_t config_bind(object config)
 * 
 * cdef struct CurvePoint_t:             # <<<<<<<<<<<<<<
 *     double a, b, c
 * 
 */
struct __pyx_t_16CyEulerBallistic_8cy_euler_CurvePoint_t {
  double a;
  double b;
  double c;
};

/* "CyEulerBallistic/cy_euler.pxd":18
 *     double a, b, c
 * 
 * cdef struct Curve_t:             # <<<<<<<<<<<<<<
 *     CurvePoint_t * points
 *     double length
 */
struct __pyx_t_16CyEulerBallistic_8cy_euler_Curve_t {
  struct __pyx_t_16CyEulerBallistic_8cy_euler_CurvePoint_t *points;
  double length;
};

/* "CyEulerBallistic/cy_euler.pxd":22
 *     double length
 * 
 * cdef struct MachList_t:             # <<<<<<<<<<<<<<
 *     double * array
 *     double length
 */
struct __pyx_t_16CyEulerBallistic_8cy_euler_MachList_t {
  double *array;
  double length;
};

/* "CyEulerBallistic/cy_euler.pxd":26
 *     double length
 * 
 * cdef struct Atmosphere_t:             # <<<<<<<<<<<<<<
 *     double _t0
 *     double _a0
 */
struct __pyx_t_16CyEulerBallistic_8cy_euler_Atmosphere_t {
  double _t0;
  double _a0;
  double _p0;
  double _mach1;
  double density_ratio;
};

/* "CyEulerBallistic/cy_euler.pxd":37
 * )
 * 
 * cdef struct ShotData_t:             # <<<<<<<<<<<<<<
 *     double bc
 *     Curve_t curve
 */
struct __pyx_t_16CyEulerBallistic_8cy_euler_ShotData_t {
  double bc;
  struct __pyx_t_16CyEulerBallistic_8cy_euler_Curve_t curve;
  struct __pyx_t_16CyEulerBallistic_8cy_euler_MachList_t mach_list;
  double look_angle;
  double twist;
  double length;
  double diameter;
  double weight;
  double barrel_elevation;
  double barrel_azimuth;
  double sight_height;
  double cant_cosine;
  double cant_sine;
  double alt0;
  double calc_step;
  double muzzle_velocity;
  double stability_coefficient;
  struct __pyx_t_16CyEulerBallistic_8cy_euler_Atmosphere_t atmo;
};

/* "CyEulerBallistic/cy_euler.pxd":57
 *     Atmosphere_t atmo
 * 
 * cdef double cy_get_calc_step(Config_t * config, double step = ?)             # <<<<<<<<<<<<<<
 * cdef MachList_t cy_table_to_mach(list[object] data)
 * cdef Curve_t cy_calculate_curve(list[object] data_points)
 */
struct __pyx_opt_args_16CyEulerBallistic_8cy_euler_cy_get_calc_step {
  int __pyx_n;
  double step;
};

/* "CyEulerBallistic/cy_euler.pxd":77
 * cdef double cLapseRateImperial
 * 
 * cdef struct Wind_t:             # <<<<<<<<<<<<<<
 *     double velocity
 *     double direction_from
 */
struct __pyx_t_16CyEulerBallistic_8cy_euler_Wind_t {
  double velocity;
  double direction_from;
  double until_distance;
  double MAX_DISTANCE_FEET;
};
struct __pyx_opt_args_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_get_calc_step;
struct __pyx_opt_args_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc__integrate;

/* "CyEulerBallistic/trajectory_calc.pyx":203
 *     #     free_trajectory(&self.__shot)
 * 
 *     cdef double get_calc_step(self, double step = 0):             # <<<<<<<<<<<<<<
 *         cdef double preferred_step = self.__config.max_calc_step_size_feet
 *         # cdef double defined_max = 0.5  # const will be better optimized with cython
 */
struct __pyx_opt_args_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_get_calc_step {
  int __pyx_n;
  double step;
};

/* "CyEulerBallistic/trajectory_calc.pyx":315
 * 
 * 
 *     cdef list[object] _integrate(EulerTrajectoryCalc self,             # <<<<<<<<<<<<<<
 *                           double maximum_range, double step, int filter_flags, double time_step = 0.0):
 *         cdef:
 */
struct __pyx_opt_args_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc__integrate {
  int __pyx_n;
  double time_step;
};

/* "CyEulerBallistic/vector.pxd":5
 * 
 * 
 * cdef class Vector:             # <<<<<<<<<<<<<<
 *     cdef double _x
 *     cdef double _y
 */
struct __pyx_obj_16CyEulerBallistic_6vector_Vector {
  PyObject_HEAD
  struct __pyx_vtabstruct_16CyEulerBallistic_6vector_Vector *__pyx_vtab;
  double _x;
  double _y;
  double _z;
};


/* "CyEulerBallistic/trajectory_data.pxd":12
 * 
 * 
 * cdef class TrajectoryData:             # <<<<<<<<<<<<<<
 *     cdef:
 *         readonly double time
 */
struct __pyx_obj_16CyEulerBallistic_15trajectory_data_TrajectoryData {
  PyObject_HEAD
  double time;
  PyObject *distance;
  PyObject *velocity;
  double mach;
  PyObject *height;
  PyObject *target_drop;
  PyObject *drop_adj;
  PyObject *windage;
  PyObject *windage_adj;
  PyObject *look_distance;
  PyObject *angle;
  double density_factor;
  double drag;
  PyObject *energy;
  PyObject *ogw;
  int flag;
};


/* "CyEulerBallistic/trajectory_calc.pyx":51
 * 
 * @final
 * cdef class _TrajectoryDataFilter:             # <<<<<<<<<<<<<<
 *     cdef:
 *         int filter, current_flag, seen_zero
 */
struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter {
  PyObject_HEAD
  struct __pyx_vtabstruct_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *__pyx_vtab;
  int filter;
  int current_flag;
  int seen_zero;
  int current_item;
  int ranges_length;
  double previous_mach;
  double previous_time;
  double next_range_distance;
  double time_step;
  double look_angle;
};


/* "CyEulerBallistic/trajectory_calc.pyx":137
 * 
 * @final
 * cdef class _WindSock:             # <<<<<<<<<<<<<<
 *     cdef:
 *         list[Wind_t] winds
 */
struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock {
  PyObject_HEAD
  struct __pyx_vtabstruct_16CyEulerBallistic_15trajectory_calc__WindSock *__pyx_vtab;
  PyObject *winds;
  int current;
  double next_range;
  struct __pyx_t_16CyEulerBallistic_6vector_CVector _last_vector_cache;
  int _length;
};


/* "CyEulerBallistic/trajectory_calc.pyx":187
 * 
 * 
 * cdef class EulerTrajectoryCalc:             # <<<<<<<<<<<<<<
 *     cdef:
 *         list[object] _table_data
 */
struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc {
  PyObject_HEAD
  struct __pyx_vtabstruct_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *__pyx_vtab;
  PyObject *_table_data;
  struct __pyx_t_16CyEulerBallistic_6vector_CVector gravity_vector;
  PyObject *_config;
  struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock *ws;
  struct __pyx_t_16CyEulerBallistic_8cy_euler_Config_t _EulerTrajectoryCalc__config;
  struct __pyx_t_16CyEulerBallistic_8cy_euler_ShotData_t _EulerTrajectoryCalc__shot;
};



/* "CyEulerBallistic/vector.pxd":5
 * 
 * 
 * cdef class Vector:             # <<<<<<<<<<<<<<
 *     cdef double _x
 *     cdef double _y
 */

struct __pyx_vtabstruct_16CyEulerBallistic_6vector_Vector {
  double (*_magnitude)(struct __pyx_obj_16CyEulerBallistic_6vector_Vector *);
  struct __pyx_obj_16CyEulerBallistic_6vector_Vector *(*_mul_by_const)(struct __pyx_obj_16CyEulerBallistic_6vector_Vector *, double);
  double (*_mul_by_vector)(struct __pyx_obj_16CyEulerBallistic_6vector_Vector *, struct __pyx_obj_16CyEulerBallistic_6vector_Vector *);
  struct __pyx_obj_16CyEulerBallistic_6vector_Vector *(*_add)(struct __pyx_obj_16CyEulerBallistic_6vector_Vector *, struct __pyx_obj_16CyEulerBallistic_6vector_Vector *);
  struct __pyx_obj_16CyEulerBallistic_6vector_Vector *(*_subtract)(struct __pyx_obj_16CyEulerBallistic_6vector_Vector *, struct __pyx_obj_16CyEulerBallistic_6vector_Vector *);
  struct __pyx_obj_16CyEulerBallistic_6vector_Vector *(*_negate)(struct __pyx_obj_16CyEulerBallistic_6vector_Vector *);
  struct __pyx_obj_16CyEulerBallistic_6vector_Vector *(*_normalize)(struct __pyx_obj_16CyEulerBallistic_6vector_Vector *);
  struct __pyx_t_16CyEulerBallistic_6vector_CVector (*c_vector)(struct __pyx_obj_16CyEulerBallistic_6vector_Vector *);
};
static struct __pyx_vtabstruct_16CyEulerBallistic_6vector_Vector *__pyx_vtabptr_16CyEulerBallistic_6vector_Vector;


/* "CyEulerBallistic/trajectory_calc.pyx":51
 * 
 * @final
 * cdef class _TrajectoryDataFilter:             # <<<<<<<<<<<<<<
 *     cdef:
 *         int filter, current_flag, seen_zero
 */

struct __pyx_vtabstruct_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter {
  void (*setup_seen_zero)(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *, double, double, double);
  void (*clear_current_flag)(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *);
  int (*should_record)(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *, struct __pyx_t_16CyEulerBallistic_6vector_CVector, double, double, double, double);
  int (*should_break)(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *);
  int (*check_next_range)(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *, double, double);
  void (*check_next_time)(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *, double);
  void (*check_mach_crossing)(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *, double, double);
  void (*check_zero_crossing)(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *, struct __pyx_t_16CyEulerBallistic_6vector_CVector);
};
static struct __pyx_vtabstruct_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *__pyx_vtabptr_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter;
static void __pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_setup_seen_zero(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *, double, double, double);
static void __pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_clear_current_flag(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *);
static int __pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_should_record(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *, struct __pyx_t_16CyEulerBallistic_6vector_CVector, double, double, double, double);
static int __pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_should_break(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *);
static int __pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_check_next_range(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *, double, double);
static void __pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_check_next_time(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *, double);
static void __pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_check_mach_crossing(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *, double, double);
static void __pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_check_zero_crossing(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *, struct __pyx_t_16CyEulerBallistic_6vector_CVector);


/* "CyEulerBallistic/trajectory_calc.pyx":137
 * 
 * @final
 * cdef class _WindSock:             # <<<<<<<<<<<<<<
 *     cdef:
 *         list[Wind_t] winds
 */

struct __pyx_vtabstruct_16CyEulerBallistic_15trajectory_calc__WindSock {
  struct __pyx_t_16CyEulerBallistic_6vector_CVector (*current_vector)(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock *);
  void (*update_cache)(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock *);
  struct __pyx_t_16CyEulerBallistic_6vector_CVector (*vector_for_range)(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock *, double);
};
static struct __pyx_vtabstruct_16CyEulerBallistic_15trajectory_calc__WindSock *__pyx_vtabptr_16CyEulerBallistic_15trajectory_calc__WindSock;
static struct __pyx_t_16CyEulerBallistic_6vector_CVector __pyx_f_16CyEulerBallistic_15trajectory_calc_9_WindSock_current_vector(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock *);
static void __pyx_f_16CyEulerBallistic_15trajectory_calc_9_WindSock_update_cache(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock *);
static struct __pyx_t_16CyEulerBallistic_6vector_CVector __pyx_f_16CyEulerBallistic_15trajectory_calc_9_WindSock_vector_for_range(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock *, double);


/* "CyEulerBallistic/trajectory_calc.pyx":187
 * 
 * 
 * cdef class EulerTrajectoryCalc:             # <<<<<<<<<<<<<<
 *     cdef:
 *         list[object] _table_data
 */

struct __pyx_vtabstruct_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc {
  double (*get_calc_step)(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *, struct __pyx_opt_args_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_get_calc_step *__pyx_optional_args);
  void (*_free_trajectory)(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *);
  void (*_init_trajectory)(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *, PyObject *);
  PyObject *(*_zero_angle)(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *, PyObject *, PyObject *);
  PyObject *(*_integrate)(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *, double, double, int, struct __pyx_opt_args_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc__integrate *__pyx_optional_args);
};
static struct __pyx_vtabstruct_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *__pyx_vtabptr_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc;
/* #### Code section: utility_code_proto ### */

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, Py_ssize_t);
    void (*DECREF)(void*, PyObject*, Py_ssize_t);
    void (*GOTREF)(void*, PyObject*, Py_ssize_t);
    void (*GIVEREF)(void*, PyObject*, Py_ssize_t);
    void* (*SetupContext)(const char*, Py_ssize_t, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__));\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__));\
          }
  #define __Pyx_RefNannyFinishContextNogil() {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __Pyx_RefNannyFinishContext();\
              PyGILState_Release(__pyx_gilstate_save);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__))
  #define __Pyx_RefNannyFinishContextNogil() __Pyx_RefNannyFinishContext()
#endif
  #define __Pyx_RefNannyFinishContextNogil() {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __Pyx_RefNannyFinishContext();\
              PyGILState_Release(__pyx_gilstate_save);\
          }
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_XINCREF(r)  do { if((r) == NULL); else {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) == NULL); else {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) == NULL); else {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) == NULL); else {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContextNogil()
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_Py_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; Py_XDECREF(tmp);\
    } while (0)
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#if PY_VERSION_HEX >= 0x030C00A6
#define __Pyx_PyErr_Occurred()  (__pyx_tstate->current_exception != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  (__pyx_tstate->current_exception ? (PyObject*) Py_TYPE(__pyx_tstate->current_exception) : (PyObject*) NULL)
#else
#define __Pyx_PyErr_Occurred()  (__pyx_tstate->curexc_type != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  (__pyx_tstate->curexc_type)
#endif
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  (PyErr_Occurred() != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A6
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* PyObjectGetAttrStrNoError.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* DictGetItem.proto */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key);
#define __Pyx_PyObject_Dict_GetItem(obj, name)\
    (likely(PyDict_CheckExact(obj)) ?\
     __Pyx_PyDict_GetItem(obj, name) : PyObject_GetItem(obj, name))
#else
#define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#define __Pyx_PyObject_Dict_GetItem(obj, name)  PyObject_GetItem(obj, name)
#endif

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK && CYTHON_FAST_THREAD_STATE
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* TupleAndListFromArray.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyList_FromArray(PyObject *const *src, Py_ssize_t n);
static CYTHON_INLINE PyObject* __Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n);
#endif

/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* fastcall.proto */
#if CYTHON_AVOID_BORROWED_REFS
    #define __Pyx_Arg_VARARGS(args, i) PySequence_GetItem(args, i)
#elif CYTHON_ASSUME_SAFE_MACROS
    #define __Pyx_Arg_VARARGS(args, i) PyTuple_GET_ITEM(args, i)
#else
    #define __Pyx_Arg_VARARGS(args, i) PyTuple_GetItem(args, i)
#endif
#if CYTHON_AVOID_BORROWED_REFS
    #define __Pyx_Arg_NewRef_VARARGS(arg) __Pyx_NewRef(arg)
    #define __Pyx_Arg_XDECREF_VARARGS(arg) Py_XDECREF(arg)
#else
    #define __Pyx_Arg_NewRef_VARARGS(arg) arg
    #define __Pyx_Arg_XDECREF_VARARGS(arg)
#endif
#define __Pyx_NumKwargs_VARARGS(kwds) PyDict_Size(kwds)
#define __Pyx_KwValues_VARARGS(args, nargs) NULL
#define __Pyx_GetKwValue_VARARGS(kw, kwvalues, s) __Pyx_PyDict_GetItemStrWithError(kw, s)
#define __Pyx_KwargsAsDict_VARARGS(kw, kwvalues) PyDict_Copy(kw)
#if CYTHON_METH_FASTCALL
    #define __Pyx_Arg_FASTCALL(args, i) args[i]
    #define __Pyx_NumKwargs_FASTCALL(kwds) PyTuple_GET_SIZE(kwds)
    #define __Pyx_KwValues_FASTCALL(args, nargs) ((args) + (nargs))
    static CYTHON_INLINE PyObject * __Pyx_GetKwValue_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues, PyObject *s);
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000
    CYTHON_UNUSED static PyObject *__Pyx_KwargsAsDict_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues);
  #else
    #define __Pyx_KwargsAsDict_FASTCALL(kw, kwvalues) _PyStack_AsDict(kwvalues, kw)
  #endif
    #define __Pyx_Arg_NewRef_FASTCALL(arg) arg  /* no-op, __Pyx_Arg_FASTCALL is direct and this needs
                                                   to have the same reference counting */
    #define __Pyx_Arg_XDECREF_FASTCALL(arg)
#else
    #define __Pyx_Arg_FASTCALL __Pyx_Arg_VARARGS
    #define __Pyx_NumKwargs_FASTCALL __Pyx_NumKwargs_VARARGS
    #define __Pyx_KwValues_FASTCALL __Pyx_KwValues_VARARGS
    #define __Pyx_GetKwValue_FASTCALL __Pyx_GetKwValue_VARARGS
    #define __Pyx_KwargsAsDict_FASTCALL __Pyx_KwargsAsDict_VARARGS
    #define __Pyx_Arg_NewRef_FASTCALL(arg) __Pyx_Arg_NewRef_VARARGS(arg)
    #define __Pyx_Arg_XDECREF_FASTCALL(arg) __Pyx_Arg_XDECREF_VARARGS(arg)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
#define __Pyx_ArgsSlice_VARARGS(args, start, stop) __Pyx_PyTuple_FromArray(&__Pyx_Arg_VARARGS(args, start), stop - start)
#define __Pyx_ArgsSlice_FASTCALL(args, start, stop) __Pyx_PyTuple_FromArray(&__Pyx_Arg_FASTCALL(args, start), stop - start)
#else
#define __Pyx_ArgsSlice_VARARGS(args, start, stop) PyTuple_GetSlice(args, start, stop)
#define __Pyx_ArgsSlice_FASTCALL(args, start, stop) PyTuple_GetSlice(args, start, stop)
#endif

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject *const *kwvalues,
    PyObject **argnames[],
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,
    const char* function_name);

/* KeywordStringCheck.proto */
static int __Pyx_CheckKeywordStrings(PyObject *kw, const char* function_name, int kw_allowed);

/* ListCompAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000
        L->ob_item[len] = x;
        #else
        PyList_SET_ITEM(list, len, x);
        #endif
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  do {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  do {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#if !CYTHON_VECTORCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
#if !CYTHON_VECTORCALL
#if PY_VERSION_HEX >= 0x03080000
  #include "frameobject.h"
#if PY_VERSION_HEX >= 0x030b00a6 && !CYTHON_COMPILING_IN_LIMITED_API
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
  #define __Pxy_PyFrame_Initialize_Offsets()
  #define __Pyx_PyFrame_GetLocalsplus(frame)  ((frame)->f_localsplus)
#else
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif
#endif
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectFastCall.proto */
#define __Pyx_PyObject_FastCall(func, args, nargs)  __Pyx_PyObject_FastCallDict(func, args, (size_t)(nargs), NULL)
static CYTHON_INLINE PyObject* __Pyx_PyObject_FastCallDict(PyObject *func, PyObject **args, size_t nargs, PyObject *kwargs);

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* RaiseUnexpectedTypeError.proto */
static int __Pyx_RaiseUnexpectedTypeError(const char *expected, PyObject *obj);

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000
        L->ob_item[len] = x;
        #else
        PyList_SET_ITEM(list, len, x);
        #endif
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* GCCDiagnostics.proto */
#if !defined(__INTEL_COMPILER) && defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* BuildPyUnicode.proto */
static PyObject* __Pyx_PyUnicode_BuildFromAscii(Py_ssize_t ulength, char* chars, int clength,
                                                int prepend_sign, char padding_char);

/* CIntToPyUnicode.proto */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_From_int(int value, Py_ssize_t width, char padding_char, char format_char);

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o, n, NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value);
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* IncludeStructmemberH.proto */
#include <structmember.h>

/* FixUpExtensionType.proto */
#if CYTHON_USE_TYPE_SPECS
static int __Pyx_fix_up_extension_type_from_spec(PyType_Spec *spec, PyTypeObject *type);
#endif

/* PyObjectCallNoArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);

/* PyObjectGetMethod.proto */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method);

/* PyObjectCallMethod0.proto */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name);

/* ValidateBasesTuple.proto */
#if CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_USE_TYPE_SPECS
static int __Pyx_validate_bases_tuple(const char *type_name, Py_ssize_t dictoffset, PyObject *bases);
#endif

/* PyType_Ready.proto */
CYTHON_UNUSED static int __Pyx_PyType_Ready(PyTypeObject *t);

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* SetVTable.proto */
static int __Pyx_SetVtable(PyTypeObject* typeptr , void* vtable);

/* GetVTable.proto */
static void* __Pyx_GetVtable(PyTypeObject *type);

/* MergeVTables.proto */
#if !CYTHON_COMPILING_IN_LIMITED_API
static int __Pyx_MergeVtables(PyTypeObject *type);
#endif

/* SetupReduce.proto */
#if !CYTHON_COMPILING_IN_LIMITED_API
static int __Pyx_setup_reduce(PyObject* type_obj);
#endif

/* PyObject_GenericGetAttr.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttr PyObject_GenericGetAttr
#endif

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto_3_0_11
#define __PYX_HAVE_RT_ImportType_proto_3_0_11
#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 201112L
#include <stdalign.h>
#endif
#if (defined (__STDC_VERSION__) && __STDC_VERSION__ >= 201112L) || __cplusplus >= 201103L
#define __PYX_GET_STRUCT_ALIGNMENT_3_0_11(s) alignof(s)
#else
#define __PYX_GET_STRUCT_ALIGNMENT_3_0_11(s) sizeof(void*)
#endif
enum __Pyx_ImportType_CheckSize_3_0_11 {
   __Pyx_ImportType_CheckSize_Error_3_0_11 = 0,
   __Pyx_ImportType_CheckSize_Warn_3_0_11 = 1,
   __Pyx_ImportType_CheckSize_Ignore_3_0_11 = 2
};
static PyTypeObject *__Pyx_ImportType_3_0_11(PyObject* module, const char *module_name, const char *class_name, size_t size, size_t alignment, enum __Pyx_ImportType_CheckSize_3_0_11 check_size);
#endif

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportDottedModule.proto */
static PyObject *__Pyx_ImportDottedModule(PyObject *name, PyObject *parts_tuple);
#if PY_MAJOR_VERSION >= 3
static PyObject *__Pyx_ImportDottedModule_WalkParts(PyObject *module, PyObject *name, PyObject *parts_tuple);
#endif

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
#if !CYTHON_COMPILING_IN_LIMITED_API
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);
#endif

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* FormatTypeName.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
typedef PyObject *__Pyx_TypeName;
#define __Pyx_FMT_TYPENAME "%U"
static __Pyx_TypeName __Pyx_PyType_GetName(PyTypeObject* tp);
#define __Pyx_DECREF_TypeName(obj) Py_XDECREF(obj)
#else
typedef const char *__Pyx_TypeName;
#define __Pyx_FMT_TYPENAME "%.200s"
#define __Pyx_PyType_GetName(tp) ((tp)->tp_name)
#define __Pyx_DECREF_TypeName(obj)
#endif

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
#define __Pyx_TypeCheck2(obj, type1, type2) __Pyx_IsAnySubtype2(Py_TYPE(obj), (PyTypeObject *)type1, (PyTypeObject *)type2)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_IsAnySubtype2(PyTypeObject *cls, PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_TypeCheck2(obj, type1, type2) (PyObject_TypeCheck(obj, (PyTypeObject *)type1) || PyObject_TypeCheck(obj, (PyTypeObject *)type2))
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyErr_ExceptionMatches2(err1, err2)  __Pyx_PyErr_GivenExceptionMatches2(__Pyx_PyErr_CurrentExceptionType(), err1, err2)
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* CheckBinaryVersion.proto */
static unsigned long __Pyx_get_runtime_version(void);
static int __Pyx_check_binary_version(unsigned long ct_version, unsigned long rt_version, int allow_newer);

/* VoidPtrImport.proto */
static int __Pyx_ImportVoidPtr_3_0_11(PyObject *module, const char *name, void **p, const char *sig);

/* FunctionImport.proto */
static int __Pyx_ImportFunction_3_0_11(PyObject *module, const char *funcname, void (**f)(void), const char *sig);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

/* #### Code section: module_declarations ### */
static void __pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_setup_seen_zero(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *__pyx_v_self, double __pyx_v_height, double __pyx_v_barrel_elevation, double __pyx_v_look_angle); /* proto*/
static void __pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_clear_current_flag(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *__pyx_v_self); /* proto*/
static int __pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_should_record(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *__pyx_v_self, struct __pyx_t_16CyEulerBallistic_6vector_CVector __pyx_v_range_vector, double __pyx_v_velocity, double __pyx_v_mach, double __pyx_v_step, double __pyx_v_time); /* proto*/
static int __pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_should_break(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *__pyx_v_self); /* proto*/
static int __pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_check_next_range(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *__pyx_v_self, double __pyx_v_next_range, double __pyx_v_step); /* proto*/
static void __pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_check_next_time(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *__pyx_v_self, double __pyx_v_time); /* proto*/
static void __pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_check_mach_crossing(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *__pyx_v_self, double __pyx_v_velocity, double __pyx_v_mach); /* proto*/
static void __pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_check_zero_crossing(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *__pyx_v_self, struct __pyx_t_16CyEulerBallistic_6vector_CVector __pyx_v_range_vector); /* proto*/
static struct __pyx_t_16CyEulerBallistic_6vector_CVector __pyx_f_16CyEulerBallistic_15trajectory_calc_9_WindSock_current_vector(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock *__pyx_v_self); /* proto*/
static void __pyx_f_16CyEulerBallistic_15trajectory_calc_9_WindSock_update_cache(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock *__pyx_v_self); /* proto*/
static struct __pyx_t_16CyEulerBallistic_6vector_CVector __pyx_f_16CyEulerBallistic_15trajectory_calc_9_WindSock_vector_for_range(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock *__pyx_v_self, double __pyx_v_next_range); /* proto*/
static double __pyx_f_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_get_calc_step(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *__pyx_v_self, struct __pyx_opt_args_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_get_calc_step *__pyx_optional_args); /* proto*/
static void __pyx_f_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc__free_trajectory(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *__pyx_v_self); /* proto*/
static void __pyx_f_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc__init_trajectory(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *__pyx_v_self, PyObject *__pyx_v_shot_info); /* proto*/
static PyObject *__pyx_f_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc__zero_angle(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *__pyx_v_self, PyObject *__pyx_v_shot_info, PyObject *__pyx_v_distance); /* proto*/
static PyObject *__pyx_f_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc__integrate(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *__pyx_v_self, double __pyx_v_maximum_range, double __pyx_v_step, int __pyx_v_filter_flags, struct __pyx_opt_args_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc__integrate *__pyx_optional_args); /* proto*/

/* Module declarations from "libc.math" */

/* Module declarations from "CyEulerBallistic.vector" */
static double (*__pyx_f_16CyEulerBallistic_6vector_mag)(struct __pyx_t_16CyEulerBallistic_6vector_CVector *); /*proto*/
static struct __pyx_t_16CyEulerBallistic_6vector_CVector (*__pyx_f_16CyEulerBallistic_6vector_mul_c)(struct __pyx_t_16CyEulerBallistic_6vector_CVector *, double); /*proto*/
static double (*__pyx_f_16CyEulerBallistic_6vector_mul_v)(struct __pyx_t_16CyEulerBallistic_6vector_CVector *, struct __pyx_t_16CyEulerBallistic_6vector_CVector *); /*proto*/
static struct __pyx_t_16CyEulerBallistic_6vector_CVector (*__pyx_f_16CyEulerBallistic_6vector_add)(struct __pyx_t_16CyEulerBallistic_6vector_CVector *, struct __pyx_t_16CyEulerBallistic_6vector_CVector *); /*proto*/
static struct __pyx_t_16CyEulerBallistic_6vector_CVector (*__pyx_f_16CyEulerBallistic_6vector_sub)(struct __pyx_t_16CyEulerBallistic_6vector_CVector *, struct __pyx_t_16CyEulerBallistic_6vector_CVector *); /*proto*/
static struct __pyx_t_16CyEulerBallistic_6vector_CVector (*__pyx_f_16CyEulerBallistic_6vector_neg)(struct __pyx_t_16CyEulerBallistic_6vector_CVector *); /*proto*/
static struct __pyx_t_16CyEulerBallistic_6vector_CVector (*__pyx_f_16CyEulerBallistic_6vector_norm)(struct __pyx_t_16CyEulerBallistic_6vector_CVector *); /*proto*/

/* Module declarations from "CyEulerBallistic.trajectory_data" */

/* Module declarations from "CyEulerBallistic.cy_euler" */
static double *__pyx_vp_16CyEulerBallistic_8cy_euler_cDegreesFtoR = 0;
#define __pyx_v_16CyEulerBallistic_8cy_euler_cDegreesFtoR (*__pyx_vp_16CyEulerBallistic_8cy_euler_cDegreesFtoR)
static double *__pyx_vp_16CyEulerBallistic_8cy_euler_cSpeedOfSoundImperial = 0;
#define __pyx_v_16CyEulerBallistic_8cy_euler_cSpeedOfSoundImperial (*__pyx_vp_16CyEulerBallistic_8cy_euler_cSpeedOfSoundImperial)
static double *__pyx_vp_16CyEulerBallistic_8cy_euler_cLapseRateImperial = 0;
#define __pyx_v_16CyEulerBallistic_8cy_euler_cLapseRateImperial (*__pyx_vp_16CyEulerBallistic_8cy_euler_cLapseRateImperial)
static struct __pyx_t_16CyEulerBallistic_8cy_euler_Config_t (*__pyx_f_16CyEulerBallistic_8cy_euler_config_bind)(PyObject *); /*proto*/
static void (*__pyx_f_16CyEulerBallistic_8cy_euler_update_density_factor_and_mach_for_altitude)(struct __pyx_t_16CyEulerBallistic_8cy_euler_Atmosphere_t *, double, double *, double *); /*proto*/
static struct __pyx_t_16CyEulerBallistic_8cy_euler_MachList_t (*__pyx_f_16CyEulerBallistic_8cy_euler_cy_table_to_mach)(PyObject *); /*proto*/
static struct __pyx_t_16CyEulerBallistic_8cy_euler_Curve_t (*__pyx_f_16CyEulerBallistic_8cy_euler_cy_calculate_curve)(PyObject *); /*proto*/
static double (*__pyx_f_16CyEulerBallistic_8cy_euler_cy_calculate_by_curve_and_mach_list)(struct __pyx_t_16CyEulerBallistic_8cy_euler_MachList_t *, struct __pyx_t_16CyEulerBallistic_8cy_euler_Curve_t *, double); /*proto*/
static double (*__pyx_f_16CyEulerBallistic_8cy_euler_cy_spin_drift)(struct __pyx_t_16CyEulerBallistic_8cy_euler_ShotData_t *, double); /*proto*/
static double (*__pyx_f_16CyEulerBallistic_8cy_euler_cy_drag_by_mach)(struct __pyx_t_16CyEulerBallistic_8cy_euler_ShotData_t *, double); /*proto*/
static void (*__pyx_f_16CyEulerBallistic_8cy_euler_cy_update_stability_coefficient)(struct __pyx_t_16CyEulerBallistic_8cy_euler_ShotData_t *); /*proto*/
static double (*__pyx_f_16CyEulerBallistic_8cy_euler_cy_get_correction)(double, double); /*proto*/
static double (*__pyx_f_16CyEulerBallistic_8cy_euler_cy_calculate_energy)(double, double); /*proto*/
static double (*__pyx_f_16CyEulerBallistic_8cy_euler_cy_calculate_ogw)(double, double); /*proto*/
static void (*__pyx_f_16CyEulerBallistic_8cy_euler_free_trajectory)(struct __pyx_t_16CyEulerBallistic_8cy_euler_ShotData_t *); /*proto*/
static struct __pyx_t_16CyEulerBallistic_6vector_CVector (*__pyx_f_16CyEulerBallistic_8cy_euler_wind_to_c_vector)(struct __pyx_t_16CyEulerBallistic_8cy_euler_Wind_t *); /*proto*/

/* Module declarations from "CyEulerBallistic.trajectory_calc" */
static PyObject *__pyx_f_16CyEulerBallistic_15trajectory_calc_create_trajectory_row(double, struct __pyx_t_16CyEulerBallistic_6vector_CVector, struct __pyx_t_16CyEulerBallistic_6vector_CVector, double, double, double, double, double, double, double, int); /*proto*/
static PyObject *__pyx_f_16CyEulerBallistic_15trajectory_calc__new_feet(double); /*proto*/
static PyObject *__pyx_f_16CyEulerBallistic_15trajectory_calc__new_fps(double); /*proto*/
static PyObject *__pyx_f_16CyEulerBallistic_15trajectory_calc__new_rad(double); /*proto*/
static PyObject *__pyx_f_16CyEulerBallistic_15trajectory_calc__new_ft_lb(double); /*proto*/
static PyObject *__pyx_f_16CyEulerBallistic_15trajectory_calc__new_lb(double); /*proto*/
static struct __pyx_t_16CyEulerBallistic_8cy_euler_Wind_t __pyx_convert__from_py_struct____pyx_t_16CyEulerBallistic_8cy_euler_Wind_t(PyObject *); /*proto*/
/* #### Code section: typeinfo ### */
/* #### Code section: before_global_var ### */
#define __Pyx_MODULE_NAME "CyEulerBallistic.trajectory_calc"
extern int __pyx_module_is_main_CyEulerBallistic__trajectory_calc;
int __pyx_module_is_main_CyEulerBallistic__trajectory_calc = 0;

/* Implementation of "CyEulerBallistic.trajectory_calc" */
/* #### Code section: global_var ### */
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_object;
static PyObject *__pyx_builtin_KeyError;
static PyObject *__pyx_builtin_ValueError;
/* #### Code section: string_decls ### */
static const char __pyx_k_x[] = "x";
static const char __pyx_k_y[] = "y";
static const char __pyx_k_z[] = "z";
static const char __pyx_k_BC[] = "BC";
static const char __pyx_k__5[] = "*";
static const char __pyx_k__6[] = ".";
static const char __pyx_k__8[] = "?";
static const char __pyx_k_a0[] = "_a0";
static const char __pyx_k_bc[] = "bc";
static const char __pyx_k_dm[] = "dm";
static const char __pyx_k_gc[] = "gc";
static const char __pyx_k_p0[] = "_p0";
static const char __pyx_k_t0[] = "_t0";
static const char __pyx_k_FPS[] = "FPS";
static const char __pyx_k_all[] = "__all__";
static const char __pyx_k_fps[] = "_fps";
static const char __pyx_k_new[] = "__new__";
static const char __pyx_k_ogw[] = "ogw";
static const char __pyx_k_rad[] = "_rad";
static const char __pyx_k_Foot[] = "Foot";
static const char __pyx_k_InHg[] = "InHg";
static const char __pyx_k_Unit[] = "Unit";
static const char __pyx_k_alt0[] = "alt0";
static const char __pyx_k_ammo[] = "ammo";
static const char __pyx_k_atmo[] = "atmo";
static const char __pyx_k_drag[] = "drag";
static const char __pyx_k_feet[] = "_feet";
static const char __pyx_k_flag[] = "flag";
static const char __pyx_k_inch[] = "_inch";
static const char __pyx_k_mach[] = "mach";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_once[] = "once";
static const char __pyx_k_spec[] = "__spec__";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_time[] = "time";
static const char __pyx_k_Pound[] = "Pound";
static const char __pyx_k_angle[] = "angle";
static const char __pyx_k_curve[] = "curve";
static const char __pyx_k_debug[] = "debug";
static const char __pyx_k_grain[] = "_grain";
static const char __pyx_k_mach1[] = "_mach1";
static const char __pyx_k_twist[] = "twist";
static const char __pyx_k_value[] = "_value";
static const char __pyx_k_winds[] = "winds";
static const char __pyx_k_Energy[] = "Energy";
static const char __pyx_k_Radian[] = "Radian";
static const char __pyx_k_Weight[] = "Weight";
static const char __pyx_k_config[] = "_config";
static const char __pyx_k_enable[] = "enable";
static const char __pyx_k_energy[] = "energy";
static const char __pyx_k_get_in[] = "get_in";
static const char __pyx_k_height[] = "height";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_length[] = "length";
static const char __pyx_k_logger[] = "logger";
static const char __pyx_k_object[] = "object";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_weapon[] = "weapon";
static const char __pyx_k_weight[] = "weight";
static const char __pyx_k_Angular[] = "Angular";
static const char __pyx_k_disable[] = "disable";
static const char __pyx_k_windage[] = "windage";
static const char __pyx_k_Distance[] = "Distance";
static const char __pyx_k_KeyError[] = "KeyError";
static const char __pyx_k_Velocity[] = "Velocity";
static const char __pyx_k_WindSock[] = "_WindSock";
static const char __pyx_k_altitude[] = "altitude";
static const char __pyx_k_diameter[] = "diameter";
static const char __pyx_k_distance[] = "distance";
static const char __pyx_k_drop_adj[] = "drop_adj";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_pressure[] = "pressure";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_velocity[] = "velocity";
static const char __pyx_k_warnings[] = "warnings";
static const char __pyx_k_FootPound[] = "FootPound";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_calc_step[] = "calc_step";
static const char __pyx_k_cant_sine[] = "cant_sine";
static const char __pyx_k_dist_step[] = "dist_step";
static const char __pyx_k_isenabled[] = "isenabled";
static const char __pyx_k_mach_list[] = "mach_list";
static const char __pyx_k_max_range[] = "max_range";
static const char __pyx_k_pyx_state[] = "__pyx_state";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_shot_info[] = "shot_info";
static const char __pyx_k_time_step[] = "time_step";
static const char __pyx_k_RangeError[] = "RangeError";
static const char __pyx_k_ValueError[] = "ValueError";
static const char __pyx_k_cant_angle[] = "cant_angle";
static const char __pyx_k_drag_table[] = "drag_table";
static const char __pyx_k_extra_data[] = "extra_data";
static const char __pyx_k_look_angle[] = "look_angle";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_cant_cosine[] = "cant_cosine";
static const char __pyx_k_euler_cy_it[] = "euler cy it ";
static const char __pyx_k_powder_temp[] = "powder_temp";
static const char __pyx_k_target_drop[] = "target_drop";
static const char __pyx_k_windage_adj[] = "windage_adj";
static const char __pyx_k_filter_flags[] = "filter_flags";
static const char __pyx_k_initializing[] = "_initializing";
static const char __pyx_k_sight_height[] = "sight_height";
static const char __pyx_k_simplefilter[] = "simplefilter";
static const char __pyx_k_defined_units[] = "_defined_units";
static const char __pyx_k_density_ratio[] = "density_ratio";
static const char __pyx_k_look_distance[] = "look_distance";
static const char __pyx_k_ranges_length[] = "ranges_length";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_barrel_azimuth[] = "barrel_azimuth";
static const char __pyx_k_density_factor[] = "density_factor";
static const char __pyx_k_direction_from[] = "direction_from";
static const char __pyx_k_until_distance[] = "until_distance";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_ZeroFindingError[] = "ZeroFindingError";
static const char __pyx_k_barrel_elevation[] = "barrel_elevation";
static const char __pyx_k_cGravityConstant[] = "cGravityConstant";
static const char __pyx_k_MAX_DISTANCE_FEET[] = "MAX_DISTANCE_FEET";
static const char __pyx_k_MaximumDropReached[] = "MaximumDropReached";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_EulerTrajectoryCalc[] = "EulerTrajectoryCalc";
static const char __pyx_k_TrajectoryDataFilter[] = "_TrajectoryDataFilter";
static const char __pyx_k_cMaxWindDistanceFeet[] = "cMaxWindDistanceFeet";
static const char __pyx_k_get_velocity_for_temp[] = "get_velocity_for_temp";
static const char __pyx_k_py_ballisticcalc_unit[] = "py_ballisticcalc.unit";
static const char __pyx_k_stability_coefficient[] = "stability_coefficient";
static const char __pyx_k_MinimumAltitudeReached[] = "MinimumAltitudeReached";
static const char __pyx_k_MinimumVelocityReached[] = "MinimumVelocityReached";
static const char __pyx_k_py_ballisticcalc_logger[] = "py_ballisticcalc.logger";
static const char __pyx_k_py_ballisticcalc_constants[] = "py_ballisticcalc.constants";
static const char __pyx_k_py_ballisticcalc_exceptions[] = "py_ballisticcalc.exceptions";
static const char __pyx_k_Total_Score_2394_Possible_Score[] = "\n# Total Score: 2394, Possible Score: 45200\n# Total Non-Empty Lines: 452\n# Python Overhead Lines: 152\n# Cythonization Percentage: 94.70%\n# Python Overhead Lines Percentage: 33.63%\n";
static const char __pyx_k_No_value_specified_for_struct_at[] = "No value specified for struct attribute 'velocity'";
static const char __pyx_k_no_default___reduce___due_to_non[] = "no default __reduce__ due to non-trivial __cinit__";
static const char __pyx_k_No_value_specified_for_struct_at_2[] = "No value specified for struct attribute 'direction_from'";
static const char __pyx_k_No_value_specified_for_struct_at_3[] = "No value specified for struct attribute 'until_distance'";
static const char __pyx_k_No_value_specified_for_struct_at_4[] = "No value specified for struct attribute 'MAX_DISTANCE_FEET'";
/* #### Code section: decls ### */
static int __pyx_pf_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter___cinit__(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *__pyx_v_self, int __pyx_v_filter_flags, int __pyx_v_ranges_length, double __pyx_v_time_step); /* proto */
static PyObject *__pyx_pf_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_16CyEulerBallistic_15trajectory_calc_9_WindSock___cinit__(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock *__pyx_v_self, PyObject *__pyx_v_winds); /* proto */
static PyObject *__pyx_pf_16CyEulerBallistic_15trajectory_calc_9_WindSock_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_16CyEulerBallistic_15trajectory_calc_9_WindSock_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc___cinit__(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *__pyx_v_self, PyObject *__pyx_v__config); /* proto */
static PyObject *__pyx_pf_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_10table_data___get__(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_2zero_angle(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *__pyx_v_self, PyObject *__pyx_v_shot_info, PyObject *__pyx_v_distance); /* proto */
static PyObject *__pyx_pf_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_4trajectory(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *__pyx_v_self, PyObject *__pyx_v_shot_info, PyObject *__pyx_v_max_range, PyObject *__pyx_v_dist_step, int __pyx_v_extra_data, double __pyx_v_time_step); /* proto */
static PyObject *__pyx_pf_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_7_config___get__(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *__pyx_v_self); /* proto */
static int __pyx_pf_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_7_config_2__set__(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_7_config_4__del__(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_16CyEulerBallistic_15trajectory_calc__WindSock(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
/* #### Code section: late_includes ### */
/* #### Code section: module_state ### */
typedef struct {
  PyObject *__pyx_d;
  PyObject *__pyx_b;
  PyObject *__pyx_cython_runtime;
  PyObject *__pyx_empty_tuple;
  PyObject *__pyx_empty_bytes;
  PyObject *__pyx_empty_unicode;
  #ifdef __Pyx_CyFunction_USED
  PyTypeObject *__pyx_CyFunctionType;
  #endif
  #ifdef __Pyx_FusedFunction_USED
  PyTypeObject *__pyx_FusedFunctionType;
  #endif
  #ifdef __Pyx_Generator_USED
  PyTypeObject *__pyx_GeneratorType;
  #endif
  #ifdef __Pyx_IterableCoroutine_USED
  PyTypeObject *__pyx_IterableCoroutineType;
  #endif
  #ifdef __Pyx_Coroutine_USED
  PyTypeObject *__pyx_CoroutineAwaitType;
  #endif
  #ifdef __Pyx_Coroutine_USED
  PyTypeObject *__pyx_CoroutineType;
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  PyTypeObject *__pyx_ptype_16CyEulerBallistic_6vector_Vector;
  #if CYTHON_USE_MODULE_STATE
  #endif
  PyTypeObject *__pyx_ptype_16CyEulerBallistic_15trajectory_data_TrajectoryData;
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  PyObject *__pyx_type_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter;
  PyObject *__pyx_type_16CyEulerBallistic_15trajectory_calc__WindSock;
  PyObject *__pyx_type_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc;
  #endif
  PyTypeObject *__pyx_ptype_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter;
  PyTypeObject *__pyx_ptype_16CyEulerBallistic_15trajectory_calc__WindSock;
  PyTypeObject *__pyx_ptype_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc;
  PyObject *__pyx_n_s_Angular;
  PyObject *__pyx_n_s_BC;
  PyObject *__pyx_n_s_Distance;
  PyObject *__pyx_n_s_Energy;
  PyObject *__pyx_n_s_EulerTrajectoryCalc;
  PyObject *__pyx_n_u_EulerTrajectoryCalc;
  PyObject *__pyx_n_s_FPS;
  PyObject *__pyx_n_s_Foot;
  PyObject *__pyx_n_s_FootPound;
  PyObject *__pyx_n_s_InHg;
  PyObject *__pyx_n_s_KeyError;
  PyObject *__pyx_n_s_MAX_DISTANCE_FEET;
  PyObject *__pyx_n_s_MaximumDropReached;
  PyObject *__pyx_n_s_MinimumAltitudeReached;
  PyObject *__pyx_n_s_MinimumVelocityReached;
  PyObject *__pyx_kp_s_No_value_specified_for_struct_at;
  PyObject *__pyx_kp_s_No_value_specified_for_struct_at_2;
  PyObject *__pyx_kp_s_No_value_specified_for_struct_at_3;
  PyObject *__pyx_kp_s_No_value_specified_for_struct_at_4;
  PyObject *__pyx_n_s_Pound;
  PyObject *__pyx_n_s_Radian;
  PyObject *__pyx_n_s_RangeError;
  PyObject *__pyx_n_s_TrajectoryDataFilter;
  PyObject *__pyx_n_s_TypeError;
  PyObject *__pyx_n_s_Unit;
  PyObject *__pyx_n_s_ValueError;
  PyObject *__pyx_n_s_Velocity;
  PyObject *__pyx_n_s_Weight;
  PyObject *__pyx_n_s_WindSock;
  PyObject *__pyx_n_s_ZeroFindingError;
  PyObject *__pyx_n_s__5;
  PyObject *__pyx_kp_u__6;
  PyObject *__pyx_n_s__8;
  PyObject *__pyx_n_s_a0;
  PyObject *__pyx_n_s_all;
  PyObject *__pyx_n_s_alt0;
  PyObject *__pyx_n_s_altitude;
  PyObject *__pyx_n_s_ammo;
  PyObject *__pyx_n_s_angle;
  PyObject *__pyx_n_s_atmo;
  PyObject *__pyx_n_s_barrel_azimuth;
  PyObject *__pyx_n_s_barrel_elevation;
  PyObject *__pyx_n_s_bc;
  PyObject *__pyx_n_s_cGravityConstant;
  PyObject *__pyx_n_s_cMaxWindDistanceFeet;
  PyObject *__pyx_n_s_calc_step;
  PyObject *__pyx_n_s_cant_angle;
  PyObject *__pyx_n_s_cant_cosine;
  PyObject *__pyx_n_s_cant_sine;
  PyObject *__pyx_n_s_cline_in_traceback;
  PyObject *__pyx_n_s_config;
  PyObject *__pyx_n_s_curve;
  PyObject *__pyx_n_s_debug;
  PyObject *__pyx_n_s_defined_units;
  PyObject *__pyx_n_s_density_factor;
  PyObject *__pyx_n_s_density_ratio;
  PyObject *__pyx_n_s_diameter;
  PyObject *__pyx_n_s_direction_from;
  PyObject *__pyx_kp_u_disable;
  PyObject *__pyx_n_s_dist_step;
  PyObject *__pyx_n_s_distance;
  PyObject *__pyx_n_s_dm;
  PyObject *__pyx_n_s_drag;
  PyObject *__pyx_n_s_drag_table;
  PyObject *__pyx_n_s_drop_adj;
  PyObject *__pyx_kp_u_enable;
  PyObject *__pyx_n_s_energy;
  PyObject *__pyx_kp_u_euler_cy_it;
  PyObject *__pyx_n_s_extra_data;
  PyObject *__pyx_n_s_feet;
  PyObject *__pyx_n_s_filter_flags;
  PyObject *__pyx_n_s_flag;
  PyObject *__pyx_n_s_fps;
  PyObject *__pyx_kp_u_gc;
  PyObject *__pyx_n_s_get_in;
  PyObject *__pyx_n_s_get_velocity_for_temp;
  PyObject *__pyx_n_s_getstate;
  PyObject *__pyx_n_s_grain;
  PyObject *__pyx_n_s_height;
  PyObject *__pyx_n_s_import;
  PyObject *__pyx_n_s_inch;
  PyObject *__pyx_n_s_initializing;
  PyObject *__pyx_kp_u_isenabled;
  PyObject *__pyx_n_s_length;
  PyObject *__pyx_n_s_logger;
  PyObject *__pyx_n_s_look_angle;
  PyObject *__pyx_n_s_look_distance;
  PyObject *__pyx_n_s_mach;
  PyObject *__pyx_n_s_mach1;
  PyObject *__pyx_n_s_mach_list;
  PyObject *__pyx_n_s_main;
  PyObject *__pyx_n_s_max_range;
  PyObject *__pyx_n_s_name;
  PyObject *__pyx_n_s_new;
  PyObject *__pyx_kp_s_no_default___reduce___due_to_non;
  PyObject *__pyx_n_s_object;
  PyObject *__pyx_n_s_ogw;
  PyObject *__pyx_n_u_once;
  PyObject *__pyx_n_s_p0;
  PyObject *__pyx_n_s_powder_temp;
  PyObject *__pyx_n_s_pressure;
  PyObject *__pyx_n_s_py_ballisticcalc_constants;
  PyObject *__pyx_n_s_py_ballisticcalc_exceptions;
  PyObject *__pyx_n_s_py_ballisticcalc_logger;
  PyObject *__pyx_n_s_py_ballisticcalc_unit;
  PyObject *__pyx_n_s_pyx_state;
  PyObject *__pyx_n_s_pyx_vtable;
  PyObject *__pyx_n_s_rad;
  PyObject *__pyx_n_s_ranges_length;
  PyObject *__pyx_n_s_reduce;
  PyObject *__pyx_n_s_reduce_cython;
  PyObject *__pyx_n_s_reduce_ex;
  PyObject *__pyx_n_s_setstate;
  PyObject *__pyx_n_s_setstate_cython;
  PyObject *__pyx_n_s_shot_info;
  PyObject *__pyx_n_s_sight_height;
  PyObject *__pyx_n_s_simplefilter;
  PyObject *__pyx_n_s_spec;
  PyObject *__pyx_n_s_stability_coefficient;
  PyObject *__pyx_n_s_t0;
  PyObject *__pyx_n_s_target_drop;
  PyObject *__pyx_n_s_test;
  PyObject *__pyx_n_s_time;
  PyObject *__pyx_n_s_time_step;
  PyObject *__pyx_n_s_twist;
  PyObject *__pyx_n_s_until_distance;
  PyObject *__pyx_n_s_value;
  PyObject *__pyx_n_s_velocity;
  PyObject *__pyx_n_s_warnings;
  PyObject *__pyx_n_s_weapon;
  PyObject *__pyx_n_s_weight;
  PyObject *__pyx_n_s_windage;
  PyObject *__pyx_n_s_windage_adj;
  PyObject *__pyx_n_s_winds;
  PyObject *__pyx_n_s_x;
  PyObject *__pyx_n_s_y;
  PyObject *__pyx_n_s_z;
  PyObject *__pyx_tuple_;
  PyObject *__pyx_tuple__2;
  PyObject *__pyx_tuple__3;
  PyObject *__pyx_tuple__4;
  PyObject *__pyx_tuple__7;
} __pyx_mstate;

#if CYTHON_USE_MODULE_STATE
#ifdef __cplusplus
namespace {
  extern struct PyModuleDef __pyx_moduledef;
} /* anonymous namespace */
#else
static struct PyModuleDef __pyx_moduledef;
#endif

#define __pyx_mstate(o) ((__pyx_mstate *)__Pyx_PyModule_GetState(o))

#define __pyx_mstate_global (__pyx_mstate(PyState_FindModule(&__pyx_moduledef)))

#define __pyx_m (PyState_FindModule(&__pyx_moduledef))
#else
static __pyx_mstate __pyx_mstate_global_static =
#ifdef __cplusplus
    {};
#else
    {0};
#endif
static __pyx_mstate *__pyx_mstate_global = &__pyx_mstate_global_static;
#endif
/* #### Code section: module_state_clear ### */
#if CYTHON_USE_MODULE_STATE
static int __pyx_m_clear(PyObject *m) {
  __pyx_mstate *clear_module_state = __pyx_mstate(m);
  if (!clear_module_state) return 0;
  Py_CLEAR(clear_module_state->__pyx_d);
  Py_CLEAR(clear_module_state->__pyx_b);
  Py_CLEAR(clear_module_state->__pyx_cython_runtime);
  Py_CLEAR(clear_module_state->__pyx_empty_tuple);
  Py_CLEAR(clear_module_state->__pyx_empty_bytes);
  Py_CLEAR(clear_module_state->__pyx_empty_unicode);
  #ifdef __Pyx_CyFunction_USED
  Py_CLEAR(clear_module_state->__pyx_CyFunctionType);
  #endif
  #ifdef __Pyx_FusedFunction_USED
  Py_CLEAR(clear_module_state->__pyx_FusedFunctionType);
  #endif
  Py_CLEAR(clear_module_state->__pyx_ptype_16CyEulerBallistic_6vector_Vector);
  Py_CLEAR(clear_module_state->__pyx_ptype_16CyEulerBallistic_15trajectory_data_TrajectoryData);
  Py_CLEAR(clear_module_state->__pyx_ptype_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter);
  Py_CLEAR(clear_module_state->__pyx_type_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter);
  Py_CLEAR(clear_module_state->__pyx_ptype_16CyEulerBallistic_15trajectory_calc__WindSock);
  Py_CLEAR(clear_module_state->__pyx_type_16CyEulerBallistic_15trajectory_calc__WindSock);
  Py_CLEAR(clear_module_state->__pyx_ptype_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc);
  Py_CLEAR(clear_module_state->__pyx_type_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc);
  Py_CLEAR(clear_module_state->__pyx_n_s_Angular);
  Py_CLEAR(clear_module_state->__pyx_n_s_BC);
  Py_CLEAR(clear_module_state->__pyx_n_s_Distance);
  Py_CLEAR(clear_module_state->__pyx_n_s_Energy);
  Py_CLEAR(clear_module_state->__pyx_n_s_EulerTrajectoryCalc);
  Py_CLEAR(clear_module_state->__pyx_n_u_EulerTrajectoryCalc);
  Py_CLEAR(clear_module_state->__pyx_n_s_FPS);
  Py_CLEAR(clear_module_state->__pyx_n_s_Foot);
  Py_CLEAR(clear_module_state->__pyx_n_s_FootPound);
  Py_CLEAR(clear_module_state->__pyx_n_s_InHg);
  Py_CLEAR(clear_module_state->__pyx_n_s_KeyError);
  Py_CLEAR(clear_module_state->__pyx_n_s_MAX_DISTANCE_FEET);
  Py_CLEAR(clear_module_state->__pyx_n_s_MaximumDropReached);
  Py_CLEAR(clear_module_state->__pyx_n_s_MinimumAltitudeReached);
  Py_CLEAR(clear_module_state->__pyx_n_s_MinimumVelocityReached);
  Py_CLEAR(clear_module_state->__pyx_kp_s_No_value_specified_for_struct_at);
  Py_CLEAR(clear_module_state->__pyx_kp_s_No_value_specified_for_struct_at_2);
  Py_CLEAR(clear_module_state->__pyx_kp_s_No_value_specified_for_struct_at_3);
  Py_CLEAR(clear_module_state->__pyx_kp_s_No_value_specified_for_struct_at_4);
  Py_CLEAR(clear_module_state->__pyx_n_s_Pound);
  Py_CLEAR(clear_module_state->__pyx_n_s_Radian);
  Py_CLEAR(clear_module_state->__pyx_n_s_RangeError);
  Py_CLEAR(clear_module_state->__pyx_n_s_TrajectoryDataFilter);
  Py_CLEAR(clear_module_state->__pyx_n_s_TypeError);
  Py_CLEAR(clear_module_state->__pyx_n_s_Unit);
  Py_CLEAR(clear_module_state->__pyx_n_s_ValueError);
  Py_CLEAR(clear_module_state->__pyx_n_s_Velocity);
  Py_CLEAR(clear_module_state->__pyx_n_s_Weight);
  Py_CLEAR(clear_module_state->__pyx_n_s_WindSock);
  Py_CLEAR(clear_module_state->__pyx_n_s_ZeroFindingError);
  Py_CLEAR(clear_module_state->__pyx_n_s__5);
  Py_CLEAR(clear_module_state->__pyx_kp_u__6);
  Py_CLEAR(clear_module_state->__pyx_n_s__8);
  Py_CLEAR(clear_module_state->__pyx_n_s_a0);
  Py_CLEAR(clear_module_state->__pyx_n_s_all);
  Py_CLEAR(clear_module_state->__pyx_n_s_alt0);
  Py_CLEAR(clear_module_state->__pyx_n_s_altitude);
  Py_CLEAR(clear_module_state->__pyx_n_s_ammo);
  Py_CLEAR(clear_module_state->__pyx_n_s_angle);
  Py_CLEAR(clear_module_state->__pyx_n_s_atmo);
  Py_CLEAR(clear_module_state->__pyx_n_s_barrel_azimuth);
  Py_CLEAR(clear_module_state->__pyx_n_s_barrel_elevation);
  Py_CLEAR(clear_module_state->__pyx_n_s_bc);
  Py_CLEAR(clear_module_state->__pyx_n_s_cGravityConstant);
  Py_CLEAR(clear_module_state->__pyx_n_s_cMaxWindDistanceFeet);
  Py_CLEAR(clear_module_state->__pyx_n_s_calc_step);
  Py_CLEAR(clear_module_state->__pyx_n_s_cant_angle);
  Py_CLEAR(clear_module_state->__pyx_n_s_cant_cosine);
  Py_CLEAR(clear_module_state->__pyx_n_s_cant_sine);
  Py_CLEAR(clear_module_state->__pyx_n_s_cline_in_traceback);
  Py_CLEAR(clear_module_state->__pyx_n_s_config);
  Py_CLEAR(clear_module_state->__pyx_n_s_curve);
  Py_CLEAR(clear_module_state->__pyx_n_s_debug);
  Py_CLEAR(clear_module_state->__pyx_n_s_defined_units);
  Py_CLEAR(clear_module_state->__pyx_n_s_density_factor);
  Py_CLEAR(clear_module_state->__pyx_n_s_density_ratio);
  Py_CLEAR(clear_module_state->__pyx_n_s_diameter);
  Py_CLEAR(clear_module_state->__pyx_n_s_direction_from);
  Py_CLEAR(clear_module_state->__pyx_kp_u_disable);
  Py_CLEAR(clear_module_state->__pyx_n_s_dist_step);
  Py_CLEAR(clear_module_state->__pyx_n_s_distance);
  Py_CLEAR(clear_module_state->__pyx_n_s_dm);
  Py_CLEAR(clear_module_state->__pyx_n_s_drag);
  Py_CLEAR(clear_module_state->__pyx_n_s_drag_table);
  Py_CLEAR(clear_module_state->__pyx_n_s_drop_adj);
  Py_CLEAR(clear_module_state->__pyx_kp_u_enable);
  Py_CLEAR(clear_module_state->__pyx_n_s_energy);
  Py_CLEAR(clear_module_state->__pyx_kp_u_euler_cy_it);
  Py_CLEAR(clear_module_state->__pyx_n_s_extra_data);
  Py_CLEAR(clear_module_state->__pyx_n_s_feet);
  Py_CLEAR(clear_module_state->__pyx_n_s_filter_flags);
  Py_CLEAR(clear_module_state->__pyx_n_s_flag);
  Py_CLEAR(clear_module_state->__pyx_n_s_fps);
  Py_CLEAR(clear_module_state->__pyx_kp_u_gc);
  Py_CLEAR(clear_module_state->__pyx_n_s_get_in);
  Py_CLEAR(clear_module_state->__pyx_n_s_get_velocity_for_temp);
  Py_CLEAR(clear_module_state->__pyx_n_s_getstate);
  Py_CLEAR(clear_module_state->__pyx_n_s_grain);
  Py_CLEAR(clear_module_state->__pyx_n_s_height);
  Py_CLEAR(clear_module_state->__pyx_n_s_import);
  Py_CLEAR(clear_module_state->__pyx_n_s_inch);
  Py_CLEAR(clear_module_state->__pyx_n_s_initializing);
  Py_CLEAR(clear_module_state->__pyx_kp_u_isenabled);
  Py_CLEAR(clear_module_state->__pyx_n_s_length);
  Py_CLEAR(clear_module_state->__pyx_n_s_logger);
  Py_CLEAR(clear_module_state->__pyx_n_s_look_angle);
  Py_CLEAR(clear_module_state->__pyx_n_s_look_distance);
  Py_CLEAR(clear_module_state->__pyx_n_s_mach);
  Py_CLEAR(clear_module_state->__pyx_n_s_mach1);
  Py_CLEAR(clear_module_state->__pyx_n_s_mach_list);
  Py_CLEAR(clear_module_state->__pyx_n_s_main);
  Py_CLEAR(clear_module_state->__pyx_n_s_max_range);
  Py_CLEAR(clear_module_state->__pyx_n_s_name);
  Py_CLEAR(clear_module_state->__pyx_n_s_new);
  Py_CLEAR(clear_module_state->__pyx_kp_s_no_default___reduce___due_to_non);
  Py_CLEAR(clear_module_state->__pyx_n_s_object);
  Py_CLEAR(clear_module_state->__pyx_n_s_ogw);
  Py_CLEAR(clear_module_state->__pyx_n_u_once);
  Py_CLEAR(clear_module_state->__pyx_n_s_p0);
  Py_CLEAR(clear_module_state->__pyx_n_s_powder_temp);
  Py_CLEAR(clear_module_state->__pyx_n_s_pressure);
  Py_CLEAR(clear_module_state->__pyx_n_s_py_ballisticcalc_constants);
  Py_CLEAR(clear_module_state->__pyx_n_s_py_ballisticcalc_exceptions);
  Py_CLEAR(clear_module_state->__pyx_n_s_py_ballisticcalc_logger);
  Py_CLEAR(clear_module_state->__pyx_n_s_py_ballisticcalc_unit);
  Py_CLEAR(clear_module_state->__pyx_n_s_pyx_state);
  Py_CLEAR(clear_module_state->__pyx_n_s_pyx_vtable);
  Py_CLEAR(clear_module_state->__pyx_n_s_rad);
  Py_CLEAR(clear_module_state->__pyx_n_s_ranges_length);
  Py_CLEAR(clear_module_state->__pyx_n_s_reduce);
  Py_CLEAR(clear_module_state->__pyx_n_s_reduce_cython);
  Py_CLEAR(clear_module_state->__pyx_n_s_reduce_ex);
  Py_CLEAR(clear_module_state->__pyx_n_s_setstate);
  Py_CLEAR(clear_module_state->__pyx_n_s_setstate_cython);
  Py_CLEAR(clear_module_state->__pyx_n_s_shot_info);
  Py_CLEAR(clear_module_state->__pyx_n_s_sight_height);
  Py_CLEAR(clear_module_state->__pyx_n_s_simplefilter);
  Py_CLEAR(clear_module_state->__pyx_n_s_spec);
  Py_CLEAR(clear_module_state->__pyx_n_s_stability_coefficient);
  Py_CLEAR(clear_module_state->__pyx_n_s_t0);
  Py_CLEAR(clear_module_state->__pyx_n_s_target_drop);
  Py_CLEAR(clear_module_state->__pyx_n_s_test);
  Py_CLEAR(clear_module_state->__pyx_n_s_time);
  Py_CLEAR(clear_module_state->__pyx_n_s_time_step);
  Py_CLEAR(clear_module_state->__pyx_n_s_twist);
  Py_CLEAR(clear_module_state->__pyx_n_s_until_distance);
  Py_CLEAR(clear_module_state->__pyx_n_s_value);
  Py_CLEAR(clear_module_state->__pyx_n_s_velocity);
  Py_CLEAR(clear_module_state->__pyx_n_s_warnings);
  Py_CLEAR(clear_module_state->__pyx_n_s_weapon);
  Py_CLEAR(clear_module_state->__pyx_n_s_weight);
  Py_CLEAR(clear_module_state->__pyx_n_s_windage);
  Py_CLEAR(clear_module_state->__pyx_n_s_windage_adj);
  Py_CLEAR(clear_module_state->__pyx_n_s_winds);
  Py_CLEAR(clear_module_state->__pyx_n_s_x);
  Py_CLEAR(clear_module_state->__pyx_n_s_y);
  Py_CLEAR(clear_module_state->__pyx_n_s_z);
  Py_CLEAR(clear_module_state->__pyx_tuple_);
  Py_CLEAR(clear_module_state->__pyx_tuple__2);
  Py_CLEAR(clear_module_state->__pyx_tuple__3);
  Py_CLEAR(clear_module_state->__pyx_tuple__4);
  Py_CLEAR(clear_module_state->__pyx_tuple__7);
  return 0;
}
#endif
/* #### Code section: module_state_traverse ### */
#if CYTHON_USE_MODULE_STATE
static int __pyx_m_traverse(PyObject *m, visitproc visit, void *arg) {
  __pyx_mstate *traverse_module_state = __pyx_mstate(m);
  if (!traverse_module_state) return 0;
  Py_VISIT(traverse_module_state->__pyx_d);
  Py_VISIT(traverse_module_state->__pyx_b);
  Py_VISIT(traverse_module_state->__pyx_cython_runtime);
  Py_VISIT(traverse_module_state->__pyx_empty_tuple);
  Py_VISIT(traverse_module_state->__pyx_empty_bytes);
  Py_VISIT(traverse_module_state->__pyx_empty_unicode);
  #ifdef __Pyx_CyFunction_USED
  Py_VISIT(traverse_module_state->__pyx_CyFunctionType);
  #endif
  #ifdef __Pyx_FusedFunction_USED
  Py_VISIT(traverse_module_state->__pyx_FusedFunctionType);
  #endif
  Py_VISIT(traverse_module_state->__pyx_ptype_16CyEulerBallistic_6vector_Vector);
  Py_VISIT(traverse_module_state->__pyx_ptype_16CyEulerBallistic_15trajectory_data_TrajectoryData);
  Py_VISIT(traverse_module_state->__pyx_ptype_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter);
  Py_VISIT(traverse_module_state->__pyx_type_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter);
  Py_VISIT(traverse_module_state->__pyx_ptype_16CyEulerBallistic_15trajectory_calc__WindSock);
  Py_VISIT(traverse_module_state->__pyx_type_16CyEulerBallistic_15trajectory_calc__WindSock);
  Py_VISIT(traverse_module_state->__pyx_ptype_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc);
  Py_VISIT(traverse_module_state->__pyx_type_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc);
  Py_VISIT(traverse_module_state->__pyx_n_s_Angular);
  Py_VISIT(traverse_module_state->__pyx_n_s_BC);
  Py_VISIT(traverse_module_state->__pyx_n_s_Distance);
  Py_VISIT(traverse_module_state->__pyx_n_s_Energy);
  Py_VISIT(traverse_module_state->__pyx_n_s_EulerTrajectoryCalc);
  Py_VISIT(traverse_module_state->__pyx_n_u_EulerTrajectoryCalc);
  Py_VISIT(traverse_module_state->__pyx_n_s_FPS);
  Py_VISIT(traverse_module_state->__pyx_n_s_Foot);
  Py_VISIT(traverse_module_state->__pyx_n_s_FootPound);
  Py_VISIT(traverse_module_state->__pyx_n_s_InHg);
  Py_VISIT(traverse_module_state->__pyx_n_s_KeyError);
  Py_VISIT(traverse_module_state->__pyx_n_s_MAX_DISTANCE_FEET);
  Py_VISIT(traverse_module_state->__pyx_n_s_MaximumDropReached);
  Py_VISIT(traverse_module_state->__pyx_n_s_MinimumAltitudeReached);
  Py_VISIT(traverse_module_state->__pyx_n_s_MinimumVelocityReached);
  Py_VISIT(traverse_module_state->__pyx_kp_s_No_value_specified_for_struct_at);
  Py_VISIT(traverse_module_state->__pyx_kp_s_No_value_specified_for_struct_at_2);
  Py_VISIT(traverse_module_state->__pyx_kp_s_No_value_specified_for_struct_at_3);
  Py_VISIT(traverse_module_state->__pyx_kp_s_No_value_specified_for_struct_at_4);
  Py_VISIT(traverse_module_state->__pyx_n_s_Pound);
  Py_VISIT(traverse_module_state->__pyx_n_s_Radian);
  Py_VISIT(traverse_module_state->__pyx_n_s_RangeError);
  Py_VISIT(traverse_module_state->__pyx_n_s_TrajectoryDataFilter);
  Py_VISIT(traverse_module_state->__pyx_n_s_TypeError);
  Py_VISIT(traverse_module_state->__pyx_n_s_Unit);
  Py_VISIT(traverse_module_state->__pyx_n_s_ValueError);
  Py_VISIT(traverse_module_state->__pyx_n_s_Velocity);
  Py_VISIT(traverse_module_state->__pyx_n_s_Weight);
  Py_VISIT(traverse_module_state->__pyx_n_s_WindSock);
  Py_VISIT(traverse_module_state->__pyx_n_s_ZeroFindingError);
  Py_VISIT(traverse_module_state->__pyx_n_s__5);
  Py_VISIT(traverse_module_state->__pyx_kp_u__6);
  Py_VISIT(traverse_module_state->__pyx_n_s__8);
  Py_VISIT(traverse_module_state->__pyx_n_s_a0);
  Py_VISIT(traverse_module_state->__pyx_n_s_all);
  Py_VISIT(traverse_module_state->__pyx_n_s_alt0);
  Py_VISIT(traverse_module_state->__pyx_n_s_altitude);
  Py_VISIT(traverse_module_state->__pyx_n_s_ammo);
  Py_VISIT(traverse_module_state->__pyx_n_s_angle);
  Py_VISIT(traverse_module_state->__pyx_n_s_atmo);
  Py_VISIT(traverse_module_state->__pyx_n_s_barrel_azimuth);
  Py_VISIT(traverse_module_state->__pyx_n_s_barrel_elevation);
  Py_VISIT(traverse_module_state->__pyx_n_s_bc);
  Py_VISIT(traverse_module_state->__pyx_n_s_cGravityConstant);
  Py_VISIT(traverse_module_state->__pyx_n_s_cMaxWindDistanceFeet);
  Py_VISIT(traverse_module_state->__pyx_n_s_calc_step);
  Py_VISIT(traverse_module_state->__pyx_n_s_cant_angle);
  Py_VISIT(traverse_module_state->__pyx_n_s_cant_cosine);
  Py_VISIT(traverse_module_state->__pyx_n_s_cant_sine);
  Py_VISIT(traverse_module_state->__pyx_n_s_cline_in_traceback);
  Py_VISIT(traverse_module_state->__pyx_n_s_config);
  Py_VISIT(traverse_module_state->__pyx_n_s_curve);
  Py_VISIT(traverse_module_state->__pyx_n_s_debug);
  Py_VISIT(traverse_module_state->__pyx_n_s_defined_units);
  Py_VISIT(traverse_module_state->__pyx_n_s_density_factor);
  Py_VISIT(traverse_module_state->__pyx_n_s_density_ratio);
  Py_VISIT(traverse_module_state->__pyx_n_s_diameter);
  Py_VISIT(traverse_module_state->__pyx_n_s_direction_from);
  Py_VISIT(traverse_module_state->__pyx_kp_u_disable);
  Py_VISIT(traverse_module_state->__pyx_n_s_dist_step);
  Py_VISIT(traverse_module_state->__pyx_n_s_distance);
  Py_VISIT(traverse_module_state->__pyx_n_s_dm);
  Py_VISIT(traverse_module_state->__pyx_n_s_drag);
  Py_VISIT(traverse_module_state->__pyx_n_s_drag_table);
  Py_VISIT(traverse_module_state->__pyx_n_s_drop_adj);
  Py_VISIT(traverse_module_state->__pyx_kp_u_enable);
  Py_VISIT(traverse_module_state->__pyx_n_s_energy);
  Py_VISIT(traverse_module_state->__pyx_kp_u_euler_cy_it);
  Py_VISIT(traverse_module_state->__pyx_n_s_extra_data);
  Py_VISIT(traverse_module_state->__pyx_n_s_feet);
  Py_VISIT(traverse_module_state->__pyx_n_s_filter_flags);
  Py_VISIT(traverse_module_state->__pyx_n_s_flag);
  Py_VISIT(traverse_module_state->__pyx_n_s_fps);
  Py_VISIT(traverse_module_state->__pyx_kp_u_gc);
  Py_VISIT(traverse_module_state->__pyx_n_s_get_in);
  Py_VISIT(traverse_module_state->__pyx_n_s_get_velocity_for_temp);
  Py_VISIT(traverse_module_state->__pyx_n_s_getstate);
  Py_VISIT(traverse_module_state->__pyx_n_s_grain);
  Py_VISIT(traverse_module_state->__pyx_n_s_height);
  Py_VISIT(traverse_module_state->__pyx_n_s_import);
  Py_VISIT(traverse_module_state->__pyx_n_s_inch);
  Py_VISIT(traverse_module_state->__pyx_n_s_initializing);
  Py_VISIT(traverse_module_state->__pyx_kp_u_isenabled);
  Py_VISIT(traverse_module_state->__pyx_n_s_length);
  Py_VISIT(traverse_module_state->__pyx_n_s_logger);
  Py_VISIT(traverse_module_state->__pyx_n_s_look_angle);
  Py_VISIT(traverse_module_state->__pyx_n_s_look_distance);
  Py_VISIT(traverse_module_state->__pyx_n_s_mach);
  Py_VISIT(traverse_module_state->__pyx_n_s_mach1);
  Py_VISIT(traverse_module_state->__pyx_n_s_mach_list);
  Py_VISIT(traverse_module_state->__pyx_n_s_main);
  Py_VISIT(traverse_module_state->__pyx_n_s_max_range);
  Py_VISIT(traverse_module_state->__pyx_n_s_name);
  Py_VISIT(traverse_module_state->__pyx_n_s_new);
  Py_VISIT(traverse_module_state->__pyx_kp_s_no_default___reduce___due_to_non);
  Py_VISIT(traverse_module_state->__pyx_n_s_object);
  Py_VISIT(traverse_module_state->__pyx_n_s_ogw);
  Py_VISIT(traverse_module_state->__pyx_n_u_once);
  Py_VISIT(traverse_module_state->__pyx_n_s_p0);
  Py_VISIT(traverse_module_state->__pyx_n_s_powder_temp);
  Py_VISIT(traverse_module_state->__pyx_n_s_pressure);
  Py_VISIT(traverse_module_state->__pyx_n_s_py_ballisticcalc_constants);
  Py_VISIT(traverse_module_state->__pyx_n_s_py_ballisticcalc_exceptions);
  Py_VISIT(traverse_module_state->__pyx_n_s_py_ballisticcalc_logger);
  Py_VISIT(traverse_module_state->__pyx_n_s_py_ballisticcalc_unit);
  Py_VISIT(traverse_module_state->__pyx_n_s_pyx_state);
  Py_VISIT(traverse_module_state->__pyx_n_s_pyx_vtable);
  Py_VISIT(traverse_module_state->__pyx_n_s_rad);
  Py_VISIT(traverse_module_state->__pyx_n_s_ranges_length);
  Py_VISIT(traverse_module_state->__pyx_n_s_reduce);
  Py_VISIT(traverse_module_state->__pyx_n_s_reduce_cython);
  Py_VISIT(traverse_module_state->__pyx_n_s_reduce_ex);
  Py_VISIT(traverse_module_state->__pyx_n_s_setstate);
  Py_VISIT(traverse_module_state->__pyx_n_s_setstate_cython);
  Py_VISIT(traverse_module_state->__pyx_n_s_shot_info);
  Py_VISIT(traverse_module_state->__pyx_n_s_sight_height);
  Py_VISIT(traverse_module_state->__pyx_n_s_simplefilter);
  Py_VISIT(traverse_module_state->__pyx_n_s_spec);
  Py_VISIT(traverse_module_state->__pyx_n_s_stability_coefficient);
  Py_VISIT(traverse_module_state->__pyx_n_s_t0);
  Py_VISIT(traverse_module_state->__pyx_n_s_target_drop);
  Py_VISIT(traverse_module_state->__pyx_n_s_test);
  Py_VISIT(traverse_module_state->__pyx_n_s_time);
  Py_VISIT(traverse_module_state->__pyx_n_s_time_step);
  Py_VISIT(traverse_module_state->__pyx_n_s_twist);
  Py_VISIT(traverse_module_state->__pyx_n_s_until_distance);
  Py_VISIT(traverse_module_state->__pyx_n_s_value);
  Py_VISIT(traverse_module_state->__pyx_n_s_velocity);
  Py_VISIT(traverse_module_state->__pyx_n_s_warnings);
  Py_VISIT(traverse_module_state->__pyx_n_s_weapon);
  Py_VISIT(traverse_module_state->__pyx_n_s_weight);
  Py_VISIT(traverse_module_state->__pyx_n_s_windage);
  Py_VISIT(traverse_module_state->__pyx_n_s_windage_adj);
  Py_VISIT(traverse_module_state->__pyx_n_s_winds);
  Py_VISIT(traverse_module_state->__pyx_n_s_x);
  Py_VISIT(traverse_module_state->__pyx_n_s_y);
  Py_VISIT(traverse_module_state->__pyx_n_s_z);
  Py_VISIT(traverse_module_state->__pyx_tuple_);
  Py_VISIT(traverse_module_state->__pyx_tuple__2);
  Py_VISIT(traverse_module_state->__pyx_tuple__3);
  Py_VISIT(traverse_module_state->__pyx_tuple__4);
  Py_VISIT(traverse_module_state->__pyx_tuple__7);
  return 0;
}
#endif
/* #### Code section: module_state_defines ### */
#define __pyx_d __pyx_mstate_global->__pyx_d
#define __pyx_b __pyx_mstate_global->__pyx_b
#define __pyx_cython_runtime __pyx_mstate_global->__pyx_cython_runtime
#define __pyx_empty_tuple __pyx_mstate_global->__pyx_empty_tuple
#define __pyx_empty_bytes __pyx_mstate_global->__pyx_empty_bytes
#define __pyx_empty_unicode __pyx_mstate_global->__pyx_empty_unicode
#ifdef __Pyx_CyFunction_USED
#define __pyx_CyFunctionType __pyx_mstate_global->__pyx_CyFunctionType
#endif
#ifdef __Pyx_FusedFunction_USED
#define __pyx_FusedFunctionType __pyx_mstate_global->__pyx_FusedFunctionType
#endif
#ifdef __Pyx_Generator_USED
#define __pyx_GeneratorType __pyx_mstate_global->__pyx_GeneratorType
#endif
#ifdef __Pyx_IterableCoroutine_USED
#define __pyx_IterableCoroutineType __pyx_mstate_global->__pyx_IterableCoroutineType
#endif
#ifdef __Pyx_Coroutine_USED
#define __pyx_CoroutineAwaitType __pyx_mstate_global->__pyx_CoroutineAwaitType
#endif
#ifdef __Pyx_Coroutine_USED
#define __pyx_CoroutineType __pyx_mstate_global->__pyx_CoroutineType
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#define __pyx_ptype_16CyEulerBallistic_6vector_Vector __pyx_mstate_global->__pyx_ptype_16CyEulerBallistic_6vector_Vector
#if CYTHON_USE_MODULE_STATE
#endif
#define __pyx_ptype_16CyEulerBallistic_15trajectory_data_TrajectoryData __pyx_mstate_global->__pyx_ptype_16CyEulerBallistic_15trajectory_data_TrajectoryData
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#define __pyx_type_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter __pyx_mstate_global->__pyx_type_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter
#define __pyx_type_16CyEulerBallistic_15trajectory_calc__WindSock __pyx_mstate_global->__pyx_type_16CyEulerBallistic_15trajectory_calc__WindSock
#define __pyx_type_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc __pyx_mstate_global->__pyx_type_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc
#endif
#define __pyx_ptype_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter __pyx_mstate_global->__pyx_ptype_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter
#define __pyx_ptype_16CyEulerBallistic_15trajectory_calc__WindSock __pyx_mstate_global->__pyx_ptype_16CyEulerBallistic_15trajectory_calc__WindSock
#define __pyx_ptype_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc __pyx_mstate_global->__pyx_ptype_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc
#define __pyx_n_s_Angular __pyx_mstate_global->__pyx_n_s_Angular
#define __pyx_n_s_BC __pyx_mstate_global->__pyx_n_s_BC
#define __pyx_n_s_Distance __pyx_mstate_global->__pyx_n_s_Distance
#define __pyx_n_s_Energy __pyx_mstate_global->__pyx_n_s_Energy
#define __pyx_n_s_EulerTrajectoryCalc __pyx_mstate_global->__pyx_n_s_EulerTrajectoryCalc
#define __pyx_n_u_EulerTrajectoryCalc __pyx_mstate_global->__pyx_n_u_EulerTrajectoryCalc
#define __pyx_n_s_FPS __pyx_mstate_global->__pyx_n_s_FPS
#define __pyx_n_s_Foot __pyx_mstate_global->__pyx_n_s_Foot
#define __pyx_n_s_FootPound __pyx_mstate_global->__pyx_n_s_FootPound
#define __pyx_n_s_InHg __pyx_mstate_global->__pyx_n_s_InHg
#define __pyx_n_s_KeyError __pyx_mstate_global->__pyx_n_s_KeyError
#define __pyx_n_s_MAX_DISTANCE_FEET __pyx_mstate_global->__pyx_n_s_MAX_DISTANCE_FEET
#define __pyx_n_s_MaximumDropReached __pyx_mstate_global->__pyx_n_s_MaximumDropReached
#define __pyx_n_s_MinimumAltitudeReached __pyx_mstate_global->__pyx_n_s_MinimumAltitudeReached
#define __pyx_n_s_MinimumVelocityReached __pyx_mstate_global->__pyx_n_s_MinimumVelocityReached
#define __pyx_kp_s_No_value_specified_for_struct_at __pyx_mstate_global->__pyx_kp_s_No_value_specified_for_struct_at
#define __pyx_kp_s_No_value_specified_for_struct_at_2 __pyx_mstate_global->__pyx_kp_s_No_value_specified_for_struct_at_2
#define __pyx_kp_s_No_value_specified_for_struct_at_3 __pyx_mstate_global->__pyx_kp_s_No_value_specified_for_struct_at_3
#define __pyx_kp_s_No_value_specified_for_struct_at_4 __pyx_mstate_global->__pyx_kp_s_No_value_specified_for_struct_at_4
#define __pyx_n_s_Pound __pyx_mstate_global->__pyx_n_s_Pound
#define __pyx_n_s_Radian __pyx_mstate_global->__pyx_n_s_Radian
#define __pyx_n_s_RangeError __pyx_mstate_global->__pyx_n_s_RangeError
#define __pyx_n_s_TrajectoryDataFilter __pyx_mstate_global->__pyx_n_s_TrajectoryDataFilter
#define __pyx_n_s_TypeError __pyx_mstate_global->__pyx_n_s_TypeError
#define __pyx_n_s_Unit __pyx_mstate_global->__pyx_n_s_Unit
#define __pyx_n_s_ValueError __pyx_mstate_global->__pyx_n_s_ValueError
#define __pyx_n_s_Velocity __pyx_mstate_global->__pyx_n_s_Velocity
#define __pyx_n_s_Weight __pyx_mstate_global->__pyx_n_s_Weight
#define __pyx_n_s_WindSock __pyx_mstate_global->__pyx_n_s_WindSock
#define __pyx_n_s_ZeroFindingError __pyx_mstate_global->__pyx_n_s_ZeroFindingError
#define __pyx_n_s__5 __pyx_mstate_global->__pyx_n_s__5
#define __pyx_kp_u__6 __pyx_mstate_global->__pyx_kp_u__6
#define __pyx_n_s__8 __pyx_mstate_global->__pyx_n_s__8
#define __pyx_n_s_a0 __pyx_mstate_global->__pyx_n_s_a0
#define __pyx_n_s_all __pyx_mstate_global->__pyx_n_s_all
#define __pyx_n_s_alt0 __pyx_mstate_global->__pyx_n_s_alt0
#define __pyx_n_s_altitude __pyx_mstate_global->__pyx_n_s_altitude
#define __pyx_n_s_ammo __pyx_mstate_global->__pyx_n_s_ammo
#define __pyx_n_s_angle __pyx_mstate_global->__pyx_n_s_angle
#define __pyx_n_s_atmo __pyx_mstate_global->__pyx_n_s_atmo
#define __pyx_n_s_barrel_azimuth __pyx_mstate_global->__pyx_n_s_barrel_azimuth
#define __pyx_n_s_barrel_elevation __pyx_mstate_global->__pyx_n_s_barrel_elevation
#define __pyx_n_s_bc __pyx_mstate_global->__pyx_n_s_bc
#define __pyx_n_s_cGravityConstant __pyx_mstate_global->__pyx_n_s_cGravityConstant
#define __pyx_n_s_cMaxWindDistanceFeet __pyx_mstate_global->__pyx_n_s_cMaxWindDistanceFeet
#define __pyx_n_s_calc_step __pyx_mstate_global->__pyx_n_s_calc_step
#define __pyx_n_s_cant_angle __pyx_mstate_global->__pyx_n_s_cant_angle
#define __pyx_n_s_cant_cosine __pyx_mstate_global->__pyx_n_s_cant_cosine
#define __pyx_n_s_cant_sine __pyx_mstate_global->__pyx_n_s_cant_sine
#define __pyx_n_s_cline_in_traceback __pyx_mstate_global->__pyx_n_s_cline_in_traceback
#define __pyx_n_s_config __pyx_mstate_global->__pyx_n_s_config
#define __pyx_n_s_curve __pyx_mstate_global->__pyx_n_s_curve
#define __pyx_n_s_debug __pyx_mstate_global->__pyx_n_s_debug
#define __pyx_n_s_defined_units __pyx_mstate_global->__pyx_n_s_defined_units
#define __pyx_n_s_density_factor __pyx_mstate_global->__pyx_n_s_density_factor
#define __pyx_n_s_density_ratio __pyx_mstate_global->__pyx_n_s_density_ratio
#define __pyx_n_s_diameter __pyx_mstate_global->__pyx_n_s_diameter
#define __pyx_n_s_direction_from __pyx_mstate_global->__pyx_n_s_direction_from
#define __pyx_kp_u_disable __pyx_mstate_global->__pyx_kp_u_disable
#define __pyx_n_s_dist_step __pyx_mstate_global->__pyx_n_s_dist_step
#define __pyx_n_s_distance __pyx_mstate_global->__pyx_n_s_distance
#define __pyx_n_s_dm __pyx_mstate_global->__pyx_n_s_dm
#define __pyx_n_s_drag __pyx_mstate_global->__pyx_n_s_drag
#define __pyx_n_s_drag_table __pyx_mstate_global->__pyx_n_s_drag_table
#define __pyx_n_s_drop_adj __pyx_mstate_global->__pyx_n_s_drop_adj
#define __pyx_kp_u_enable __pyx_mstate_global->__pyx_kp_u_enable
#define __pyx_n_s_energy __pyx_mstate_global->__pyx_n_s_energy
#define __pyx_kp_u_euler_cy_it __pyx_mstate_global->__pyx_kp_u_euler_cy_it
#define __pyx_n_s_extra_data __pyx_mstate_global->__pyx_n_s_extra_data
#define __pyx_n_s_feet __pyx_mstate_global->__pyx_n_s_feet
#define __pyx_n_s_filter_flags __pyx_mstate_global->__pyx_n_s_filter_flags
#define __pyx_n_s_flag __pyx_mstate_global->__pyx_n_s_flag
#define __pyx_n_s_fps __pyx_mstate_global->__pyx_n_s_fps
#define __pyx_kp_u_gc __pyx_mstate_global->__pyx_kp_u_gc
#define __pyx_n_s_get_in __pyx_mstate_global->__pyx_n_s_get_in
#define __pyx_n_s_get_velocity_for_temp __pyx_mstate_global->__pyx_n_s_get_velocity_for_temp
#define __pyx_n_s_getstate __pyx_mstate_global->__pyx_n_s_getstate
#define __pyx_n_s_grain __pyx_mstate_global->__pyx_n_s_grain
#define __pyx_n_s_height __pyx_mstate_global->__pyx_n_s_height
#define __pyx_n_s_import __pyx_mstate_global->__pyx_n_s_import
#define __pyx_n_s_inch __pyx_mstate_global->__pyx_n_s_inch
#define __pyx_n_s_initializing __pyx_mstate_global->__pyx_n_s_initializing
#define __pyx_kp_u_isenabled __pyx_mstate_global->__pyx_kp_u_isenabled
#define __pyx_n_s_length __pyx_mstate_global->__pyx_n_s_length
#define __pyx_n_s_logger __pyx_mstate_global->__pyx_n_s_logger
#define __pyx_n_s_look_angle __pyx_mstate_global->__pyx_n_s_look_angle
#define __pyx_n_s_look_distance __pyx_mstate_global->__pyx_n_s_look_distance
#define __pyx_n_s_mach __pyx_mstate_global->__pyx_n_s_mach
#define __pyx_n_s_mach1 __pyx_mstate_global->__pyx_n_s_mach1
#define __pyx_n_s_mach_list __pyx_mstate_global->__pyx_n_s_mach_list
#define __pyx_n_s_main __pyx_mstate_global->__pyx_n_s_main
#define __pyx_n_s_max_range __pyx_mstate_global->__pyx_n_s_max_range
#define __pyx_n_s_name __pyx_mstate_global->__pyx_n_s_name
#define __pyx_n_s_new __pyx_mstate_global->__pyx_n_s_new
#define __pyx_kp_s_no_default___reduce___due_to_non __pyx_mstate_global->__pyx_kp_s_no_default___reduce___due_to_non
#define __pyx_n_s_object __pyx_mstate_global->__pyx_n_s_object
#define __pyx_n_s_ogw __pyx_mstate_global->__pyx_n_s_ogw
#define __pyx_n_u_once __pyx_mstate_global->__pyx_n_u_once
#define __pyx_n_s_p0 __pyx_mstate_global->__pyx_n_s_p0
#define __pyx_n_s_powder_temp __pyx_mstate_global->__pyx_n_s_powder_temp
#define __pyx_n_s_pressure __pyx_mstate_global->__pyx_n_s_pressure
#define __pyx_n_s_py_ballisticcalc_constants __pyx_mstate_global->__pyx_n_s_py_ballisticcalc_constants
#define __pyx_n_s_py_ballisticcalc_exceptions __pyx_mstate_global->__pyx_n_s_py_ballisticcalc_exceptions
#define __pyx_n_s_py_ballisticcalc_logger __pyx_mstate_global->__pyx_n_s_py_ballisticcalc_logger
#define __pyx_n_s_py_ballisticcalc_unit __pyx_mstate_global->__pyx_n_s_py_ballisticcalc_unit
#define __pyx_n_s_pyx_state __pyx_mstate_global->__pyx_n_s_pyx_state
#define __pyx_n_s_pyx_vtable __pyx_mstate_global->__pyx_n_s_pyx_vtable
#define __pyx_n_s_rad __pyx_mstate_global->__pyx_n_s_rad
#define __pyx_n_s_ranges_length __pyx_mstate_global->__pyx_n_s_ranges_length
#define __pyx_n_s_reduce __pyx_mstate_global->__pyx_n_s_reduce
#define __pyx_n_s_reduce_cython __pyx_mstate_global->__pyx_n_s_reduce_cython
#define __pyx_n_s_reduce_ex __pyx_mstate_global->__pyx_n_s_reduce_ex
#define __pyx_n_s_setstate __pyx_mstate_global->__pyx_n_s_setstate
#define __pyx_n_s_setstate_cython __pyx_mstate_global->__pyx_n_s_setstate_cython
#define __pyx_n_s_shot_info __pyx_mstate_global->__pyx_n_s_shot_info
#define __pyx_n_s_sight_height __pyx_mstate_global->__pyx_n_s_sight_height
#define __pyx_n_s_simplefilter __pyx_mstate_global->__pyx_n_s_simplefilter
#define __pyx_n_s_spec __pyx_mstate_global->__pyx_n_s_spec
#define __pyx_n_s_stability_coefficient __pyx_mstate_global->__pyx_n_s_stability_coefficient
#define __pyx_n_s_t0 __pyx_mstate_global->__pyx_n_s_t0
#define __pyx_n_s_target_drop __pyx_mstate_global->__pyx_n_s_target_drop
#define __pyx_n_s_test __pyx_mstate_global->__pyx_n_s_test
#define __pyx_n_s_time __pyx_mstate_global->__pyx_n_s_time
#define __pyx_n_s_time_step __pyx_mstate_global->__pyx_n_s_time_step
#define __pyx_n_s_twist __pyx_mstate_global->__pyx_n_s_twist
#define __pyx_n_s_until_distance __pyx_mstate_global->__pyx_n_s_until_distance
#define __pyx_n_s_value __pyx_mstate_global->__pyx_n_s_value
#define __pyx_n_s_velocity __pyx_mstate_global->__pyx_n_s_velocity
#define __pyx_n_s_warnings __pyx_mstate_global->__pyx_n_s_warnings
#define __pyx_n_s_weapon __pyx_mstate_global->__pyx_n_s_weapon
#define __pyx_n_s_weight __pyx_mstate_global->__pyx_n_s_weight
#define __pyx_n_s_windage __pyx_mstate_global->__pyx_n_s_windage
#define __pyx_n_s_windage_adj __pyx_mstate_global->__pyx_n_s_windage_adj
#define __pyx_n_s_winds __pyx_mstate_global->__pyx_n_s_winds
#define __pyx_n_s_x __pyx_mstate_global->__pyx_n_s_x
#define __pyx_n_s_y __pyx_mstate_global->__pyx_n_s_y
#define __pyx_n_s_z __pyx_mstate_global->__pyx_n_s_z
#define __pyx_tuple_ __pyx_mstate_global->__pyx_tuple_
#define __pyx_tuple__2 __pyx_mstate_global->__pyx_tuple__2
#define __pyx_tuple__3 __pyx_mstate_global->__pyx_tuple__3
#define __pyx_tuple__4 __pyx_mstate_global->__pyx_tuple__4
#define __pyx_tuple__7 __pyx_mstate_global->__pyx_tuple__7
/* #### Code section: module_code ### */

/* "FromPyStructUtility":12
 * 
 * @cname("__pyx_convert__from_py_struct____pyx_t_16CyEulerBallistic_8cy_euler_Wind_t")
 * cdef struct_type __pyx_convert__from_py_struct____pyx_t_16CyEulerBallistic_8cy_euler_Wind_t(obj) except *:             # <<<<<<<<<<<<<<
 *     cdef struct_type result
 *     if not PyMapping_Check(obj):
 */

static struct __pyx_t_16CyEulerBallistic_8cy_euler_Wind_t __pyx_convert__from_py_struct____pyx_t_16CyEulerBallistic_8cy_euler_Wind_t(PyObject *__pyx_v_obj) {
  struct __pyx_t_16CyEulerBallistic_8cy_euler_Wind_t __pyx_v_result;
  PyObject *__pyx_v_value = NULL;
  struct __pyx_t_16CyEulerBallistic_8cy_euler_Wind_t __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  double __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_convert__from_py_struct____pyx_t_16CyEulerBallistic_8cy_euler_Wind_t", 1);

  /* "FromPyStructUtility":14
 * cdef struct_type __pyx_convert__from_py_struct____pyx_t_16CyEulerBallistic_8cy_euler_Wind_t(obj) except *:
 *     cdef struct_type result
 *     if not PyMapping_Check(obj):             # <<<<<<<<<<<<<<
 *         __Pyx_RaiseUnexpectedTypeError(b"a mapping", obj)
 * 
 */
  __pyx_t_1 = (!PyMapping_Check(__pyx_v_obj));
  if (__pyx_t_1) {

    /* "FromPyStructUtility":15
 *     cdef struct_type result
 *     if not PyMapping_Check(obj):
 *         __Pyx_RaiseUnexpectedTypeError(b"a mapping", obj)             # <<<<<<<<<<<<<<
 * 
 *     try:
 */
    __pyx_t_2 = __Pyx_RaiseUnexpectedTypeError(((char const *)"a mapping"), __pyx_v_obj); if (unlikely(__pyx_t_2 == ((int)0))) __PYX_ERR(0, 15, __pyx_L1_error)

    /* "FromPyStructUtility":14
 * cdef struct_type __pyx_convert__from_py_struct____pyx_t_16CyEulerBallistic_8cy_euler_Wind_t(obj) except *:
 *     cdef struct_type result
 *     if not PyMapping_Check(obj):             # <<<<<<<<<<<<<<
 *         __Pyx_RaiseUnexpectedTypeError(b"a mapping", obj)
 * 
 */
  }

  /* "FromPyStructUtility":17
 *         __Pyx_RaiseUnexpectedTypeError(b"a mapping", obj)
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         value = obj['velocity']
 *     except KeyError:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_3, &__pyx_t_4, &__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    /*try:*/ {

      /* "FromPyStructUtility":18
 * 
 *     try:
 *         value = obj['velocity']             # <<<<<<<<<<<<<<
 *     except KeyError:
 *         raise ValueError("No value specified for struct attribute 'velocity'")
 */
      __pyx_t_6 = __Pyx_PyObject_Dict_GetItem(__pyx_v_obj, __pyx_n_s_velocity); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 18, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_v_value = __pyx_t_6;
      __pyx_t_6 = 0;

      /* "FromPyStructUtility":17
 *         __Pyx_RaiseUnexpectedTypeError(b"a mapping", obj)
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         value = obj['velocity']
 *     except KeyError:
 */
    }
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    goto __pyx_L9_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "FromPyStructUtility":19
 *     try:
 *         value = obj['velocity']
 *     except KeyError:             # <<<<<<<<<<<<<<
 *         raise ValueError("No value specified for struct attribute 'velocity'")
 *     result.velocity = value
 */
    __pyx_t_2 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_KeyError);
    if (__pyx_t_2) {
      __Pyx_AddTraceback("FromPyStructUtility.__pyx_convert__from_py_struct____pyx_t_16CyEulerBallistic_8cy_euler_Wind_t", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8) < 0) __PYX_ERR(0, 19, __pyx_L6_except_error)
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_7);
      __Pyx_XGOTREF(__pyx_t_8);

      /* "FromPyStructUtility":20
 *         value = obj['velocity']
 *     except KeyError:
 *         raise ValueError("No value specified for struct attribute 'velocity'")             # <<<<<<<<<<<<<<
 *     result.velocity = value
 *     try:
 */
      __pyx_t_9 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 20, __pyx_L6_except_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_Raise(__pyx_t_9, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __PYX_ERR(0, 20, __pyx_L6_except_error)
    }
    goto __pyx_L6_except_error;

    /* "FromPyStructUtility":17
 *         __Pyx_RaiseUnexpectedTypeError(b"a mapping", obj)
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         value = obj['velocity']
 *     except KeyError:
 */
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
    goto __pyx_L1_error;
    __pyx_L9_try_end:;
  }

  /* "FromPyStructUtility":21
 *     except KeyError:
 *         raise ValueError("No value specified for struct attribute 'velocity'")
 *     result.velocity = value             # <<<<<<<<<<<<<<
 *     try:
 *         value = obj['direction_from']
 */
  __pyx_t_10 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_10 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 21, __pyx_L1_error)
  __pyx_v_result.velocity = __pyx_t_10;

  /* "FromPyStructUtility":22
 *         raise ValueError("No value specified for struct attribute 'velocity'")
 *     result.velocity = value
 *     try:             # <<<<<<<<<<<<<<
 *         value = obj['direction_from']
 *     except KeyError:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_5, &__pyx_t_4, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "FromPyStructUtility":23
 *     result.velocity = value
 *     try:
 *         value = obj['direction_from']             # <<<<<<<<<<<<<<
 *     except KeyError:
 *         raise ValueError("No value specified for struct attribute 'direction_from'")
 */
      __pyx_t_8 = __Pyx_PyObject_Dict_GetItem(__pyx_v_obj, __pyx_n_s_direction_from); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 23, __pyx_L12_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF_SET(__pyx_v_value, __pyx_t_8);
      __pyx_t_8 = 0;

      /* "FromPyStructUtility":22
 *         raise ValueError("No value specified for struct attribute 'velocity'")
 *     result.velocity = value
 *     try:             # <<<<<<<<<<<<<<
 *         value = obj['direction_from']
 *     except KeyError:
 */
    }
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L17_try_end;
    __pyx_L12_error:;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;

    /* "FromPyStructUtility":24
 *     try:
 *         value = obj['direction_from']
 *     except KeyError:             # <<<<<<<<<<<<<<
 *         raise ValueError("No value specified for struct attribute 'direction_from'")
 *     result.direction_from = value
 */
    __pyx_t_2 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_KeyError);
    if (__pyx_t_2) {
      __Pyx_AddTraceback("FromPyStructUtility.__pyx_convert__from_py_struct____pyx_t_16CyEulerBallistic_8cy_euler_Wind_t", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_8, &__pyx_t_7, &__pyx_t_6) < 0) __PYX_ERR(0, 24, __pyx_L14_except_error)
      __Pyx_XGOTREF(__pyx_t_8);
      __Pyx_XGOTREF(__pyx_t_7);
      __Pyx_XGOTREF(__pyx_t_6);

      /* "FromPyStructUtility":25
 *         value = obj['direction_from']
 *     except KeyError:
 *         raise ValueError("No value specified for struct attribute 'direction_from'")             # <<<<<<<<<<<<<<
 *     result.direction_from = value
 *     try:
 */
      __pyx_t_9 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 25, __pyx_L14_except_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_Raise(__pyx_t_9, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __PYX_ERR(0, 25, __pyx_L14_except_error)
    }
    goto __pyx_L14_except_error;

    /* "FromPyStructUtility":22
 *         raise ValueError("No value specified for struct attribute 'velocity'")
 *     result.velocity = value
 *     try:             # <<<<<<<<<<<<<<
 *         value = obj['direction_from']
 *     except KeyError:
 */
    __pyx_L14_except_error:;
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_4, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L17_try_end:;
  }

  /* "FromPyStructUtility":26
 *     except KeyError:
 *         raise ValueError("No value specified for struct attribute 'direction_from'")
 *     result.direction_from = value             # <<<<<<<<<<<<<<
 *     try:
 *         value = obj['until_distance']
 */
  __pyx_t_10 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_10 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 26, __pyx_L1_error)
  __pyx_v_result.direction_from = __pyx_t_10;

  /* "FromPyStructUtility":27
 *         raise ValueError("No value specified for struct attribute 'direction_from'")
 *     result.direction_from = value
 *     try:             # <<<<<<<<<<<<<<
 *         value = obj['until_distance']
 *     except KeyError:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_3, &__pyx_t_4, &__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    /*try:*/ {

      /* "FromPyStructUtility":28
 *     result.direction_from = value
 *     try:
 *         value = obj['until_distance']             # <<<<<<<<<<<<<<
 *     except KeyError:
 *         raise ValueError("No value specified for struct attribute 'until_distance'")
 */
      __pyx_t_6 = __Pyx_PyObject_Dict_GetItem(__pyx_v_obj, __pyx_n_s_until_distance); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 28, __pyx_L20_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF_SET(__pyx_v_value, __pyx_t_6);
      __pyx_t_6 = 0;

      /* "FromPyStructUtility":27
 *         raise ValueError("No value specified for struct attribute 'direction_from'")
 *     result.direction_from = value
 *     try:             # <<<<<<<<<<<<<<
 *         value = obj['until_distance']
 *     except KeyError:
 */
    }
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    goto __pyx_L25_try_end;
    __pyx_L20_error:;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;

    /* "FromPyStructUtility":29
 *     try:
 *         value = obj['until_distance']
 *     except KeyError:             # <<<<<<<<<<<<<<
 *         raise ValueError("No value specified for struct attribute 'until_distance'")
 *     result.until_distance = value
 */
    __pyx_t_2 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_KeyError);
    if (__pyx_t_2) {
      __Pyx_AddTraceback("FromPyStructUtility.__pyx_convert__from_py_struct____pyx_t_16CyEulerBallistic_8cy_euler_Wind_t", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8) < 0) __PYX_ERR(0, 29, __pyx_L22_except_error)
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_7);
      __Pyx_XGOTREF(__pyx_t_8);

      /* "FromPyStructUtility":30
 *         value = obj['until_distance']
 *     except KeyError:
 *         raise ValueError("No value specified for struct attribute 'until_distance'")             # <<<<<<<<<<<<<<
 *     result.until_distance = value
 *     try:
 */
      __pyx_t_9 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 30, __pyx_L22_except_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_Raise(__pyx_t_9, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __PYX_ERR(0, 30, __pyx_L22_except_error)
    }
    goto __pyx_L22_except_error;

    /* "FromPyStructUtility":27
 *         raise ValueError("No value specified for struct attribute 'direction_from'")
 *     result.direction_from = value
 *     try:             # <<<<<<<<<<<<<<
 *         value = obj['until_distance']
 *     except KeyError:
 */
    __pyx_L22_except_error:;
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
    goto __pyx_L1_error;
    __pyx_L25_try_end:;
  }

  /* "FromPyStructUtility":31
 *     except KeyError:
 *         raise ValueError("No value specified for struct attribute 'until_distance'")
 *     result.until_distance = value             # <<<<<<<<<<<<<<
 *     try:
 *         value = obj['MAX_DISTANCE_FEET']
 */
  __pyx_t_10 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_10 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 31, __pyx_L1_error)
  __pyx_v_result.until_distance = __pyx_t_10;

  /* "FromPyStructUtility":32
 *         raise ValueError("No value specified for struct attribute 'until_distance'")
 *     result.until_distance = value
 *     try:             # <<<<<<<<<<<<<<
 *         value = obj['MAX_DISTANCE_FEET']
 *     except KeyError:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_5, &__pyx_t_4, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "FromPyStructUtility":33
 *     result.until_distance = value
 *     try:
 *         value = obj['MAX_DISTANCE_FEET']             # <<<<<<<<<<<<<<
 *     except KeyError:
 *         raise ValueError("No value specified for struct attribute 'MAX_DISTANCE_FEET'")
 */
      __pyx_t_8 = __Pyx_PyObject_Dict_GetItem(__pyx_v_obj, __pyx_n_s_MAX_DISTANCE_FEET); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 33, __pyx_L28_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF_SET(__pyx_v_value, __pyx_t_8);
      __pyx_t_8 = 0;

      /* "FromPyStructUtility":32
 *         raise ValueError("No value specified for struct attribute 'until_distance'")
 *     result.until_distance = value
 *     try:             # <<<<<<<<<<<<<<
 *         value = obj['MAX_DISTANCE_FEET']
 *     except KeyError:
 */
    }
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L33_try_end;
    __pyx_L28_error:;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;

    /* "FromPyStructUtility":34
 *     try:
 *         value = obj['MAX_DISTANCE_FEET']
 *     except KeyError:             # <<<<<<<<<<<<<<
 *         raise ValueError("No value specified for struct attribute 'MAX_DISTANCE_FEET'")
 *     result.MAX_DISTANCE_FEET = value
 */
    __pyx_t_2 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_KeyError);
    if (__pyx_t_2) {
      __Pyx_AddTraceback("FromPyStructUtility.__pyx_convert__from_py_struct____pyx_t_16CyEulerBallistic_8cy_euler_Wind_t", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_8, &__pyx_t_7, &__pyx_t_6) < 0) __PYX_ERR(0, 34, __pyx_L30_except_error)
      __Pyx_XGOTREF(__pyx_t_8);
      __Pyx_XGOTREF(__pyx_t_7);
      __Pyx_XGOTREF(__pyx_t_6);

      /* "FromPyStructUtility":35
 *         value = obj['MAX_DISTANCE_FEET']
 *     except KeyError:
 *         raise ValueError("No value specified for struct attribute 'MAX_DISTANCE_FEET'")             # <<<<<<<<<<<<<<
 *     result.MAX_DISTANCE_FEET = value
 *     return result
 */
      __pyx_t_9 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 35, __pyx_L30_except_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_Raise(__pyx_t_9, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __PYX_ERR(0, 35, __pyx_L30_except_error)
    }
    goto __pyx_L30_except_error;

    /* "FromPyStructUtility":32
 *         raise ValueError("No value specified for struct attribute 'until_distance'")
 *     result.until_distance = value
 *     try:             # <<<<<<<<<<<<<<
 *         value = obj['MAX_DISTANCE_FEET']
 *     except KeyError:
 */
    __pyx_L30_except_error:;
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_4, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L33_try_end:;
  }

  /* "FromPyStructUtility":36
 *     except KeyError:
 *         raise ValueError("No value specified for struct attribute 'MAX_DISTANCE_FEET'")
 *     result.MAX_DISTANCE_FEET = value             # <<<<<<<<<<<<<<
 *     return result
 * 
 */
  __pyx_t_10 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_10 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 36, __pyx_L1_error)
  __pyx_v_result.MAX_DISTANCE_FEET = __pyx_t_10;

  /* "FromPyStructUtility":37
 *         raise ValueError("No value specified for struct attribute 'MAX_DISTANCE_FEET'")
 *     result.MAX_DISTANCE_FEET = value
 *     return result             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "FromPyStructUtility":12
 * 
 * @cname("__pyx_convert__from_py_struct____pyx_t_16CyEulerBallistic_8cy_euler_Wind_t")
 * cdef struct_type __pyx_convert__from_py_struct____pyx_t_16CyEulerBallistic_8cy_euler_Wind_t(obj) except *:             # <<<<<<<<<<<<<<
 *     cdef struct_type result
 *     if not PyMapping_Check(obj):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("FromPyStructUtility.__pyx_convert__from_py_struct____pyx_t_16CyEulerBallistic_8cy_euler_Wind_t", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_pretend_to_initialize(&__pyx_r);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "CyEulerBallistic/trajectory_calc.pyx":57
 *         double previous_mach, previous_time, next_range_distance, time_step, look_angle
 * 
 *     def __cinit__(_TrajectoryDataFilter self,             # <<<<<<<<<<<<<<
 *                   int filter_flags, int ranges_length, double time_step = 0.0) -> None:
 *         self.filter = filter_flags
 */

/* Python wrapper */
static int __pyx_pw_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_v_filter_flags;
  int __pyx_v_ranges_length;
  double __pyx_v_time_step;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_filter_flags,&__pyx_n_s_ranges_length,&__pyx_n_s_time_step,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  3: values[2] = __Pyx_Arg_VARARGS(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = __Pyx_Arg_VARARGS(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_VARARGS(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_VARARGS(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_VARARGS(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_filter_flags)) != 0)) {
          (void)__Pyx_Arg_NewRef_VARARGS(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 57, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_VARARGS(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_ranges_length)) != 0)) {
          (void)__Pyx_Arg_NewRef_VARARGS(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 57, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 2, 3, 1); __PYX_ERR(1, 57, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_VARARGS(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_time_step);
          if (value) { values[2] = __Pyx_Arg_NewRef_VARARGS(value); kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 57, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "__cinit__") < 0)) __PYX_ERR(1, 57, __pyx_L3_error)
      }
    } else {
      switch (__pyx_nargs) {
        case  3: values[2] = __Pyx_Arg_VARARGS(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = __Pyx_Arg_VARARGS(__pyx_args, 1);
        values[0] = __Pyx_Arg_VARARGS(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_filter_flags = __Pyx_PyInt_As_int(values[0]); if (unlikely((__pyx_v_filter_flags == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 58, __pyx_L3_error)
    __pyx_v_ranges_length = __Pyx_PyInt_As_int(values[1]); if (unlikely((__pyx_v_ranges_length == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 58, __pyx_L3_error)
    if (values[2]) {
      __pyx_v_time_step = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_time_step == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 58, __pyx_L3_error)
    } else {
      __pyx_v_time_step = ((double)0.0);
    }
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 2, 3, __pyx_nargs); __PYX_ERR(1, 57, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_VARARGS(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("CyEulerBallistic.trajectory_calc._TrajectoryDataFilter.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter___cinit__(((struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *)__pyx_v_self), __pyx_v_filter_flags, __pyx_v_ranges_length, __pyx_v_time_step);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_VARARGS(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter___cinit__(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *__pyx_v_self, int __pyx_v_filter_flags, int __pyx_v_ranges_length, double __pyx_v_time_step) {
  int __pyx_r;

  /* "CyEulerBallistic/trajectory_calc.pyx":59
 *     def __cinit__(_TrajectoryDataFilter self,
 *                   int filter_flags, int ranges_length, double time_step = 0.0) -> None:
 *         self.filter = filter_flags             # <<<<<<<<<<<<<<
 *         self.current_flag = CTrajFlag.NONE
 *         self.seen_zero = CTrajFlag.NONE
 */
  __pyx_v_self->filter = __pyx_v_filter_flags;

  /* "CyEulerBallistic/trajectory_calc.pyx":60
 *                   int filter_flags, int ranges_length, double time_step = 0.0) -> None:
 *         self.filter = filter_flags
 *         self.current_flag = CTrajFlag.NONE             # <<<<<<<<<<<<<<
 *         self.seen_zero = CTrajFlag.NONE
 *         self.time_step = time_step
 */
  __pyx_v_self->current_flag = __pyx_e_16CyEulerBallistic_15trajectory_data_NONE;

  /* "CyEulerBallistic/trajectory_calc.pyx":61
 *         self.filter = filter_flags
 *         self.current_flag = CTrajFlag.NONE
 *         self.seen_zero = CTrajFlag.NONE             # <<<<<<<<<<<<<<
 *         self.time_step = time_step
 *         self.current_item = 0
 */
  __pyx_v_self->seen_zero = __pyx_e_16CyEulerBallistic_15trajectory_data_NONE;

  /* "CyEulerBallistic/trajectory_calc.pyx":62
 *         self.current_flag = CTrajFlag.NONE
 *         self.seen_zero = CTrajFlag.NONE
 *         self.time_step = time_step             # <<<<<<<<<<<<<<
 *         self.current_item = 0
 *         self.ranges_length = ranges_length
 */
  __pyx_v_self->time_step = __pyx_v_time_step;

  /* "CyEulerBallistic/trajectory_calc.pyx":63
 *         self.seen_zero = CTrajFlag.NONE
 *         self.time_step = time_step
 *         self.current_item = 0             # <<<<<<<<<<<<<<
 *         self.ranges_length = ranges_length
 *         self.previous_mach = 0.0
 */
  __pyx_v_self->current_item = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":64
 *         self.time_step = time_step
 *         self.current_item = 0
 *         self.ranges_length = ranges_length             # <<<<<<<<<<<<<<
 *         self.previous_mach = 0.0
 *         self.previous_time = 0.0
 */
  __pyx_v_self->ranges_length = __pyx_v_ranges_length;

  /* "CyEulerBallistic/trajectory_calc.pyx":65
 *         self.current_item = 0
 *         self.ranges_length = ranges_length
 *         self.previous_mach = 0.0             # <<<<<<<<<<<<<<
 *         self.previous_time = 0.0
 *         self.next_range_distance = 0.0
 */
  __pyx_v_self->previous_mach = 0.0;

  /* "CyEulerBallistic/trajectory_calc.pyx":66
 *         self.ranges_length = ranges_length
 *         self.previous_mach = 0.0
 *         self.previous_time = 0.0             # <<<<<<<<<<<<<<
 *         self.next_range_distance = 0.0
 *         self.look_angle = 0
 */
  __pyx_v_self->previous_time = 0.0;

  /* "CyEulerBallistic/trajectory_calc.pyx":67
 *         self.previous_mach = 0.0
 *         self.previous_time = 0.0
 *         self.next_range_distance = 0.0             # <<<<<<<<<<<<<<
 *         self.look_angle = 0
 * 
 */
  __pyx_v_self->next_range_distance = 0.0;

  /* "CyEulerBallistic/trajectory_calc.pyx":68
 *         self.previous_time = 0.0
 *         self.next_range_distance = 0.0
 *         self.look_angle = 0             # <<<<<<<<<<<<<<
 * 
 *     cdef void setup_seen_zero(_TrajectoryDataFilter self, double height, double barrel_elevation, double look_angle):
 */
  __pyx_v_self->look_angle = 0.0;

  /* "CyEulerBallistic/trajectory_calc.pyx":57
 *         double previous_mach, previous_time, next_range_distance, time_step, look_angle
 * 
 *     def __cinit__(_TrajectoryDataFilter self,             # <<<<<<<<<<<<<<
 *                   int filter_flags, int ranges_length, double time_step = 0.0) -> None:
 *         self.filter = filter_flags
 */

  /* function exit code */
  __pyx_r = 0;
  return __pyx_r;
}

/* "CyEulerBallistic/trajectory_calc.pyx":70
 *         self.look_angle = 0
 * 
 *     cdef void setup_seen_zero(_TrajectoryDataFilter self, double height, double barrel_elevation, double look_angle):             # <<<<<<<<<<<<<<
 *         if height >= 0:
 *             self.seen_zero |= CTrajFlag.ZERO_UP
 */

static void __pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_setup_seen_zero(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *__pyx_v_self, double __pyx_v_height, double __pyx_v_barrel_elevation, double __pyx_v_look_angle) {
  int __pyx_t_1;
  int __pyx_t_2;

  /* "CyEulerBallistic/trajectory_calc.pyx":71
 * 
 *     cdef void setup_seen_zero(_TrajectoryDataFilter self, double height, double barrel_elevation, double look_angle):
 *         if height >= 0:             # <<<<<<<<<<<<<<
 *             self.seen_zero |= CTrajFlag.ZERO_UP
 *         elif height < 0 and barrel_elevation < look_angle:
 */
  __pyx_t_1 = (__pyx_v_height >= 0.0);
  if (__pyx_t_1) {

    /* "CyEulerBallistic/trajectory_calc.pyx":72
 *     cdef void setup_seen_zero(_TrajectoryDataFilter self, double height, double barrel_elevation, double look_angle):
 *         if height >= 0:
 *             self.seen_zero |= CTrajFlag.ZERO_UP             # <<<<<<<<<<<<<<
 *         elif height < 0 and barrel_elevation < look_angle:
 *             self.seen_zero |= CTrajFlag.ZERO_DOWN
 */
    __pyx_v_self->seen_zero = (__pyx_v_self->seen_zero | __pyx_e_16CyEulerBallistic_15trajectory_data_ZERO_UP);

    /* "CyEulerBallistic/trajectory_calc.pyx":71
 * 
 *     cdef void setup_seen_zero(_TrajectoryDataFilter self, double height, double barrel_elevation, double look_angle):
 *         if height >= 0:             # <<<<<<<<<<<<<<
 *             self.seen_zero |= CTrajFlag.ZERO_UP
 *         elif height < 0 and barrel_elevation < look_angle:
 */
    goto __pyx_L3;
  }

  /* "CyEulerBallistic/trajectory_calc.pyx":73
 *         if height >= 0:
 *             self.seen_zero |= CTrajFlag.ZERO_UP
 *         elif height < 0 and barrel_elevation < look_angle:             # <<<<<<<<<<<<<<
 *             self.seen_zero |= CTrajFlag.ZERO_DOWN
 *         self.look_angle = look_angle
 */
  __pyx_t_2 = (__pyx_v_height < 0.0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (__pyx_v_barrel_elevation < __pyx_v_look_angle);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "CyEulerBallistic/trajectory_calc.pyx":74
 *             self.seen_zero |= CTrajFlag.ZERO_UP
 *         elif height < 0 and barrel_elevation < look_angle:
 *             self.seen_zero |= CTrajFlag.ZERO_DOWN             # <<<<<<<<<<<<<<
 *         self.look_angle = look_angle
 * 
 */
    __pyx_v_self->seen_zero = (__pyx_v_self->seen_zero | __pyx_e_16CyEulerBallistic_15trajectory_data_ZERO_DOWN);

    /* "CyEulerBallistic/trajectory_calc.pyx":73
 *         if height >= 0:
 *             self.seen_zero |= CTrajFlag.ZERO_UP
 *         elif height < 0 and barrel_elevation < look_angle:             # <<<<<<<<<<<<<<
 *             self.seen_zero |= CTrajFlag.ZERO_DOWN
 *         self.look_angle = look_angle
 */
  }
  __pyx_L3:;

  /* "CyEulerBallistic/trajectory_calc.pyx":75
 *         elif height < 0 and barrel_elevation < look_angle:
 *             self.seen_zero |= CTrajFlag.ZERO_DOWN
 *         self.look_angle = look_angle             # <<<<<<<<<<<<<<
 * 
 *     cdef void clear_current_flag(_TrajectoryDataFilter self):
 */
  __pyx_v_self->look_angle = __pyx_v_look_angle;

  /* "CyEulerBallistic/trajectory_calc.pyx":70
 *         self.look_angle = 0
 * 
 *     cdef void setup_seen_zero(_TrajectoryDataFilter self, double height, double barrel_elevation, double look_angle):             # <<<<<<<<<<<<<<
 *         if height >= 0:
 *             self.seen_zero |= CTrajFlag.ZERO_UP
 */

  /* function exit code */
}

/* "CyEulerBallistic/trajectory_calc.pyx":77
 *         self.look_angle = look_angle
 * 
 *     cdef void clear_current_flag(_TrajectoryDataFilter self):             # <<<<<<<<<<<<<<
 *         self.current_flag = CTrajFlag.NONE
 * 
 */

static void __pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_clear_current_flag(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *__pyx_v_self) {

  /* "CyEulerBallistic/trajectory_calc.pyx":78
 * 
 *     cdef void clear_current_flag(_TrajectoryDataFilter self):
 *         self.current_flag = CTrajFlag.NONE             # <<<<<<<<<<<<<<
 * 
 *     cdef bint should_record(_TrajectoryDataFilter self,
 */
  __pyx_v_self->current_flag = __pyx_e_16CyEulerBallistic_15trajectory_data_NONE;

  /* "CyEulerBallistic/trajectory_calc.pyx":77
 *         self.look_angle = look_angle
 * 
 *     cdef void clear_current_flag(_TrajectoryDataFilter self):             # <<<<<<<<<<<<<<
 *         self.current_flag = CTrajFlag.NONE
 * 
 */

  /* function exit code */
}

/* "CyEulerBallistic/trajectory_calc.pyx":80
 *         self.current_flag = CTrajFlag.NONE
 * 
 *     cdef bint should_record(_TrajectoryDataFilter self,             # <<<<<<<<<<<<<<
 *                             CVector range_vector,
 *                             double velocity,
 */

static int __pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_should_record(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *__pyx_v_self, struct __pyx_t_16CyEulerBallistic_6vector_CVector __pyx_v_range_vector, double __pyx_v_velocity, double __pyx_v_mach, double __pyx_v_step, double __pyx_v_time) {
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":87
 *                             double time,
 *                             ):
 *         self.check_zero_crossing(range_vector)             # <<<<<<<<<<<<<<
 *         self.check_mach_crossing(velocity, mach)
 *         if self.check_next_range(range_vector.x, step):
 */
  __pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_check_zero_crossing(__pyx_v_self, __pyx_v_range_vector); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 87, __pyx_L1_error)

  /* "CyEulerBallistic/trajectory_calc.pyx":88
 *                             ):
 *         self.check_zero_crossing(range_vector)
 *         self.check_mach_crossing(velocity, mach)             # <<<<<<<<<<<<<<
 *         if self.check_next_range(range_vector.x, step):
 *             self.previous_time = time
 */
  __pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_check_mach_crossing(__pyx_v_self, __pyx_v_velocity, __pyx_v_mach); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 88, __pyx_L1_error)

  /* "CyEulerBallistic/trajectory_calc.pyx":89
 *         self.check_zero_crossing(range_vector)
 *         self.check_mach_crossing(velocity, mach)
 *         if self.check_next_range(range_vector.x, step):             # <<<<<<<<<<<<<<
 *             self.previous_time = time
 *         elif self.time_step > 0:
 */
  __pyx_t_1 = __pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_check_next_range(__pyx_v_self, __pyx_v_range_vector.x, __pyx_v_step); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 89, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "CyEulerBallistic/trajectory_calc.pyx":90
 *         self.check_mach_crossing(velocity, mach)
 *         if self.check_next_range(range_vector.x, step):
 *             self.previous_time = time             # <<<<<<<<<<<<<<
 *         elif self.time_step > 0:
 *             self.check_next_time(time)
 */
    __pyx_v_self->previous_time = __pyx_v_time;

    /* "CyEulerBallistic/trajectory_calc.pyx":89
 *         self.check_zero_crossing(range_vector)
 *         self.check_mach_crossing(velocity, mach)
 *         if self.check_next_range(range_vector.x, step):             # <<<<<<<<<<<<<<
 *             self.previous_time = time
 *         elif self.time_step > 0:
 */
    goto __pyx_L3;
  }

  /* "CyEulerBallistic/trajectory_calc.pyx":91
 *         if self.check_next_range(range_vector.x, step):
 *             self.previous_time = time
 *         elif self.time_step > 0:             # <<<<<<<<<<<<<<
 *             self.check_next_time(time)
 *         return (self.current_flag & self.filter) != 0
 */
  __pyx_t_1 = (__pyx_v_self->time_step > 0.0);
  if (__pyx_t_1) {

    /* "CyEulerBallistic/trajectory_calc.pyx":92
 *             self.previous_time = time
 *         elif self.time_step > 0:
 *             self.check_next_time(time)             # <<<<<<<<<<<<<<
 *         return (self.current_flag & self.filter) != 0
 * 
 */
    __pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_check_next_time(__pyx_v_self, __pyx_v_time); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 92, __pyx_L1_error)

    /* "CyEulerBallistic/trajectory_calc.pyx":91
 *         if self.check_next_range(range_vector.x, step):
 *             self.previous_time = time
 *         elif self.time_step > 0:             # <<<<<<<<<<<<<<
 *             self.check_next_time(time)
 *         return (self.current_flag & self.filter) != 0
 */
  }
  __pyx_L3:;

  /* "CyEulerBallistic/trajectory_calc.pyx":93
 *         elif self.time_step > 0:
 *             self.check_next_time(time)
 *         return (self.current_flag & self.filter) != 0             # <<<<<<<<<<<<<<
 * 
 *     cdef bint should_break(_TrajectoryDataFilter self):
 */
  __pyx_r = ((__pyx_v_self->current_flag & __pyx_v_self->filter) != 0);
  goto __pyx_L0;

  /* "CyEulerBallistic/trajectory_calc.pyx":80
 *         self.current_flag = CTrajFlag.NONE
 * 
 *     cdef bint should_record(_TrajectoryDataFilter self,             # <<<<<<<<<<<<<<
 *                             CVector range_vector,
 *                             double velocity,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("CyEulerBallistic.trajectory_calc._TrajectoryDataFilter.should_record", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  return __pyx_r;
}

/* "CyEulerBallistic/trajectory_calc.pyx":95
 *         return (self.current_flag & self.filter) != 0
 * 
 *     cdef bint should_break(_TrajectoryDataFilter self):             # <<<<<<<<<<<<<<
 *         return self.current_item == self.ranges_length
 * 
 */

static int __pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_should_break(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *__pyx_v_self) {
  int __pyx_r;

  /* "CyEulerBallistic/trajectory_calc.pyx":96
 * 
 *     cdef bint should_break(_TrajectoryDataFilter self):
 *         return self.current_item == self.ranges_length             # <<<<<<<<<<<<<<
 * 
 *     cdef bint check_next_range(_TrajectoryDataFilter self, double next_range, double step):
 */
  __pyx_r = (__pyx_v_self->current_item == __pyx_v_self->ranges_length);
  goto __pyx_L0;

  /* "CyEulerBallistic/trajectory_calc.pyx":95
 *         return (self.current_flag & self.filter) != 0
 * 
 *     cdef bint should_break(_TrajectoryDataFilter self):             # <<<<<<<<<<<<<<
 *         return self.current_item == self.ranges_length
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "CyEulerBallistic/trajectory_calc.pyx":98
 *         return self.current_item == self.ranges_length
 * 
 *     cdef bint check_next_range(_TrajectoryDataFilter self, double next_range, double step):             # <<<<<<<<<<<<<<
 *         # Next range check
 *         if next_range >= self.next_range_distance:
 */

static int __pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_check_next_range(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *__pyx_v_self, double __pyx_v_next_range, double __pyx_v_step) {
  int __pyx_r;
  int __pyx_t_1;

  /* "CyEulerBallistic/trajectory_calc.pyx":100
 *     cdef bint check_next_range(_TrajectoryDataFilter self, double next_range, double step):
 *         # Next range check
 *         if next_range >= self.next_range_distance:             # <<<<<<<<<<<<<<
 *             self.current_flag |= CTrajFlag.RANGE
 *             self.next_range_distance += step
 */
  __pyx_t_1 = (__pyx_v_next_range >= __pyx_v_self->next_range_distance);
  if (__pyx_t_1) {

    /* "CyEulerBallistic/trajectory_calc.pyx":101
 *         # Next range check
 *         if next_range >= self.next_range_distance:
 *             self.current_flag |= CTrajFlag.RANGE             # <<<<<<<<<<<<<<
 *             self.next_range_distance += step
 *             self.current_item += 1
 */
    __pyx_v_self->current_flag = (__pyx_v_self->current_flag | __pyx_e_16CyEulerBallistic_15trajectory_data_RANGE);

    /* "CyEulerBallistic/trajectory_calc.pyx":102
 *         if next_range >= self.next_range_distance:
 *             self.current_flag |= CTrajFlag.RANGE
 *             self.next_range_distance += step             # <<<<<<<<<<<<<<
 *             self.current_item += 1
 *             return True
 */
    __pyx_v_self->next_range_distance = (__pyx_v_self->next_range_distance + __pyx_v_step);

    /* "CyEulerBallistic/trajectory_calc.pyx":103
 *             self.current_flag |= CTrajFlag.RANGE
 *             self.next_range_distance += step
 *             self.current_item += 1             # <<<<<<<<<<<<<<
 *             return True
 *         return False
 */
    __pyx_v_self->current_item = (__pyx_v_self->current_item + 1);

    /* "CyEulerBallistic/trajectory_calc.pyx":104
 *             self.next_range_distance += step
 *             self.current_item += 1
 *             return True             # <<<<<<<<<<<<<<
 *         return False
 * 
 */
    __pyx_r = 1;
    goto __pyx_L0;

    /* "CyEulerBallistic/trajectory_calc.pyx":100
 *     cdef bint check_next_range(_TrajectoryDataFilter self, double next_range, double step):
 *         # Next range check
 *         if next_range >= self.next_range_distance:             # <<<<<<<<<<<<<<
 *             self.current_flag |= CTrajFlag.RANGE
 *             self.next_range_distance += step
 */
  }

  /* "CyEulerBallistic/trajectory_calc.pyx":105
 *             self.current_item += 1
 *             return True
 *         return False             # <<<<<<<<<<<<<<
 * 
 *     cdef void check_next_time(self, double time):
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "CyEulerBallistic/trajectory_calc.pyx":98
 *         return self.current_item == self.ranges_length
 * 
 *     cdef bint check_next_range(_TrajectoryDataFilter self, double next_range, double step):             # <<<<<<<<<<<<<<
 *         # Next range check
 *         if next_range >= self.next_range_distance:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "CyEulerBallistic/trajectory_calc.pyx":107
 *         return False
 * 
 *     cdef void check_next_time(self, double time):             # <<<<<<<<<<<<<<
 *         if time > self.previous_time + self.time_step:
 *             self.current_flag |= CTrajFlag.RANGE
 */

static void __pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_check_next_time(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *__pyx_v_self, double __pyx_v_time) {
  int __pyx_t_1;

  /* "CyEulerBallistic/trajectory_calc.pyx":108
 * 
 *     cdef void check_next_time(self, double time):
 *         if time > self.previous_time + self.time_step:             # <<<<<<<<<<<<<<
 *             self.current_flag |= CTrajFlag.RANGE
 *             self.previous_time = time
 */
  __pyx_t_1 = (__pyx_v_time > (__pyx_v_self->previous_time + __pyx_v_self->time_step));
  if (__pyx_t_1) {

    /* "CyEulerBallistic/trajectory_calc.pyx":109
 *     cdef void check_next_time(self, double time):
 *         if time > self.previous_time + self.time_step:
 *             self.current_flag |= CTrajFlag.RANGE             # <<<<<<<<<<<<<<
 *             self.previous_time = time
 * 
 */
    __pyx_v_self->current_flag = (__pyx_v_self->current_flag | __pyx_e_16CyEulerBallistic_15trajectory_data_RANGE);

    /* "CyEulerBallistic/trajectory_calc.pyx":110
 *         if time > self.previous_time + self.time_step:
 *             self.current_flag |= CTrajFlag.RANGE
 *             self.previous_time = time             # <<<<<<<<<<<<<<
 * 
 *     cdef void check_mach_crossing(_TrajectoryDataFilter self, double velocity, double mach):
 */
    __pyx_v_self->previous_time = __pyx_v_time;

    /* "CyEulerBallistic/trajectory_calc.pyx":108
 * 
 *     cdef void check_next_time(self, double time):
 *         if time > self.previous_time + self.time_step:             # <<<<<<<<<<<<<<
 *             self.current_flag |= CTrajFlag.RANGE
 *             self.previous_time = time
 */
  }

  /* "CyEulerBallistic/trajectory_calc.pyx":107
 *         return False
 * 
 *     cdef void check_next_time(self, double time):             # <<<<<<<<<<<<<<
 *         if time > self.previous_time + self.time_step:
 *             self.current_flag |= CTrajFlag.RANGE
 */

  /* function exit code */
}

/* "CyEulerBallistic/trajectory_calc.pyx":112
 *             self.previous_time = time
 * 
 *     cdef void check_mach_crossing(_TrajectoryDataFilter self, double velocity, double mach):             # <<<<<<<<<<<<<<
 *         # Mach crossing check
 *         cdef double current_mach = velocity / mach
 */

static void __pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_check_mach_crossing(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *__pyx_v_self, double __pyx_v_velocity, double __pyx_v_mach) {
  double __pyx_v_current_mach;
  int __pyx_t_1;

  /* "CyEulerBallistic/trajectory_calc.pyx":114
 *     cdef void check_mach_crossing(_TrajectoryDataFilter self, double velocity, double mach):
 *         # Mach crossing check
 *         cdef double current_mach = velocity / mach             # <<<<<<<<<<<<<<
 *         if self.previous_mach > 1 >= current_mach:
 *             self.current_flag |= CTrajFlag.MACH
 */
  __pyx_v_current_mach = (__pyx_v_velocity / __pyx_v_mach);

  /* "CyEulerBallistic/trajectory_calc.pyx":115
 *         # Mach crossing check
 *         cdef double current_mach = velocity / mach
 *         if self.previous_mach > 1 >= current_mach:             # <<<<<<<<<<<<<<
 *             self.current_flag |= CTrajFlag.MACH
 *         self.previous_mach = current_mach
 */
  __pyx_t_1 = (__pyx_v_self->previous_mach > 1.0);
  if (__pyx_t_1) {
    __pyx_t_1 = (1.0 >= __pyx_v_current_mach);
  }
  if (__pyx_t_1) {

    /* "CyEulerBallistic/trajectory_calc.pyx":116
 *         cdef double current_mach = velocity / mach
 *         if self.previous_mach > 1 >= current_mach:
 *             self.current_flag |= CTrajFlag.MACH             # <<<<<<<<<<<<<<
 *         self.previous_mach = current_mach
 * 
 */
    __pyx_v_self->current_flag = (__pyx_v_self->current_flag | __pyx_e_16CyEulerBallistic_15trajectory_data_MACH);

    /* "CyEulerBallistic/trajectory_calc.pyx":115
 *         # Mach crossing check
 *         cdef double current_mach = velocity / mach
 *         if self.previous_mach > 1 >= current_mach:             # <<<<<<<<<<<<<<
 *             self.current_flag |= CTrajFlag.MACH
 *         self.previous_mach = current_mach
 */
  }

  /* "CyEulerBallistic/trajectory_calc.pyx":117
 *         if self.previous_mach > 1 >= current_mach:
 *             self.current_flag |= CTrajFlag.MACH
 *         self.previous_mach = current_mach             # <<<<<<<<<<<<<<
 * 
 *     cdef void check_zero_crossing(_TrajectoryDataFilter self, CVector range_vector):
 */
  __pyx_v_self->previous_mach = __pyx_v_current_mach;

  /* "CyEulerBallistic/trajectory_calc.pyx":112
 *             self.previous_time = time
 * 
 *     cdef void check_mach_crossing(_TrajectoryDataFilter self, double velocity, double mach):             # <<<<<<<<<<<<<<
 *         # Mach crossing check
 *         cdef double current_mach = velocity / mach
 */

  /* function exit code */
}

/* "CyEulerBallistic/trajectory_calc.pyx":119
 *         self.previous_mach = current_mach
 * 
 *     cdef void check_zero_crossing(_TrajectoryDataFilter self, CVector range_vector):             # <<<<<<<<<<<<<<
 *         # Zero-crossing checks
 * 
 */

static void __pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_check_zero_crossing(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *__pyx_v_self, struct __pyx_t_16CyEulerBallistic_6vector_CVector __pyx_v_range_vector) {
  double __pyx_v_reference_height;
  int __pyx_t_1;

  /* "CyEulerBallistic/trajectory_calc.pyx":122
 *         # Zero-crossing checks
 * 
 *         if range_vector.x > 0:             # <<<<<<<<<<<<<<
 *             # Zero reference line is the sight line defined by look_angle
 *             reference_height = range_vector.x * tan(self.look_angle)
 */
  __pyx_t_1 = (__pyx_v_range_vector.x > 0.0);
  if (__pyx_t_1) {

    /* "CyEulerBallistic/trajectory_calc.pyx":124
 *         if range_vector.x > 0:
 *             # Zero reference line is the sight line defined by look_angle
 *             reference_height = range_vector.x * tan(self.look_angle)             # <<<<<<<<<<<<<<
 *             # If we haven't seen ZERO_UP, we look for that first
 *             if not (self.seen_zero & CTrajFlag.ZERO_UP):
 */
    __pyx_v_reference_height = (__pyx_v_range_vector.x * tan(__pyx_v_self->look_angle));

    /* "CyEulerBallistic/trajectory_calc.pyx":126
 *             reference_height = range_vector.x * tan(self.look_angle)
 *             # If we haven't seen ZERO_UP, we look for that first
 *             if not (self.seen_zero & CTrajFlag.ZERO_UP):             # <<<<<<<<<<<<<<
 *                 if range_vector.x >= reference_height:
 *                     self.current_flag |= CTrajFlag.ZERO_UP
 */
    __pyx_t_1 = (!((__pyx_v_self->seen_zero & __pyx_e_16CyEulerBallistic_15trajectory_data_ZERO_UP) != 0));
    if (__pyx_t_1) {

      /* "CyEulerBallistic/trajectory_calc.pyx":127
 *             # If we haven't seen ZERO_UP, we look for that first
 *             if not (self.seen_zero & CTrajFlag.ZERO_UP):
 *                 if range_vector.x >= reference_height:             # <<<<<<<<<<<<<<
 *                     self.current_flag |= CTrajFlag.ZERO_UP
 *                     self.seen_zero |= CTrajFlag.ZERO_UP
 */
      __pyx_t_1 = (__pyx_v_range_vector.x >= __pyx_v_reference_height);
      if (__pyx_t_1) {

        /* "CyEulerBallistic/trajectory_calc.pyx":128
 *             if not (self.seen_zero & CTrajFlag.ZERO_UP):
 *                 if range_vector.x >= reference_height:
 *                     self.current_flag |= CTrajFlag.ZERO_UP             # <<<<<<<<<<<<<<
 *                     self.seen_zero |= CTrajFlag.ZERO_UP
 *             # We've crossed above sight line; now look for crossing back through it
 */
        __pyx_v_self->current_flag = (__pyx_v_self->current_flag | __pyx_e_16CyEulerBallistic_15trajectory_data_ZERO_UP);

        /* "CyEulerBallistic/trajectory_calc.pyx":129
 *                 if range_vector.x >= reference_height:
 *                     self.current_flag |= CTrajFlag.ZERO_UP
 *                     self.seen_zero |= CTrajFlag.ZERO_UP             # <<<<<<<<<<<<<<
 *             # We've crossed above sight line; now look for crossing back through it
 *             elif not (self.seen_zero & CTrajFlag.ZERO_DOWN):
 */
        __pyx_v_self->seen_zero = (__pyx_v_self->seen_zero | __pyx_e_16CyEulerBallistic_15trajectory_data_ZERO_UP);

        /* "CyEulerBallistic/trajectory_calc.pyx":127
 *             # If we haven't seen ZERO_UP, we look for that first
 *             if not (self.seen_zero & CTrajFlag.ZERO_UP):
 *                 if range_vector.x >= reference_height:             # <<<<<<<<<<<<<<
 *                     self.current_flag |= CTrajFlag.ZERO_UP
 *                     self.seen_zero |= CTrajFlag.ZERO_UP
 */
      }

      /* "CyEulerBallistic/trajectory_calc.pyx":126
 *             reference_height = range_vector.x * tan(self.look_angle)
 *             # If we haven't seen ZERO_UP, we look for that first
 *             if not (self.seen_zero & CTrajFlag.ZERO_UP):             # <<<<<<<<<<<<<<
 *                 if range_vector.x >= reference_height:
 *                     self.current_flag |= CTrajFlag.ZERO_UP
 */
      goto __pyx_L4;
    }

    /* "CyEulerBallistic/trajectory_calc.pyx":131
 *                     self.seen_zero |= CTrajFlag.ZERO_UP
 *             # We've crossed above sight line; now look for crossing back through it
 *             elif not (self.seen_zero & CTrajFlag.ZERO_DOWN):             # <<<<<<<<<<<<<<
 *                 if range_vector.x < reference_height:
 *                     self.current_flag |= CTrajFlag.ZERO_DOWN
 */
    __pyx_t_1 = (!((__pyx_v_self->seen_zero & __pyx_e_16CyEulerBallistic_15trajectory_data_ZERO_DOWN) != 0));
    if (__pyx_t_1) {

      /* "CyEulerBallistic/trajectory_calc.pyx":132
 *             # We've crossed above sight line; now look for crossing back through it
 *             elif not (self.seen_zero & CTrajFlag.ZERO_DOWN):
 *                 if range_vector.x < reference_height:             # <<<<<<<<<<<<<<
 *                     self.current_flag |= CTrajFlag.ZERO_DOWN
 *                     self.seen_zero |= CTrajFlag.ZERO_DOWN
 */
      __pyx_t_1 = (__pyx_v_range_vector.x < __pyx_v_reference_height);
      if (__pyx_t_1) {

        /* "CyEulerBallistic/trajectory_calc.pyx":133
 *             elif not (self.seen_zero & CTrajFlag.ZERO_DOWN):
 *                 if range_vector.x < reference_height:
 *                     self.current_flag |= CTrajFlag.ZERO_DOWN             # <<<<<<<<<<<<<<
 *                     self.seen_zero |= CTrajFlag.ZERO_DOWN
 * 
 */
        __pyx_v_self->current_flag = (__pyx_v_self->current_flag | __pyx_e_16CyEulerBallistic_15trajectory_data_ZERO_DOWN);

        /* "CyEulerBallistic/trajectory_calc.pyx":134
 *                 if range_vector.x < reference_height:
 *                     self.current_flag |= CTrajFlag.ZERO_DOWN
 *                     self.seen_zero |= CTrajFlag.ZERO_DOWN             # <<<<<<<<<<<<<<
 * 
 * @final
 */
        __pyx_v_self->seen_zero = (__pyx_v_self->seen_zero | __pyx_e_16CyEulerBallistic_15trajectory_data_ZERO_DOWN);

        /* "CyEulerBallistic/trajectory_calc.pyx":132
 *             # We've crossed above sight line; now look for crossing back through it
 *             elif not (self.seen_zero & CTrajFlag.ZERO_DOWN):
 *                 if range_vector.x < reference_height:             # <<<<<<<<<<<<<<
 *                     self.current_flag |= CTrajFlag.ZERO_DOWN
 *                     self.seen_zero |= CTrajFlag.ZERO_DOWN
 */
      }

      /* "CyEulerBallistic/trajectory_calc.pyx":131
 *                     self.seen_zero |= CTrajFlag.ZERO_UP
 *             # We've crossed above sight line; now look for crossing back through it
 *             elif not (self.seen_zero & CTrajFlag.ZERO_DOWN):             # <<<<<<<<<<<<<<
 *                 if range_vector.x < reference_height:
 *                     self.current_flag |= CTrajFlag.ZERO_DOWN
 */
    }
    __pyx_L4:;

    /* "CyEulerBallistic/trajectory_calc.pyx":122
 *         # Zero-crossing checks
 * 
 *         if range_vector.x > 0:             # <<<<<<<<<<<<<<
 *             # Zero reference line is the sight line defined by look_angle
 *             reference_height = range_vector.x * tan(self.look_angle)
 */
  }

  /* "CyEulerBallistic/trajectory_calc.pyx":119
 *         self.previous_mach = current_mach
 * 
 *     cdef void check_zero_crossing(_TrajectoryDataFilter self, CVector range_vector):             # <<<<<<<<<<<<<<
 *         # Zero-crossing checks
 * 
 */

  /* function exit code */
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_3__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_2__reduce_cython__, "_TrajectoryDataFilter.__reduce_cython__(self)");
static PyObject *__pyx_pw_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_3__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) {
    __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL;}
  if (unlikely(__pyx_kwds) && __Pyx_NumKwargs_FASTCALL(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__reduce_cython__", 0))) return NULL;
  __pyx_r = __pyx_pf_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_2__reduce_cython__(((struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 1);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 */
  __Pyx_Raise(__pyx_builtin_TypeError, __pyx_kp_s_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(0, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("CyEulerBallistic.trajectory_calc._TrajectoryDataFilter.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 */

/* Python wrapper */
static PyObject *__pyx_pw_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_5__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_4__setstate_cython__, "_TrajectoryDataFilter.__setstate_cython__(self, __pyx_state)");
static PyObject *__pyx_pw_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_5__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_state,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_pyx_state)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 3, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "__setstate_cython__") < 0)) __PYX_ERR(0, 3, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("CyEulerBallistic.trajectory_calc._TrajectoryDataFilter.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_4__setstate_cython__(((struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 1);

  /* "(tree fragment)":4
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
 */
  __Pyx_Raise(__pyx_builtin_TypeError, __pyx_kp_s_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(0, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("CyEulerBallistic.trajectory_calc._TrajectoryDataFilter.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "CyEulerBallistic/trajectory_calc.pyx":145
 *         int _length
 * 
 *     def __cinit__(_WindSock self, object winds):             # <<<<<<<<<<<<<<
 *         self.winds = [
 *             Wind_t(
 */

/* Python wrapper */
static int __pyx_pw_16CyEulerBallistic_15trajectory_calc_9_WindSock_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_16CyEulerBallistic_15trajectory_calc_9_WindSock_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_winds = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_winds,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_VARARGS(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_VARARGS(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_VARARGS(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_winds)) != 0)) {
          (void)__Pyx_Arg_NewRef_VARARGS(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 145, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "__cinit__") < 0)) __PYX_ERR(1, 145, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_VARARGS(__pyx_args, 0);
    }
    __pyx_v_winds = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 145, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_VARARGS(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("CyEulerBallistic.trajectory_calc._WindSock.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_16CyEulerBallistic_15trajectory_calc_9_WindSock___cinit__(((struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock *)__pyx_v_self), __pyx_v_winds);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_VARARGS(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_16CyEulerBallistic_15trajectory_calc_9_WindSock___cinit__(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock *__pyx_v_self, PyObject *__pyx_v_winds) {
  PyObject *__pyx_7genexpr__pyx_v_w = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *(*__pyx_t_4)(PyObject *);
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  unsigned int __pyx_t_10;
  double __pyx_t_11;
  struct __pyx_t_16CyEulerBallistic_6vector_CVector __pyx_t_12;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__cinit__", 1);

  /* "CyEulerBallistic/trajectory_calc.pyx":146
 * 
 *     def __cinit__(_WindSock self, object winds):
 *         self.winds = [             # <<<<<<<<<<<<<<
 *             Wind_t(
 *                 w.velocity.get_in(Unit.FPS),
 */
  { /* enter inner scope */
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 146, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_1);

    /* "CyEulerBallistic/trajectory_calc.pyx":152
 *                 w.until_distance.get_in(Unit.Foot),
 *                 w.MAX_DISTANCE_FEET
 *             ) for w in winds             # <<<<<<<<<<<<<<
 *         ]
 *         self.current = 0
 */
    if (likely(PyList_CheckExact(__pyx_v_winds)) || PyTuple_CheckExact(__pyx_v_winds)) {
      __pyx_t_2 = __pyx_v_winds; __Pyx_INCREF(__pyx_t_2);
      __pyx_t_3 = 0;
      __pyx_t_4 = NULL;
    } else {
      __pyx_t_3 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_v_winds); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 152, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 152, __pyx_L5_error)
    }
    for (;;) {
      if (likely(!__pyx_t_4)) {
        if (likely(PyList_CheckExact(__pyx_t_2))) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_2);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(1, 152, __pyx_L5_error)
            #endif
            if (__pyx_t_3 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_5 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely((0 < 0))) __PYX_ERR(1, 152, __pyx_L5_error)
          #else
          __pyx_t_5 = __Pyx_PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 152, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_5);
          #endif
        } else {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_2);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(1, 152, __pyx_L5_error)
            #endif
            if (__pyx_t_3 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely((0 < 0))) __PYX_ERR(1, 152, __pyx_L5_error)
          #else
          __pyx_t_5 = __Pyx_PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 152, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_5);
          #endif
        }
      } else {
        __pyx_t_5 = __pyx_t_4(__pyx_t_2);
        if (unlikely(!__pyx_t_5)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(1, 152, __pyx_L5_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_5);
      }
      __Pyx_XDECREF_SET(__pyx_7genexpr__pyx_v_w, __pyx_t_5);
      __pyx_t_5 = 0;

      /* "CyEulerBallistic/trajectory_calc.pyx":147
 *     def __cinit__(_WindSock self, object winds):
 *         self.winds = [
 *             Wind_t(             # <<<<<<<<<<<<<<
 *                 w.velocity.get_in(Unit.FPS),
 *                 w.direction_from.get_in(Unit.Radian),
 */
      __pyx_t_5 = __Pyx_PyDict_NewPresized(4); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 147, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_5);

      /* "CyEulerBallistic/trajectory_calc.pyx":148
 *         self.winds = [
 *             Wind_t(
 *                 w.velocity.get_in(Unit.FPS),             # <<<<<<<<<<<<<<
 *                 w.direction_from.get_in(Unit.Radian),
 *                 w.until_distance.get_in(Unit.Foot),
 */
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_7genexpr__pyx_v_w, __pyx_n_s_velocity); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 148, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_get_in); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 148, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_Unit); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 148, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_FPS); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 148, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = NULL;
      __pyx_t_10 = 0;
      #if CYTHON_UNPACK_METHODS
      if (likely(PyMethod_Check(__pyx_t_8))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_8);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_8, function);
          __pyx_t_10 = 1;
        }
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_t_9};
        __pyx_t_6 = __Pyx_PyObject_FastCall(__pyx_t_8, __pyx_callargs+1-__pyx_t_10, 1+__pyx_t_10);
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 148, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      }
      __pyx_t_11 = __pyx_PyFloat_AsDouble(__pyx_t_6); if (unlikely((__pyx_t_11 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 148, __pyx_L5_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = PyFloat_FromDouble(__pyx_t_11); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 148, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_velocity, __pyx_t_6) < 0) __PYX_ERR(1, 147, __pyx_L5_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "CyEulerBallistic/trajectory_calc.pyx":149
 *             Wind_t(
 *                 w.velocity.get_in(Unit.FPS),
 *                 w.direction_from.get_in(Unit.Radian),             # <<<<<<<<<<<<<<
 *                 w.until_distance.get_in(Unit.Foot),
 *                 w.MAX_DISTANCE_FEET
 */
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_7genexpr__pyx_v_w, __pyx_n_s_direction_from); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 149, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_get_in); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 149, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_n_s_Unit); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 149, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_Radian); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 149, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = NULL;
      __pyx_t_10 = 0;
      #if CYTHON_UNPACK_METHODS
      if (likely(PyMethod_Check(__pyx_t_9))) {
        __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_9);
        if (likely(__pyx_t_8)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
          __Pyx_INCREF(__pyx_t_8);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_9, function);
          __pyx_t_10 = 1;
        }
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_t_7};
        __pyx_t_6 = __Pyx_PyObject_FastCall(__pyx_t_9, __pyx_callargs+1-__pyx_t_10, 1+__pyx_t_10);
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 149, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      }
      __pyx_t_11 = __pyx_PyFloat_AsDouble(__pyx_t_6); if (unlikely((__pyx_t_11 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 149, __pyx_L5_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = PyFloat_FromDouble(__pyx_t_11); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 149, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_direction_from, __pyx_t_6) < 0) __PYX_ERR(1, 147, __pyx_L5_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "CyEulerBallistic/trajectory_calc.pyx":150
 *                 w.velocity.get_in(Unit.FPS),
 *                 w.direction_from.get_in(Unit.Radian),
 *                 w.until_distance.get_in(Unit.Foot),             # <<<<<<<<<<<<<<
 *                 w.MAX_DISTANCE_FEET
 *             ) for w in winds
 */
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_7genexpr__pyx_v_w, __pyx_n_s_until_distance); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 150, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_n_s_get_in); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 150, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_n_s_Unit); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 150, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_n_s_Foot); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 150, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_t_9 = NULL;
      __pyx_t_10 = 0;
      #if CYTHON_UNPACK_METHODS
      if (likely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_9)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_9);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
          __pyx_t_10 = 1;
        }
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_t_8};
        __pyx_t_6 = __Pyx_PyObject_FastCall(__pyx_t_7, __pyx_callargs+1-__pyx_t_10, 1+__pyx_t_10);
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 150, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      }
      __pyx_t_11 = __pyx_PyFloat_AsDouble(__pyx_t_6); if (unlikely((__pyx_t_11 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 150, __pyx_L5_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = PyFloat_FromDouble(__pyx_t_11); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 150, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_until_distance, __pyx_t_6) < 0) __PYX_ERR(1, 147, __pyx_L5_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "CyEulerBallistic/trajectory_calc.pyx":151
 *                 w.direction_from.get_in(Unit.Radian),
 *                 w.until_distance.get_in(Unit.Foot),
 *                 w.MAX_DISTANCE_FEET             # <<<<<<<<<<<<<<
 *             ) for w in winds
 *         ]
 */
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_7genexpr__pyx_v_w, __pyx_n_s_MAX_DISTANCE_FEET); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 151, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_11 = __pyx_PyFloat_AsDouble(__pyx_t_6); if (unlikely((__pyx_t_11 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 151, __pyx_L5_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = PyFloat_FromDouble(__pyx_t_11); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 151, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_MAX_DISTANCE_FEET, __pyx_t_6) < 0) __PYX_ERR(1, 147, __pyx_L5_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_5))) __PYX_ERR(1, 146, __pyx_L5_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

      /* "CyEulerBallistic/trajectory_calc.pyx":152
 *                 w.until_distance.get_in(Unit.Foot),
 *                 w.MAX_DISTANCE_FEET
 *             ) for w in winds             # <<<<<<<<<<<<<<
 *         ]
 *         self.current = 0
 */
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_7genexpr__pyx_v_w); __pyx_7genexpr__pyx_v_w = 0;
    goto __pyx_L9_exit_scope;
    __pyx_L5_error:;
    __Pyx_XDECREF(__pyx_7genexpr__pyx_v_w); __pyx_7genexpr__pyx_v_w = 0;
    goto __pyx_L1_error;
    __pyx_L9_exit_scope:;
  } /* exit inner scope */

  /* "CyEulerBallistic/trajectory_calc.pyx":146
 * 
 *     def __cinit__(_WindSock self, object winds):
 *         self.winds = [             # <<<<<<<<<<<<<<
 *             Wind_t(
 *                 w.velocity.get_in(Unit.FPS),
 */
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->winds);
  __Pyx_DECREF(__pyx_v_self->winds);
  __pyx_v_self->winds = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":154
 *             ) for w in winds
 *         ]
 *         self.current = 0             # <<<<<<<<<<<<<<
 *         self.next_range = cMaxWindDistanceFeet
 *         self._last_vector_cache = CVector(0.0, 0.0, 0.0)
 */
  __pyx_v_self->current = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":155
 *         ]
 *         self.current = 0
 *         self.next_range = cMaxWindDistanceFeet             # <<<<<<<<<<<<<<
 *         self._last_vector_cache = CVector(0.0, 0.0, 0.0)
 *         self._length = len(self.winds)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_cMaxWindDistanceFeet); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 155, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_11 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_11 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 155, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->next_range = __pyx_t_11;

  /* "CyEulerBallistic/trajectory_calc.pyx":156
 *         self.current = 0
 *         self.next_range = cMaxWindDistanceFeet
 *         self._last_vector_cache = CVector(0.0, 0.0, 0.0)             # <<<<<<<<<<<<<<
 *         self._length = len(self.winds)
 * 
 */
  __pyx_t_12.x = 0.0;
  __pyx_t_12.y = 0.0;
  __pyx_t_12.z = 0.0;
  __pyx_v_self->_last_vector_cache = __pyx_t_12;

  /* "CyEulerBallistic/trajectory_calc.pyx":157
 *         self.next_range = cMaxWindDistanceFeet
 *         self._last_vector_cache = CVector(0.0, 0.0, 0.0)
 *         self._length = len(self.winds)             # <<<<<<<<<<<<<<
 * 
 *         # Initialize cache correctly
 */
  __pyx_t_1 = __pyx_v_self->winds;
  __Pyx_INCREF(__pyx_t_1);
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 157, __pyx_L1_error)
  }
  __pyx_t_3 = __Pyx_PyList_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 157, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->_length = __pyx_t_3;

  /* "CyEulerBallistic/trajectory_calc.pyx":160
 * 
 *         # Initialize cache correctly
 *         self.update_cache()             # <<<<<<<<<<<<<<
 * 
 *     cdef CVector current_vector(_WindSock self):
 */
  __pyx_f_16CyEulerBallistic_15trajectory_calc_9_WindSock_update_cache(__pyx_v_self); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 160, __pyx_L1_error)

  /* "CyEulerBallistic/trajectory_calc.pyx":145
 *         int _length
 * 
 *     def __cinit__(_WindSock self, object winds):             # <<<<<<<<<<<<<<
 *         self.winds = [
 *             Wind_t(
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("CyEulerBallistic.trajectory_calc._WindSock.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_7genexpr__pyx_v_w);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "CyEulerBallistic/trajectory_calc.pyx":162
 *         self.update_cache()
 * 
 *     cdef CVector current_vector(_WindSock self):             # <<<<<<<<<<<<<<
 *         return self._last_vector_cache
 * 
 */

static struct __pyx_t_16CyEulerBallistic_6vector_CVector __pyx_f_16CyEulerBallistic_15trajectory_calc_9_WindSock_current_vector(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock *__pyx_v_self) {
  struct __pyx_t_16CyEulerBallistic_6vector_CVector __pyx_r;

  /* "CyEulerBallistic/trajectory_calc.pyx":163
 * 
 *     cdef CVector current_vector(_WindSock self):
 *         return self._last_vector_cache             # <<<<<<<<<<<<<<
 * 
 *     cdef void update_cache(_WindSock self):
 */
  __pyx_r = __pyx_v_self->_last_vector_cache;
  goto __pyx_L0;

  /* "CyEulerBallistic/trajectory_calc.pyx":162
 *         self.update_cache()
 * 
 *     cdef CVector current_vector(_WindSock self):             # <<<<<<<<<<<<<<
 *         return self._last_vector_cache
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "CyEulerBallistic/trajectory_calc.pyx":165
 *         return self._last_vector_cache
 * 
 *     cdef void update_cache(_WindSock self):             # <<<<<<<<<<<<<<
 *         cdef Wind_t cur_wind
 *         if self.current < self._length:
 */

static void __pyx_f_16CyEulerBallistic_15trajectory_calc_9_WindSock_update_cache(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock *__pyx_v_self) {
  struct __pyx_t_16CyEulerBallistic_8cy_euler_Wind_t __pyx_v_cur_wind;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  struct __pyx_t_16CyEulerBallistic_8cy_euler_Wind_t __pyx_t_3;
  struct __pyx_t_16CyEulerBallistic_6vector_CVector __pyx_t_4;
  double __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("update_cache", 1);

  /* "CyEulerBallistic/trajectory_calc.pyx":167
 *     cdef void update_cache(_WindSock self):
 *         cdef Wind_t cur_wind
 *         if self.current < self._length:             # <<<<<<<<<<<<<<
 *             cur_wind = self.winds[self.current]
 *             self._last_vector_cache = wind_to_c_vector(&cur_wind)
 */
  __pyx_t_1 = (__pyx_v_self->current < __pyx_v_self->_length);
  if (__pyx_t_1) {

    /* "CyEulerBallistic/trajectory_calc.pyx":168
 *         cdef Wind_t cur_wind
 *         if self.current < self._length:
 *             cur_wind = self.winds[self.current]             # <<<<<<<<<<<<<<
 *             self._last_vector_cache = wind_to_c_vector(&cur_wind)
 *             self.next_range = cur_wind.until_distance
 */
    if (unlikely(__pyx_v_self->winds == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 168, __pyx_L1_error)
    }
    __pyx_t_2 = __Pyx_GetItemInt_List(__pyx_v_self->winds, __pyx_v_self->current, int, 1, __Pyx_PyInt_From_int, 1, 1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 168, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __pyx_convert__from_py_struct____pyx_t_16CyEulerBallistic_8cy_euler_Wind_t(__pyx_t_2); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 168, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_cur_wind = __pyx_t_3;

    /* "CyEulerBallistic/trajectory_calc.pyx":169
 *         if self.current < self._length:
 *             cur_wind = self.winds[self.current]
 *             self._last_vector_cache = wind_to_c_vector(&cur_wind)             # <<<<<<<<<<<<<<
 *             self.next_range = cur_wind.until_distance
 *         else:
 */
    __pyx_t_4 = __pyx_f_16CyEulerBallistic_8cy_euler_wind_to_c_vector((&__pyx_v_cur_wind)); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 169, __pyx_L1_error)
    __pyx_v_self->_last_vector_cache = __pyx_t_4;

    /* "CyEulerBallistic/trajectory_calc.pyx":170
 *             cur_wind = self.winds[self.current]
 *             self._last_vector_cache = wind_to_c_vector(&cur_wind)
 *             self.next_range = cur_wind.until_distance             # <<<<<<<<<<<<<<
 *         else:
 *             self._last_vector_cache = CVector(0.0, 0.0, 0.0)
 */
    __pyx_t_5 = __pyx_v_cur_wind.until_distance;
    __pyx_v_self->next_range = __pyx_t_5;

    /* "CyEulerBallistic/trajectory_calc.pyx":167
 *     cdef void update_cache(_WindSock self):
 *         cdef Wind_t cur_wind
 *         if self.current < self._length:             # <<<<<<<<<<<<<<
 *             cur_wind = self.winds[self.current]
 *             self._last_vector_cache = wind_to_c_vector(&cur_wind)
 */
    goto __pyx_L3;
  }

  /* "CyEulerBallistic/trajectory_calc.pyx":172
 *             self.next_range = cur_wind.until_distance
 *         else:
 *             self._last_vector_cache = CVector(0.0, 0.0, 0.0)             # <<<<<<<<<<<<<<
 *             self.next_range = cMaxWindDistanceFeet
 * 
 */
  /*else*/ {
    __pyx_t_4.x = 0.0;
    __pyx_t_4.y = 0.0;
    __pyx_t_4.z = 0.0;
    __pyx_v_self->_last_vector_cache = __pyx_t_4;

    /* "CyEulerBallistic/trajectory_calc.pyx":173
 *         else:
 *             self._last_vector_cache = CVector(0.0, 0.0, 0.0)
 *             self.next_range = cMaxWindDistanceFeet             # <<<<<<<<<<<<<<
 * 
 *     cdef CVector vector_for_range(_WindSock self, double next_range):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_cMaxWindDistanceFeet); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 173, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 173, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_self->next_range = __pyx_t_5;
  }
  __pyx_L3:;

  /* "CyEulerBallistic/trajectory_calc.pyx":165
 *         return self._last_vector_cache
 * 
 *     cdef void update_cache(_WindSock self):             # <<<<<<<<<<<<<<
 *         cdef Wind_t cur_wind
 *         if self.current < self._length:
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("CyEulerBallistic.trajectory_calc._WindSock.update_cache", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "CyEulerBallistic/trajectory_calc.pyx":175
 *             self.next_range = cMaxWindDistanceFeet
 * 
 *     cdef CVector vector_for_range(_WindSock self, double next_range):             # <<<<<<<<<<<<<<
 *         if next_range >= self.next_range:
 *         # if next_range + 1e-6 >= self.next_range:
 */

static struct __pyx_t_16CyEulerBallistic_6vector_CVector __pyx_f_16CyEulerBallistic_15trajectory_calc_9_WindSock_vector_for_range(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock *__pyx_v_self, double __pyx_v_next_range) {
  struct __pyx_t_16CyEulerBallistic_6vector_CVector __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  struct __pyx_t_16CyEulerBallistic_6vector_CVector __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  double __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("vector_for_range", 1);

  /* "CyEulerBallistic/trajectory_calc.pyx":176
 * 
 *     cdef CVector vector_for_range(_WindSock self, double next_range):
 *         if next_range >= self.next_range:             # <<<<<<<<<<<<<<
 *         # if next_range + 1e-6 >= self.next_range:
 *             self.current += 1
 */
  __pyx_t_1 = (__pyx_v_next_range >= __pyx_v_self->next_range);
  if (__pyx_t_1) {

    /* "CyEulerBallistic/trajectory_calc.pyx":178
 *         if next_range >= self.next_range:
 *         # if next_range + 1e-6 >= self.next_range:
 *             self.current += 1             # <<<<<<<<<<<<<<
 *             if self.current >= self._length:
 *                 self._last_vector_cache = CVector(0.0, 0.0, 0.0)
 */
    __pyx_v_self->current = (__pyx_v_self->current + 1);

    /* "CyEulerBallistic/trajectory_calc.pyx":179
 *         # if next_range + 1e-6 >= self.next_range:
 *             self.current += 1
 *             if self.current >= self._length:             # <<<<<<<<<<<<<<
 *                 self._last_vector_cache = CVector(0.0, 0.0, 0.0)
 *                 self.next_range = cMaxWindDistanceFeet
 */
    __pyx_t_1 = (__pyx_v_self->current >= __pyx_v_self->_length);
    if (__pyx_t_1) {

      /* "CyEulerBallistic/trajectory_calc.pyx":180
 *             self.current += 1
 *             if self.current >= self._length:
 *                 self._last_vector_cache = CVector(0.0, 0.0, 0.0)             # <<<<<<<<<<<<<<
 *                 self.next_range = cMaxWindDistanceFeet
 *             else:
 */
      __pyx_t_2.x = 0.0;
      __pyx_t_2.y = 0.0;
      __pyx_t_2.z = 0.0;
      __pyx_v_self->_last_vector_cache = __pyx_t_2;

      /* "CyEulerBallistic/trajectory_calc.pyx":181
 *             if self.current >= self._length:
 *                 self._last_vector_cache = CVector(0.0, 0.0, 0.0)
 *                 self.next_range = cMaxWindDistanceFeet             # <<<<<<<<<<<<<<
 *             else:
 *                 self.update_cache()  # This will trigger cache updates.
 */
      __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_cMaxWindDistanceFeet); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 181, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_3); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 181, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_v_self->next_range = __pyx_t_4;

      /* "CyEulerBallistic/trajectory_calc.pyx":179
 *         # if next_range + 1e-6 >= self.next_range:
 *             self.current += 1
 *             if self.current >= self._length:             # <<<<<<<<<<<<<<
 *                 self._last_vector_cache = CVector(0.0, 0.0, 0.0)
 *                 self.next_range = cMaxWindDistanceFeet
 */
      goto __pyx_L4;
    }

    /* "CyEulerBallistic/trajectory_calc.pyx":183
 *                 self.next_range = cMaxWindDistanceFeet
 *             else:
 *                 self.update_cache()  # This will trigger cache updates.             # <<<<<<<<<<<<<<
 *         return self._last_vector_cache
 * 
 */
    /*else*/ {
      __pyx_f_16CyEulerBallistic_15trajectory_calc_9_WindSock_update_cache(__pyx_v_self); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 183, __pyx_L1_error)
    }
    __pyx_L4:;

    /* "CyEulerBallistic/trajectory_calc.pyx":176
 * 
 *     cdef CVector vector_for_range(_WindSock self, double next_range):
 *         if next_range >= self.next_range:             # <<<<<<<<<<<<<<
 *         # if next_range + 1e-6 >= self.next_range:
 *             self.current += 1
 */
  }

  /* "CyEulerBallistic/trajectory_calc.pyx":184
 *             else:
 *                 self.update_cache()  # This will trigger cache updates.
 *         return self._last_vector_cache             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_self->_last_vector_cache;
  goto __pyx_L0;

  /* "CyEulerBallistic/trajectory_calc.pyx":175
 *             self.next_range = cMaxWindDistanceFeet
 * 
 *     cdef CVector vector_for_range(_WindSock self, double next_range):             # <<<<<<<<<<<<<<
 *         if next_range >= self.next_range:
 *         # if next_range + 1e-6 >= self.next_range:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("CyEulerBallistic.trajectory_calc._WindSock.vector_for_range", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_pretend_to_initialize(&__pyx_r);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_16CyEulerBallistic_15trajectory_calc_9_WindSock_3__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_16CyEulerBallistic_15trajectory_calc_9_WindSock_2__reduce_cython__, "_WindSock.__reduce_cython__(self)");
static PyObject *__pyx_pw_16CyEulerBallistic_15trajectory_calc_9_WindSock_3__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) {
    __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL;}
  if (unlikely(__pyx_kwds) && __Pyx_NumKwargs_FASTCALL(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__reduce_cython__", 0))) return NULL;
  __pyx_r = __pyx_pf_16CyEulerBallistic_15trajectory_calc_9_WindSock_2__reduce_cython__(((struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_16CyEulerBallistic_15trajectory_calc_9_WindSock_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 1);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 */
  __Pyx_Raise(__pyx_builtin_TypeError, __pyx_kp_s_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(0, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("CyEulerBallistic.trajectory_calc._WindSock.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 */

/* Python wrapper */
static PyObject *__pyx_pw_16CyEulerBallistic_15trajectory_calc_9_WindSock_5__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_16CyEulerBallistic_15trajectory_calc_9_WindSock_4__setstate_cython__, "_WindSock.__setstate_cython__(self, __pyx_state)");
static PyObject *__pyx_pw_16CyEulerBallistic_15trajectory_calc_9_WindSock_5__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_state,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_pyx_state)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 3, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "__setstate_cython__") < 0)) __PYX_ERR(0, 3, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("CyEulerBallistic.trajectory_calc._WindSock.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_16CyEulerBallistic_15trajectory_calc_9_WindSock_4__setstate_cython__(((struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_16CyEulerBallistic_15trajectory_calc_9_WindSock_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 1);

  /* "(tree fragment)":4
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
 */
  __Pyx_Raise(__pyx_builtin_TypeError, __pyx_kp_s_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(0, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("CyEulerBallistic.trajectory_calc._WindSock.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "CyEulerBallistic/trajectory_calc.pyx":196
 *         ShotData_t __shot
 * 
 *     def __cinit__(EulerTrajectoryCalc self, object _config):             # <<<<<<<<<<<<<<
 *         self._config = _config
 *         self.gravity_vector = CVector(.0, self._config.cGravityConstant, .0)
 */

/* Python wrapper */
static int __pyx_pw_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v__config = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_config,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_VARARGS(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_VARARGS(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_VARARGS(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_config)) != 0)) {
          (void)__Pyx_Arg_NewRef_VARARGS(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 196, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "__cinit__") < 0)) __PYX_ERR(1, 196, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_VARARGS(__pyx_args, 0);
    }
    __pyx_v__config = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 196, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_VARARGS(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("CyEulerBallistic.trajectory_calc.EulerTrajectoryCalc.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc___cinit__(((struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *)__pyx_v_self), __pyx_v__config);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_VARARGS(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc___cinit__(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *__pyx_v_self, PyObject *__pyx_v__config) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  struct __pyx_t_16CyEulerBallistic_6vector_CVector __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  double __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__cinit__", 1);

  /* "CyEulerBallistic/trajectory_calc.pyx":197
 * 
 *     def __cinit__(EulerTrajectoryCalc self, object _config):
 *         self._config = _config             # <<<<<<<<<<<<<<
 *         self.gravity_vector = CVector(.0, self._config.cGravityConstant, .0)
 * 
 */
  __Pyx_INCREF(__pyx_v__config);
  __Pyx_GIVEREF(__pyx_v__config);
  __Pyx_GOTREF(__pyx_v_self->_config);
  __Pyx_DECREF(__pyx_v_self->_config);
  __pyx_v_self->_config = __pyx_v__config;

  /* "CyEulerBallistic/trajectory_calc.pyx":198
 *     def __cinit__(EulerTrajectoryCalc self, object _config):
 *         self._config = _config
 *         self.gravity_vector = CVector(.0, self._config.cGravityConstant, .0)             # <<<<<<<<<<<<<<
 * 
 *     # def __dealloc__(EulerTrajectoryCalc self):
 */
  __pyx_t_1.x = .0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->_config, __pyx_n_s_cGravityConstant); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 198, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_3 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 198, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_1.y = __pyx_t_3;
  __pyx_t_1.z = .0;
  __pyx_v_self->gravity_vector = __pyx_t_1;

  /* "CyEulerBallistic/trajectory_calc.pyx":196
 *         ShotData_t __shot
 * 
 *     def __cinit__(EulerTrajectoryCalc self, object _config):             # <<<<<<<<<<<<<<
 *         self._config = _config
 *         self.gravity_vector = CVector(.0, self._config.cGravityConstant, .0)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("CyEulerBallistic.trajectory_calc.EulerTrajectoryCalc.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "CyEulerBallistic/trajectory_calc.pyx":203
 *     #     free_trajectory(&self.__shot)
 * 
 *     cdef double get_calc_step(self, double step = 0):             # <<<<<<<<<<<<<<
 *         cdef double preferred_step = self.__config.max_calc_step_size_feet
 *         # cdef double defined_max = 0.5  # const will be better optimized with cython
 */

static double __pyx_f_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_get_calc_step(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *__pyx_v_self, struct __pyx_opt_args_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_get_calc_step *__pyx_optional_args) {
  double __pyx_v_step = ((double)0.0);
  double __pyx_v_preferred_step;
  double __pyx_r;
  double __pyx_t_1;
  int __pyx_t_2;
  double __pyx_t_3;
  double __pyx_t_4;
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_step = __pyx_optional_args->step;
    }
  }

  /* "CyEulerBallistic/trajectory_calc.pyx":204
 * 
 *     cdef double get_calc_step(self, double step = 0):
 *         cdef double preferred_step = self.__config.max_calc_step_size_feet             # <<<<<<<<<<<<<<
 *         # cdef double defined_max = 0.5  # const will be better optimized with cython
 *         if step == 0:
 */
  __pyx_t_1 = __pyx_v_self->_EulerTrajectoryCalc__config.max_calc_step_size_feet;
  __pyx_v_preferred_step = __pyx_t_1;

  /* "CyEulerBallistic/trajectory_calc.pyx":206
 *         cdef double preferred_step = self.__config.max_calc_step_size_feet
 *         # cdef double defined_max = 0.5  # const will be better optimized with cython
 *         if step == 0:             # <<<<<<<<<<<<<<
 *             return preferred_step / 2.0
 *         return min(step, preferred_step) / 2.0
 */
  __pyx_t_2 = (__pyx_v_step == 0.0);
  if (__pyx_t_2) {

    /* "CyEulerBallistic/trajectory_calc.pyx":207
 *         # cdef double defined_max = 0.5  # const will be better optimized with cython
 *         if step == 0:
 *             return preferred_step / 2.0             # <<<<<<<<<<<<<<
 *         return min(step, preferred_step) / 2.0
 * 
 */
    __pyx_r = (__pyx_v_preferred_step / 2.0);
    goto __pyx_L0;

    /* "CyEulerBallistic/trajectory_calc.pyx":206
 *         cdef double preferred_step = self.__config.max_calc_step_size_feet
 *         # cdef double defined_max = 0.5  # const will be better optimized with cython
 *         if step == 0:             # <<<<<<<<<<<<<<
 *             return preferred_step / 2.0
 *         return min(step, preferred_step) / 2.0
 */
  }

  /* "CyEulerBallistic/trajectory_calc.pyx":208
 *         if step == 0:
 *             return preferred_step / 2.0
 *         return min(step, preferred_step) / 2.0             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __pyx_t_1 = __pyx_v_preferred_step;
  __pyx_t_3 = __pyx_v_step;
  __pyx_t_2 = (__pyx_t_1 < __pyx_t_3);
  if (__pyx_t_2) {
    __pyx_t_4 = __pyx_t_1;
  } else {
    __pyx_t_4 = __pyx_t_3;
  }
  __pyx_r = (__pyx_t_4 / 2.0);
  goto __pyx_L0;

  /* "CyEulerBallistic/trajectory_calc.pyx":203
 *     #     free_trajectory(&self.__shot)
 * 
 *     cdef double get_calc_step(self, double step = 0):             # <<<<<<<<<<<<<<
 *         cdef double preferred_step = self.__config.max_calc_step_size_feet
 *         # cdef double defined_max = 0.5  # const will be better optimized with cython
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "CyEulerBallistic/trajectory_calc.pyx":210
 *         return min(step, preferred_step) / 2.0
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def table_data(self) -> list[object]:
 *         return self._table_data
 */

/* Python wrapper */
static PyObject *__pyx_pw_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_10table_data_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_10table_data_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_10table_data___get__(((struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_10table_data___get__(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 1);

  /* "CyEulerBallistic/trajectory_calc.pyx":212
 *     @property
 *     def table_data(self) -> list[object]:
 *         return self._table_data             # <<<<<<<<<<<<<<
 * 
 *     def zero_angle(self, object shot_info, object distance) -> Angular:
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_table_data);
  __pyx_r = __pyx_v_self->_table_data;
  goto __pyx_L0;

  /* "CyEulerBallistic/trajectory_calc.pyx":210
 *         return min(step, preferred_step) / 2.0
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def table_data(self) -> list[object]:
 *         return self._table_data
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "CyEulerBallistic/trajectory_calc.pyx":214
 *         return self._table_data
 * 
 *     def zero_angle(self, object shot_info, object distance) -> Angular:             # <<<<<<<<<<<<<<
 *         return self._zero_angle(shot_info, distance)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_3zero_angle(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_2zero_angle, "EulerTrajectoryCalc.zero_angle(self, shot_info, distance) -> Angular");
static PyObject *__pyx_pw_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_3zero_angle(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_shot_info = 0;
  PyObject *__pyx_v_distance = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("zero_angle (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_shot_info,&__pyx_n_s_distance,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_shot_info)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 214, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_distance)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 214, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("zero_angle", 1, 2, 2, 1); __PYX_ERR(1, 214, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "zero_angle") < 0)) __PYX_ERR(1, 214, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
      values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
    }
    __pyx_v_shot_info = values[0];
    __pyx_v_distance = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("zero_angle", 1, 2, 2, __pyx_nargs); __PYX_ERR(1, 214, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("CyEulerBallistic.trajectory_calc.EulerTrajectoryCalc.zero_angle", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_2zero_angle(((struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *)__pyx_v_self), __pyx_v_shot_info, __pyx_v_distance);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_2zero_angle(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *__pyx_v_self, PyObject *__pyx_v_shot_info, PyObject *__pyx_v_distance) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("zero_angle", 1);

  /* "CyEulerBallistic/trajectory_calc.pyx":215
 * 
 *     def zero_angle(self, object shot_info, object distance) -> Angular:
 *         return self._zero_angle(shot_info, distance)             # <<<<<<<<<<<<<<
 * 
 *     def trajectory(self, object shot_info, object max_range, object dist_step,
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *)__pyx_v_self->__pyx_vtab)->_zero_angle(__pyx_v_self, __pyx_v_shot_info, __pyx_v_distance); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 215, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "CyEulerBallistic/trajectory_calc.pyx":214
 *         return self._table_data
 * 
 *     def zero_angle(self, object shot_info, object distance) -> Angular:             # <<<<<<<<<<<<<<
 *         return self._zero_angle(shot_info, distance)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("CyEulerBallistic.trajectory_calc.EulerTrajectoryCalc.zero_angle", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "CyEulerBallistic/trajectory_calc.pyx":217
 *         return self._zero_angle(shot_info, distance)
 * 
 *     def trajectory(self, object shot_info, object max_range, object dist_step,             # <<<<<<<<<<<<<<
 *                    bint extra_data = False, double time_step = 0.0) -> object:
 *         # hack to reload config if it was changed explicit on existed instance
 */

/* Python wrapper */
static PyObject *__pyx_pw_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_5trajectory(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_4trajectory, "EulerTrajectoryCalc.trajectory(self, shot_info, max_range, dist_step, bool extra_data=False, double time_step=0.0) -> object");
static PyObject *__pyx_pw_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_5trajectory(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_shot_info = 0;
  PyObject *__pyx_v_max_range = 0;
  PyObject *__pyx_v_dist_step = 0;
  int __pyx_v_extra_data;
  double __pyx_v_time_step;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[5] = {0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("trajectory (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_shot_info,&__pyx_n_s_max_range,&__pyx_n_s_dist_step,&__pyx_n_s_extra_data,&__pyx_n_s_time_step,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  5: values[4] = __Pyx_Arg_FASTCALL(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = __Pyx_Arg_FASTCALL(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_shot_info)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 217, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_max_range)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 217, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("trajectory", 0, 3, 5, 1); __PYX_ERR(1, 217, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_dist_step)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[2]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 217, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("trajectory", 0, 3, 5, 2); __PYX_ERR(1, 217, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_extra_data);
          if (value) { values[3] = __Pyx_Arg_NewRef_FASTCALL(value); kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 217, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_time_step);
          if (value) { values[4] = __Pyx_Arg_NewRef_FASTCALL(value); kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 217, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "trajectory") < 0)) __PYX_ERR(1, 217, __pyx_L3_error)
      }
    } else {
      switch (__pyx_nargs) {
        case  5: values[4] = __Pyx_Arg_FASTCALL(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = __Pyx_Arg_FASTCALL(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
        values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_shot_info = values[0];
    __pyx_v_max_range = values[1];
    __pyx_v_dist_step = values[2];
    if (values[3]) {
      __pyx_v_extra_data = __Pyx_PyObject_IsTrue(values[3]); if (unlikely((__pyx_v_extra_data == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 218, __pyx_L3_error)
    } else {

      /* "CyEulerBallistic/trajectory_calc.pyx":218
 * 
 *     def trajectory(self, object shot_info, object max_range, object dist_step,
 *                    bint extra_data = False, double time_step = 0.0) -> object:             # <<<<<<<<<<<<<<
 *         # hack to reload config if it was changed explicit on existed instance
 *         self.__config = config_bind(self._config)
 */
      __pyx_v_extra_data = ((int)0);
    }
    if (values[4]) {
      __pyx_v_time_step = __pyx_PyFloat_AsDouble(values[4]); if (unlikely((__pyx_v_time_step == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 218, __pyx_L3_error)
    } else {
      __pyx_v_time_step = ((double)0.0);
    }
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("trajectory", 0, 3, 5, __pyx_nargs); __PYX_ERR(1, 217, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("CyEulerBallistic.trajectory_calc.EulerTrajectoryCalc.trajectory", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_4trajectory(((struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *)__pyx_v_self), __pyx_v_shot_info, __pyx_v_max_range, __pyx_v_dist_step, __pyx_v_extra_data, __pyx_v_time_step);

  /* "CyEulerBallistic/trajectory_calc.pyx":217
 *         return self._zero_angle(shot_info, distance)
 * 
 *     def trajectory(self, object shot_info, object max_range, object dist_step,             # <<<<<<<<<<<<<<
 *                    bint extra_data = False, double time_step = 0.0) -> object:
 *         # hack to reload config if it was changed explicit on existed instance
 */

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_4trajectory(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *__pyx_v_self, PyObject *__pyx_v_shot_info, PyObject *__pyx_v_max_range, PyObject *__pyx_v_dist_step, int __pyx_v_extra_data, double __pyx_v_time_step) {
  enum __pyx_t_16CyEulerBallistic_15trajectory_data_CTrajFlag __pyx_v_filter_flags;
  PyObject *__pyx_v_t = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_t_16CyEulerBallistic_8cy_euler_Config_t __pyx_t_2;
  struct __pyx_t_16CyEulerBallistic_6vector_CVector __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  unsigned int __pyx_t_7;
  double __pyx_t_8;
  double __pyx_t_9;
  struct __pyx_opt_args_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc__integrate __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("trajectory", 0);
  __Pyx_INCREF(__pyx_v_dist_step);

  /* "CyEulerBallistic/trajectory_calc.pyx":220
 *                    bint extra_data = False, double time_step = 0.0) -> object:
 *         # hack to reload config if it was changed explicit on existed instance
 *         self.__config = config_bind(self._config)             # <<<<<<<<<<<<<<
 *         self.gravity_vector = CVector(.0, self.__config.cGravityConstant, .0)
 * 
 */
  __pyx_t_1 = __pyx_v_self->_config;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = __pyx_f_16CyEulerBallistic_8cy_euler_config_bind(__pyx_t_1); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 220, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->_EulerTrajectoryCalc__config = __pyx_t_2;

  /* "CyEulerBallistic/trajectory_calc.pyx":221
 *         # hack to reload config if it was changed explicit on existed instance
 *         self.__config = config_bind(self._config)
 *         self.gravity_vector = CVector(.0, self.__config.cGravityConstant, .0)             # <<<<<<<<<<<<<<
 * 
 *         cdef:
 */
  __pyx_t_3.x = .0;
  __pyx_t_3.y = __pyx_v_self->_EulerTrajectoryCalc__config.cGravityConstant;
  __pyx_t_3.z = .0;
  __pyx_v_self->gravity_vector = __pyx_t_3;

  /* "CyEulerBallistic/trajectory_calc.pyx":224
 * 
 *         cdef:
 *             CTrajFlag filter_flags = CTrajFlag.RANGE             # <<<<<<<<<<<<<<
 * 
 *         if extra_data:
 */
  __pyx_v_filter_flags = __pyx_e_16CyEulerBallistic_15trajectory_data_RANGE;

  /* "CyEulerBallistic/trajectory_calc.pyx":226
 *             CTrajFlag filter_flags = CTrajFlag.RANGE
 * 
 *         if extra_data:             # <<<<<<<<<<<<<<
 *             dist_step = Distance.Foot(self.__config.chart_resolution)
 *             filter_flags = CTrajFlag.ALL
 */
  if (__pyx_v_extra_data) {

    /* "CyEulerBallistic/trajectory_calc.pyx":227
 * 
 *         if extra_data:
 *             dist_step = Distance.Foot(self.__config.chart_resolution)             # <<<<<<<<<<<<<<
 *             filter_flags = CTrajFlag.ALL
 * 
 */
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_Distance); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 227, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_Foot); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 227, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = PyFloat_FromDouble(__pyx_v_self->_EulerTrajectoryCalc__config.chart_resolution); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 227, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = NULL;
    __pyx_t_7 = 0;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_7 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_t_4};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+1-__pyx_t_7, 1+__pyx_t_7);
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 227, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_DECREF_SET(__pyx_v_dist_step, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "CyEulerBallistic/trajectory_calc.pyx":228
 *         if extra_data:
 *             dist_step = Distance.Foot(self.__config.chart_resolution)
 *             filter_flags = CTrajFlag.ALL             # <<<<<<<<<<<<<<
 * 
 *         self._init_trajectory(shot_info)
 */
    __pyx_v_filter_flags = __pyx_e_16CyEulerBallistic_15trajectory_data_ALL;

    /* "CyEulerBallistic/trajectory_calc.pyx":226
 *             CTrajFlag filter_flags = CTrajFlag.RANGE
 * 
 *         if extra_data:             # <<<<<<<<<<<<<<
 *             dist_step = Distance.Foot(self.__config.chart_resolution)
 *             filter_flags = CTrajFlag.ALL
 */
  }

  /* "CyEulerBallistic/trajectory_calc.pyx":230
 *             filter_flags = CTrajFlag.ALL
 * 
 *         self._init_trajectory(shot_info)             # <<<<<<<<<<<<<<
 *         cdef list[object] t = self._integrate(max_range._feet, dist_step._feet, filter_flags, time_step)
 *         self._free_trajectory()
 */
  ((struct __pyx_vtabstruct_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *)__pyx_v_self->__pyx_vtab)->_init_trajectory(__pyx_v_self, __pyx_v_shot_info); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 230, __pyx_L1_error)

  /* "CyEulerBallistic/trajectory_calc.pyx":231
 * 
 *         self._init_trajectory(shot_info)
 *         cdef list[object] t = self._integrate(max_range._feet, dist_step._feet, filter_flags, time_step)             # <<<<<<<<<<<<<<
 *         self._free_trajectory()
 *         return t
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_max_range, __pyx_n_s_feet); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 231, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_8 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_8 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 231, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dist_step, __pyx_n_s_feet); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 231, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_9 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_9 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 231, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_10.__pyx_n = 1;
  __pyx_t_10.time_step = __pyx_v_time_step;
  __pyx_t_1 = ((struct __pyx_vtabstruct_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *)__pyx_v_self->__pyx_vtab)->_integrate(__pyx_v_self, __pyx_t_8, __pyx_t_9, __pyx_v_filter_flags, &__pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 231, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_t = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":232
 *         self._init_trajectory(shot_info)
 *         cdef list[object] t = self._integrate(max_range._feet, dist_step._feet, filter_flags, time_step)
 *         self._free_trajectory()             # <<<<<<<<<<<<<<
 *         return t
 * 
 */
  ((struct __pyx_vtabstruct_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *)__pyx_v_self->__pyx_vtab)->_free_trajectory(__pyx_v_self); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 232, __pyx_L1_error)

  /* "CyEulerBallistic/trajectory_calc.pyx":233
 *         cdef list[object] t = self._integrate(max_range._feet, dist_step._feet, filter_flags, time_step)
 *         self._free_trajectory()
 *         return t             # <<<<<<<<<<<<<<
 * 
 *     cdef void _free_trajectory(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_t);
  __pyx_r = __pyx_v_t;
  goto __pyx_L0;

  /* "CyEulerBallistic/trajectory_calc.pyx":217
 *         return self._zero_angle(shot_info, distance)
 * 
 *     def trajectory(self, object shot_info, object max_range, object dist_step,             # <<<<<<<<<<<<<<
 *                    bint extra_data = False, double time_step = 0.0) -> object:
 *         # hack to reload config if it was changed explicit on existed instance
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("CyEulerBallistic.trajectory_calc.EulerTrajectoryCalc.trajectory", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_t);
  __Pyx_XDECREF(__pyx_v_dist_step);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "CyEulerBallistic/trajectory_calc.pyx":235
 *         return t
 * 
 *     cdef void _free_trajectory(self):             # <<<<<<<<<<<<<<
 *         free_trajectory(&self.__shot)
 * 
 */

static void __pyx_f_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc__free_trajectory(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *__pyx_v_self) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":236
 * 
 *     cdef void _free_trajectory(self):
 *         free_trajectory(&self.__shot)             # <<<<<<<<<<<<<<
 * 
 *     cdef void _init_trajectory(self, object shot_info):
 */
  __pyx_f_16CyEulerBallistic_8cy_euler_free_trajectory((&__pyx_v_self->_EulerTrajectoryCalc__shot)); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 236, __pyx_L1_error)

  /* "CyEulerBallistic/trajectory_calc.pyx":235
 *         return t
 * 
 *     cdef void _free_trajectory(self):             # <<<<<<<<<<<<<<
 *         free_trajectory(&self.__shot)
 * 
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("CyEulerBallistic.trajectory_calc.EulerTrajectoryCalc._free_trajectory", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
}

/* "CyEulerBallistic/trajectory_calc.pyx":238
 *         free_trajectory(&self.__shot)
 * 
 *     cdef void _init_trajectory(self, object shot_info):             # <<<<<<<<<<<<<<
 *         self._table_data = shot_info.ammo.dm.drag_table
 *         self.__shot = ShotData_t(
 */

static void __pyx_f_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc__init_trajectory(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *__pyx_v_self, PyObject *__pyx_v_shot_info) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  struct __pyx_t_16CyEulerBallistic_8cy_euler_ShotData_t __pyx_t_3;
  double __pyx_t_4;
  struct __pyx_t_16CyEulerBallistic_8cy_euler_Curve_t __pyx_t_5;
  struct __pyx_t_16CyEulerBallistic_8cy_euler_MachList_t __pyx_t_6;
  struct __pyx_t_16CyEulerBallistic_8cy_euler_Atmosphere_t __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  unsigned int __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_init_trajectory", 1);

  /* "CyEulerBallistic/trajectory_calc.pyx":239
 * 
 *     cdef void _init_trajectory(self, object shot_info):
 *         self._table_data = shot_info.ammo.dm.drag_table             # <<<<<<<<<<<<<<
 *         self.__shot = ShotData_t(
 *             bc=shot_info.ammo.dm.BC,
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_shot_info, __pyx_n_s_ammo); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 239, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dm); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 239, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_drag_table); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 239, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("list", __pyx_t_1))) __PYX_ERR(1, 239, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_table_data);
  __Pyx_DECREF(__pyx_v_self->_table_data);
  __pyx_v_self->_table_data = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":241
 *         self._table_data = shot_info.ammo.dm.drag_table
 *         self.__shot = ShotData_t(
 *             bc=shot_info.ammo.dm.BC,             # <<<<<<<<<<<<<<
 *             curve=cy_calculate_curve(self._table_data),
 *             mach_list=cy_table_to_mach(self._table_data),
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_shot_info, __pyx_n_s_ammo); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 241, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dm); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 241, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_BC); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 241, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 241, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3.bc = __pyx_t_4;

  /* "CyEulerBallistic/trajectory_calc.pyx":242
 *         self.__shot = ShotData_t(
 *             bc=shot_info.ammo.dm.BC,
 *             curve=cy_calculate_curve(self._table_data),             # <<<<<<<<<<<<<<
 *             mach_list=cy_table_to_mach(self._table_data),
 *             look_angle=shot_info.look_angle._rad,
 */
  __pyx_t_1 = __pyx_v_self->_table_data;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_5 = __pyx_f_16CyEulerBallistic_8cy_euler_cy_calculate_curve(((PyObject*)__pyx_t_1)); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 242, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3.curve = __pyx_t_5;

  /* "CyEulerBallistic/trajectory_calc.pyx":243
 *             bc=shot_info.ammo.dm.BC,
 *             curve=cy_calculate_curve(self._table_data),
 *             mach_list=cy_table_to_mach(self._table_data),             # <<<<<<<<<<<<<<
 *             look_angle=shot_info.look_angle._rad,
 *             twist=shot_info.weapon.twist._inch,
 */
  __pyx_t_1 = __pyx_v_self->_table_data;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_6 = __pyx_f_16CyEulerBallistic_8cy_euler_cy_table_to_mach(((PyObject*)__pyx_t_1)); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 243, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3.mach_list = __pyx_t_6;

  /* "CyEulerBallistic/trajectory_calc.pyx":244
 *             curve=cy_calculate_curve(self._table_data),
 *             mach_list=cy_table_to_mach(self._table_data),
 *             look_angle=shot_info.look_angle._rad,             # <<<<<<<<<<<<<<
 *             twist=shot_info.weapon.twist._inch,
 *             length=shot_info.ammo.dm.length._inch,
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_shot_info, __pyx_n_s_look_angle); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 244, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_rad); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 244, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 244, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3.look_angle = __pyx_t_4;

  /* "CyEulerBallistic/trajectory_calc.pyx":245
 *             mach_list=cy_table_to_mach(self._table_data),
 *             look_angle=shot_info.look_angle._rad,
 *             twist=shot_info.weapon.twist._inch,             # <<<<<<<<<<<<<<
 *             length=shot_info.ammo.dm.length._inch,
 *             diameter=shot_info.ammo.dm.diameter._inch,
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_shot_info, __pyx_n_s_weapon); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 245, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_twist); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 245, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_inch); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 245, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 245, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3.twist = __pyx_t_4;

  /* "CyEulerBallistic/trajectory_calc.pyx":246
 *             look_angle=shot_info.look_angle._rad,
 *             twist=shot_info.weapon.twist._inch,
 *             length=shot_info.ammo.dm.length._inch,             # <<<<<<<<<<<<<<
 *             diameter=shot_info.ammo.dm.diameter._inch,
 *             weight=shot_info.ammo.dm.weight._grain,
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_shot_info, __pyx_n_s_ammo); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 246, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_dm); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 246, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_length); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 246, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_inch); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 246, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 246, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3.length = __pyx_t_4;

  /* "CyEulerBallistic/trajectory_calc.pyx":247
 *             twist=shot_info.weapon.twist._inch,
 *             length=shot_info.ammo.dm.length._inch,
 *             diameter=shot_info.ammo.dm.diameter._inch,             # <<<<<<<<<<<<<<
 *             weight=shot_info.ammo.dm.weight._grain,
 *             barrel_elevation=shot_info.barrel_elevation._rad,
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_shot_info, __pyx_n_s_ammo); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 247, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dm); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 247, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_diameter); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 247, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_inch); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 247, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 247, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3.diameter = __pyx_t_4;

  /* "CyEulerBallistic/trajectory_calc.pyx":248
 *             length=shot_info.ammo.dm.length._inch,
 *             diameter=shot_info.ammo.dm.diameter._inch,
 *             weight=shot_info.ammo.dm.weight._grain,             # <<<<<<<<<<<<<<
 *             barrel_elevation=shot_info.barrel_elevation._rad,
 *             barrel_azimuth=shot_info.barrel_azimuth._rad,
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_shot_info, __pyx_n_s_ammo); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_dm); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_weight); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_grain); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 248, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3.weight = __pyx_t_4;

  /* "CyEulerBallistic/trajectory_calc.pyx":249
 *             diameter=shot_info.ammo.dm.diameter._inch,
 *             weight=shot_info.ammo.dm.weight._grain,
 *             barrel_elevation=shot_info.barrel_elevation._rad,             # <<<<<<<<<<<<<<
 *             barrel_azimuth=shot_info.barrel_azimuth._rad,
 *             sight_height=shot_info.weapon.sight_height._feet,
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_shot_info, __pyx_n_s_barrel_elevation); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_rad); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 249, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3.barrel_elevation = __pyx_t_4;

  /* "CyEulerBallistic/trajectory_calc.pyx":250
 *             weight=shot_info.ammo.dm.weight._grain,
 *             barrel_elevation=shot_info.barrel_elevation._rad,
 *             barrel_azimuth=shot_info.barrel_azimuth._rad,             # <<<<<<<<<<<<<<
 *             sight_height=shot_info.weapon.sight_height._feet,
 *             cant_cosine=cos(shot_info.cant_angle._rad),
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_shot_info, __pyx_n_s_barrel_azimuth); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 250, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_rad); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 250, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 250, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3.barrel_azimuth = __pyx_t_4;

  /* "CyEulerBallistic/trajectory_calc.pyx":251
 *             barrel_elevation=shot_info.barrel_elevation._rad,
 *             barrel_azimuth=shot_info.barrel_azimuth._rad,
 *             sight_height=shot_info.weapon.sight_height._feet,             # <<<<<<<<<<<<<<
 *             cant_cosine=cos(shot_info.cant_angle._rad),
 *             cant_sine=sin(shot_info.cant_angle._rad),
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_shot_info, __pyx_n_s_weapon); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 251, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_sight_height); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 251, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_feet); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 251, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 251, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3.sight_height = __pyx_t_4;

  /* "CyEulerBallistic/trajectory_calc.pyx":252
 *             barrel_azimuth=shot_info.barrel_azimuth._rad,
 *             sight_height=shot_info.weapon.sight_height._feet,
 *             cant_cosine=cos(shot_info.cant_angle._rad),             # <<<<<<<<<<<<<<
 *             cant_sine=sin(shot_info.cant_angle._rad),
 *             alt0=shot_info.atmo.altitude._feet,
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_shot_info, __pyx_n_s_cant_angle); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 252, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_rad); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 252, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 252, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3.cant_cosine = cos(__pyx_t_4);

  /* "CyEulerBallistic/trajectory_calc.pyx":253
 *             sight_height=shot_info.weapon.sight_height._feet,
 *             cant_cosine=cos(shot_info.cant_angle._rad),
 *             cant_sine=sin(shot_info.cant_angle._rad),             # <<<<<<<<<<<<<<
 *             alt0=shot_info.atmo.altitude._feet,
 *             calc_step=self.get_calc_step(),
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_shot_info, __pyx_n_s_cant_angle); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 253, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_rad); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 253, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 253, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3.cant_sine = sin(__pyx_t_4);

  /* "CyEulerBallistic/trajectory_calc.pyx":254
 *             cant_cosine=cos(shot_info.cant_angle._rad),
 *             cant_sine=sin(shot_info.cant_angle._rad),
 *             alt0=shot_info.atmo.altitude._feet,             # <<<<<<<<<<<<<<
 *             calc_step=self.get_calc_step(),
 *             # calc_step=cy_get_calc_step(self.__config),
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_shot_info, __pyx_n_s_atmo); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 254, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_altitude); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 254, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_feet); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 254, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 254, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3.alt0 = __pyx_t_4;

  /* "CyEulerBallistic/trajectory_calc.pyx":255
 *             cant_sine=sin(shot_info.cant_angle._rad),
 *             alt0=shot_info.atmo.altitude._feet,
 *             calc_step=self.get_calc_step(),             # <<<<<<<<<<<<<<
 *             # calc_step=cy_get_calc_step(self.__config),
 *             diameter=shot_info.ammo.dm.diameter._inch,
 */
  __pyx_t_4 = ((struct __pyx_vtabstruct_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *)__pyx_v_self->__pyx_vtab)->get_calc_step(__pyx_v_self, NULL); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 255, __pyx_L1_error)
  __pyx_t_3.calc_step = __pyx_t_4;

  /* "CyEulerBallistic/trajectory_calc.pyx":257
 *             calc_step=self.get_calc_step(),
 *             # calc_step=cy_get_calc_step(self.__config),
 *             diameter=shot_info.ammo.dm.diameter._inch,             # <<<<<<<<<<<<<<
 *             stability_coefficient=0.0,
 *             atmo=Atmosphere_t(
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_shot_info, __pyx_n_s_ammo); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 257, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dm); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 257, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_diameter); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 257, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_inch); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 257, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 257, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3.diameter = __pyx_t_4;
  __pyx_t_3.stability_coefficient = 0.0;

  /* "CyEulerBallistic/trajectory_calc.pyx":260
 *             stability_coefficient=0.0,
 *             atmo=Atmosphere_t(
 *                 _t0=shot_info.atmo._t0,             # <<<<<<<<<<<<<<
 *                 _a0=shot_info.atmo._a0,
 *                 _p0=shot_info.atmo.pressure.get_in(Unit.InHg),
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_shot_info, __pyx_n_s_atmo); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 260, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_t0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 260, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 260, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7._t0 = __pyx_t_4;

  /* "CyEulerBallistic/trajectory_calc.pyx":261
 *             atmo=Atmosphere_t(
 *                 _t0=shot_info.atmo._t0,
 *                 _a0=shot_info.atmo._a0,             # <<<<<<<<<<<<<<
 *                 _p0=shot_info.atmo.pressure.get_in(Unit.InHg),
 *                 _mach1=shot_info.atmo._mach1,
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_shot_info, __pyx_n_s_atmo); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 261, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_a0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 261, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 261, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7._a0 = __pyx_t_4;

  /* "CyEulerBallistic/trajectory_calc.pyx":262
 *                 _t0=shot_info.atmo._t0,
 *                 _a0=shot_info.atmo._a0,
 *                 _p0=shot_info.atmo.pressure.get_in(Unit.InHg),             # <<<<<<<<<<<<<<
 *                 _mach1=shot_info.atmo._mach1,
 *                 density_ratio=shot_info.atmo.density_ratio,
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_shot_info, __pyx_n_s_atmo); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 262, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_pressure); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 262, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_get_in); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 262, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_n_s_Unit); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 262, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_InHg); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 262, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = NULL;
  __pyx_t_10 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_8)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
      __pyx_t_10 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_t_9};
    __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_1, __pyx_callargs+1-__pyx_t_10, 1+__pyx_t_10);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 262, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 262, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7._p0 = __pyx_t_4;

  /* "CyEulerBallistic/trajectory_calc.pyx":263
 *                 _a0=shot_info.atmo._a0,
 *                 _p0=shot_info.atmo.pressure.get_in(Unit.InHg),
 *                 _mach1=shot_info.atmo._mach1,             # <<<<<<<<<<<<<<
 *                 density_ratio=shot_info.atmo.density_ratio,
 *             )
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_shot_info, __pyx_n_s_atmo); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_mach1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 263, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7._mach1 = __pyx_t_4;

  /* "CyEulerBallistic/trajectory_calc.pyx":264
 *                 _p0=shot_info.atmo.pressure.get_in(Unit.InHg),
 *                 _mach1=shot_info.atmo._mach1,
 *                 density_ratio=shot_info.atmo.density_ratio,             # <<<<<<<<<<<<<<
 *             )
 *         )
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_shot_info, __pyx_n_s_atmo); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_density_ratio); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 264, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7.density_ratio = __pyx_t_4;
  __pyx_t_3.atmo = __pyx_t_7;

  /* "CyEulerBallistic/trajectory_calc.pyx":240
 *     cdef void _init_trajectory(self, object shot_info):
 *         self._table_data = shot_info.ammo.dm.drag_table
 *         self.__shot = ShotData_t(             # <<<<<<<<<<<<<<
 *             bc=shot_info.ammo.dm.BC,
 *             curve=cy_calculate_curve(self._table_data),
 */
  __pyx_v_self->_EulerTrajectoryCalc__shot = __pyx_t_3;

  /* "CyEulerBallistic/trajectory_calc.pyx":267
 *             )
 *         )
 *         self.__shot.muzzle_velocity = shot_info.ammo.get_velocity_for_temp(shot_info.atmo.powder_temp)._fps             # <<<<<<<<<<<<<<
 *         cy_update_stability_coefficient(&self.__shot)
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_shot_info, __pyx_n_s_ammo); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 267, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_get_velocity_for_temp); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 267, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_shot_info, __pyx_n_s_atmo); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 267, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_powder_temp); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 267, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = NULL;
  __pyx_t_10 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_9))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_9);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_9, function);
      __pyx_t_10 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_t_8};
    __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_9, __pyx_callargs+1-__pyx_t_10, 1+__pyx_t_10);
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 267, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  }
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_fps); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 267, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_9); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 267, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_v_self->_EulerTrajectoryCalc__shot.muzzle_velocity = __pyx_t_4;

  /* "CyEulerBallistic/trajectory_calc.pyx":268
 *         )
 *         self.__shot.muzzle_velocity = shot_info.ammo.get_velocity_for_temp(shot_info.atmo.powder_temp)._fps
 *         cy_update_stability_coefficient(&self.__shot)             # <<<<<<<<<<<<<<
 * 
 *         self.ws = _WindSock(shot_info.winds)
 */
  __pyx_f_16CyEulerBallistic_8cy_euler_cy_update_stability_coefficient((&__pyx_v_self->_EulerTrajectoryCalc__shot)); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 268, __pyx_L1_error)

  /* "CyEulerBallistic/trajectory_calc.pyx":270
 *         cy_update_stability_coefficient(&self.__shot)
 * 
 *         self.ws = _WindSock(shot_info.winds)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_shot_info, __pyx_n_s_winds); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 270, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_16CyEulerBallistic_15trajectory_calc__WindSock), __pyx_t_9); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 270, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->ws);
  __Pyx_DECREF((PyObject *)__pyx_v_self->ws);
  __pyx_v_self->ws = ((struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":238
 *         free_trajectory(&self.__shot)
 * 
 *     cdef void _init_trajectory(self, object shot_info):             # <<<<<<<<<<<<<<
 *         self._table_data = shot_info.ammo.dm.drag_table
 *         self.__shot = ShotData_t(
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("CyEulerBallistic.trajectory_calc.EulerTrajectoryCalc._init_trajectory", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "CyEulerBallistic/trajectory_calc.pyx":273
 * 
 * 
 *     cdef object _zero_angle(EulerTrajectoryCalc self, object shot_info, object distance):             # <<<<<<<<<<<<<<
 *         # hack to reload config if it was changed explicit on existed instance
 *         self.__config = config_bind(self._config)
 */

static PyObject *__pyx_f_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc__zero_angle(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *__pyx_v_self, PyObject *__pyx_v_shot_info, PyObject *__pyx_v_distance) {
  double __pyx_v__cZeroFindingAccuracy;
  double __pyx_v__cMaxIterations;
  double __pyx_v_zero_distance;
  double __pyx_v_height_at_zero;
  double __pyx_v_maximum_range;
  int __pyx_v_iterations_count;
  double __pyx_v_zero_finding_error;
  PyObject *__pyx_v_t = 0;
  double __pyx_v_height;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_t_16CyEulerBallistic_8cy_euler_Config_t __pyx_t_2;
  struct __pyx_t_16CyEulerBallistic_6vector_CVector __pyx_t_3;
  double __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  unsigned int __pyx_t_15;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_zero_angle", 1);

  /* "CyEulerBallistic/trajectory_calc.pyx":275
 *     cdef object _zero_angle(EulerTrajectoryCalc self, object shot_info, object distance):
 *         # hack to reload config if it was changed explicit on existed instance
 *         self.__config = config_bind(self._config)             # <<<<<<<<<<<<<<
 *         self.gravity_vector = CVector(.0, self.__config.cGravityConstant, .0)
 * 
 */
  __pyx_t_1 = __pyx_v_self->_config;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = __pyx_f_16CyEulerBallistic_8cy_euler_config_bind(__pyx_t_1); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 275, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->_EulerTrajectoryCalc__config = __pyx_t_2;

  /* "CyEulerBallistic/trajectory_calc.pyx":276
 *         # hack to reload config if it was changed explicit on existed instance
 *         self.__config = config_bind(self._config)
 *         self.gravity_vector = CVector(.0, self.__config.cGravityConstant, .0)             # <<<<<<<<<<<<<<
 * 
 *         cdef:
 */
  __pyx_t_3.x = .0;
  __pyx_t_3.y = __pyx_v_self->_EulerTrajectoryCalc__config.cGravityConstant;
  __pyx_t_3.z = .0;
  __pyx_v_self->gravity_vector = __pyx_t_3;

  /* "CyEulerBallistic/trajectory_calc.pyx":280
 *         cdef:
 *             # early bindings
 *             double _cZeroFindingAccuracy = self.__config.cZeroFindingAccuracy             # <<<<<<<<<<<<<<
 *             double _cMaxIterations = self.__config.cMaxIterations
 * 
 */
  __pyx_t_4 = __pyx_v_self->_EulerTrajectoryCalc__config.cZeroFindingAccuracy;
  __pyx_v__cZeroFindingAccuracy = __pyx_t_4;

  /* "CyEulerBallistic/trajectory_calc.pyx":281
 *             # early bindings
 *             double _cZeroFindingAccuracy = self.__config.cZeroFindingAccuracy
 *             double _cMaxIterations = self.__config.cMaxIterations             # <<<<<<<<<<<<<<
 * 
 *         cdef:
 */
  __pyx_t_5 = __pyx_v_self->_EulerTrajectoryCalc__config.cMaxIterations;
  __pyx_v__cMaxIterations = __pyx_t_5;

  /* "CyEulerBallistic/trajectory_calc.pyx":284
 * 
 *         cdef:
 *             double zero_distance = cos(shot_info.look_angle._rad) * distance._feet             # <<<<<<<<<<<<<<
 *             double height_at_zero = sin(shot_info.look_angle._rad) * distance._feet
 *             double maximum_range = zero_distance
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_shot_info, __pyx_n_s_look_angle); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 284, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_rad); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 284, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_6); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 284, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = PyFloat_FromDouble(cos(__pyx_t_4)); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 284, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_distance, __pyx_n_s_feet); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 284, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = PyNumber_Multiply(__pyx_t_6, __pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 284, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_7); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 284, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_v_zero_distance = __pyx_t_4;

  /* "CyEulerBallistic/trajectory_calc.pyx":285
 *         cdef:
 *             double zero_distance = cos(shot_info.look_angle._rad) * distance._feet
 *             double height_at_zero = sin(shot_info.look_angle._rad) * distance._feet             # <<<<<<<<<<<<<<
 *             double maximum_range = zero_distance
 *             int iterations_count = 0
 */
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_shot_info, __pyx_n_s_look_angle); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 285, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_rad); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 285, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 285, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyFloat_FromDouble(sin(__pyx_t_4)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 285, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_distance, __pyx_n_s_feet); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 285, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_6 = PyNumber_Multiply(__pyx_t_1, __pyx_t_7); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 285, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_6); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 285, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_v_height_at_zero = __pyx_t_4;

  /* "CyEulerBallistic/trajectory_calc.pyx":286
 *             double zero_distance = cos(shot_info.look_angle._rad) * distance._feet
 *             double height_at_zero = sin(shot_info.look_angle._rad) * distance._feet
 *             double maximum_range = zero_distance             # <<<<<<<<<<<<<<
 *             int iterations_count = 0
 *             double zero_finding_error = _cZeroFindingAccuracy * 2
 */
  __pyx_v_maximum_range = __pyx_v_zero_distance;

  /* "CyEulerBallistic/trajectory_calc.pyx":287
 *             double height_at_zero = sin(shot_info.look_angle._rad) * distance._feet
 *             double maximum_range = zero_distance
 *             int iterations_count = 0             # <<<<<<<<<<<<<<
 *             double zero_finding_error = _cZeroFindingAccuracy * 2
 * 
 */
  __pyx_v_iterations_count = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":288
 *             double maximum_range = zero_distance
 *             int iterations_count = 0
 *             double zero_finding_error = _cZeroFindingAccuracy * 2             # <<<<<<<<<<<<<<
 * 
 *             object t
 */
  __pyx_v_zero_finding_error = (__pyx_v__cZeroFindingAccuracy * 2.0);

  /* "CyEulerBallistic/trajectory_calc.pyx":293
 *             double height
 * 
 *         self._init_trajectory(shot_info)             # <<<<<<<<<<<<<<
 *         self.__shot.barrel_azimuth = 0.0
 *         self.__shot.barrel_elevation = atan(height_at_zero / zero_distance)
 */
  ((struct __pyx_vtabstruct_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *)__pyx_v_self->__pyx_vtab)->_init_trajectory(__pyx_v_self, __pyx_v_shot_info); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 293, __pyx_L1_error)

  /* "CyEulerBallistic/trajectory_calc.pyx":294
 * 
 *         self._init_trajectory(shot_info)
 *         self.__shot.barrel_azimuth = 0.0             # <<<<<<<<<<<<<<
 *         self.__shot.barrel_elevation = atan(height_at_zero / zero_distance)
 *         self.__shot.twist = 0
 */
  __pyx_v_self->_EulerTrajectoryCalc__shot.barrel_azimuth = 0.0;

  /* "CyEulerBallistic/trajectory_calc.pyx":295
 *         self._init_trajectory(shot_info)
 *         self.__shot.barrel_azimuth = 0.0
 *         self.__shot.barrel_elevation = atan(height_at_zero / zero_distance)             # <<<<<<<<<<<<<<
 *         self.__shot.twist = 0
 *         maximum_range -= 1.5 * self.__shot.calc_step
 */
  __pyx_v_self->_EulerTrajectoryCalc__shot.barrel_elevation = atan((__pyx_v_height_at_zero / __pyx_v_zero_distance));

  /* "CyEulerBallistic/trajectory_calc.pyx":296
 *         self.__shot.barrel_azimuth = 0.0
 *         self.__shot.barrel_elevation = atan(height_at_zero / zero_distance)
 *         self.__shot.twist = 0             # <<<<<<<<<<<<<<
 *         maximum_range -= 1.5 * self.__shot.calc_step
 * 
 */
  __pyx_v_self->_EulerTrajectoryCalc__shot.twist = 0.0;

  /* "CyEulerBallistic/trajectory_calc.pyx":297
 *         self.__shot.barrel_elevation = atan(height_at_zero / zero_distance)
 *         self.__shot.twist = 0
 *         maximum_range -= 1.5 * self.__shot.calc_step             # <<<<<<<<<<<<<<
 * 
 *         # x = horizontal distance down range, y = drop, z = windage
 */
  __pyx_v_maximum_range = (__pyx_v_maximum_range - (1.5 * __pyx_v_self->_EulerTrajectoryCalc__shot.calc_step));

  /* "CyEulerBallistic/trajectory_calc.pyx":300
 * 
 *         # x = horizontal distance down range, y = drop, z = windage
 *         while zero_finding_error > _cZeroFindingAccuracy and iterations_count < _cMaxIterations:             # <<<<<<<<<<<<<<
 *             t = self._integrate(maximum_range, zero_distance, CTrajFlag.NONE)[0]
 *             height = t.height._feet  # use there internal shortcut instead of (t.height >> Distance.Foot)
 */
  while (1) {
    __pyx_t_9 = (__pyx_v_zero_finding_error > __pyx_v__cZeroFindingAccuracy);
    if (__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L5_bool_binop_done;
    }
    __pyx_t_9 = (__pyx_v_iterations_count < __pyx_v__cMaxIterations);
    __pyx_t_8 = __pyx_t_9;
    __pyx_L5_bool_binop_done:;
    if (!__pyx_t_8) break;

    /* "CyEulerBallistic/trajectory_calc.pyx":301
 *         # x = horizontal distance down range, y = drop, z = windage
 *         while zero_finding_error > _cZeroFindingAccuracy and iterations_count < _cMaxIterations:
 *             t = self._integrate(maximum_range, zero_distance, CTrajFlag.NONE)[0]             # <<<<<<<<<<<<<<
 *             height = t.height._feet  # use there internal shortcut instead of (t.height >> Distance.Foot)
 *             zero_finding_error = fabs(height - height_at_zero)
 */
    __pyx_t_6 = ((struct __pyx_vtabstruct_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *)__pyx_v_self->__pyx_vtab)->_integrate(__pyx_v_self, __pyx_v_maximum_range, __pyx_v_zero_distance, __pyx_e_16CyEulerBallistic_15trajectory_data_NONE, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 301, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (unlikely(__pyx_t_6 == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 301, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_GetItemInt_List(__pyx_t_6, 0, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 301, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF_SET(__pyx_v_t, __pyx_t_7);
    __pyx_t_7 = 0;

    /* "CyEulerBallistic/trajectory_calc.pyx":302
 *         while zero_finding_error > _cZeroFindingAccuracy and iterations_count < _cMaxIterations:
 *             t = self._integrate(maximum_range, zero_distance, CTrajFlag.NONE)[0]
 *             height = t.height._feet  # use there internal shortcut instead of (t.height >> Distance.Foot)             # <<<<<<<<<<<<<<
 *             zero_finding_error = fabs(height - height_at_zero)
 *             if zero_finding_error > _cZeroFindingAccuracy:
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_t, __pyx_n_s_height); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 302, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_feet); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 302, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_6); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 302, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_v_height = __pyx_t_4;

    /* "CyEulerBallistic/trajectory_calc.pyx":303
 *             t = self._integrate(maximum_range, zero_distance, CTrajFlag.NONE)[0]
 *             height = t.height._feet  # use there internal shortcut instead of (t.height >> Distance.Foot)
 *             zero_finding_error = fabs(height - height_at_zero)             # <<<<<<<<<<<<<<
 *             if zero_finding_error > _cZeroFindingAccuracy:
 *                 self.__shot.barrel_elevation -= (height - height_at_zero) / zero_distance
 */
    __pyx_v_zero_finding_error = fabs((__pyx_v_height - __pyx_v_height_at_zero));

    /* "CyEulerBallistic/trajectory_calc.pyx":304
 *             height = t.height._feet  # use there internal shortcut instead of (t.height >> Distance.Foot)
 *             zero_finding_error = fabs(height - height_at_zero)
 *             if zero_finding_error > _cZeroFindingAccuracy:             # <<<<<<<<<<<<<<
 *                 self.__shot.barrel_elevation -= (height - height_at_zero) / zero_distance
 *             else:  # last barrel_elevation hit zero!
 */
    __pyx_t_8 = (__pyx_v_zero_finding_error > __pyx_v__cZeroFindingAccuracy);
    if (__pyx_t_8) {

      /* "CyEulerBallistic/trajectory_calc.pyx":305
 *             zero_finding_error = fabs(height - height_at_zero)
 *             if zero_finding_error > _cZeroFindingAccuracy:
 *                 self.__shot.barrel_elevation -= (height - height_at_zero) / zero_distance             # <<<<<<<<<<<<<<
 *             else:  # last barrel_elevation hit zero!
 *                 break
 */
      __pyx_v_self->_EulerTrajectoryCalc__shot.barrel_elevation = (__pyx_v_self->_EulerTrajectoryCalc__shot.barrel_elevation - ((__pyx_v_height - __pyx_v_height_at_zero) / __pyx_v_zero_distance));

      /* "CyEulerBallistic/trajectory_calc.pyx":304
 *             height = t.height._feet  # use there internal shortcut instead of (t.height >> Distance.Foot)
 *             zero_finding_error = fabs(height - height_at_zero)
 *             if zero_finding_error > _cZeroFindingAccuracy:             # <<<<<<<<<<<<<<
 *                 self.__shot.barrel_elevation -= (height - height_at_zero) / zero_distance
 *             else:  # last barrel_elevation hit zero!
 */
      goto __pyx_L7;
    }

    /* "CyEulerBallistic/trajectory_calc.pyx":307
 *                 self.__shot.barrel_elevation -= (height - height_at_zero) / zero_distance
 *             else:  # last barrel_elevation hit zero!
 *                 break             # <<<<<<<<<<<<<<
 *             iterations_count += 1
 *         self._free_trajectory()
 */
    /*else*/ {
      goto __pyx_L4_break;
    }
    __pyx_L7:;

    /* "CyEulerBallistic/trajectory_calc.pyx":308
 *             else:  # last barrel_elevation hit zero!
 *                 break
 *             iterations_count += 1             # <<<<<<<<<<<<<<
 *         self._free_trajectory()
 *         if zero_finding_error > _cZeroFindingAccuracy:
 */
    __pyx_v_iterations_count = (__pyx_v_iterations_count + 1);
  }
  __pyx_L4_break:;

  /* "CyEulerBallistic/trajectory_calc.pyx":309
 *                 break
 *             iterations_count += 1
 *         self._free_trajectory()             # <<<<<<<<<<<<<<
 *         if zero_finding_error > _cZeroFindingAccuracy:
 *             raise ZeroFindingError(zero_finding_error, iterations_count, Angular.Radian(self.__shot.barrel_elevation))
 */
  ((struct __pyx_vtabstruct_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *)__pyx_v_self->__pyx_vtab)->_free_trajectory(__pyx_v_self); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 309, __pyx_L1_error)

  /* "CyEulerBallistic/trajectory_calc.pyx":310
 *             iterations_count += 1
 *         self._free_trajectory()
 *         if zero_finding_error > _cZeroFindingAccuracy:             # <<<<<<<<<<<<<<
 *             raise ZeroFindingError(zero_finding_error, iterations_count, Angular.Radian(self.__shot.barrel_elevation))
 *         return Angular.Radian(self.__shot.barrel_elevation)
 */
  __pyx_t_8 = (__pyx_v_zero_finding_error > __pyx_v__cZeroFindingAccuracy);
  if (unlikely(__pyx_t_8)) {

    /* "CyEulerBallistic/trajectory_calc.pyx":311
 *         self._free_trajectory()
 *         if zero_finding_error > _cZeroFindingAccuracy:
 *             raise ZeroFindingError(zero_finding_error, iterations_count, Angular.Radian(self.__shot.barrel_elevation))             # <<<<<<<<<<<<<<
 *         return Angular.Radian(self.__shot.barrel_elevation)
 * 
 */
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_ZeroFindingError); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 311, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_1 = PyFloat_FromDouble(__pyx_v_zero_finding_error); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 311, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_10 = __Pyx_PyInt_From_int(__pyx_v_iterations_count); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 311, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_n_s_Angular); if (unlikely(!__pyx_t_12)) __PYX_ERR(1, 311, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_Radian); if (unlikely(!__pyx_t_13)) __PYX_ERR(1, 311, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __pyx_t_12 = PyFloat_FromDouble(__pyx_v_self->_EulerTrajectoryCalc__shot.barrel_elevation); if (unlikely(!__pyx_t_12)) __PYX_ERR(1, 311, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __pyx_t_14 = NULL;
    __pyx_t_15 = 0;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_13))) {
      __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_13);
      if (likely(__pyx_t_14)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
        __Pyx_INCREF(__pyx_t_14);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_13, function);
        __pyx_t_15 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_14, __pyx_t_12};
      __pyx_t_11 = __Pyx_PyObject_FastCall(__pyx_t_13, __pyx_callargs+1-__pyx_t_15, 1+__pyx_t_15);
      __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      if (unlikely(!__pyx_t_11)) __PYX_ERR(1, 311, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
    }
    __pyx_t_13 = NULL;
    __pyx_t_15 = 0;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_13)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_13);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
        __pyx_t_15 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[4] = {__pyx_t_13, __pyx_t_1, __pyx_t_10, __pyx_t_11};
      __pyx_t_6 = __Pyx_PyObject_FastCall(__pyx_t_7, __pyx_callargs+1-__pyx_t_15, 3+__pyx_t_15);
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 311, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __PYX_ERR(1, 311, __pyx_L1_error)

    /* "CyEulerBallistic/trajectory_calc.pyx":310
 *             iterations_count += 1
 *         self._free_trajectory()
 *         if zero_finding_error > _cZeroFindingAccuracy:             # <<<<<<<<<<<<<<
 *             raise ZeroFindingError(zero_finding_error, iterations_count, Angular.Radian(self.__shot.barrel_elevation))
 *         return Angular.Radian(self.__shot.barrel_elevation)
 */
  }

  /* "CyEulerBallistic/trajectory_calc.pyx":312
 *         if zero_finding_error > _cZeroFindingAccuracy:
 *             raise ZeroFindingError(zero_finding_error, iterations_count, Angular.Radian(self.__shot.barrel_elevation))
 *         return Angular.Radian(self.__shot.barrel_elevation)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_Angular); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 312, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_Radian); if (unlikely(!__pyx_t_11)) __PYX_ERR(1, 312, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = PyFloat_FromDouble(__pyx_v_self->_EulerTrajectoryCalc__shot.barrel_elevation); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 312, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_10 = NULL;
  __pyx_t_15 = 0;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_11))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_11);
    if (likely(__pyx_t_10)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_11);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_11, function);
      __pyx_t_15 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_10, __pyx_t_7};
    __pyx_t_6 = __Pyx_PyObject_FastCall(__pyx_t_11, __pyx_callargs+1-__pyx_t_15, 1+__pyx_t_15);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 312, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  }
  __pyx_r = __pyx_t_6;
  __pyx_t_6 = 0;
  goto __pyx_L0;

  /* "CyEulerBallistic/trajectory_calc.pyx":273
 * 
 * 
 *     cdef object _zero_angle(EulerTrajectoryCalc self, object shot_info, object distance):             # <<<<<<<<<<<<<<
 *         # hack to reload config if it was changed explicit on existed instance
 *         self.__config = config_bind(self._config)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_AddTraceback("CyEulerBallistic.trajectory_calc.EulerTrajectoryCalc._zero_angle", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_t);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "CyEulerBallistic/trajectory_calc.pyx":315
 * 
 * 
 *     cdef list[object] _integrate(EulerTrajectoryCalc self,             # <<<<<<<<<<<<<<
 *                           double maximum_range, double step, int filter_flags, double time_step = 0.0):
 *         cdef:
 */

static PyObject *__pyx_f_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc__integrate(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *__pyx_v_self, double __pyx_v_maximum_range, double __pyx_v_step, int __pyx_v_filter_flags, struct __pyx_opt_args_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc__integrate *__pyx_optional_args) {
  double __pyx_v_time_step = ((double)0.0);
  double __pyx_v_velocity;
  double __pyx_v_delta_time;
  double __pyx_v_density_factor;
  double __pyx_v_mach;
  PyObject *__pyx_v_ranges = 0;
  double __pyx_v_time;
  double __pyx_v_drag;
  struct __pyx_t_16CyEulerBallistic_6vector_CVector __pyx_v_range_vector;
  struct __pyx_t_16CyEulerBallistic_6vector_CVector __pyx_v_velocity_vector;
  struct __pyx_t_16CyEulerBallistic_6vector_CVector __pyx_v_delta_range_vector;
  struct __pyx_t_16CyEulerBallistic_6vector_CVector __pyx_v_velocity_adjusted;
  struct __pyx_t_16CyEulerBallistic_6vector_CVector __pyx_v_gravity_vector;
  double __pyx_v_calc_step;
  struct __pyx_t_16CyEulerBallistic_6vector_CVector __pyx_v_wind_vector;
  struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *__pyx_v_data_filter = 0;
  double __pyx_v__cMinimumVelocity;
  double __pyx_v__cMaximumDrop;
  double __pyx_v__cMinimumAltitude;
  struct __pyx_t_16CyEulerBallistic_6vector_CVector __pyx_v__dir_vector;
  struct __pyx_t_16CyEulerBallistic_6vector_CVector __pyx_v__temp1;
  struct __pyx_t_16CyEulerBallistic_6vector_CVector __pyx_v__temp2;
  struct __pyx_t_16CyEulerBallistic_6vector_CVector __pyx_v__temp3;
  int __pyx_v_it;
  PyObject *__pyx_v_reason = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_t_16CyEulerBallistic_6vector_CVector __pyx_t_2;
  double __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  unsigned int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  double __pyx_t_9;
  double __pyx_t_10;
  int __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_integrate", 1);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_time_step = __pyx_optional_args->time_step;
    }
  }

  /* "CyEulerBallistic/trajectory_calc.pyx":319
 *         cdef:
 *             double velocity, delta_time
 *             double density_factor = .0             # <<<<<<<<<<<<<<
 *             double mach = .0
 *             list[object] ranges = []
 */
  __pyx_v_density_factor = .0;

  /* "CyEulerBallistic/trajectory_calc.pyx":320
 *             double velocity, delta_time
 *             double density_factor = .0
 *             double mach = .0             # <<<<<<<<<<<<<<
 *             list[object] ranges = []
 *             double time = .0
 */
  __pyx_v_mach = .0;

  /* "CyEulerBallistic/trajectory_calc.pyx":321
 *             double density_factor = .0
 *             double mach = .0
 *             list[object] ranges = []             # <<<<<<<<<<<<<<
 *             double time = .0
 *             double drag = .0
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 321, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_ranges = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":322
 *             double mach = .0
 *             list[object] ranges = []
 *             double time = .0             # <<<<<<<<<<<<<<
 *             double drag = .0
 *             CVector range_vector, velocity_vector
 */
  __pyx_v_time = .0;

  /* "CyEulerBallistic/trajectory_calc.pyx":323
 *             list[object] ranges = []
 *             double time = .0
 *             double drag = .0             # <<<<<<<<<<<<<<
 *             CVector range_vector, velocity_vector
 *             CVector delta_range_vector, velocity_adjusted
 */
  __pyx_v_drag = .0;

  /* "CyEulerBallistic/trajectory_calc.pyx":326
 *             CVector range_vector, velocity_vector
 *             CVector delta_range_vector, velocity_adjusted
 *             CVector gravity_vector = CVector(.0, self.__config.cGravityConstant, .0)             # <<<<<<<<<<<<<<
 *             double calc_step = self.__shot.calc_step
 * 
 */
  __pyx_t_2.x = .0;
  __pyx_t_2.y = __pyx_v_self->_EulerTrajectoryCalc__config.cGravityConstant;
  __pyx_t_2.z = .0;
  __pyx_v_gravity_vector = __pyx_t_2;

  /* "CyEulerBallistic/trajectory_calc.pyx":327
 *             CVector delta_range_vector, velocity_adjusted
 *             CVector gravity_vector = CVector(.0, self.__config.cGravityConstant, .0)
 *             double calc_step = self.__shot.calc_step             # <<<<<<<<<<<<<<
 * 
 *             # region Initialize wind-related variables to first wind reading (if any)
 */
  __pyx_t_3 = __pyx_v_self->_EulerTrajectoryCalc__shot.calc_step;
  __pyx_v_calc_step = __pyx_t_3;

  /* "CyEulerBallistic/trajectory_calc.pyx":330
 * 
 *             # region Initialize wind-related variables to first wind reading (if any)
 *             CVector wind_vector = self.ws.current_vector()             # <<<<<<<<<<<<<<
 *             # endregion
 * 
 */
  __pyx_t_2 = __pyx_f_16CyEulerBallistic_15trajectory_calc_9_WindSock_current_vector(__pyx_v_self->ws); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 330, __pyx_L1_error)
  __pyx_v_wind_vector = __pyx_t_2;

  /* "CyEulerBallistic/trajectory_calc.pyx":337
 *         cdef:
 *             # early bindings
 *             double _cMinimumVelocity = self.__config.cMinimumVelocity             # <<<<<<<<<<<<<<
 *             double _cMaximumDrop = self.__config.cMaximumDrop
 *             double _cMinimumAltitude = self.__config.cMinimumAltitude
 */
  __pyx_t_3 = __pyx_v_self->_EulerTrajectoryCalc__config.cMinimumVelocity;
  __pyx_v__cMinimumVelocity = __pyx_t_3;

  /* "CyEulerBallistic/trajectory_calc.pyx":338
 *             # early bindings
 *             double _cMinimumVelocity = self.__config.cMinimumVelocity
 *             double _cMaximumDrop = self.__config.cMaximumDrop             # <<<<<<<<<<<<<<
 *             double _cMinimumAltitude = self.__config.cMinimumAltitude
 * 
 */
  __pyx_t_3 = __pyx_v_self->_EulerTrajectoryCalc__config.cMaximumDrop;
  __pyx_v__cMaximumDrop = __pyx_t_3;

  /* "CyEulerBallistic/trajectory_calc.pyx":339
 *             double _cMinimumVelocity = self.__config.cMinimumVelocity
 *             double _cMaximumDrop = self.__config.cMaximumDrop
 *             double _cMinimumAltitude = self.__config.cMinimumAltitude             # <<<<<<<<<<<<<<
 * 
 *         cdef:
 */
  __pyx_t_3 = __pyx_v_self->_EulerTrajectoryCalc__config.cMinimumAltitude;
  __pyx_v__cMinimumAltitude = __pyx_t_3;

  /* "CyEulerBallistic/trajectory_calc.pyx":346
 * 
 *         # region Initialize velocity and position of projectile
 *         velocity = self.__shot.muzzle_velocity             # <<<<<<<<<<<<<<
 *         # x: downrange distance, y: drop, z: windage
 *         range_vector = CVector(.0, -self.__shot.cant_cosine * self.__shot.sight_height, -self.__shot.cant_sine * self.__shot.sight_height)
 */
  __pyx_t_3 = __pyx_v_self->_EulerTrajectoryCalc__shot.muzzle_velocity;
  __pyx_v_velocity = __pyx_t_3;

  /* "CyEulerBallistic/trajectory_calc.pyx":348
 *         velocity = self.__shot.muzzle_velocity
 *         # x: downrange distance, y: drop, z: windage
 *         range_vector = CVector(.0, -self.__shot.cant_cosine * self.__shot.sight_height, -self.__shot.cant_sine * self.__shot.sight_height)             # <<<<<<<<<<<<<<
 *         _dir_vector = CVector(cos(self.__shot.barrel_elevation) * cos(self.__shot.barrel_azimuth),
 *                                  sin(self.__shot.barrel_elevation),
 */
  __pyx_t_2.x = .0;
  __pyx_t_2.y = ((-__pyx_v_self->_EulerTrajectoryCalc__shot.cant_cosine) * __pyx_v_self->_EulerTrajectoryCalc__shot.sight_height);
  __pyx_t_2.z = ((-__pyx_v_self->_EulerTrajectoryCalc__shot.cant_sine) * __pyx_v_self->_EulerTrajectoryCalc__shot.sight_height);
  __pyx_v_range_vector = __pyx_t_2;

  /* "CyEulerBallistic/trajectory_calc.pyx":349
 *         # x: downrange distance, y: drop, z: windage
 *         range_vector = CVector(.0, -self.__shot.cant_cosine * self.__shot.sight_height, -self.__shot.cant_sine * self.__shot.sight_height)
 *         _dir_vector = CVector(cos(self.__shot.barrel_elevation) * cos(self.__shot.barrel_azimuth),             # <<<<<<<<<<<<<<
 *                                  sin(self.__shot.barrel_elevation),
 *                                  cos(self.__shot.barrel_elevation) * sin(self.__shot.barrel_azimuth))
 */
  __pyx_t_2.x = (cos(__pyx_v_self->_EulerTrajectoryCalc__shot.barrel_elevation) * cos(__pyx_v_self->_EulerTrajectoryCalc__shot.barrel_azimuth));

  /* "CyEulerBallistic/trajectory_calc.pyx":350
 *         range_vector = CVector(.0, -self.__shot.cant_cosine * self.__shot.sight_height, -self.__shot.cant_sine * self.__shot.sight_height)
 *         _dir_vector = CVector(cos(self.__shot.barrel_elevation) * cos(self.__shot.barrel_azimuth),
 *                                  sin(self.__shot.barrel_elevation),             # <<<<<<<<<<<<<<
 *                                  cos(self.__shot.barrel_elevation) * sin(self.__shot.barrel_azimuth))
 *         velocity_vector = mul_c(&_dir_vector, velocity)
 */
  __pyx_t_2.y = sin(__pyx_v_self->_EulerTrajectoryCalc__shot.barrel_elevation);

  /* "CyEulerBallistic/trajectory_calc.pyx":351
 *         _dir_vector = CVector(cos(self.__shot.barrel_elevation) * cos(self.__shot.barrel_azimuth),
 *                                  sin(self.__shot.barrel_elevation),
 *                                  cos(self.__shot.barrel_elevation) * sin(self.__shot.barrel_azimuth))             # <<<<<<<<<<<<<<
 *         velocity_vector = mul_c(&_dir_vector, velocity)
 *         # endregion
 */
  __pyx_t_2.z = (cos(__pyx_v_self->_EulerTrajectoryCalc__shot.barrel_elevation) * sin(__pyx_v_self->_EulerTrajectoryCalc__shot.barrel_azimuth));
  __pyx_v__dir_vector = __pyx_t_2;

  /* "CyEulerBallistic/trajectory_calc.pyx":352
 *                                  sin(self.__shot.barrel_elevation),
 *                                  cos(self.__shot.barrel_elevation) * sin(self.__shot.barrel_azimuth))
 *         velocity_vector = mul_c(&_dir_vector, velocity)             # <<<<<<<<<<<<<<
 *         # endregion
 * 
 */
  __pyx_t_2 = __pyx_f_16CyEulerBallistic_6vector_mul_c((&__pyx_v__dir_vector), __pyx_v_velocity); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 352, __pyx_L1_error)
  __pyx_v_velocity_vector = __pyx_t_2;

  /* "CyEulerBallistic/trajectory_calc.pyx":356
 * 
 *         # With non-zero look_angle, rounding can suggest multiple adjacent zero-crossings
 *         data_filter = _TrajectoryDataFilter(filter_flags=filter_flags,             # <<<<<<<<<<<<<<
 *                                             ranges_length=<int> ((maximum_range / step) + 1),
 *                                             time_step=time_step)
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 356, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_filter_flags); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 356, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_filter_flags, __pyx_t_4) < 0) __PYX_ERR(1, 356, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":357
 *         # With non-zero look_angle, rounding can suggest multiple adjacent zero-crossings
 *         data_filter = _TrajectoryDataFilter(filter_flags=filter_flags,
 *                                             ranges_length=<int> ((maximum_range / step) + 1),             # <<<<<<<<<<<<<<
 *                                             time_step=time_step)
 *         data_filter.setup_seen_zero(range_vector.y, self.__shot.barrel_elevation, self.__shot.look_angle)
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(((int)((__pyx_v_maximum_range / __pyx_v_step) + 1.0))); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 357, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_ranges_length, __pyx_t_4) < 0) __PYX_ERR(1, 356, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":358
 *         data_filter = _TrajectoryDataFilter(filter_flags=filter_flags,
 *                                             ranges_length=<int> ((maximum_range / step) + 1),
 *                                             time_step=time_step)             # <<<<<<<<<<<<<<
 *         data_filter.setup_seen_zero(range_vector.y, self.__shot.barrel_elevation, self.__shot.look_angle)
 * 
 */
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_time_step); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 358, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_time_step, __pyx_t_4) < 0) __PYX_ERR(1, 356, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":356
 * 
 *         # With non-zero look_angle, rounding can suggest multiple adjacent zero-crossings
 *         data_filter = _TrajectoryDataFilter(filter_flags=filter_flags,             # <<<<<<<<<<<<<<
 *                                             ranges_length=<int> ((maximum_range / step) + 1),
 *                                             time_step=time_step)
 */
  __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter), __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 356, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_data_filter = ((struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":359
 *                                             ranges_length=<int> ((maximum_range / step) + 1),
 *                                             time_step=time_step)
 *         data_filter.setup_seen_zero(range_vector.y, self.__shot.barrel_elevation, self.__shot.look_angle)             # <<<<<<<<<<<<<<
 * 
 *         #region Trajectory Loop
 */
  __pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_setup_seen_zero(__pyx_v_data_filter, __pyx_v_range_vector.y, __pyx_v_self->_EulerTrajectoryCalc__shot.barrel_elevation, __pyx_v_self->_EulerTrajectoryCalc__shot.look_angle); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 359, __pyx_L1_error)

  /* "CyEulerBallistic/trajectory_calc.pyx":362
 * 
 *         #region Trajectory Loop
 *         warnings.simplefilter("once")  # used to avoid multiple warnings in a loop             # <<<<<<<<<<<<<<
 *         cdef int it = 0
 *         while range_vector.x <= maximum_range + calc_step:
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_warnings); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 362, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_simplefilter); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 362, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = NULL;
  __pyx_t_6 = 0;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
      __pyx_t_6 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_n_u_once};
    __pyx_t_4 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+1-__pyx_t_6, 1+__pyx_t_6);
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 362, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":363
 *         #region Trajectory Loop
 *         warnings.simplefilter("once")  # used to avoid multiple warnings in a loop
 *         cdef int it = 0             # <<<<<<<<<<<<<<
 *         while range_vector.x <= maximum_range + calc_step:
 *             it += 1
 */
  __pyx_v_it = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":364
 *         warnings.simplefilter("once")  # used to avoid multiple warnings in a loop
 *         cdef int it = 0
 *         while range_vector.x <= maximum_range + calc_step:             # <<<<<<<<<<<<<<
 *             it += 1
 *             data_filter.current_flag = CTrajFlag.NONE
 */
  while (1) {
    __pyx_t_7 = (__pyx_v_range_vector.x <= (__pyx_v_maximum_range + __pyx_v_calc_step));
    if (!__pyx_t_7) break;

    /* "CyEulerBallistic/trajectory_calc.pyx":365
 *         cdef int it = 0
 *         while range_vector.x <= maximum_range + calc_step:
 *             it += 1             # <<<<<<<<<<<<<<
 *             data_filter.current_flag = CTrajFlag.NONE
 * 
 */
    __pyx_v_it = (__pyx_v_it + 1);

    /* "CyEulerBallistic/trajectory_calc.pyx":366
 *         while range_vector.x <= maximum_range + calc_step:
 *             it += 1
 *             data_filter.current_flag = CTrajFlag.NONE             # <<<<<<<<<<<<<<
 * 
 *             # Update wind reading at current point in trajectory
 */
    __pyx_v_data_filter->current_flag = __pyx_e_16CyEulerBallistic_15trajectory_data_NONE;

    /* "CyEulerBallistic/trajectory_calc.pyx":369
 * 
 *             # Update wind reading at current point in trajectory
 *             if range_vector.x >= self.ws.next_range:  # require check before call to improve performance             # <<<<<<<<<<<<<<
 *                 wind_vector = self.ws.vector_for_range(range_vector.x)
 * 
 */
    __pyx_t_7 = (__pyx_v_range_vector.x >= __pyx_v_self->ws->next_range);
    if (__pyx_t_7) {

      /* "CyEulerBallistic/trajectory_calc.pyx":370
 *             # Update wind reading at current point in trajectory
 *             if range_vector.x >= self.ws.next_range:  # require check before call to improve performance
 *                 wind_vector = self.ws.vector_for_range(range_vector.x)             # <<<<<<<<<<<<<<
 * 
 *             # overwrite density_factor and mach by pointer
 */
      __pyx_t_2 = __pyx_f_16CyEulerBallistic_15trajectory_calc_9_WindSock_vector_for_range(__pyx_v_self->ws, __pyx_v_range_vector.x); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 370, __pyx_L1_error)
      __pyx_v_wind_vector = __pyx_t_2;

      /* "CyEulerBallistic/trajectory_calc.pyx":369
 * 
 *             # Update wind reading at current point in trajectory
 *             if range_vector.x >= self.ws.next_range:  # require check before call to improve performance             # <<<<<<<<<<<<<<
 *                 wind_vector = self.ws.vector_for_range(range_vector.x)
 * 
 */
    }

    /* "CyEulerBallistic/trajectory_calc.pyx":373
 * 
 *             # overwrite density_factor and mach by pointer
 *             update_density_factor_and_mach_for_altitude(&self.__shot.atmo,             # <<<<<<<<<<<<<<
 *                 self.__shot.alt0 + range_vector.y, &density_factor, &mach)
 * 
 */
    __pyx_f_16CyEulerBallistic_8cy_euler_update_density_factor_and_mach_for_altitude((&__pyx_v_self->_EulerTrajectoryCalc__shot.atmo), (__pyx_v_self->_EulerTrajectoryCalc__shot.alt0 + __pyx_v_range_vector.y), (&__pyx_v_density_factor), (&__pyx_v_mach)); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 373, __pyx_L1_error)

    /* "CyEulerBallistic/trajectory_calc.pyx":376
 *                 self.__shot.alt0 + range_vector.y, &density_factor, &mach)
 * 
 *             if filter_flags:             # <<<<<<<<<<<<<<
 * 
 *                 # Record TrajectoryData row
 */
    __pyx_t_7 = (__pyx_v_filter_flags != 0);
    if (__pyx_t_7) {

      /* "CyEulerBallistic/trajectory_calc.pyx":380
 *                 # Record TrajectoryData row
 *                 # if data_filter.should_record(range_vector, velocity, mach, step, self.look_angle, time):
 *                 if data_filter.should_record(range_vector, velocity, mach, step, time):             # <<<<<<<<<<<<<<
 *                     ranges.append(create_trajectory_row(
 *                         time, range_vector, velocity_vector,
 */
      __pyx_t_7 = __pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_should_record(__pyx_v_data_filter, __pyx_v_range_vector, __pyx_v_velocity, __pyx_v_mach, __pyx_v_step, __pyx_v_time); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 380, __pyx_L1_error)
      if (__pyx_t_7) {

        /* "CyEulerBallistic/trajectory_calc.pyx":383
 *                     ranges.append(create_trajectory_row(
 *                         time, range_vector, velocity_vector,
 *                         velocity, mach, cy_spin_drift(&self.__shot, time), self.__shot.look_angle,             # <<<<<<<<<<<<<<
 *                         density_factor, drag, self.__shot.weight, data_filter.current_flag
 *                     ))
 */
        __pyx_t_3 = __pyx_f_16CyEulerBallistic_8cy_euler_cy_spin_drift((&__pyx_v_self->_EulerTrajectoryCalc__shot), __pyx_v_time); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 383, __pyx_L1_error)

        /* "CyEulerBallistic/trajectory_calc.pyx":381
 *                 # if data_filter.should_record(range_vector, velocity, mach, step, self.look_angle, time):
 *                 if data_filter.should_record(range_vector, velocity, mach, step, time):
 *                     ranges.append(create_trajectory_row(             # <<<<<<<<<<<<<<
 *                         time, range_vector, velocity_vector,
 *                         velocity, mach, cy_spin_drift(&self.__shot, time), self.__shot.look_angle,
 */
        __pyx_t_4 = __pyx_f_16CyEulerBallistic_15trajectory_calc_create_trajectory_row(__pyx_v_time, __pyx_v_range_vector, __pyx_v_velocity_vector, __pyx_v_velocity, __pyx_v_mach, __pyx_t_3, __pyx_v_self->_EulerTrajectoryCalc__shot.look_angle, __pyx_v_density_factor, __pyx_v_drag, __pyx_v_self->_EulerTrajectoryCalc__shot.weight, __pyx_v_data_filter->current_flag); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 381, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_8 = __Pyx_PyList_Append(__pyx_v_ranges, __pyx_t_4); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(1, 381, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

        /* "CyEulerBallistic/trajectory_calc.pyx":386
 *                         density_factor, drag, self.__shot.weight, data_filter.current_flag
 *                     ))
 *                     if data_filter.should_break():             # <<<<<<<<<<<<<<
 *                         break
 * 
 */
        __pyx_t_7 = __pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_should_break(__pyx_v_data_filter); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 386, __pyx_L1_error)
        if (__pyx_t_7) {

          /* "CyEulerBallistic/trajectory_calc.pyx":387
 *                     ))
 *                     if data_filter.should_break():
 *                         break             # <<<<<<<<<<<<<<
 * 
 *             #region Ballistic calculation step
 */
          goto __pyx_L4_break;

          /* "CyEulerBallistic/trajectory_calc.pyx":386
 *                         density_factor, drag, self.__shot.weight, data_filter.current_flag
 *                     ))
 *                     if data_filter.should_break():             # <<<<<<<<<<<<<<
 *                         break
 * 
 */
        }

        /* "CyEulerBallistic/trajectory_calc.pyx":380
 *                 # Record TrajectoryData row
 *                 # if data_filter.should_record(range_vector, velocity, mach, step, self.look_angle, time):
 *                 if data_filter.should_record(range_vector, velocity, mach, step, time):             # <<<<<<<<<<<<<<
 *                     ranges.append(create_trajectory_row(
 *                         time, range_vector, velocity_vector,
 */
      }

      /* "CyEulerBallistic/trajectory_calc.pyx":376
 *                 self.__shot.alt0 + range_vector.y, &density_factor, &mach)
 * 
 *             if filter_flags:             # <<<<<<<<<<<<<<
 * 
 *                 # Record TrajectoryData row
 */
    }

    /* "CyEulerBallistic/trajectory_calc.pyx":392
 *             # use just cdef methods to
 * 
 *             velocity_adjusted = sub(&velocity_vector, &wind_vector)             # <<<<<<<<<<<<<<
 *             velocity = mag(&velocity_adjusted)
 *             delta_time = calc_step / max(1.0, velocity)
 */
    __pyx_t_2 = __pyx_f_16CyEulerBallistic_6vector_sub((&__pyx_v_velocity_vector), (&__pyx_v_wind_vector)); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 392, __pyx_L1_error)
    __pyx_v_velocity_adjusted = __pyx_t_2;

    /* "CyEulerBallistic/trajectory_calc.pyx":393
 * 
 *             velocity_adjusted = sub(&velocity_vector, &wind_vector)
 *             velocity = mag(&velocity_adjusted)             # <<<<<<<<<<<<<<
 *             delta_time = calc_step / max(1.0, velocity)
 *             drag = density_factor * velocity * cy_drag_by_mach(&self.__shot, velocity / mach)
 */
    __pyx_t_3 = __pyx_f_16CyEulerBallistic_6vector_mag((&__pyx_v_velocity_adjusted)); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 393, __pyx_L1_error)
    __pyx_v_velocity = __pyx_t_3;

    /* "CyEulerBallistic/trajectory_calc.pyx":394
 *             velocity_adjusted = sub(&velocity_vector, &wind_vector)
 *             velocity = mag(&velocity_adjusted)
 *             delta_time = calc_step / max(1.0, velocity)             # <<<<<<<<<<<<<<
 *             drag = density_factor * velocity * cy_drag_by_mach(&self.__shot, velocity / mach)
 * 
 */
    __pyx_t_3 = __pyx_v_velocity;
    __pyx_t_9 = 1.0;
    __pyx_t_7 = (__pyx_t_3 > __pyx_t_9);
    if (__pyx_t_7) {
      __pyx_t_10 = __pyx_t_3;
    } else {
      __pyx_t_10 = __pyx_t_9;
    }
    __pyx_v_delta_time = (__pyx_v_calc_step / __pyx_t_10);

    /* "CyEulerBallistic/trajectory_calc.pyx":395
 *             velocity = mag(&velocity_adjusted)
 *             delta_time = calc_step / max(1.0, velocity)
 *             drag = density_factor * velocity * cy_drag_by_mach(&self.__shot, velocity / mach)             # <<<<<<<<<<<<<<
 * 
 *             _temp1 = mul_c(&velocity_adjusted, drag)
 */
    __pyx_t_10 = __pyx_f_16CyEulerBallistic_8cy_euler_cy_drag_by_mach((&__pyx_v_self->_EulerTrajectoryCalc__shot), (__pyx_v_velocity / __pyx_v_mach)); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 395, __pyx_L1_error)
    __pyx_v_drag = ((__pyx_v_density_factor * __pyx_v_velocity) * __pyx_t_10);

    /* "CyEulerBallistic/trajectory_calc.pyx":397
 *             drag = density_factor * velocity * cy_drag_by_mach(&self.__shot, velocity / mach)
 * 
 *             _temp1 = mul_c(&velocity_adjusted, drag)             # <<<<<<<<<<<<<<
 *             _temp2 = sub(&_temp1, &gravity_vector)
 *             _temp3 = mul_c(&_temp2, delta_time)
 */
    __pyx_t_2 = __pyx_f_16CyEulerBallistic_6vector_mul_c((&__pyx_v_velocity_adjusted), __pyx_v_drag); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 397, __pyx_L1_error)
    __pyx_v__temp1 = __pyx_t_2;

    /* "CyEulerBallistic/trajectory_calc.pyx":398
 * 
 *             _temp1 = mul_c(&velocity_adjusted, drag)
 *             _temp2 = sub(&_temp1, &gravity_vector)             # <<<<<<<<<<<<<<
 *             _temp3 = mul_c(&_temp2, delta_time)
 * 
 */
    __pyx_t_2 = __pyx_f_16CyEulerBallistic_6vector_sub((&__pyx_v__temp1), (&__pyx_v_gravity_vector)); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 398, __pyx_L1_error)
    __pyx_v__temp2 = __pyx_t_2;

    /* "CyEulerBallistic/trajectory_calc.pyx":399
 *             _temp1 = mul_c(&velocity_adjusted, drag)
 *             _temp2 = sub(&_temp1, &gravity_vector)
 *             _temp3 = mul_c(&_temp2, delta_time)             # <<<<<<<<<<<<<<
 * 
 *             velocity_vector = sub(&velocity_vector, &_temp3)
 */
    __pyx_t_2 = __pyx_f_16CyEulerBallistic_6vector_mul_c((&__pyx_v__temp2), __pyx_v_delta_time); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 399, __pyx_L1_error)
    __pyx_v__temp3 = __pyx_t_2;

    /* "CyEulerBallistic/trajectory_calc.pyx":401
 *             _temp3 = mul_c(&_temp2, delta_time)
 * 
 *             velocity_vector = sub(&velocity_vector, &_temp3)             # <<<<<<<<<<<<<<
 * 
 *             delta_range_vector = mul_c(&velocity_vector, delta_time)
 */
    __pyx_t_2 = __pyx_f_16CyEulerBallistic_6vector_sub((&__pyx_v_velocity_vector), (&__pyx_v__temp3)); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 401, __pyx_L1_error)
    __pyx_v_velocity_vector = __pyx_t_2;

    /* "CyEulerBallistic/trajectory_calc.pyx":403
 *             velocity_vector = sub(&velocity_vector, &_temp3)
 * 
 *             delta_range_vector = mul_c(&velocity_vector, delta_time)             # <<<<<<<<<<<<<<
 *             range_vector = add(&range_vector, &delta_range_vector)
 * 
 */
    __pyx_t_2 = __pyx_f_16CyEulerBallistic_6vector_mul_c((&__pyx_v_velocity_vector), __pyx_v_delta_time); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 403, __pyx_L1_error)
    __pyx_v_delta_range_vector = __pyx_t_2;

    /* "CyEulerBallistic/trajectory_calc.pyx":404
 * 
 *             delta_range_vector = mul_c(&velocity_vector, delta_time)
 *             range_vector = add(&range_vector, &delta_range_vector)             # <<<<<<<<<<<<<<
 * 
 *             velocity = mag(&velocity_vector)
 */
    __pyx_t_2 = __pyx_f_16CyEulerBallistic_6vector_add((&__pyx_v_range_vector), (&__pyx_v_delta_range_vector)); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 404, __pyx_L1_error)
    __pyx_v_range_vector = __pyx_t_2;

    /* "CyEulerBallistic/trajectory_calc.pyx":406
 *             range_vector = add(&range_vector, &delta_range_vector)
 * 
 *             velocity = mag(&velocity_vector)             # <<<<<<<<<<<<<<
 *             time += delta_time
 * 
 */
    __pyx_t_10 = __pyx_f_16CyEulerBallistic_6vector_mag((&__pyx_v_velocity_vector)); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 406, __pyx_L1_error)
    __pyx_v_velocity = __pyx_t_10;

    /* "CyEulerBallistic/trajectory_calc.pyx":407
 * 
 *             velocity = mag(&velocity_vector)
 *             time += delta_time             # <<<<<<<<<<<<<<
 * 
 *             if (
 */
    __pyx_v_time = (__pyx_v_time + __pyx_v_delta_time);

    /* "CyEulerBallistic/trajectory_calc.pyx":410
 * 
 *             if (
 *                     velocity < _cMinimumVelocity             # <<<<<<<<<<<<<<
 *                     or range_vector.y < _cMaximumDrop
 *                     or self.__shot.alt0 + range_vector.y < _cMinimumAltitude
 */
    __pyx_t_11 = (__pyx_v_velocity < __pyx_v__cMinimumVelocity);
    if (!__pyx_t_11) {
    } else {
      __pyx_t_7 = __pyx_t_11;
      goto __pyx_L10_bool_binop_done;
    }

    /* "CyEulerBallistic/trajectory_calc.pyx":411
 *             if (
 *                     velocity < _cMinimumVelocity
 *                     or range_vector.y < _cMaximumDrop             # <<<<<<<<<<<<<<
 *                     or self.__shot.alt0 + range_vector.y < _cMinimumAltitude
 *             ):
 */
    __pyx_t_11 = (__pyx_v_range_vector.y < __pyx_v__cMaximumDrop);
    if (!__pyx_t_11) {
    } else {
      __pyx_t_7 = __pyx_t_11;
      goto __pyx_L10_bool_binop_done;
    }

    /* "CyEulerBallistic/trajectory_calc.pyx":412
 *                     velocity < _cMinimumVelocity
 *                     or range_vector.y < _cMaximumDrop
 *                     or self.__shot.alt0 + range_vector.y < _cMinimumAltitude             # <<<<<<<<<<<<<<
 *             ):
 *                 if velocity < _cMinimumVelocity:
 */
    __pyx_t_11 = ((__pyx_v_self->_EulerTrajectoryCalc__shot.alt0 + __pyx_v_range_vector.y) < __pyx_v__cMinimumAltitude);
    __pyx_t_7 = __pyx_t_11;
    __pyx_L10_bool_binop_done:;

    /* "CyEulerBallistic/trajectory_calc.pyx":409
 *             time += delta_time
 * 
 *             if (             # <<<<<<<<<<<<<<
 *                     velocity < _cMinimumVelocity
 *                     or range_vector.y < _cMaximumDrop
 */
    if (__pyx_t_7) {

      /* "CyEulerBallistic/trajectory_calc.pyx":414
 *                     or self.__shot.alt0 + range_vector.y < _cMinimumAltitude
 *             ):
 *                 if velocity < _cMinimumVelocity:             # <<<<<<<<<<<<<<
 *                     reason = RangeError.MinimumVelocityReached
 *                 elif range_vector.y < _cMaximumDrop:
 */
      __pyx_t_7 = (__pyx_v_velocity < __pyx_v__cMinimumVelocity);
      if (__pyx_t_7) {

        /* "CyEulerBallistic/trajectory_calc.pyx":415
 *             ):
 *                 if velocity < _cMinimumVelocity:
 *                     reason = RangeError.MinimumVelocityReached             # <<<<<<<<<<<<<<
 *                 elif range_vector.y < _cMaximumDrop:
 *                     reason = RangeError.MaximumDropReached
 */
        __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_RangeError); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 415, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_MinimumVelocityReached); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 415, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_v_reason = __pyx_t_5;
        __pyx_t_5 = 0;

        /* "CyEulerBallistic/trajectory_calc.pyx":414
 *                     or self.__shot.alt0 + range_vector.y < _cMinimumAltitude
 *             ):
 *                 if velocity < _cMinimumVelocity:             # <<<<<<<<<<<<<<
 *                     reason = RangeError.MinimumVelocityReached
 *                 elif range_vector.y < _cMaximumDrop:
 */
        goto __pyx_L13;
      }

      /* "CyEulerBallistic/trajectory_calc.pyx":416
 *                 if velocity < _cMinimumVelocity:
 *                     reason = RangeError.MinimumVelocityReached
 *                 elif range_vector.y < _cMaximumDrop:             # <<<<<<<<<<<<<<
 *                     reason = RangeError.MaximumDropReached
 *                 else:
 */
      __pyx_t_7 = (__pyx_v_range_vector.y < __pyx_v__cMaximumDrop);
      if (__pyx_t_7) {

        /* "CyEulerBallistic/trajectory_calc.pyx":417
 *                     reason = RangeError.MinimumVelocityReached
 *                 elif range_vector.y < _cMaximumDrop:
 *                     reason = RangeError.MaximumDropReached             # <<<<<<<<<<<<<<
 *                 else:
 *                     reason = RangeError.MinimumAltitudeReached
 */
        __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_RangeError); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 417, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_MaximumDropReached); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 417, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_v_reason = __pyx_t_4;
        __pyx_t_4 = 0;

        /* "CyEulerBallistic/trajectory_calc.pyx":416
 *                 if velocity < _cMinimumVelocity:
 *                     reason = RangeError.MinimumVelocityReached
 *                 elif range_vector.y < _cMaximumDrop:             # <<<<<<<<<<<<<<
 *                     reason = RangeError.MaximumDropReached
 *                 else:
 */
        goto __pyx_L13;
      }

      /* "CyEulerBallistic/trajectory_calc.pyx":419
 *                     reason = RangeError.MaximumDropReached
 *                 else:
 *                     reason = RangeError.MinimumAltitudeReached             # <<<<<<<<<<<<<<
 *                 raise RangeError(reason, ranges)
 *             #endregion
 */
      /*else*/ {
        __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_RangeError); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 419, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_MinimumAltitudeReached); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 419, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_v_reason = __pyx_t_5;
        __pyx_t_5 = 0;
      }
      __pyx_L13:;

      /* "CyEulerBallistic/trajectory_calc.pyx":420
 *                 else:
 *                     reason = RangeError.MinimumAltitudeReached
 *                 raise RangeError(reason, ranges)             # <<<<<<<<<<<<<<
 *             #endregion
 * 
 */
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_RangeError); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 420, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_1 = NULL;
      __pyx_t_6 = 0;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_1)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_1);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
          __pyx_t_6 = 1;
        }
      }
      #endif
      {
        PyObject *__pyx_callargs[3] = {__pyx_t_1, __pyx_v_reason, __pyx_v_ranges};
        __pyx_t_5 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+1-__pyx_t_6, 2+__pyx_t_6);
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 420, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      }
      __Pyx_Raise(__pyx_t_5, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __PYX_ERR(1, 420, __pyx_L1_error)

      /* "CyEulerBallistic/trajectory_calc.pyx":409
 *             time += delta_time
 * 
 *             if (             # <<<<<<<<<<<<<<
 *                     velocity < _cMinimumVelocity
 *                     or range_vector.y < _cMaximumDrop
 */
    }
  }
  __pyx_L4_break:;

  /* "CyEulerBallistic/trajectory_calc.pyx":425
 *         #endregion
 *         # If filter_flags == 0 then all we want is the ending value
 *         if not filter_flags:             # <<<<<<<<<<<<<<
 *             ranges.append(create_trajectory_row(
 *                 time, range_vector, velocity_vector,
 */
  __pyx_t_7 = (!(__pyx_v_filter_flags != 0));
  if (__pyx_t_7) {

    /* "CyEulerBallistic/trajectory_calc.pyx":428
 *             ranges.append(create_trajectory_row(
 *                 time, range_vector, velocity_vector,
 *                 velocity, mach, cy_spin_drift(&self.__shot, time), self.__shot.look_angle,             # <<<<<<<<<<<<<<
 *                 density_factor, drag, self.__shot.weight, CTrajFlag.NONE))
 *         logger.debug(f"euler cy it {it}")
 */
    __pyx_t_10 = __pyx_f_16CyEulerBallistic_8cy_euler_cy_spin_drift((&__pyx_v_self->_EulerTrajectoryCalc__shot), __pyx_v_time); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 428, __pyx_L1_error)

    /* "CyEulerBallistic/trajectory_calc.pyx":426
 *         # If filter_flags == 0 then all we want is the ending value
 *         if not filter_flags:
 *             ranges.append(create_trajectory_row(             # <<<<<<<<<<<<<<
 *                 time, range_vector, velocity_vector,
 *                 velocity, mach, cy_spin_drift(&self.__shot, time), self.__shot.look_angle,
 */
    __pyx_t_5 = __pyx_f_16CyEulerBallistic_15trajectory_calc_create_trajectory_row(__pyx_v_time, __pyx_v_range_vector, __pyx_v_velocity_vector, __pyx_v_velocity, __pyx_v_mach, __pyx_t_10, __pyx_v_self->_EulerTrajectoryCalc__shot.look_angle, __pyx_v_density_factor, __pyx_v_drag, __pyx_v_self->_EulerTrajectoryCalc__shot.weight, __pyx_e_16CyEulerBallistic_15trajectory_data_NONE); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 426, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_8 = __Pyx_PyList_Append(__pyx_v_ranges, __pyx_t_5); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(1, 426, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "CyEulerBallistic/trajectory_calc.pyx":425
 *         #endregion
 *         # If filter_flags == 0 then all we want is the ending value
 *         if not filter_flags:             # <<<<<<<<<<<<<<
 *             ranges.append(create_trajectory_row(
 *                 time, range_vector, velocity_vector,
 */
  }

  /* "CyEulerBallistic/trajectory_calc.pyx":430
 *                 velocity, mach, cy_spin_drift(&self.__shot, time), self.__shot.look_angle,
 *                 density_factor, drag, self.__shot.weight, CTrajFlag.NONE))
 *         logger.debug(f"euler cy it {it}")             # <<<<<<<<<<<<<<
 *         return ranges
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_logger); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 430, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_debug); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 430, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyUnicode_From_int(__pyx_v_it, 0, ' ', 'd'); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 430, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_12 = __Pyx_PyUnicode_Concat(__pyx_kp_u_euler_cy_it, __pyx_t_4); if (unlikely(!__pyx_t_12)) __PYX_ERR(1, 430, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  __pyx_t_6 = 0;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
      __pyx_t_6 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_t_12};
    __pyx_t_5 = __Pyx_PyObject_FastCall(__pyx_t_1, __pyx_callargs+1-__pyx_t_6, 1+__pyx_t_6);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 430, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":431
 *                 density_factor, drag, self.__shot.weight, CTrajFlag.NONE))
 *         logger.debug(f"euler cy it {it}")
 *         return ranges             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_ranges);
  __pyx_r = __pyx_v_ranges;
  goto __pyx_L0;

  /* "CyEulerBallistic/trajectory_calc.pyx":315
 * 
 * 
 *     cdef list[object] _integrate(EulerTrajectoryCalc self,             # <<<<<<<<<<<<<<
 *                           double maximum_range, double step, int filter_flags, double time_step = 0.0):
 *         cdef:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("CyEulerBallistic.trajectory_calc.EulerTrajectoryCalc._integrate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ranges);
  __Pyx_XDECREF((PyObject *)__pyx_v_data_filter);
  __Pyx_XDECREF(__pyx_v_reason);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "CyEulerBallistic/trajectory_calc.pyx":191
 *         list[object] _table_data
 *         CVector gravity_vector
 *         public object _config             # <<<<<<<<<<<<<<
 *         _WindSock ws
 *         Config_t __config
 */

/* Python wrapper */
static PyObject *__pyx_pw_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_7_config_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_7_config_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_7_config___get__(((struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_7_config___get__(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 1);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_config);
  __pyx_r = __pyx_v_self->_config;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_7_config_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_7_config_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_7_config_2__set__(((struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_7_config_2__set__(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 1);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->_config);
  __Pyx_DECREF(__pyx_v_self->_config);
  __pyx_v_self->_config = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_7_config_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_7_config_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_7_config_4__del__(((struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_7_config_4__del__(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_config);
  __Pyx_DECREF(__pyx_v_self->_config);
  __pyx_v_self->_config = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_7__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_6__reduce_cython__, "EulerTrajectoryCalc.__reduce_cython__(self)");
static PyObject *__pyx_pw_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_7__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) {
    __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL;}
  if (unlikely(__pyx_kwds) && __Pyx_NumKwargs_FASTCALL(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__reduce_cython__", 0))) return NULL;
  __pyx_r = __pyx_pf_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_6__reduce_cython__(((struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 1);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 */
  __Pyx_Raise(__pyx_builtin_TypeError, __pyx_kp_s_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(0, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("CyEulerBallistic.trajectory_calc.EulerTrajectoryCalc.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 */

/* Python wrapper */
static PyObject *__pyx_pw_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_9__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_8__setstate_cython__, "EulerTrajectoryCalc.__setstate_cython__(self, __pyx_state)");
static PyObject *__pyx_pw_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_9__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_state,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_pyx_state)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 3, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "__setstate_cython__") < 0)) __PYX_ERR(0, 3, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("CyEulerBallistic.trajectory_calc.EulerTrajectoryCalc.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_8__setstate_cython__(((struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 1);

  /* "(tree fragment)":4
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
 */
  __Pyx_Raise(__pyx_builtin_TypeError, __pyx_kp_s_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(0, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("CyEulerBallistic.trajectory_calc.EulerTrajectoryCalc.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "CyEulerBallistic/trajectory_calc.pyx":434
 * 
 * 
 * cdef object create_trajectory_row(double time, CVector range_vector, CVector velocity_vector,             # <<<<<<<<<<<<<<
 *                            double velocity, double mach, double spin_drift, double look_angle,
 *                            double density_factor, double drag, double weight, int flag):
 */

static PyObject *__pyx_f_16CyEulerBallistic_15trajectory_calc_create_trajectory_row(double __pyx_v_time, struct __pyx_t_16CyEulerBallistic_6vector_CVector __pyx_v_range_vector, struct __pyx_t_16CyEulerBallistic_6vector_CVector __pyx_v_velocity_vector, double __pyx_v_velocity, double __pyx_v_mach, double __pyx_v_spin_drift, double __pyx_v_look_angle, double __pyx_v_density_factor, double __pyx_v_drag, double __pyx_v_weight, int __pyx_v_flag) {
  double __pyx_v_windage;
  double __pyx_v_drop_adjustment;
  double __pyx_v_windage_adjustment;
  double __pyx_v_trajectory_angle;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("create_trajectory_row", 1);

  /* "CyEulerBallistic/trajectory_calc.pyx":439
 * 
 *     cdef:
 *         double windage = range_vector.z + spin_drift             # <<<<<<<<<<<<<<
 *         double drop_adjustment = cy_get_correction(range_vector.x, range_vector.y)
 *         double windage_adjustment = cy_get_correction(range_vector.x, windage)
 */
  __pyx_v_windage = (__pyx_v_range_vector.z + __pyx_v_spin_drift);

  /* "CyEulerBallistic/trajectory_calc.pyx":440
 *     cdef:
 *         double windage = range_vector.z + spin_drift
 *         double drop_adjustment = cy_get_correction(range_vector.x, range_vector.y)             # <<<<<<<<<<<<<<
 *         double windage_adjustment = cy_get_correction(range_vector.x, windage)
 *         double trajectory_angle = atan2(velocity_vector.y, velocity_vector.x);
 */
  __pyx_t_1 = __pyx_f_16CyEulerBallistic_8cy_euler_cy_get_correction(__pyx_v_range_vector.x, __pyx_v_range_vector.y); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 440, __pyx_L1_error)
  __pyx_v_drop_adjustment = __pyx_t_1;

  /* "CyEulerBallistic/trajectory_calc.pyx":441
 *         double windage = range_vector.z + spin_drift
 *         double drop_adjustment = cy_get_correction(range_vector.x, range_vector.y)
 *         double windage_adjustment = cy_get_correction(range_vector.x, windage)             # <<<<<<<<<<<<<<
 *         double trajectory_angle = atan2(velocity_vector.y, velocity_vector.x);
 * 
 */
  __pyx_t_1 = __pyx_f_16CyEulerBallistic_8cy_euler_cy_get_correction(__pyx_v_range_vector.x, __pyx_v_windage); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 441, __pyx_L1_error)
  __pyx_v_windage_adjustment = __pyx_t_1;

  /* "CyEulerBallistic/trajectory_calc.pyx":442
 *         double drop_adjustment = cy_get_correction(range_vector.x, range_vector.y)
 *         double windage_adjustment = cy_get_correction(range_vector.x, windage)
 *         double trajectory_angle = atan2(velocity_vector.y, velocity_vector.x);             # <<<<<<<<<<<<<<
 * 
 *     return TrajectoryData(
 */
  __pyx_v_trajectory_angle = atan2(__pyx_v_velocity_vector.y, __pyx_v_velocity_vector.x);

  /* "CyEulerBallistic/trajectory_calc.pyx":444
 *         double trajectory_angle = atan2(velocity_vector.y, velocity_vector.x);
 * 
 *     return TrajectoryData(             # <<<<<<<<<<<<<<
 *         time=time,
 *         distance=_new_feet(range_vector.x),
 */
  __Pyx_XDECREF(__pyx_r);

  /* "CyEulerBallistic/trajectory_calc.pyx":445
 * 
 *     return TrajectoryData(
 *         time=time,             # <<<<<<<<<<<<<<
 *         distance=_new_feet(range_vector.x),
 *         velocity=_new_fps(velocity),
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(16); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 445, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_time); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 445, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_time, __pyx_t_3) < 0) __PYX_ERR(1, 445, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":446
 *     return TrajectoryData(
 *         time=time,
 *         distance=_new_feet(range_vector.x),             # <<<<<<<<<<<<<<
 *         velocity=_new_fps(velocity),
 *         mach=velocity / mach,
 */
  __pyx_t_3 = __pyx_f_16CyEulerBallistic_15trajectory_calc__new_feet(__pyx_v_range_vector.x); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 446, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_distance, __pyx_t_3) < 0) __PYX_ERR(1, 445, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":447
 *         time=time,
 *         distance=_new_feet(range_vector.x),
 *         velocity=_new_fps(velocity),             # <<<<<<<<<<<<<<
 *         mach=velocity / mach,
 *         height=_new_feet(range_vector.y),
 */
  __pyx_t_3 = __pyx_f_16CyEulerBallistic_15trajectory_calc__new_fps(__pyx_v_velocity); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 447, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_velocity, __pyx_t_3) < 0) __PYX_ERR(1, 445, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":448
 *         distance=_new_feet(range_vector.x),
 *         velocity=_new_fps(velocity),
 *         mach=velocity / mach,             # <<<<<<<<<<<<<<
 *         height=_new_feet(range_vector.y),
 *         target_drop=_new_feet(
 */
  __pyx_t_3 = PyFloat_FromDouble((__pyx_v_velocity / __pyx_v_mach)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 448, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_mach, __pyx_t_3) < 0) __PYX_ERR(1, 445, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":449
 *         velocity=_new_fps(velocity),
 *         mach=velocity / mach,
 *         height=_new_feet(range_vector.y),             # <<<<<<<<<<<<<<
 *         target_drop=_new_feet(
 *             (range_vector.y - range_vector.x * tan(look_angle)) * cos(look_angle)
 */
  __pyx_t_3 = __pyx_f_16CyEulerBallistic_15trajectory_calc__new_feet(__pyx_v_range_vector.y); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 449, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_height, __pyx_t_3) < 0) __PYX_ERR(1, 445, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":450
 *         mach=velocity / mach,
 *         height=_new_feet(range_vector.y),
 *         target_drop=_new_feet(             # <<<<<<<<<<<<<<
 *             (range_vector.y - range_vector.x * tan(look_angle)) * cos(look_angle)
 *         ),
 */
  __pyx_t_3 = __pyx_f_16CyEulerBallistic_15trajectory_calc__new_feet(((__pyx_v_range_vector.y - (__pyx_v_range_vector.x * tan(__pyx_v_look_angle))) * cos(__pyx_v_look_angle))); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 450, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_target_drop, __pyx_t_3) < 0) __PYX_ERR(1, 445, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":453
 *             (range_vector.y - range_vector.x * tan(look_angle)) * cos(look_angle)
 *         ),
 *         drop_adj=_new_rad(drop_adjustment - (look_angle if range_vector.x else 0)),             # <<<<<<<<<<<<<<
 *         windage=_new_feet(windage),
 *         windage_adj=_new_rad(windage_adjustment),
 */
  __pyx_t_4 = (__pyx_v_range_vector.x != 0);
  if (__pyx_t_4) {
    __pyx_t_1 = __pyx_v_look_angle;
  } else {
    __pyx_t_1 = 0.0;
  }
  __pyx_t_3 = __pyx_f_16CyEulerBallistic_15trajectory_calc__new_rad((__pyx_v_drop_adjustment - __pyx_t_1)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 453, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_drop_adj, __pyx_t_3) < 0) __PYX_ERR(1, 445, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":454
 *         ),
 *         drop_adj=_new_rad(drop_adjustment - (look_angle if range_vector.x else 0)),
 *         windage=_new_feet(windage),             # <<<<<<<<<<<<<<
 *         windage_adj=_new_rad(windage_adjustment),
 *         look_distance=_new_feet(range_vector.x / cos(look_angle)),
 */
  __pyx_t_3 = __pyx_f_16CyEulerBallistic_15trajectory_calc__new_feet(__pyx_v_windage); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 454, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_windage, __pyx_t_3) < 0) __PYX_ERR(1, 445, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":455
 *         drop_adj=_new_rad(drop_adjustment - (look_angle if range_vector.x else 0)),
 *         windage=_new_feet(windage),
 *         windage_adj=_new_rad(windage_adjustment),             # <<<<<<<<<<<<<<
 *         look_distance=_new_feet(range_vector.x / cos(look_angle)),
 *         angle=_new_rad(trajectory_angle),
 */
  __pyx_t_3 = __pyx_f_16CyEulerBallistic_15trajectory_calc__new_rad(__pyx_v_windage_adjustment); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 455, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_windage_adj, __pyx_t_3) < 0) __PYX_ERR(1, 445, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":456
 *         windage=_new_feet(windage),
 *         windage_adj=_new_rad(windage_adjustment),
 *         look_distance=_new_feet(range_vector.x / cos(look_angle)),             # <<<<<<<<<<<<<<
 *         angle=_new_rad(trajectory_angle),
 *         density_factor=density_factor - 1,
 */
  __pyx_t_3 = __pyx_f_16CyEulerBallistic_15trajectory_calc__new_feet((__pyx_v_range_vector.x / cos(__pyx_v_look_angle))); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 456, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_look_distance, __pyx_t_3) < 0) __PYX_ERR(1, 445, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":457
 *         windage_adj=_new_rad(windage_adjustment),
 *         look_distance=_new_feet(range_vector.x / cos(look_angle)),
 *         angle=_new_rad(trajectory_angle),             # <<<<<<<<<<<<<<
 *         density_factor=density_factor - 1,
 *         drag=drag,
 */
  __pyx_t_3 = __pyx_f_16CyEulerBallistic_15trajectory_calc__new_rad(__pyx_v_trajectory_angle); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 457, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_angle, __pyx_t_3) < 0) __PYX_ERR(1, 445, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":458
 *         look_distance=_new_feet(range_vector.x / cos(look_angle)),
 *         angle=_new_rad(trajectory_angle),
 *         density_factor=density_factor - 1,             # <<<<<<<<<<<<<<
 *         drag=drag,
 *         energy=_new_ft_lb(cy_calculate_energy(weight, velocity)),
 */
  __pyx_t_3 = PyFloat_FromDouble((__pyx_v_density_factor - 1.0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 458, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_density_factor, __pyx_t_3) < 0) __PYX_ERR(1, 445, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":459
 *         angle=_new_rad(trajectory_angle),
 *         density_factor=density_factor - 1,
 *         drag=drag,             # <<<<<<<<<<<<<<
 *         energy=_new_ft_lb(cy_calculate_energy(weight, velocity)),
 *         ogw=_new_lb(cy_calculate_ogw(weight, velocity)),
 */
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_drag); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 459, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_drag, __pyx_t_3) < 0) __PYX_ERR(1, 445, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":460
 *         density_factor=density_factor - 1,
 *         drag=drag,
 *         energy=_new_ft_lb(cy_calculate_energy(weight, velocity)),             # <<<<<<<<<<<<<<
 *         ogw=_new_lb(cy_calculate_ogw(weight, velocity)),
 *         flag=flag
 */
  __pyx_t_1 = __pyx_f_16CyEulerBallistic_8cy_euler_cy_calculate_energy(__pyx_v_weight, __pyx_v_velocity); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 460, __pyx_L1_error)
  __pyx_t_3 = __pyx_f_16CyEulerBallistic_15trajectory_calc__new_ft_lb(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 460, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_energy, __pyx_t_3) < 0) __PYX_ERR(1, 445, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":461
 *         drag=drag,
 *         energy=_new_ft_lb(cy_calculate_energy(weight, velocity)),
 *         ogw=_new_lb(cy_calculate_ogw(weight, velocity)),             # <<<<<<<<<<<<<<
 *         flag=flag
 *     )
 */
  __pyx_t_1 = __pyx_f_16CyEulerBallistic_8cy_euler_cy_calculate_ogw(__pyx_v_weight, __pyx_v_velocity); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 461, __pyx_L1_error)
  __pyx_t_3 = __pyx_f_16CyEulerBallistic_15trajectory_calc__new_lb(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 461, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_ogw, __pyx_t_3) < 0) __PYX_ERR(1, 445, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":462
 *         energy=_new_ft_lb(cy_calculate_energy(weight, velocity)),
 *         ogw=_new_lb(cy_calculate_ogw(weight, velocity)),
 *         flag=flag             # <<<<<<<<<<<<<<
 *     )
 * 
 */
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_flag); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 462, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_flag, __pyx_t_3) < 0) __PYX_ERR(1, 445, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":444
 *         double trajectory_angle = atan2(velocity_vector.y, velocity_vector.x);
 * 
 *     return TrajectoryData(             # <<<<<<<<<<<<<<
 *         time=time,
 *         distance=_new_feet(range_vector.x),
 */
  __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_16CyEulerBallistic_15trajectory_data_TrajectoryData), __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 444, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "CyEulerBallistic/trajectory_calc.pyx":434
 * 
 * 
 * cdef object create_trajectory_row(double time, CVector range_vector, CVector velocity_vector,             # <<<<<<<<<<<<<<
 *                            double velocity, double mach, double spin_drift, double look_angle,
 *                            double density_factor, double drag, double weight, int flag):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("CyEulerBallistic.trajectory_calc.create_trajectory_row", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "CyEulerBallistic/trajectory_calc.pyx":466
 * 
 * 
 * cdef object _new_feet(double v):             # <<<<<<<<<<<<<<
 *     d = object.__new__(Distance)
 *     d._value = v * 12
 */

static PyObject *__pyx_f_16CyEulerBallistic_15trajectory_calc__new_feet(double __pyx_v_v) {
  PyObject *__pyx_v_d = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  unsigned int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_new_feet", 1);

  /* "CyEulerBallistic/trajectory_calc.pyx":467
 * 
 * cdef object _new_feet(double v):
 *     d = object.__new__(Distance)             # <<<<<<<<<<<<<<
 *     d._value = v * 12
 *     d._defined_units = Unit.Foot
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_builtin_object, __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 467, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_Distance); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 467, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 467, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __pyx_v_d = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":468
 * cdef object _new_feet(double v):
 *     d = object.__new__(Distance)
 *     d._value = v * 12             # <<<<<<<<<<<<<<
 *     d._defined_units = Unit.Foot
 *     return d
 */
  __pyx_t_1 = PyFloat_FromDouble((__pyx_v_v * 12.0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 468, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_d, __pyx_n_s_value, __pyx_t_1) < 0) __PYX_ERR(1, 468, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":469
 *     d = object.__new__(Distance)
 *     d._value = v * 12
 *     d._defined_units = Unit.Foot             # <<<<<<<<<<<<<<
 *     return d
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_Unit); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 469, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Foot); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 469, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_d, __pyx_n_s_defined_units, __pyx_t_2) < 0) __PYX_ERR(1, 469, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":470
 *     d._value = v * 12
 *     d._defined_units = Unit.Foot
 *     return d             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_d);
  __pyx_r = __pyx_v_d;
  goto __pyx_L0;

  /* "CyEulerBallistic/trajectory_calc.pyx":466
 * 
 * 
 * cdef object _new_feet(double v):             # <<<<<<<<<<<<<<
 *     d = object.__new__(Distance)
 *     d._value = v * 12
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("CyEulerBallistic.trajectory_calc._new_feet", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_d);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "CyEulerBallistic/trajectory_calc.pyx":473
 * 
 * 
 * cdef object _new_fps(double v):             # <<<<<<<<<<<<<<
 *     d = object.__new__(Velocity)
 *     d._value = v / 3.2808399
 */

static PyObject *__pyx_f_16CyEulerBallistic_15trajectory_calc__new_fps(double __pyx_v_v) {
  PyObject *__pyx_v_d = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  unsigned int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_new_fps", 1);

  /* "CyEulerBallistic/trajectory_calc.pyx":474
 * 
 * cdef object _new_fps(double v):
 *     d = object.__new__(Velocity)             # <<<<<<<<<<<<<<
 *     d._value = v / 3.2808399
 *     d._defined_units = Unit.FPS
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_builtin_object, __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 474, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_Velocity); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 474, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 474, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __pyx_v_d = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":475
 * cdef object _new_fps(double v):
 *     d = object.__new__(Velocity)
 *     d._value = v / 3.2808399             # <<<<<<<<<<<<<<
 *     d._defined_units = Unit.FPS
 *     return d
 */
  __pyx_t_1 = PyFloat_FromDouble((__pyx_v_v / 3.2808399)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 475, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_d, __pyx_n_s_value, __pyx_t_1) < 0) __PYX_ERR(1, 475, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":476
 *     d = object.__new__(Velocity)
 *     d._value = v / 3.2808399
 *     d._defined_units = Unit.FPS             # <<<<<<<<<<<<<<
 *     return d
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_Unit); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 476, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_FPS); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 476, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_d, __pyx_n_s_defined_units, __pyx_t_2) < 0) __PYX_ERR(1, 476, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":477
 *     d._value = v / 3.2808399
 *     d._defined_units = Unit.FPS
 *     return d             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_d);
  __pyx_r = __pyx_v_d;
  goto __pyx_L0;

  /* "CyEulerBallistic/trajectory_calc.pyx":473
 * 
 * 
 * cdef object _new_fps(double v):             # <<<<<<<<<<<<<<
 *     d = object.__new__(Velocity)
 *     d._value = v / 3.2808399
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("CyEulerBallistic.trajectory_calc._new_fps", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_d);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "CyEulerBallistic/trajectory_calc.pyx":480
 * 
 * 
 * cdef object _new_rad(double v):             # <<<<<<<<<<<<<<
 *     d = object.__new__(Angular)
 *     d._value = v
 */

static PyObject *__pyx_f_16CyEulerBallistic_15trajectory_calc__new_rad(double __pyx_v_v) {
  PyObject *__pyx_v_d = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  unsigned int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_new_rad", 1);

  /* "CyEulerBallistic/trajectory_calc.pyx":481
 * 
 * cdef object _new_rad(double v):
 *     d = object.__new__(Angular)             # <<<<<<<<<<<<<<
 *     d._value = v
 *     d._defined_units = Unit.Radian
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_builtin_object, __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 481, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_Angular); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 481, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 481, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __pyx_v_d = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":482
 * cdef object _new_rad(double v):
 *     d = object.__new__(Angular)
 *     d._value = v             # <<<<<<<<<<<<<<
 *     d._defined_units = Unit.Radian
 *     return d
 */
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_v); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 482, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_d, __pyx_n_s_value, __pyx_t_1) < 0) __PYX_ERR(1, 482, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":483
 *     d = object.__new__(Angular)
 *     d._value = v
 *     d._defined_units = Unit.Radian             # <<<<<<<<<<<<<<
 *     return d
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_Unit); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 483, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Radian); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 483, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_d, __pyx_n_s_defined_units, __pyx_t_2) < 0) __PYX_ERR(1, 483, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":484
 *     d._value = v
 *     d._defined_units = Unit.Radian
 *     return d             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_d);
  __pyx_r = __pyx_v_d;
  goto __pyx_L0;

  /* "CyEulerBallistic/trajectory_calc.pyx":480
 * 
 * 
 * cdef object _new_rad(double v):             # <<<<<<<<<<<<<<
 *     d = object.__new__(Angular)
 *     d._value = v
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("CyEulerBallistic.trajectory_calc._new_rad", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_d);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "CyEulerBallistic/trajectory_calc.pyx":487
 * 
 * 
 * cdef object _new_ft_lb(double v):             # <<<<<<<<<<<<<<
 *     d = object.__new__(Energy)
 *     d._value = v
 */

static PyObject *__pyx_f_16CyEulerBallistic_15trajectory_calc__new_ft_lb(double __pyx_v_v) {
  PyObject *__pyx_v_d = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  unsigned int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_new_ft_lb", 1);

  /* "CyEulerBallistic/trajectory_calc.pyx":488
 * 
 * cdef object _new_ft_lb(double v):
 *     d = object.__new__(Energy)             # <<<<<<<<<<<<<<
 *     d._value = v
 *     d._defined_units = Unit.FootPound
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_builtin_object, __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 488, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_Energy); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 488, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 488, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __pyx_v_d = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":489
 * cdef object _new_ft_lb(double v):
 *     d = object.__new__(Energy)
 *     d._value = v             # <<<<<<<<<<<<<<
 *     d._defined_units = Unit.FootPound
 *     return d
 */
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_v); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 489, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_d, __pyx_n_s_value, __pyx_t_1) < 0) __PYX_ERR(1, 489, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":490
 *     d = object.__new__(Energy)
 *     d._value = v
 *     d._defined_units = Unit.FootPound             # <<<<<<<<<<<<<<
 *     return d
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_Unit); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 490, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_FootPound); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 490, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_d, __pyx_n_s_defined_units, __pyx_t_2) < 0) __PYX_ERR(1, 490, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":491
 *     d._value = v
 *     d._defined_units = Unit.FootPound
 *     return d             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_d);
  __pyx_r = __pyx_v_d;
  goto __pyx_L0;

  /* "CyEulerBallistic/trajectory_calc.pyx":487
 * 
 * 
 * cdef object _new_ft_lb(double v):             # <<<<<<<<<<<<<<
 *     d = object.__new__(Energy)
 *     d._value = v
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("CyEulerBallistic.trajectory_calc._new_ft_lb", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_d);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "CyEulerBallistic/trajectory_calc.pyx":494
 * 
 * 
 * cdef object _new_lb(double v):             # <<<<<<<<<<<<<<
 *     d = object.__new__(Weight)
 *     d._value = v / 0.000142857143
 */

static PyObject *__pyx_f_16CyEulerBallistic_15trajectory_calc__new_lb(double __pyx_v_v) {
  PyObject *__pyx_v_d = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  unsigned int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_new_lb", 1);

  /* "CyEulerBallistic/trajectory_calc.pyx":495
 * 
 * cdef object _new_lb(double v):
 *     d = object.__new__(Weight)             # <<<<<<<<<<<<<<
 *     d._value = v / 0.000142857143
 *     d._defined_units = Unit.Pound
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_builtin_object, __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 495, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_Weight); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 495, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 495, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __pyx_v_d = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":496
 * cdef object _new_lb(double v):
 *     d = object.__new__(Weight)
 *     d._value = v / 0.000142857143             # <<<<<<<<<<<<<<
 *     d._defined_units = Unit.Pound
 *     return d
 */
  __pyx_t_1 = PyFloat_FromDouble((__pyx_v_v / 0.000142857143)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 496, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_d, __pyx_n_s_value, __pyx_t_1) < 0) __PYX_ERR(1, 496, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":497
 *     d = object.__new__(Weight)
 *     d._value = v / 0.000142857143
 *     d._defined_units = Unit.Pound             # <<<<<<<<<<<<<<
 *     return d
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_Unit); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Pound); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_d, __pyx_n_s_defined_units, __pyx_t_2) < 0) __PYX_ERR(1, 497, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":498
 *     d._value = v / 0.000142857143
 *     d._defined_units = Unit.Pound
 *     return d             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_d);
  __pyx_r = __pyx_v_d;
  goto __pyx_L0;

  /* "CyEulerBallistic/trajectory_calc.pyx":494
 * 
 * 
 * cdef object _new_lb(double v):             # <<<<<<<<<<<<<<
 *     d = object.__new__(Weight)
 *     d._value = v / 0.000142857143
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("CyEulerBallistic.trajectory_calc._new_lb", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_d);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static struct __pyx_vtabstruct_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter __pyx_vtable_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter;

static PyObject *__pyx_tp_new_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *p;
  PyObject *o;
  #if CYTHON_COMPILING_IN_LIMITED_API
  allocfunc alloc_func = (allocfunc)PyType_GetSlot(t, Py_tp_alloc);
  o = alloc_func(t, 0);
  #else
  o = (*t->tp_alloc)(t, 0);
  if (unlikely(!o)) return 0;
  #endif
  p = ((struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *)o);
  p->__pyx_vtab = __pyx_vtabptr_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter;
  if (unlikely(__pyx_pw_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter(PyObject *o) {
  #if CYTHON_USE_TYPE_SLOTS || CYTHON_COMPILING_IN_PYPY
  (*Py_TYPE(o)->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(Py_TYPE(o), Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
}

static PyMethodDef __pyx_methods_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter[] = {
  {"__reduce_cython__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_3__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_2__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_5__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_4__setstate_cython__},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter},
  {Py_tp_methods, (void *)__pyx_methods_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter},
  {Py_tp_new, (void *)__pyx_tp_new_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter},
  {0, 0},
};
static PyType_Spec __pyx_type_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter_spec = {
  "CyEulerBallistic.trajectory_calc._TrajectoryDataFilter",
  sizeof(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER,
  __pyx_type_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter_slots,
};
#else

static PyTypeObject __pyx_type_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter = {
  PyVarObject_HEAD_INIT(0, 0)
  "CyEulerBallistic.trajectory_calc.""_TrajectoryDataFilter", /*tp_name*/
  sizeof(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_16CyEulerBallistic_15trajectory_calc__WindSock __pyx_vtable_16CyEulerBallistic_15trajectory_calc__WindSock;

static PyObject *__pyx_tp_new_16CyEulerBallistic_15trajectory_calc__WindSock(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock *p;
  PyObject *o;
  #if CYTHON_COMPILING_IN_LIMITED_API
  allocfunc alloc_func = (allocfunc)PyType_GetSlot(t, Py_tp_alloc);
  o = alloc_func(t, 0);
  #else
  o = (*t->tp_alloc)(t, 0);
  if (unlikely(!o)) return 0;
  #endif
  p = ((struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock *)o);
  p->__pyx_vtab = __pyx_vtabptr_16CyEulerBallistic_15trajectory_calc__WindSock;
  p->winds = ((PyObject*)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_16CyEulerBallistic_15trajectory_calc_9_WindSock_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_16CyEulerBallistic_15trajectory_calc__WindSock(PyObject *o) {
  struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock *p = (struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->winds);
  #if CYTHON_USE_TYPE_SLOTS || CYTHON_COMPILING_IN_PYPY
  (*Py_TYPE(o)->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(Py_TYPE(o), Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
}

static int __pyx_tp_traverse_16CyEulerBallistic_15trajectory_calc__WindSock(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock *p = (struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock *)o;
  if (p->winds) {
    e = (*v)(p->winds, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_16CyEulerBallistic_15trajectory_calc__WindSock(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock *p = (struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock *)o;
  tmp = ((PyObject*)p->winds);
  p->winds = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_16CyEulerBallistic_15trajectory_calc__WindSock[] = {
  {"__reduce_cython__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_16CyEulerBallistic_15trajectory_calc_9_WindSock_3__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_16CyEulerBallistic_15trajectory_calc_9_WindSock_2__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_16CyEulerBallistic_15trajectory_calc_9_WindSock_5__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_16CyEulerBallistic_15trajectory_calc_9_WindSock_4__setstate_cython__},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_16CyEulerBallistic_15trajectory_calc__WindSock_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_16CyEulerBallistic_15trajectory_calc__WindSock},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_16CyEulerBallistic_15trajectory_calc__WindSock},
  {Py_tp_clear, (void *)__pyx_tp_clear_16CyEulerBallistic_15trajectory_calc__WindSock},
  {Py_tp_methods, (void *)__pyx_methods_16CyEulerBallistic_15trajectory_calc__WindSock},
  {Py_tp_new, (void *)__pyx_tp_new_16CyEulerBallistic_15trajectory_calc__WindSock},
  {0, 0},
};
static PyType_Spec __pyx_type_16CyEulerBallistic_15trajectory_calc__WindSock_spec = {
  "CyEulerBallistic.trajectory_calc._WindSock",
  sizeof(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC,
  __pyx_type_16CyEulerBallistic_15trajectory_calc__WindSock_slots,
};
#else

static PyTypeObject __pyx_type_16CyEulerBallistic_15trajectory_calc__WindSock = {
  PyVarObject_HEAD_INIT(0, 0)
  "CyEulerBallistic.trajectory_calc.""_WindSock", /*tp_name*/
  sizeof(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_16CyEulerBallistic_15trajectory_calc__WindSock, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_16CyEulerBallistic_15trajectory_calc__WindSock, /*tp_traverse*/
  __pyx_tp_clear_16CyEulerBallistic_15trajectory_calc__WindSock, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_16CyEulerBallistic_15trajectory_calc__WindSock, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_16CyEulerBallistic_15trajectory_calc__WindSock, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc __pyx_vtable_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc;

static PyObject *__pyx_tp_new_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *p;
  PyObject *o;
  #if CYTHON_COMPILING_IN_LIMITED_API
  allocfunc alloc_func = (allocfunc)PyType_GetSlot(t, Py_tp_alloc);
  o = alloc_func(t, 0);
  #else
  if (likely(!__Pyx_PyType_HasFeature(t, Py_TPFLAGS_IS_ABSTRACT))) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  #endif
  p = ((struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *)o);
  p->__pyx_vtab = __pyx_vtabptr_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc;
  p->_table_data = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_config = Py_None; Py_INCREF(Py_None);
  p->ws = ((struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock *)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc(PyObject *o) {
  struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *p = (struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely((PY_VERSION_HEX >= 0x03080000 || __Pyx_PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE)) && __Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_table_data);
  Py_CLEAR(p->_config);
  Py_CLEAR(p->ws);
  #if CYTHON_USE_TYPE_SLOTS || CYTHON_COMPILING_IN_PYPY
  (*Py_TYPE(o)->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(Py_TYPE(o), Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
}

static int __pyx_tp_traverse_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *p = (struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *)o;
  if (p->_table_data) {
    e = (*v)(p->_table_data, a); if (e) return e;
  }
  if (p->_config) {
    e = (*v)(p->_config, a); if (e) return e;
  }
  if (p->ws) {
    e = (*v)(((PyObject *)p->ws), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *p = (struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *)o;
  tmp = ((PyObject*)p->_table_data);
  p->_table_data = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_config);
  p->_config = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->ws);
  p->ws = ((struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_table_data(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_10table_data_1__get__(o);
}

static PyObject *__pyx_getprop_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc__config(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_7_config_1__get__(o);
}

static int __pyx_setprop_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc__config(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_7_config_3__set__(o, v);
  }
  else {
    return __pyx_pw_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_7_config_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc[] = {
  {"zero_angle", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_3zero_angle, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_2zero_angle},
  {"trajectory", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_5trajectory, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_4trajectory},
  {"__reduce_cython__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_7__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_6__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_9__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_8__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc[] = {
  {(char *)"table_data", __pyx_getprop_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_table_data, 0, (char *)PyDoc_STR("EulerTrajectoryCalc.table_data: list[object]"), 0},
  {(char *)"_config", __pyx_getprop_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc__config, __pyx_setprop_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc__config, (char *)PyDoc_STR("_config: object"), 0},
  {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc},
  {Py_tp_clear, (void *)__pyx_tp_clear_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc},
  {Py_tp_methods, (void *)__pyx_methods_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc},
  {Py_tp_getset, (void *)__pyx_getsets_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc},
  {Py_tp_new, (void *)__pyx_tp_new_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc},
  {0, 0},
};
static PyType_Spec __pyx_type_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc_spec = {
  "CyEulerBallistic.trajectory_calc.EulerTrajectoryCalc",
  sizeof(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc_slots,
};
#else

static PyTypeObject __pyx_type_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc = {
  PyVarObject_HEAD_INIT(0, 0)
  "CyEulerBallistic.trajectory_calc.""EulerTrajectoryCalc", /*tp_name*/
  sizeof(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc, /*tp_traverse*/
  __pyx_tp_clear_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif
/* #### Code section: pystring_table ### */

static int __Pyx_CreateStringTabAndInitStrings(void) {
  __Pyx_StringTabEntry __pyx_string_tab[] = {
    {&__pyx_n_s_Angular, __pyx_k_Angular, sizeof(__pyx_k_Angular), 0, 0, 1, 1},
    {&__pyx_n_s_BC, __pyx_k_BC, sizeof(__pyx_k_BC), 0, 0, 1, 1},
    {&__pyx_n_s_Distance, __pyx_k_Distance, sizeof(__pyx_k_Distance), 0, 0, 1, 1},
    {&__pyx_n_s_Energy, __pyx_k_Energy, sizeof(__pyx_k_Energy), 0, 0, 1, 1},
    {&__pyx_n_s_EulerTrajectoryCalc, __pyx_k_EulerTrajectoryCalc, sizeof(__pyx_k_EulerTrajectoryCalc), 0, 0, 1, 1},
    {&__pyx_n_u_EulerTrajectoryCalc, __pyx_k_EulerTrajectoryCalc, sizeof(__pyx_k_EulerTrajectoryCalc), 0, 1, 0, 1},
    {&__pyx_n_s_FPS, __pyx_k_FPS, sizeof(__pyx_k_FPS), 0, 0, 1, 1},
    {&__pyx_n_s_Foot, __pyx_k_Foot, sizeof(__pyx_k_Foot), 0, 0, 1, 1},
    {&__pyx_n_s_FootPound, __pyx_k_FootPound, sizeof(__pyx_k_FootPound), 0, 0, 1, 1},
    {&__pyx_n_s_InHg, __pyx_k_InHg, sizeof(__pyx_k_InHg), 0, 0, 1, 1},
    {&__pyx_n_s_KeyError, __pyx_k_KeyError, sizeof(__pyx_k_KeyError), 0, 0, 1, 1},
    {&__pyx_n_s_MAX_DISTANCE_FEET, __pyx_k_MAX_DISTANCE_FEET, sizeof(__pyx_k_MAX_DISTANCE_FEET), 0, 0, 1, 1},
    {&__pyx_n_s_MaximumDropReached, __pyx_k_MaximumDropReached, sizeof(__pyx_k_MaximumDropReached), 0, 0, 1, 1},
    {&__pyx_n_s_MinimumAltitudeReached, __pyx_k_MinimumAltitudeReached, sizeof(__pyx_k_MinimumAltitudeReached), 0, 0, 1, 1},
    {&__pyx_n_s_MinimumVelocityReached, __pyx_k_MinimumVelocityReached, sizeof(__pyx_k_MinimumVelocityReached), 0, 0, 1, 1},
    {&__pyx_kp_s_No_value_specified_for_struct_at, __pyx_k_No_value_specified_for_struct_at, sizeof(__pyx_k_No_value_specified_for_struct_at), 0, 0, 1, 0},
    {&__pyx_kp_s_No_value_specified_for_struct_at_2, __pyx_k_No_value_specified_for_struct_at_2, sizeof(__pyx_k_No_value_specified_for_struct_at_2), 0, 0, 1, 0},
    {&__pyx_kp_s_No_value_specified_for_struct_at_3, __pyx_k_No_value_specified_for_struct_at_3, sizeof(__pyx_k_No_value_specified_for_struct_at_3), 0, 0, 1, 0},
    {&__pyx_kp_s_No_value_specified_for_struct_at_4, __pyx_k_No_value_specified_for_struct_at_4, sizeof(__pyx_k_No_value_specified_for_struct_at_4), 0, 0, 1, 0},
    {&__pyx_n_s_Pound, __pyx_k_Pound, sizeof(__pyx_k_Pound), 0, 0, 1, 1},
    {&__pyx_n_s_Radian, __pyx_k_Radian, sizeof(__pyx_k_Radian), 0, 0, 1, 1},
    {&__pyx_n_s_RangeError, __pyx_k_RangeError, sizeof(__pyx_k_RangeError), 0, 0, 1, 1},
    {&__pyx_n_s_TrajectoryDataFilter, __pyx_k_TrajectoryDataFilter, sizeof(__pyx_k_TrajectoryDataFilter), 0, 0, 1, 1},
    {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
    {&__pyx_n_s_Unit, __pyx_k_Unit, sizeof(__pyx_k_Unit), 0, 0, 1, 1},
    {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
    {&__pyx_n_s_Velocity, __pyx_k_Velocity, sizeof(__pyx_k_Velocity), 0, 0, 1, 1},
    {&__pyx_n_s_Weight, __pyx_k_Weight, sizeof(__pyx_k_Weight), 0, 0, 1, 1},
    {&__pyx_n_s_WindSock, __pyx_k_WindSock, sizeof(__pyx_k_WindSock), 0, 0, 1, 1},
    {&__pyx_n_s_ZeroFindingError, __pyx_k_ZeroFindingError, sizeof(__pyx_k_ZeroFindingError), 0, 0, 1, 1},
    {&__pyx_n_s__5, __pyx_k__5, sizeof(__pyx_k__5), 0, 0, 1, 1},
    {&__pyx_kp_u__6, __pyx_k__6, sizeof(__pyx_k__6), 0, 1, 0, 0},
    {&__pyx_n_s__8, __pyx_k__8, sizeof(__pyx_k__8), 0, 0, 1, 1},
    {&__pyx_n_s_a0, __pyx_k_a0, sizeof(__pyx_k_a0), 0, 0, 1, 1},
    {&__pyx_n_s_all, __pyx_k_all, sizeof(__pyx_k_all), 0, 0, 1, 1},
    {&__pyx_n_s_alt0, __pyx_k_alt0, sizeof(__pyx_k_alt0), 0, 0, 1, 1},
    {&__pyx_n_s_altitude, __pyx_k_altitude, sizeof(__pyx_k_altitude), 0, 0, 1, 1},
    {&__pyx_n_s_ammo, __pyx_k_ammo, sizeof(__pyx_k_ammo), 0, 0, 1, 1},
    {&__pyx_n_s_angle, __pyx_k_angle, sizeof(__pyx_k_angle), 0, 0, 1, 1},
    {&__pyx_n_s_atmo, __pyx_k_atmo, sizeof(__pyx_k_atmo), 0, 0, 1, 1},
    {&__pyx_n_s_barrel_azimuth, __pyx_k_barrel_azimuth, sizeof(__pyx_k_barrel_azimuth), 0, 0, 1, 1},
    {&__pyx_n_s_barrel_elevation, __pyx_k_barrel_elevation, sizeof(__pyx_k_barrel_elevation), 0, 0, 1, 1},
    {&__pyx_n_s_bc, __pyx_k_bc, sizeof(__pyx_k_bc), 0, 0, 1, 1},
    {&__pyx_n_s_cGravityConstant, __pyx_k_cGravityConstant, sizeof(__pyx_k_cGravityConstant), 0, 0, 1, 1},
    {&__pyx_n_s_cMaxWindDistanceFeet, __pyx_k_cMaxWindDistanceFeet, sizeof(__pyx_k_cMaxWindDistanceFeet), 0, 0, 1, 1},
    {&__pyx_n_s_calc_step, __pyx_k_calc_step, sizeof(__pyx_k_calc_step), 0, 0, 1, 1},
    {&__pyx_n_s_cant_angle, __pyx_k_cant_angle, sizeof(__pyx_k_cant_angle), 0, 0, 1, 1},
    {&__pyx_n_s_cant_cosine, __pyx_k_cant_cosine, sizeof(__pyx_k_cant_cosine), 0, 0, 1, 1},
    {&__pyx_n_s_cant_sine, __pyx_k_cant_sine, sizeof(__pyx_k_cant_sine), 0, 0, 1, 1},
    {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
    {&__pyx_n_s_config, __pyx_k_config, sizeof(__pyx_k_config), 0, 0, 1, 1},
    {&__pyx_n_s_curve, __pyx_k_curve, sizeof(__pyx_k_curve), 0, 0, 1, 1},
    {&__pyx_n_s_debug, __pyx_k_debug, sizeof(__pyx_k_debug), 0, 0, 1, 1},
    {&__pyx_n_s_defined_units, __pyx_k_defined_units, sizeof(__pyx_k_defined_units), 0, 0, 1, 1},
    {&__pyx_n_s_density_factor, __pyx_k_density_factor, sizeof(__pyx_k_density_factor), 0, 0, 1, 1},
    {&__pyx_n_s_density_ratio, __pyx_k_density_ratio, sizeof(__pyx_k_density_ratio), 0, 0, 1, 1},
    {&__pyx_n_s_diameter, __pyx_k_diameter, sizeof(__pyx_k_diameter), 0, 0, 1, 1},
    {&__pyx_n_s_direction_from, __pyx_k_direction_from, sizeof(__pyx_k_direction_from), 0, 0, 1, 1},
    {&__pyx_kp_u_disable, __pyx_k_disable, sizeof(__pyx_k_disable), 0, 1, 0, 0},
    {&__pyx_n_s_dist_step, __pyx_k_dist_step, sizeof(__pyx_k_dist_step), 0, 0, 1, 1},
    {&__pyx_n_s_distance, __pyx_k_distance, sizeof(__pyx_k_distance), 0, 0, 1, 1},
    {&__pyx_n_s_dm, __pyx_k_dm, sizeof(__pyx_k_dm), 0, 0, 1, 1},
    {&__pyx_n_s_drag, __pyx_k_drag, sizeof(__pyx_k_drag), 0, 0, 1, 1},
    {&__pyx_n_s_drag_table, __pyx_k_drag_table, sizeof(__pyx_k_drag_table), 0, 0, 1, 1},
    {&__pyx_n_s_drop_adj, __pyx_k_drop_adj, sizeof(__pyx_k_drop_adj), 0, 0, 1, 1},
    {&__pyx_kp_u_enable, __pyx_k_enable, sizeof(__pyx_k_enable), 0, 1, 0, 0},
    {&__pyx_n_s_energy, __pyx_k_energy, sizeof(__pyx_k_energy), 0, 0, 1, 1},
    {&__pyx_kp_u_euler_cy_it, __pyx_k_euler_cy_it, sizeof(__pyx_k_euler_cy_it), 0, 1, 0, 0},
    {&__pyx_n_s_extra_data, __pyx_k_extra_data, sizeof(__pyx_k_extra_data), 0, 0, 1, 1},
    {&__pyx_n_s_feet, __pyx_k_feet, sizeof(__pyx_k_feet), 0, 0, 1, 1},
    {&__pyx_n_s_filter_flags, __pyx_k_filter_flags, sizeof(__pyx_k_filter_flags), 0, 0, 1, 1},
    {&__pyx_n_s_flag, __pyx_k_flag, sizeof(__pyx_k_flag), 0, 0, 1, 1},
    {&__pyx_n_s_fps, __pyx_k_fps, sizeof(__pyx_k_fps), 0, 0, 1, 1},
    {&__pyx_kp_u_gc, __pyx_k_gc, sizeof(__pyx_k_gc), 0, 1, 0, 0},
    {&__pyx_n_s_get_in, __pyx_k_get_in, sizeof(__pyx_k_get_in), 0, 0, 1, 1},
    {&__pyx_n_s_get_velocity_for_temp, __pyx_k_get_velocity_for_temp, sizeof(__pyx_k_get_velocity_for_temp), 0, 0, 1, 1},
    {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
    {&__pyx_n_s_grain, __pyx_k_grain, sizeof(__pyx_k_grain), 0, 0, 1, 1},
    {&__pyx_n_s_height, __pyx_k_height, sizeof(__pyx_k_height), 0, 0, 1, 1},
    {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
    {&__pyx_n_s_inch, __pyx_k_inch, sizeof(__pyx_k_inch), 0, 0, 1, 1},
    {&__pyx_n_s_initializing, __pyx_k_initializing, sizeof(__pyx_k_initializing), 0, 0, 1, 1},
    {&__pyx_kp_u_isenabled, __pyx_k_isenabled, sizeof(__pyx_k_isenabled), 0, 1, 0, 0},
    {&__pyx_n_s_length, __pyx_k_length, sizeof(__pyx_k_length), 0, 0, 1, 1},
    {&__pyx_n_s_logger, __pyx_k_logger, sizeof(__pyx_k_logger), 0, 0, 1, 1},
    {&__pyx_n_s_look_angle, __pyx_k_look_angle, sizeof(__pyx_k_look_angle), 0, 0, 1, 1},
    {&__pyx_n_s_look_distance, __pyx_k_look_distance, sizeof(__pyx_k_look_distance), 0, 0, 1, 1},
    {&__pyx_n_s_mach, __pyx_k_mach, sizeof(__pyx_k_mach), 0, 0, 1, 1},
    {&__pyx_n_s_mach1, __pyx_k_mach1, sizeof(__pyx_k_mach1), 0, 0, 1, 1},
    {&__pyx_n_s_mach_list, __pyx_k_mach_list, sizeof(__pyx_k_mach_list), 0, 0, 1, 1},
    {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
    {&__pyx_n_s_max_range, __pyx_k_max_range, sizeof(__pyx_k_max_range), 0, 0, 1, 1},
    {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
    {&__pyx_n_s_new, __pyx_k_new, sizeof(__pyx_k_new), 0, 0, 1, 1},
    {&__pyx_kp_s_no_default___reduce___due_to_non, __pyx_k_no_default___reduce___due_to_non, sizeof(__pyx_k_no_default___reduce___due_to_non), 0, 0, 1, 0},
    {&__pyx_n_s_object, __pyx_k_object, sizeof(__pyx_k_object), 0, 0, 1, 1},
    {&__pyx_n_s_ogw, __pyx_k_ogw, sizeof(__pyx_k_ogw), 0, 0, 1, 1},
    {&__pyx_n_u_once, __pyx_k_once, sizeof(__pyx_k_once), 0, 1, 0, 1},
    {&__pyx_n_s_p0, __pyx_k_p0, sizeof(__pyx_k_p0), 0, 0, 1, 1},
    {&__pyx_n_s_powder_temp, __pyx_k_powder_temp, sizeof(__pyx_k_powder_temp), 0, 0, 1, 1},
    {&__pyx_n_s_pressure, __pyx_k_pressure, sizeof(__pyx_k_pressure), 0, 0, 1, 1},
    {&__pyx_n_s_py_ballisticcalc_constants, __pyx_k_py_ballisticcalc_constants, sizeof(__pyx_k_py_ballisticcalc_constants), 0, 0, 1, 1},
    {&__pyx_n_s_py_ballisticcalc_exceptions, __pyx_k_py_ballisticcalc_exceptions, sizeof(__pyx_k_py_ballisticcalc_exceptions), 0, 0, 1, 1},
    {&__pyx_n_s_py_ballisticcalc_logger, __pyx_k_py_ballisticcalc_logger, sizeof(__pyx_k_py_ballisticcalc_logger), 0, 0, 1, 1},
    {&__pyx_n_s_py_ballisticcalc_unit, __pyx_k_py_ballisticcalc_unit, sizeof(__pyx_k_py_ballisticcalc_unit), 0, 0, 1, 1},
    {&__pyx_n_s_pyx_state, __pyx_k_pyx_state, sizeof(__pyx_k_pyx_state), 0, 0, 1, 1},
    {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
    {&__pyx_n_s_rad, __pyx_k_rad, sizeof(__pyx_k_rad), 0, 0, 1, 1},
    {&__pyx_n_s_ranges_length, __pyx_k_ranges_length, sizeof(__pyx_k_ranges_length), 0, 0, 1, 1},
    {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
    {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
    {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
    {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
    {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
    {&__pyx_n_s_shot_info, __pyx_k_shot_info, sizeof(__pyx_k_shot_info), 0, 0, 1, 1},
    {&__pyx_n_s_sight_height, __pyx_k_sight_height, sizeof(__pyx_k_sight_height), 0, 0, 1, 1},
    {&__pyx_n_s_simplefilter, __pyx_k_simplefilter, sizeof(__pyx_k_simplefilter), 0, 0, 1, 1},
    {&__pyx_n_s_spec, __pyx_k_spec, sizeof(__pyx_k_spec), 0, 0, 1, 1},
    {&__pyx_n_s_stability_coefficient, __pyx_k_stability_coefficient, sizeof(__pyx_k_stability_coefficient), 0, 0, 1, 1},
    {&__pyx_n_s_t0, __pyx_k_t0, sizeof(__pyx_k_t0), 0, 0, 1, 1},
    {&__pyx_n_s_target_drop, __pyx_k_target_drop, sizeof(__pyx_k_target_drop), 0, 0, 1, 1},
    {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
    {&__pyx_n_s_time, __pyx_k_time, sizeof(__pyx_k_time), 0, 0, 1, 1},
    {&__pyx_n_s_time_step, __pyx_k_time_step, sizeof(__pyx_k_time_step), 0, 0, 1, 1},
    {&__pyx_n_s_twist, __pyx_k_twist, sizeof(__pyx_k_twist), 0, 0, 1, 1},
    {&__pyx_n_s_until_distance, __pyx_k_until_distance, sizeof(__pyx_k_until_distance), 0, 0, 1, 1},
    {&__pyx_n_s_value, __pyx_k_value, sizeof(__pyx_k_value), 0, 0, 1, 1},
    {&__pyx_n_s_velocity, __pyx_k_velocity, sizeof(__pyx_k_velocity), 0, 0, 1, 1},
    {&__pyx_n_s_warnings, __pyx_k_warnings, sizeof(__pyx_k_warnings), 0, 0, 1, 1},
    {&__pyx_n_s_weapon, __pyx_k_weapon, sizeof(__pyx_k_weapon), 0, 0, 1, 1},
    {&__pyx_n_s_weight, __pyx_k_weight, sizeof(__pyx_k_weight), 0, 0, 1, 1},
    {&__pyx_n_s_windage, __pyx_k_windage, sizeof(__pyx_k_windage), 0, 0, 1, 1},
    {&__pyx_n_s_windage_adj, __pyx_k_windage_adj, sizeof(__pyx_k_windage_adj), 0, 0, 1, 1},
    {&__pyx_n_s_winds, __pyx_k_winds, sizeof(__pyx_k_winds), 0, 0, 1, 1},
    {&__pyx_n_s_x, __pyx_k_x, sizeof(__pyx_k_x), 0, 0, 1, 1},
    {&__pyx_n_s_y, __pyx_k_y, sizeof(__pyx_k_y), 0, 0, 1, 1},
    {&__pyx_n_s_z, __pyx_k_z, sizeof(__pyx_k_z), 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 0, 0}
  };
  return __Pyx_InitStrings(__pyx_string_tab);
}
/* #### Code section: cached_builtins ### */
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_builtin_object = __Pyx_GetBuiltinName(__pyx_n_s_object); if (!__pyx_builtin_object) __PYX_ERR(1, 467, __pyx_L1_error)
  __pyx_builtin_KeyError = __Pyx_GetBuiltinName(__pyx_n_s_KeyError); if (!__pyx_builtin_KeyError) __PYX_ERR(0, 19, __pyx_L1_error)
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) __PYX_ERR(0, 20, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: cached_constants ### */

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "FromPyStructUtility":20
 *         value = obj['velocity']
 *     except KeyError:
 *         raise ValueError("No value specified for struct attribute 'velocity'")             # <<<<<<<<<<<<<<
 *     result.velocity = value
 *     try:
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_s_No_value_specified_for_struct_at); if (unlikely(!__pyx_tuple_)) __PYX_ERR(0, 20, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "FromPyStructUtility":25
 *         value = obj['direction_from']
 *     except KeyError:
 *         raise ValueError("No value specified for struct attribute 'direction_from'")             # <<<<<<<<<<<<<<
 *     result.direction_from = value
 *     try:
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_s_No_value_specified_for_struct_at_2); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "FromPyStructUtility":30
 *         value = obj['until_distance']
 *     except KeyError:
 *         raise ValueError("No value specified for struct attribute 'until_distance'")             # <<<<<<<<<<<<<<
 *     result.until_distance = value
 *     try:
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_kp_s_No_value_specified_for_struct_at_3); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(0, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "FromPyStructUtility":35
 *         value = obj['MAX_DISTANCE_FEET']
 *     except KeyError:
 *         raise ValueError("No value specified for struct attribute 'MAX_DISTANCE_FEET'")             # <<<<<<<<<<<<<<
 *     result.MAX_DISTANCE_FEET = value
 *     return result
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_kp_s_No_value_specified_for_struct_at_4); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "CyEulerBallistic/trajectory_calc.pyx":46
 * 
 * __all__ = (
 *     'EulerTrajectoryCalc',             # <<<<<<<<<<<<<<
 * )
 * 
 */
  __pyx_tuple__7 = PyTuple_Pack(1, __pyx_n_u_EulerTrajectoryCalc); if (unlikely(!__pyx_tuple__7)) __PYX_ERR(1, 46, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}
/* #### Code section: init_constants ### */

static CYTHON_SMALL_CODE int __Pyx_InitConstants(void) {
  if (__Pyx_CreateStringTabAndInitStrings() < 0) __PYX_ERR(1, 1, __pyx_L1_error);
  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: init_globals ### */

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  return 0;
}
/* #### Code section: init_module ### */

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __pyx_vtabptr_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter = &__pyx_vtable_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter;
  __pyx_vtable_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter.setup_seen_zero = (void (*)(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *, double, double, double))__pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_setup_seen_zero;
  __pyx_vtable_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter.clear_current_flag = (void (*)(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *))__pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_clear_current_flag;
  __pyx_vtable_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter.should_record = (int (*)(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *, struct __pyx_t_16CyEulerBallistic_6vector_CVector, double, double, double, double))__pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_should_record;
  __pyx_vtable_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter.should_break = (int (*)(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *))__pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_should_break;
  __pyx_vtable_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter.check_next_range = (int (*)(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *, double, double))__pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_check_next_range;
  __pyx_vtable_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter.check_next_time = (void (*)(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *, double))__pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_check_next_time;
  __pyx_vtable_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter.check_mach_crossing = (void (*)(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *, double, double))__pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_check_mach_crossing;
  __pyx_vtable_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter.check_zero_crossing = (void (*)(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter *, struct __pyx_t_16CyEulerBallistic_6vector_CVector))__pyx_f_16CyEulerBallistic_15trajectory_calc_21_TrajectoryDataFilter_check_zero_crossing;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_ptype_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter_spec, NULL); if (unlikely(!__pyx_ptype_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter)) __PYX_ERR(1, 51, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter_spec, __pyx_ptype_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter) < 0) __PYX_ERR(1, 51, __pyx_L1_error)
  #else
  __pyx_ptype_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter = &__pyx_type_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_ptype_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter) < 0) __PYX_ERR(1, 51, __pyx_L1_error)
  #endif
  #if PY_MAJOR_VERSION < 3
  __pyx_ptype_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter->tp_print = 0;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_ptype_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter->tp_dictoffset && __pyx_ptype_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_ptype_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter->tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_ptype_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter, __pyx_vtabptr_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter) < 0) __PYX_ERR(1, 51, __pyx_L1_error)
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if (__Pyx_MergeVtables(__pyx_ptype_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter) < 0) __PYX_ERR(1, 51, __pyx_L1_error)
  #endif
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_TrajectoryDataFilter, (PyObject *) __pyx_ptype_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter) < 0) __PYX_ERR(1, 51, __pyx_L1_error)
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if (__Pyx_setup_reduce((PyObject *) __pyx_ptype_16CyEulerBallistic_15trajectory_calc__TrajectoryDataFilter) < 0) __PYX_ERR(1, 51, __pyx_L1_error)
  #endif
  __pyx_vtabptr_16CyEulerBallistic_15trajectory_calc__WindSock = &__pyx_vtable_16CyEulerBallistic_15trajectory_calc__WindSock;
  __pyx_vtable_16CyEulerBallistic_15trajectory_calc__WindSock.current_vector = (struct __pyx_t_16CyEulerBallistic_6vector_CVector (*)(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock *))__pyx_f_16CyEulerBallistic_15trajectory_calc_9_WindSock_current_vector;
  __pyx_vtable_16CyEulerBallistic_15trajectory_calc__WindSock.update_cache = (void (*)(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock *))__pyx_f_16CyEulerBallistic_15trajectory_calc_9_WindSock_update_cache;
  __pyx_vtable_16CyEulerBallistic_15trajectory_calc__WindSock.vector_for_range = (struct __pyx_t_16CyEulerBallistic_6vector_CVector (*)(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc__WindSock *, double))__pyx_f_16CyEulerBallistic_15trajectory_calc_9_WindSock_vector_for_range;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_ptype_16CyEulerBallistic_15trajectory_calc__WindSock = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_16CyEulerBallistic_15trajectory_calc__WindSock_spec, NULL); if (unlikely(!__pyx_ptype_16CyEulerBallistic_15trajectory_calc__WindSock)) __PYX_ERR(1, 137, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_16CyEulerBallistic_15trajectory_calc__WindSock_spec, __pyx_ptype_16CyEulerBallistic_15trajectory_calc__WindSock) < 0) __PYX_ERR(1, 137, __pyx_L1_error)
  #else
  __pyx_ptype_16CyEulerBallistic_15trajectory_calc__WindSock = &__pyx_type_16CyEulerBallistic_15trajectory_calc__WindSock;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_ptype_16CyEulerBallistic_15trajectory_calc__WindSock) < 0) __PYX_ERR(1, 137, __pyx_L1_error)
  #endif
  #if PY_MAJOR_VERSION < 3
  __pyx_ptype_16CyEulerBallistic_15trajectory_calc__WindSock->tp_print = 0;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_ptype_16CyEulerBallistic_15trajectory_calc__WindSock->tp_dictoffset && __pyx_ptype_16CyEulerBallistic_15trajectory_calc__WindSock->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_ptype_16CyEulerBallistic_15trajectory_calc__WindSock->tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_ptype_16CyEulerBallistic_15trajectory_calc__WindSock, __pyx_vtabptr_16CyEulerBallistic_15trajectory_calc__WindSock) < 0) __PYX_ERR(1, 137, __pyx_L1_error)
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if (__Pyx_MergeVtables(__pyx_ptype_16CyEulerBallistic_15trajectory_calc__WindSock) < 0) __PYX_ERR(1, 137, __pyx_L1_error)
  #endif
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_WindSock, (PyObject *) __pyx_ptype_16CyEulerBallistic_15trajectory_calc__WindSock) < 0) __PYX_ERR(1, 137, __pyx_L1_error)
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if (__Pyx_setup_reduce((PyObject *) __pyx_ptype_16CyEulerBallistic_15trajectory_calc__WindSock) < 0) __PYX_ERR(1, 137, __pyx_L1_error)
  #endif
  __pyx_vtabptr_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc = &__pyx_vtable_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc;
  __pyx_vtable_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc.get_calc_step = (double (*)(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *, struct __pyx_opt_args_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_get_calc_step *__pyx_optional_args))__pyx_f_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc_get_calc_step;
  __pyx_vtable_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc._free_trajectory = (void (*)(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *))__pyx_f_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc__free_trajectory;
  __pyx_vtable_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc._init_trajectory = (void (*)(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *, PyObject *))__pyx_f_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc__init_trajectory;
  __pyx_vtable_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc._zero_angle = (PyObject *(*)(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *, PyObject *, PyObject *))__pyx_f_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc__zero_angle;
  __pyx_vtable_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc._integrate = (PyObject *(*)(struct __pyx_obj_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc *, double, double, int, struct __pyx_opt_args_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc__integrate *__pyx_optional_args))__pyx_f_16CyEulerBallistic_15trajectory_calc_19EulerTrajectoryCalc__integrate;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_ptype_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc_spec, NULL); if (unlikely(!__pyx_ptype_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc)) __PYX_ERR(1, 187, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc_spec, __pyx_ptype_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc) < 0) __PYX_ERR(1, 187, __pyx_L1_error)
  #else
  __pyx_ptype_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc = &__pyx_type_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_ptype_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc) < 0) __PYX_ERR(1, 187, __pyx_L1_error)
  #endif
  #if PY_MAJOR_VERSION < 3
  __pyx_ptype_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc->tp_print = 0;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_ptype_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc->tp_dictoffset && __pyx_ptype_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_ptype_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc->tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_ptype_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc, __pyx_vtabptr_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc) < 0) __PYX_ERR(1, 187, __pyx_L1_error)
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if (__Pyx_MergeVtables(__pyx_ptype_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc) < 0) __PYX_ERR(1, 187, __pyx_L1_error)
  #endif
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_EulerTrajectoryCalc, (PyObject *) __pyx_ptype_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc) < 0) __PYX_ERR(1, 187, __pyx_L1_error)
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if (__Pyx_setup_reduce((PyObject *) __pyx_ptype_16CyEulerBallistic_15trajectory_calc_EulerTrajectoryCalc) < 0) __PYX_ERR(1, 187, __pyx_L1_error)
  #endif
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule("CyEulerBallistic.vector"); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_16CyEulerBallistic_6vector_Vector = __Pyx_ImportType_3_0_11(__pyx_t_1, "CyEulerBallistic.vector", "Vector", sizeof(struct __pyx_obj_16CyEulerBallistic_6vector_Vector), __PYX_GET_STRUCT_ALIGNMENT_3_0_11(struct __pyx_obj_16CyEulerBallistic_6vector_Vector),__Pyx_ImportType_CheckSize_Warn_3_0_11); if (!__pyx_ptype_16CyEulerBallistic_6vector_Vector) __PYX_ERR(2, 5, __pyx_L1_error)
  __pyx_vtabptr_16CyEulerBallistic_6vector_Vector = (struct __pyx_vtabstruct_16CyEulerBallistic_6vector_Vector*)__Pyx_GetVtable(__pyx_ptype_16CyEulerBallistic_6vector_Vector); if (unlikely(!__pyx_vtabptr_16CyEulerBallistic_6vector_Vector)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("CyEulerBallistic.trajectory_data"); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_16CyEulerBallistic_15trajectory_data_TrajectoryData = __Pyx_ImportType_3_0_11(__pyx_t_1, "CyEulerBallistic.trajectory_data", "TrajectoryData", sizeof(struct __pyx_obj_16CyEulerBallistic_15trajectory_data_TrajectoryData), __PYX_GET_STRUCT_ALIGNMENT_3_0_11(struct __pyx_obj_16CyEulerBallistic_15trajectory_data_TrajectoryData),__Pyx_ImportType_CheckSize_Warn_3_0_11); if (!__pyx_ptype_16CyEulerBallistic_15trajectory_data_TrajectoryData) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __pyx_t_1 = PyImport_ImportModule("CyEulerBallistic.cy_euler"); if (!__pyx_t_1) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "cDegreesFtoR", (void **)&__pyx_vp_16CyEulerBallistic_8cy_euler_cDegreesFtoR, "double") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "cSpeedOfSoundImperial", (void **)&__pyx_vp_16CyEulerBallistic_8cy_euler_cSpeedOfSoundImperial, "double") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "cLapseRateImperial", (void **)&__pyx_vp_16CyEulerBallistic_8cy_euler_cLapseRateImperial, "double") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __pyx_t_1 = PyImport_ImportModule("CyEulerBallistic.vector"); if (!__pyx_t_1) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportFunction_3_0_11(__pyx_t_1, "mag", (void (**)(void))&__pyx_f_16CyEulerBallistic_6vector_mag, "double (struct __pyx_t_16CyEulerBallistic_6vector_CVector *)") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction_3_0_11(__pyx_t_1, "mul_c", (void (**)(void))&__pyx_f_16CyEulerBallistic_6vector_mul_c, "struct __pyx_t_16CyEulerBallistic_6vector_CVector (struct __pyx_t_16CyEulerBallistic_6vector_CVector *, double)") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction_3_0_11(__pyx_t_1, "mul_v", (void (**)(void))&__pyx_f_16CyEulerBallistic_6vector_mul_v, "double (struct __pyx_t_16CyEulerBallistic_6vector_CVector *, struct __pyx_t_16CyEulerBallistic_6vector_CVector *)") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction_3_0_11(__pyx_t_1, "add", (void (**)(void))&__pyx_f_16CyEulerBallistic_6vector_add, "struct __pyx_t_16CyEulerBallistic_6vector_CVector (struct __pyx_t_16CyEulerBallistic_6vector_CVector *, struct __pyx_t_16CyEulerBallistic_6vector_CVector *)") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction_3_0_11(__pyx_t_1, "sub", (void (**)(void))&__pyx_f_16CyEulerBallistic_6vector_sub, "struct __pyx_t_16CyEulerBallistic_6vector_CVector (struct __pyx_t_16CyEulerBallistic_6vector_CVector *, struct __pyx_t_16CyEulerBallistic_6vector_CVector *)") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction_3_0_11(__pyx_t_1, "neg", (void (**)(void))&__pyx_f_16CyEulerBallistic_6vector_neg, "struct __pyx_t_16CyEulerBallistic_6vector_CVector (struct __pyx_t_16CyEulerBallistic_6vector_CVector *)") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction_3_0_11(__pyx_t_1, "norm", (void (**)(void))&__pyx_f_16CyEulerBallistic_6vector_norm, "struct __pyx_t_16CyEulerBallistic_6vector_CVector (struct __pyx_t_16CyEulerBallistic_6vector_CVector *)") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("CyEulerBallistic.cy_euler"); if (!__pyx_t_1) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportFunction_3_0_11(__pyx_t_1, "config_bind", (void (**)(void))&__pyx_f_16CyEulerBallistic_8cy_euler_config_bind, "struct __pyx_t_16CyEulerBallistic_8cy_euler_Config_t (PyObject *)") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction_3_0_11(__pyx_t_1, "update_density_factor_and_mach_for_altitude", (void (**)(void))&__pyx_f_16CyEulerBallistic_8cy_euler_update_density_factor_and_mach_for_altitude, "void (struct __pyx_t_16CyEulerBallistic_8cy_euler_Atmosphere_t *, double, double *, double *)") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction_3_0_11(__pyx_t_1, "cy_table_to_mach", (void (**)(void))&__pyx_f_16CyEulerBallistic_8cy_euler_cy_table_to_mach, "struct __pyx_t_16CyEulerBallistic_8cy_euler_MachList_t (PyObject *)") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction_3_0_11(__pyx_t_1, "cy_calculate_curve", (void (**)(void))&__pyx_f_16CyEulerBallistic_8cy_euler_cy_calculate_curve, "struct __pyx_t_16CyEulerBallistic_8cy_euler_Curve_t (PyObject *)") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction_3_0_11(__pyx_t_1, "cy_calculate_by_curve_and_mach_list", (void (**)(void))&__pyx_f_16CyEulerBallistic_8cy_euler_cy_calculate_by_curve_and_mach_list, "double (struct __pyx_t_16CyEulerBallistic_8cy_euler_MachList_t *, struct __pyx_t_16CyEulerBallistic_8cy_euler_Curve_t *, double)") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction_3_0_11(__pyx_t_1, "cy_spin_drift", (void (**)(void))&__pyx_f_16CyEulerBallistic_8cy_euler_cy_spin_drift, "double (struct __pyx_t_16CyEulerBallistic_8cy_euler_ShotData_t *, double)") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction_3_0_11(__pyx_t_1, "cy_drag_by_mach", (void (**)(void))&__pyx_f_16CyEulerBallistic_8cy_euler_cy_drag_by_mach, "double (struct __pyx_t_16CyEulerBallistic_8cy_euler_ShotData_t *, double)") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction_3_0_11(__pyx_t_1, "cy_update_stability_coefficient", (void (**)(void))&__pyx_f_16CyEulerBallistic_8cy_euler_cy_update_stability_coefficient, "void (struct __pyx_t_16CyEulerBallistic_8cy_euler_ShotData_t *)") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction_3_0_11(__pyx_t_1, "cy_get_correction", (void (**)(void))&__pyx_f_16CyEulerBallistic_8cy_euler_cy_get_correction, "double (double, double)") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction_3_0_11(__pyx_t_1, "cy_calculate_energy", (void (**)(void))&__pyx_f_16CyEulerBallistic_8cy_euler_cy_calculate_energy, "double (double, double)") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction_3_0_11(__pyx_t_1, "cy_calculate_ogw", (void (**)(void))&__pyx_f_16CyEulerBallistic_8cy_euler_cy_calculate_ogw, "double (double, double)") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction_3_0_11(__pyx_t_1, "free_trajectory", (void (**)(void))&__pyx_f_16CyEulerBallistic_8cy_euler_free_trajectory, "void (struct __pyx_t_16CyEulerBallistic_8cy_euler_ShotData_t *)") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction_3_0_11(__pyx_t_1, "wind_to_c_vector", (void (**)(void))&__pyx_f_16CyEulerBallistic_8cy_euler_wind_to_c_vector, "struct __pyx_t_16CyEulerBallistic_6vector_CVector (struct __pyx_t_16CyEulerBallistic_8cy_euler_Wind_t *)") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}


#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_trajectory_calc(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_trajectory_calc},
  {0, NULL}
};
#endif

#ifdef __cplusplus
namespace {
  struct PyModuleDef __pyx_moduledef =
  #else
  static struct PyModuleDef __pyx_moduledef =
  #endif
  {
      PyModuleDef_HEAD_INIT,
      "trajectory_calc",
      __pyx_k_Total_Score_2394_Possible_Score, /* m_doc */
    #if CYTHON_PEP489_MULTI_PHASE_INIT
      0, /* m_size */
    #elif CYTHON_USE_MODULE_STATE
      sizeof(__pyx_mstate), /* m_size */
    #else
      -1, /* m_size */
    #endif
      __pyx_methods /* m_methods */,
    #if CYTHON_PEP489_MULTI_PHASE_INIT
      __pyx_moduledef_slots, /* m_slots */
    #else
      NULL, /* m_reload */
    #endif
    #if CYTHON_USE_MODULE_STATE
      __pyx_m_traverse, /* m_traverse */
      __pyx_m_clear, /* m_clear */
      NULL /* m_free */
    #else
      NULL, /* m_traverse */
      NULL, /* m_clear */
      NULL /* m_free */
    #endif
  };
  #ifdef __cplusplus
} /* anonymous namespace */
#endif
#endif

#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC inittrajectory_calc(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC inittrajectory_calc(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_trajectory_calc(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_trajectory_calc(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
#if CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *module, const char* from_name, const char* to_name, int allow_none)
#else
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none)
#endif
{
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
#if CYTHON_COMPILING_IN_LIMITED_API
            result = PyModule_AddObject(module, to_name, value);
#else
            result = PyDict_SetItemString(moddict, to_name, value);
#endif
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    CYTHON_UNUSED_VAR(def);
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
#if CYTHON_COMPILING_IN_LIMITED_API
    moddict = module;
#else
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
#endif
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_trajectory_calc(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  int stringtab_initialized = 0;
  #if CYTHON_USE_MODULE_STATE
  int pystate_addmodule_run = 0;
  #endif
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'trajectory_calc' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("trajectory_calc", __pyx_methods, __pyx_k_Total_Score_2394_Possible_Score, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  if (unlikely(!__pyx_m)) __PYX_ERR(1, 1, __pyx_L1_error)
  #elif CYTHON_USE_MODULE_STATE
  __pyx_t_1 = PyModule_Create(&__pyx_moduledef); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1, __pyx_L1_error)
  {
    int add_module_result = PyState_AddModule(__pyx_t_1, &__pyx_moduledef);
    __pyx_t_1 = 0; /* transfer ownership from __pyx_t_1 to "trajectory_calc" pseudovariable */
    if (unlikely((add_module_result < 0))) __PYX_ERR(1, 1, __pyx_L1_error)
    pystate_addmodule_run = 1;
  }
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  if (unlikely(!__pyx_m)) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #endif
  CYTHON_UNUSED_VAR(__pyx_t_1);
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(1, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = __Pyx_PyImport_AddModuleRef(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_cython_runtime = __Pyx_PyImport_AddModuleRef((const char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(1, 1, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_trajectory_calc(void)", 0);
  if (__Pyx_check_binary_version(__PYX_LIMITED_VERSION_HEX, __Pyx_get_runtime_version(), CYTHON_COMPILING_IN_LIMITED_API) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(1, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init(__pyx_m) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init(__pyx_m) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init(__pyx_m) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init(__pyx_m) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init(__pyx_m) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init(__pyx_m) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(WITH_THREAD) && PY_VERSION_HEX < 0x030700F0 && defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  PyEval_InitThreads();
  #endif
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitConstants() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  stringtab_initialized = 1;
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_CyEulerBallistic__trajectory_calc) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name, __pyx_n_s_main) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(1, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "CyEulerBallistic.trajectory_calc")) {
      if (unlikely((PyDict_SetItemString(modules, "CyEulerBallistic.trajectory_calc", __pyx_m) < 0))) __PYX_ERR(1, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  if (unlikely((__Pyx_modinit_type_init_code() < 0))) __PYX_ERR(1, 1, __pyx_L1_error)
  if (unlikely((__Pyx_modinit_type_import_code() < 0))) __PYX_ERR(1, 1, __pyx_L1_error)
  if (unlikely((__Pyx_modinit_variable_import_code() < 0))) __PYX_ERR(1, 1, __pyx_L1_error)
  if (unlikely((__Pyx_modinit_function_import_code() < 0))) __PYX_ERR(1, 1, __pyx_L1_error)
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif

  /* "CyEulerBallistic/trajectory_calc.pyx":37
 * )
 * 
 * import warnings             # <<<<<<<<<<<<<<
 * 
 * from py_ballisticcalc.logger import logger
 */
  __pyx_t_2 = __Pyx_ImportDottedModule(__pyx_n_s_warnings, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 37, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_warnings, __pyx_t_2) < 0) __PYX_ERR(1, 37, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":39
 * import warnings
 * 
 * from py_ballisticcalc.logger import logger             # <<<<<<<<<<<<<<
 * from py_ballisticcalc.unit import Angular, Unit, Velocity, Distance, Energy, Weight
 * from py_ballisticcalc.exceptions import ZeroFindingError, RangeError
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_logger);
  __Pyx_GIVEREF(__pyx_n_s_logger);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_logger)) __PYX_ERR(1, 39, __pyx_L1_error);
  __pyx_t_3 = __Pyx_Import(__pyx_n_s_py_ballisticcalc_logger, __pyx_t_2, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_logger); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_logger, __pyx_t_2) < 0) __PYX_ERR(1, 39, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":40
 * 
 * from py_ballisticcalc.logger import logger
 * from py_ballisticcalc.unit import Angular, Unit, Velocity, Distance, Energy, Weight             # <<<<<<<<<<<<<<
 * from py_ballisticcalc.exceptions import ZeroFindingError, RangeError
 * from py_ballisticcalc.constants import cMaxWindDistanceFeet
 */
  __pyx_t_3 = PyList_New(6); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_n_s_Angular);
  __Pyx_GIVEREF(__pyx_n_s_Angular);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 0, __pyx_n_s_Angular)) __PYX_ERR(1, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_s_Unit);
  __Pyx_GIVEREF(__pyx_n_s_Unit);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 1, __pyx_n_s_Unit)) __PYX_ERR(1, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_s_Velocity);
  __Pyx_GIVEREF(__pyx_n_s_Velocity);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 2, __pyx_n_s_Velocity)) __PYX_ERR(1, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_s_Distance);
  __Pyx_GIVEREF(__pyx_n_s_Distance);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 3, __pyx_n_s_Distance)) __PYX_ERR(1, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_s_Energy);
  __Pyx_GIVEREF(__pyx_n_s_Energy);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 4, __pyx_n_s_Energy)) __PYX_ERR(1, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_s_Weight);
  __Pyx_GIVEREF(__pyx_n_s_Weight);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 5, __pyx_n_s_Weight)) __PYX_ERR(1, 40, __pyx_L1_error);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_py_ballisticcalc_unit, __pyx_t_3, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_Angular); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Angular, __pyx_t_3) < 0) __PYX_ERR(1, 40, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_Unit); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Unit, __pyx_t_3) < 0) __PYX_ERR(1, 40, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_Velocity); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Velocity, __pyx_t_3) < 0) __PYX_ERR(1, 40, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_Distance); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Distance, __pyx_t_3) < 0) __PYX_ERR(1, 40, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_Energy); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Energy, __pyx_t_3) < 0) __PYX_ERR(1, 40, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_Weight); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Weight, __pyx_t_3) < 0) __PYX_ERR(1, 40, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":41
 * from py_ballisticcalc.logger import logger
 * from py_ballisticcalc.unit import Angular, Unit, Velocity, Distance, Energy, Weight
 * from py_ballisticcalc.exceptions import ZeroFindingError, RangeError             # <<<<<<<<<<<<<<
 * from py_ballisticcalc.constants import cMaxWindDistanceFeet
 * 
 */
  __pyx_t_2 = PyList_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_ZeroFindingError);
  __Pyx_GIVEREF(__pyx_n_s_ZeroFindingError);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_ZeroFindingError)) __PYX_ERR(1, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_s_RangeError);
  __Pyx_GIVEREF(__pyx_n_s_RangeError);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_s_RangeError)) __PYX_ERR(1, 41, __pyx_L1_error);
  __pyx_t_3 = __Pyx_Import(__pyx_n_s_py_ballisticcalc_exceptions, __pyx_t_2, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_ZeroFindingError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ZeroFindingError, __pyx_t_2) < 0) __PYX_ERR(1, 41, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_RangeError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_RangeError, __pyx_t_2) < 0) __PYX_ERR(1, 41, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":42
 * from py_ballisticcalc.unit import Angular, Unit, Velocity, Distance, Energy, Weight
 * from py_ballisticcalc.exceptions import ZeroFindingError, RangeError
 * from py_ballisticcalc.constants import cMaxWindDistanceFeet             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_3 = PyList_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_n_s_cMaxWindDistanceFeet);
  __Pyx_GIVEREF(__pyx_n_s_cMaxWindDistanceFeet);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 0, __pyx_n_s_cMaxWindDistanceFeet)) __PYX_ERR(1, 42, __pyx_L1_error);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_py_ballisticcalc_constants, __pyx_t_3, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_cMaxWindDistanceFeet); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_cMaxWindDistanceFeet, __pyx_t_3) < 0) __PYX_ERR(1, 42, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "CyEulerBallistic/trajectory_calc.pyx":46
 * 
 * __all__ = (
 *     'EulerTrajectoryCalc',             # <<<<<<<<<<<<<<
 * )
 * 
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_all, __pyx_tuple__7) < 0) __PYX_ERR(1, 45, __pyx_L1_error)

  /* "CyEulerBallistic/trajectory_calc.pyx":1
 * """             # <<<<<<<<<<<<<<
 * # Total Score: 2394, Possible Score: 45200
 * # Total Non-Empty Lines: 452
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  if (__pyx_m) {
    if (__pyx_d && stringtab_initialized) {
      __Pyx_AddTraceback("init CyEulerBallistic.trajectory_calc", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    #if !CYTHON_USE_MODULE_STATE
    Py_CLEAR(__pyx_m);
    #else
    Py_DECREF(__pyx_m);
    if (pystate_addmodule_run) {
      PyObject *tp, *value, *tb;
      PyErr_Fetch(&tp, &value, &tb);
      PyState_RemoveModule(&__pyx_moduledef);
      PyErr_Restore(tp, value, tb);
    }
    #endif
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init CyEulerBallistic.trajectory_calc");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}
/* #### Code section: cleanup_globals ### */
/* #### Code section: cleanup_module ### */
/* #### Code section: main_method ### */
/* #### Code section: utility_code_pragmas ### */
#ifdef _MSC_VER
#pragma warning( push )
/* Warning 4127: conditional expression is constant
 * Cython uses constant conditional expressions to allow in inline functions to be optimized at
 * compile-time, so this warning is not useful
 */
#pragma warning( disable : 4127 )
#endif



/* #### Code section: utility_code_def ### */

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    int result;
    PyObject *exc_type;
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject *current_exception = tstate->current_exception;
    if (unlikely(!current_exception)) return 0;
    exc_type = (PyObject*) Py_TYPE(current_exception);
    if (exc_type == err) return 1;
#else
    exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
#endif
    #if CYTHON_AVOID_BORROWED_REFS
    Py_INCREF(exc_type);
    #endif
    if (unlikely(PyTuple_Check(err))) {
        result = __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    } else {
        result = __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
    }
    #if CYTHON_AVOID_BORROWED_REFS
    Py_DECREF(exc_type);
    #endif
    return result;
}
#endif

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject *tmp_value;
    assert(type == NULL || (value != NULL && type == (PyObject*) Py_TYPE(value)));
    if (value) {
        #if CYTHON_COMPILING_IN_CPYTHON
        if (unlikely(((PyBaseExceptionObject*) value)->traceback != tb))
        #endif
            PyException_SetTraceback(value, tb);
    }
    tmp_value = tstate->current_exception;
    tstate->current_exception = value;
    Py_XDECREF(tmp_value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
#else
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#endif
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject* exc_value;
    exc_value = tstate->current_exception;
    tstate->current_exception = 0;
    *value = exc_value;
    *type = NULL;
    *tb = NULL;
    if (exc_value) {
        *type = (PyObject*) Py_TYPE(exc_value);
        Py_INCREF(*type);
        #if CYTHON_COMPILING_IN_CPYTHON
        *tb = ((PyBaseExceptionObject*) exc_value)->traceback;
        Py_XINCREF(*tb);
        #else
        *tb = PyException_GetTraceback(exc_value);
        #endif
    }
#else
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#endif
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* PyObjectGetAttrStrNoError */
#if __PYX_LIMITED_VERSION_HEX < 0x030d00A1
static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
#endif
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if __PYX_LIMITED_VERSION_HEX >= 0x030d00A1
    (void) PyObject_GetOptionalAttr(obj, attr_name, &result);
    return result;
#else
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS && PY_VERSION_HEX >= 0x030700B1
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
#endif
}

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStrNoError(__pyx_b, name);
    if (unlikely(!result) && !PyErr_Occurred()) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* DictGetItem */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (!PyErr_Occurred()) {
            if (unlikely(PyTuple_Check(key))) {
                PyObject* args = PyTuple_Pack(1, key);
                if (likely(args)) {
                    PyErr_SetObject(PyExc_KeyError, args);
                    Py_DECREF(args);
                }
            } else {
                PyErr_SetObject(PyExc_KeyError, key);
            }
        }
        return NULL;
    }
    Py_INCREF(value);
    return value;
}
#endif

/* GetTopmostException */
#if CYTHON_USE_EXC_INFO_STACK && CYTHON_FAST_THREAD_STATE
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_value == NULL || exc_info->exc_value == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    PyObject *exc_value = exc_info->exc_value;
    if (exc_value == NULL || exc_value == Py_None) {
        *value = NULL;
        *type = NULL;
        *tb = NULL;
    } else {
        *value = exc_value;
        Py_INCREF(*value);
        *type = (PyObject*) Py_TYPE(exc_value);
        Py_INCREF(*type);
        *tb = PyException_GetTraceback(exc_value);
    }
  #elif CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
  #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
  #endif
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = tstate->exc_info;
    PyObject *tmp_value = exc_info->exc_value;
    exc_info->exc_value = value;
    Py_XDECREF(tmp_value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
  #else
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
  #endif
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type = NULL, *local_value, *local_tb = NULL;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
  #if PY_VERSION_HEX >= 0x030C00A6
    local_value = tstate->current_exception;
    tstate->current_exception = 0;
    if (likely(local_value)) {
        local_type = (PyObject*) Py_TYPE(local_value);
        Py_INCREF(local_type);
        local_tb = PyException_GetTraceback(local_value);
    }
  #else
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
  #endif
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE && PY_VERSION_HEX >= 0x030C00A6
    if (unlikely(tstate->current_exception))
#elif CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
      #if PY_VERSION_HEX >= 0x030B00a4
        tmp_value = exc_info->exc_value;
        exc_info->exc_value = local_value;
        tmp_type = NULL;
        tmp_tb = NULL;
        Py_XDECREF(local_type);
        Py_XDECREF(local_tb);
      #else
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
      #endif
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    #if PY_MAJOR_VERSION < 3
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    #else
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object")))
        return NULL;
    #endif
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    __Pyx_PyThreadState_declare
    CYTHON_UNUSED_VAR(cause);
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
      #if PY_VERSION_HEX >= 0x030C00A6
        PyException_SetTraceback(value, tb);
      #elif CYTHON_FAST_THREAD_STATE
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#else
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* TupleAndListFromArray */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE void __Pyx_copy_object_array(PyObject *const *CYTHON_RESTRICT src, PyObject** CYTHON_RESTRICT dest, Py_ssize_t length) {
    PyObject *v;
    Py_ssize_t i;
    for (i = 0; i < length; i++) {
        v = dest[i] = src[i];
        Py_INCREF(v);
    }
}
static CYTHON_INLINE PyObject *
__Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    if (n <= 0) {
        Py_INCREF(__pyx_empty_tuple);
        return __pyx_empty_tuple;
    }
    res = PyTuple_New(n);
    if (unlikely(res == NULL)) return NULL;
    __Pyx_copy_object_array(src, ((PyTupleObject*)res)->ob_item, n);
    return res;
}
static CYTHON_INLINE PyObject *
__Pyx_PyList_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    if (n <= 0) {
        return PyList_New(0);
    }
    res = PyList_New(n);
    if (unlikely(res == NULL)) return NULL;
    __Pyx_copy_object_array(src, ((PyListObject*)res)->ob_item, n);
    return res;
}
#endif

/* BytesEquals */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS && (PY_VERSION_HEX < 0x030B0000)
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
        #if CYTHON_PEP393_ENABLED
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
        #else
            hash1 = ((PyUnicodeObject*)s1)->hash;
            hash2 = ((PyUnicodeObject*)s2)->hash;
        #endif
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        #if PY_MAJOR_VERSION < 3
        Py_XDECREF(owned_ref);
        #endif
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* fastcall */
#if CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject * __Pyx_GetKwValue_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues, PyObject *s)
{
    Py_ssize_t i, n = PyTuple_GET_SIZE(kwnames);
    for (i = 0; i < n; i++)
    {
        if (s == PyTuple_GET_ITEM(kwnames, i)) return kwvalues[i];
    }
    for (i = 0; i < n; i++)
    {
        int eq = __Pyx_PyUnicode_Equals(s, PyTuple_GET_ITEM(kwnames, i), Py_EQ);
        if (unlikely(eq != 0)) {
            if (unlikely(eq < 0)) return NULL;
            return kwvalues[i];
        }
    }
    return NULL;
}
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000
CYTHON_UNUSED static PyObject *__Pyx_KwargsAsDict_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues) {
    Py_ssize_t i, nkwargs = PyTuple_GET_SIZE(kwnames);
    PyObject *dict;
    dict = PyDict_New();
    if (unlikely(!dict))
        return NULL;
    for (i=0; i<nkwargs; i++) {
        PyObject *key = PyTuple_GET_ITEM(kwnames, i);
        if (unlikely(PyDict_SetItem(dict, key, kwvalues[i]) < 0))
            goto bad;
    }
    return dict;
bad:
    Py_DECREF(dict);
    return NULL;
}
#endif
#endif

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject *const *kwvalues,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    int kwds_is_tuple = CYTHON_METH_FASTCALL && likely(PyTuple_Check(kwds));
    while (1) {
        Py_XDECREF(key); key = NULL;
        Py_XDECREF(value); value = NULL;
        if (kwds_is_tuple) {
            Py_ssize_t size;
#if CYTHON_ASSUME_SAFE_MACROS
            size = PyTuple_GET_SIZE(kwds);
#else
            size = PyTuple_Size(kwds);
            if (size < 0) goto bad;
#endif
            if (pos >= size) break;
#if CYTHON_AVOID_BORROWED_REFS
            key = __Pyx_PySequence_ITEM(kwds, pos);
            if (!key) goto bad;
#elif CYTHON_ASSUME_SAFE_MACROS
            key = PyTuple_GET_ITEM(kwds, pos);
#else
            key = PyTuple_GetItem(kwds, pos);
            if (!key) goto bad;
#endif
            value = kwvalues[pos];
            pos++;
        }
        else
        {
            if (!PyDict_Next(kwds, &pos, &key, &value)) break;
#if CYTHON_AVOID_BORROWED_REFS
            Py_INCREF(key);
#endif
        }
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
#if CYTHON_AVOID_BORROWED_REFS
            Py_INCREF(value);
            Py_DECREF(key);
#endif
            key = NULL;
            value = NULL;
            continue;
        }
#if !CYTHON_AVOID_BORROWED_REFS
        Py_INCREF(key);
#endif
        Py_INCREF(value);
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
#if CYTHON_AVOID_BORROWED_REFS
                    value = NULL;
#endif
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (__Pyx_PyUnicode_GET_LENGTH(**name) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key)
                );
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
#if CYTHON_AVOID_BORROWED_REFS
                    value = NULL;
#endif
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (__Pyx_PyUnicode_GET_LENGTH(**argname) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    Py_XDECREF(key);
    Py_XDECREF(value);
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    #if PY_MAJOR_VERSION < 3
    PyErr_Format(PyExc_TypeError,
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
    PyErr_Format(PyExc_TypeError,
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    Py_XDECREF(key);
    Py_XDECREF(value);
    return -1;
}

/* KeywordStringCheck */
static int __Pyx_CheckKeywordStrings(
    PyObject *kw,
    const char* function_name,
    int kw_allowed)
{
    PyObject* key = 0;
    Py_ssize_t pos = 0;
#if CYTHON_COMPILING_IN_PYPY
    if (!kw_allowed && PyDict_Next(kw, &pos, &key, 0))
        goto invalid_keyword;
    return 1;
#else
    if (CYTHON_METH_FASTCALL && likely(PyTuple_Check(kw))) {
        Py_ssize_t kwsize;
#if CYTHON_ASSUME_SAFE_MACROS
        kwsize = PyTuple_GET_SIZE(kw);
#else
        kwsize = PyTuple_Size(kw);
        if (kwsize < 0) return 0;
#endif
        if (unlikely(kwsize == 0))
            return 1;
        if (!kw_allowed) {
#if CYTHON_ASSUME_SAFE_MACROS
            key = PyTuple_GET_ITEM(kw, 0);
#else
            key = PyTuple_GetItem(kw, pos);
            if (!key) return 0;
#endif
            goto invalid_keyword;
        }
#if PY_VERSION_HEX < 0x03090000
        for (pos = 0; pos < kwsize; pos++) {
#if CYTHON_ASSUME_SAFE_MACROS
            key = PyTuple_GET_ITEM(kw, pos);
#else
            key = PyTuple_GetItem(kw, pos);
            if (!key) return 0;
#endif
            if (unlikely(!PyUnicode_Check(key)))
                goto invalid_keyword_type;
        }
#endif
        return 1;
    }
    while (PyDict_Next(kw, &pos, &key, 0)) {
        #if PY_MAJOR_VERSION < 3
        if (unlikely(!PyString_Check(key)))
        #endif
            if (unlikely(!PyUnicode_Check(key)))
                goto invalid_keyword_type;
    }
    if (!kw_allowed && unlikely(key))
        goto invalid_keyword;
    return 1;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    return 0;
#endif
invalid_keyword:
    #if PY_MAJOR_VERSION < 3
    PyErr_Format(PyExc_TypeError,
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
    PyErr_Format(PyExc_TypeError,
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
    return 0;
}

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && PY_VERSION_HEX < 0x030d0000
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#elif CYTHON_COMPILING_IN_LIMITED_API
    if (unlikely(!__pyx_m)) {
        return NULL;
    }
    result = PyObject_GetAttr(__pyx_m, name);
    if (likely(result)) {
        return result;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL && !CYTHON_VECTORCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    #if PY_MAJOR_VERSION < 3
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object"))) {
        return NULL;
    }
    #else
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object"))) {
        return NULL;
    }
    #endif
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = __Pyx_CyOrPyCFunction_GET_FUNCTION(func);
    self = __Pyx_CyOrPyCFunction_GET_SELF(func);
    #if PY_MAJOR_VERSION < 3
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    #else
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object")))
        return NULL;
    #endif
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectFastCall */
#if PY_VERSION_HEX < 0x03090000 || CYTHON_COMPILING_IN_LIMITED_API
static PyObject* __Pyx_PyObject_FastCall_fallback(PyObject *func, PyObject **args, size_t nargs, PyObject *kwargs) {
    PyObject *argstuple;
    PyObject *result = 0;
    size_t i;
    argstuple = PyTuple_New((Py_ssize_t)nargs);
    if (unlikely(!argstuple)) return NULL;
    for (i = 0; i < nargs; i++) {
        Py_INCREF(args[i]);
        if (__Pyx_PyTuple_SET_ITEM(argstuple, (Py_ssize_t)i, args[i]) < 0) goto bad;
    }
    result = __Pyx_PyObject_Call(func, argstuple, kwargs);
  bad:
    Py_DECREF(argstuple);
    return result;
}
#endif
static CYTHON_INLINE PyObject* __Pyx_PyObject_FastCallDict(PyObject *func, PyObject **args, size_t _nargs, PyObject *kwargs) {
    Py_ssize_t nargs = __Pyx_PyVectorcall_NARGS(_nargs);
#if CYTHON_COMPILING_IN_CPYTHON
    if (nargs == 0 && kwargs == NULL) {
        if (__Pyx_CyOrPyCFunction_Check(func) && likely( __Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_NOARGS))
            return __Pyx_PyObject_CallMethO(func, NULL);
    }
    else if (nargs == 1 && kwargs == NULL) {
        if (__Pyx_CyOrPyCFunction_Check(func) && likely( __Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_O))
            return __Pyx_PyObject_CallMethO(func, args[0]);
    }
#endif
    #if PY_VERSION_HEX < 0x030800B1
    #if CYTHON_FAST_PYCCALL
    if (PyCFunction_Check(func)) {
        if (kwargs) {
            return _PyCFunction_FastCallDict(func, args, nargs, kwargs);
        } else {
            return _PyCFunction_FastCallKeywords(func, args, nargs, NULL);
        }
    }
    #if PY_VERSION_HEX >= 0x030700A1
    if (!kwargs && __Pyx_IS_TYPE(func, &PyMethodDescr_Type)) {
        return _PyMethodDescr_FastCallKeywords(func, args, nargs, NULL);
    }
    #endif
    #endif
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs);
    }
    #endif
    #endif
    if (kwargs == NULL) {
        #if CYTHON_VECTORCALL
        #if PY_VERSION_HEX < 0x03090000
        vectorcallfunc f = _PyVectorcall_Function(func);
        #else
        vectorcallfunc f = PyVectorcall_Function(func);
        #endif
        if (f) {
            return f(func, args, (size_t)nargs, NULL);
        }
        #elif defined(__Pyx_CyFunction_USED) && CYTHON_BACKPORT_VECTORCALL
        if (__Pyx_CyFunction_CheckExact(func)) {
            __pyx_vectorcallfunc f = __Pyx_CyFunction_func_vectorcall(func);
            if (f) return f(func, args, (size_t)nargs, NULL);
        }
        #endif
    }
    if (nargs == 0) {
        return __Pyx_PyObject_Call(func, __pyx_empty_tuple, kwargs);
    }
    #if PY_VERSION_HEX >= 0x03090000 && !CYTHON_COMPILING_IN_LIMITED_API
    return PyObject_VectorcallDict(func, args, (size_t)nargs, kwargs);
    #else
    return __Pyx_PyObject_FastCall_fallback(func, args, (size_t)nargs, kwargs);
    #endif
}

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (unlikely(!j)) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PyMappingMethods *mm = Py_TYPE(o)->tp_as_mapping;
        PySequenceMethods *sm = Py_TYPE(o)->tp_as_sequence;
        if (mm && mm->mp_subscript) {
            PyObject *r, *key = PyInt_FromSsize_t(i);
            if (unlikely(!key)) return NULL;
            r = mm->mp_subscript(o, key);
            Py_DECREF(key);
            return r;
        }
        if (likely(sm && sm->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(sm->sq_length)) {
                Py_ssize_t l = sm->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return sm->sq_item(o, i);
        }
    }
#else
    if (is_list || !PyMapping_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* RaiseUnexpectedTypeError */
static int
__Pyx_RaiseUnexpectedTypeError(const char *expected, PyObject *obj)
{
    __Pyx_TypeName obj_type_name = __Pyx_PyType_GetName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError, "Expected %s, got " __Pyx_FMT_TYPENAME,
                 expected, obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return 0;
}

/* PyObjectCallOneArg */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *args[2] = {NULL, arg};
    return __Pyx_PyObject_FastCall(func, args+1, 1 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* CIntToDigits */
static const char DIGIT_PAIRS_10[2*10*10+1] = {
    "00010203040506070809"
    "10111213141516171819"
    "20212223242526272829"
    "30313233343536373839"
    "40414243444546474849"
    "50515253545556575859"
    "60616263646566676869"
    "70717273747576777879"
    "80818283848586878889"
    "90919293949596979899"
};
static const char DIGIT_PAIRS_8[2*8*8+1] = {
    "0001020304050607"
    "1011121314151617"
    "2021222324252627"
    "3031323334353637"
    "4041424344454647"
    "5051525354555657"
    "6061626364656667"
    "7071727374757677"
};
static const char DIGITS_HEX[2*16+1] = {
    "0123456789abcdef"
    "0123456789ABCDEF"
};

/* BuildPyUnicode */
static PyObject* __Pyx_PyUnicode_BuildFromAscii(Py_ssize_t ulength, char* chars, int clength,
                                                int prepend_sign, char padding_char) {
    PyObject *uval;
    Py_ssize_t uoffset = ulength - clength;
#if CYTHON_USE_UNICODE_INTERNALS
    Py_ssize_t i;
#if CYTHON_PEP393_ENABLED
    void *udata;
    uval = PyUnicode_New(ulength, 127);
    if (unlikely(!uval)) return NULL;
    udata = PyUnicode_DATA(uval);
#else
    Py_UNICODE *udata;
    uval = PyUnicode_FromUnicode(NULL, ulength);
    if (unlikely(!uval)) return NULL;
    udata = PyUnicode_AS_UNICODE(uval);
#endif
    if (uoffset > 0) {
        i = 0;
        if (prepend_sign) {
            __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, 0, '-');
            i++;
        }
        for (; i < uoffset; i++) {
            __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, i, padding_char);
        }
    }
    for (i=0; i < clength; i++) {
        __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, uoffset+i, chars[i]);
    }
#else
    {
        PyObject *sign = NULL, *padding = NULL;
        uval = NULL;
        if (uoffset > 0) {
            prepend_sign = !!prepend_sign;
            if (uoffset > prepend_sign) {
                padding = PyUnicode_FromOrdinal(padding_char);
                if (likely(padding) && uoffset > prepend_sign + 1) {
                    PyObject *tmp;
                    PyObject *repeat = PyInt_FromSsize_t(uoffset - prepend_sign);
                    if (unlikely(!repeat)) goto done_or_error;
                    tmp = PyNumber_Multiply(padding, repeat);
                    Py_DECREF(repeat);
                    Py_DECREF(padding);
                    padding = tmp;
                }
                if (unlikely(!padding)) goto done_or_error;
            }
            if (prepend_sign) {
                sign = PyUnicode_FromOrdinal('-');
                if (unlikely(!sign)) goto done_or_error;
            }
        }
        uval = PyUnicode_DecodeASCII(chars, clength, NULL);
        if (likely(uval) && padding) {
            PyObject *tmp = PyNumber_Add(padding, uval);
            Py_DECREF(uval);
            uval = tmp;
        }
        if (likely(uval) && sign) {
            PyObject *tmp = PyNumber_Add(sign, uval);
            Py_DECREF(uval);
            uval = tmp;
        }
done_or_error:
        Py_XDECREF(padding);
        Py_XDECREF(sign);
    }
#endif
    return uval;
}

/* CIntToPyUnicode */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_From_int(int value, Py_ssize_t width, char padding_char, char format_char) {
    char digits[sizeof(int)*3+2];
    char *dpos, *end = digits + sizeof(int)*3+2;
    const char *hex_digits = DIGITS_HEX;
    Py_ssize_t length, ulength;
    int prepend_sign, last_one_off;
    int remaining;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (format_char == 'X') {
        hex_digits += 16;
        format_char = 'x';
    }
    remaining = value;
    last_one_off = 0;
    dpos = end;
    do {
        int digit_pos;
        switch (format_char) {
        case 'o':
            digit_pos = abs((int)(remaining % (8*8)));
            remaining = (int) (remaining / (8*8));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_8 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 8);
            break;
        case 'd':
            digit_pos = abs((int)(remaining % (10*10)));
            remaining = (int) (remaining / (10*10));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_10 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 10);
            break;
        case 'x':
            *(--dpos) = hex_digits[abs((int)(remaining % 16))];
            remaining = (int) (remaining / 16);
            break;
        default:
            assert(0);
            break;
        }
    } while (unlikely(remaining != 0));
    assert(!last_one_off || *dpos == '0');
    dpos += last_one_off;
    length = end - dpos;
    ulength = length;
    prepend_sign = 0;
    if (!is_unsigned && value <= neg_one) {
        if (padding_char == ' ' || width <= length + 1) {
            *(--dpos) = '-';
            ++length;
        } else {
            prepend_sign = 1;
        }
        ++ulength;
    }
    if (width > ulength) {
        ulength = width;
    }
    if (ulength == 1) {
        return PyUnicode_FromOrdinal(*dpos);
    }
    return __Pyx_PyUnicode_BuildFromAscii(ulength, dpos, (int) length, prepend_sign, padding_char);
}

/* PyObjectSetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_setattr))
        return tp->tp_setattr(obj, PyString_AS_STRING(attr_name), value);
#endif
    return PyObject_SetAttr(obj, attr_name, value);
}
#endif

/* FixUpExtensionType */
#if CYTHON_USE_TYPE_SPECS
static int __Pyx_fix_up_extension_type_from_spec(PyType_Spec *spec, PyTypeObject *type) {
#if PY_VERSION_HEX > 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    CYTHON_UNUSED_VAR(spec);
    CYTHON_UNUSED_VAR(type);
#else
    const PyType_Slot *slot = spec->slots;
    while (slot && slot->slot && slot->slot != Py_tp_members)
        slot++;
    if (slot && slot->slot == Py_tp_members) {
        int changed = 0;
#if !(PY_VERSION_HEX <= 0x030900b1 && CYTHON_COMPILING_IN_CPYTHON)
        const
#endif
            PyMemberDef *memb = (PyMemberDef*) slot->pfunc;
        while (memb && memb->name) {
            if (memb->name[0] == '_' && memb->name[1] == '_') {
#if PY_VERSION_HEX < 0x030900b1
                if (strcmp(memb->name, "__weaklistoffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_weaklistoffset = memb->offset;
                    changed = 1;
                }
                else if (strcmp(memb->name, "__dictoffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_dictoffset = memb->offset;
                    changed = 1;
                }
#if CYTHON_METH_FASTCALL
                else if (strcmp(memb->name, "__vectorcalloffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
#if PY_VERSION_HEX >= 0x030800b4
                    type->tp_vectorcall_offset = memb->offset;
#else
                    type->tp_print = (printfunc) memb->offset;
#endif
                    changed = 1;
                }
#endif
#else
                if ((0));
#endif
#if PY_VERSION_HEX <= 0x030900b1 && CYTHON_COMPILING_IN_CPYTHON
                else if (strcmp(memb->name, "__module__") == 0) {
                    PyObject *descr;
                    assert(memb->type == T_OBJECT);
                    assert(memb->flags == 0 || memb->flags == READONLY);
                    descr = PyDescr_NewMember(type, memb);
                    if (unlikely(!descr))
                        return -1;
                    if (unlikely(PyDict_SetItem(type->tp_dict, PyDescr_NAME(descr), descr) < 0)) {
                        Py_DECREF(descr);
                        return -1;
                    }
                    Py_DECREF(descr);
                    changed = 1;
                }
#endif
            }
            memb++;
        }
        if (changed)
            PyType_Modified(type);
    }
#endif
    return 0;
}
#endif

/* PyObjectCallNoArg */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
    PyObject *arg[2] = {NULL, NULL};
    return __Pyx_PyObject_FastCall(func, arg + 1, 0 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* PyObjectGetMethod */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method) {
    PyObject *attr;
#if CYTHON_UNPACK_METHODS && CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_PYTYPE_LOOKUP
    __Pyx_TypeName type_name;
    PyTypeObject *tp = Py_TYPE(obj);
    PyObject *descr;
    descrgetfunc f = NULL;
    PyObject **dictptr, *dict;
    int meth_found = 0;
    assert (*method == NULL);
    if (unlikely(tp->tp_getattro != PyObject_GenericGetAttr)) {
        attr = __Pyx_PyObject_GetAttrStr(obj, name);
        goto try_unpack;
    }
    if (unlikely(tp->tp_dict == NULL) && unlikely(PyType_Ready(tp) < 0)) {
        return 0;
    }
    descr = _PyType_Lookup(tp, name);
    if (likely(descr != NULL)) {
        Py_INCREF(descr);
#if defined(Py_TPFLAGS_METHOD_DESCRIPTOR) && Py_TPFLAGS_METHOD_DESCRIPTOR
        if (__Pyx_PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_METHOD_DESCRIPTOR))
#elif PY_MAJOR_VERSION >= 3
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || __Pyx_IS_TYPE(descr, &PyMethodDescr_Type) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr) || __Pyx_IS_TYPE(descr, &PyMethodDescr_Type)))
        #endif
#else
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr)))
        #endif
#endif
        {
            meth_found = 1;
        } else {
            f = Py_TYPE(descr)->tp_descr_get;
            if (f != NULL && PyDescr_IsData(descr)) {
                attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
                Py_DECREF(descr);
                goto try_unpack;
            }
        }
    }
    dictptr = _PyObject_GetDictPtr(obj);
    if (dictptr != NULL && (dict = *dictptr) != NULL) {
        Py_INCREF(dict);
        attr = __Pyx_PyDict_GetItemStr(dict, name);
        if (attr != NULL) {
            Py_INCREF(attr);
            Py_DECREF(dict);
            Py_XDECREF(descr);
            goto try_unpack;
        }
        Py_DECREF(dict);
    }
    if (meth_found) {
        *method = descr;
        return 1;
    }
    if (f != NULL) {
        attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
        Py_DECREF(descr);
        goto try_unpack;
    }
    if (likely(descr != NULL)) {
        *method = descr;
        return 0;
    }
    type_name = __Pyx_PyType_GetName(tp);
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'" __Pyx_FMT_TYPENAME "' object has no attribute '%U'",
                 type_name, name);
#else
                 "'" __Pyx_FMT_TYPENAME "' object has no attribute '%.400s'",
                 type_name, PyString_AS_STRING(name));
#endif
    __Pyx_DECREF_TypeName(type_name);
    return 0;
#else
    attr = __Pyx_PyObject_GetAttrStr(obj, name);
    goto try_unpack;
#endif
try_unpack:
#if CYTHON_UNPACK_METHODS
    if (likely(attr) && PyMethod_Check(attr) && likely(PyMethod_GET_SELF(attr) == obj)) {
        PyObject *function = PyMethod_GET_FUNCTION(attr);
        Py_INCREF(function);
        Py_DECREF(attr);
        *method = function;
        return 1;
    }
#endif
    *method = attr;
    return 0;
}

/* PyObjectCallMethod0 */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name) {
    PyObject *method = NULL, *result = NULL;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_CallOneArg(method, obj);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) goto bad;
    result = __Pyx_PyObject_CallNoArg(method);
    Py_DECREF(method);
bad:
    return result;
}

/* ValidateBasesTuple */
#if CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_USE_TYPE_SPECS
static int __Pyx_validate_bases_tuple(const char *type_name, Py_ssize_t dictoffset, PyObject *bases) {
    Py_ssize_t i, n;
#if CYTHON_ASSUME_SAFE_MACROS
    n = PyTuple_GET_SIZE(bases);
#else
    n = PyTuple_Size(bases);
    if (n < 0) return -1;
#endif
    for (i = 1; i < n; i++)
    {
#if CYTHON_AVOID_BORROWED_REFS
        PyObject *b0 = PySequence_GetItem(bases, i);
        if (!b0) return -1;
#elif CYTHON_ASSUME_SAFE_MACROS
        PyObject *b0 = PyTuple_GET_ITEM(bases, i);
#else
        PyObject *b0 = PyTuple_GetItem(bases, i);
        if (!b0) return -1;
#endif
        PyTypeObject *b;
#if PY_MAJOR_VERSION < 3
        if (PyClass_Check(b0))
        {
            PyErr_Format(PyExc_TypeError, "base class '%.200s' is an old-style class",
                         PyString_AS_STRING(((PyClassObject*)b0)->cl_name));
#if CYTHON_AVOID_BORROWED_REFS
            Py_DECREF(b0);
#endif
            return -1;
        }
#endif
        b = (PyTypeObject*) b0;
        if (!__Pyx_PyType_HasFeature(b, Py_TPFLAGS_HEAPTYPE))
        {
            __Pyx_TypeName b_name = __Pyx_PyType_GetName(b);
            PyErr_Format(PyExc_TypeError,
                "base class '" __Pyx_FMT_TYPENAME "' is not a heap type", b_name);
            __Pyx_DECREF_TypeName(b_name);
#if CYTHON_AVOID_BORROWED_REFS
            Py_DECREF(b0);
#endif
            return -1;
        }
        if (dictoffset == 0)
        {
            Py_ssize_t b_dictoffset = 0;
#if CYTHON_USE_TYPE_SLOTS || CYTHON_COMPILING_IN_PYPY
            b_dictoffset = b->tp_dictoffset;
#else
            PyObject *py_b_dictoffset = PyObject_GetAttrString((PyObject*)b, "__dictoffset__");
            if (!py_b_dictoffset) goto dictoffset_return;
            b_dictoffset = PyLong_AsSsize_t(py_b_dictoffset);
            Py_DECREF(py_b_dictoffset);
            if (b_dictoffset == -1 && PyErr_Occurred()) goto dictoffset_return;
#endif
            if (b_dictoffset) {
                {
                    __Pyx_TypeName b_name = __Pyx_PyType_GetName(b);
                    PyErr_Format(PyExc_TypeError,
                        "extension type '%.200s' has no __dict__ slot, "
                        "but base type '" __Pyx_FMT_TYPENAME "' has: "
                        "either add 'cdef dict __dict__' to the extension type "
                        "or add '__slots__ = [...]' to the base type",
                        type_name, b_name);
                    __Pyx_DECREF_TypeName(b_name);
                }
#if !(CYTHON_USE_TYPE_SLOTS || CYTHON_COMPILING_IN_PYPY)
              dictoffset_return:
#endif
#if CYTHON_AVOID_BORROWED_REFS
                Py_DECREF(b0);
#endif
                return -1;
            }
        }
#if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(b0);
#endif
    }
    return 0;
}
#endif

/* PyType_Ready */
static int __Pyx_PyType_Ready(PyTypeObject *t) {
#if CYTHON_USE_TYPE_SPECS || !(CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_LIMITED_API) || defined(PYSTON_MAJOR_VERSION)
    (void)__Pyx_PyObject_CallMethod0;
#if CYTHON_USE_TYPE_SPECS
    (void)__Pyx_validate_bases_tuple;
#endif
    return PyType_Ready(t);
#else
    int r;
    PyObject *bases = __Pyx_PyType_GetSlot(t, tp_bases, PyObject*);
    if (bases && unlikely(__Pyx_validate_bases_tuple(t->tp_name, t->tp_dictoffset, bases) == -1))
        return -1;
#if PY_VERSION_HEX >= 0x03050000 && !defined(PYSTON_MAJOR_VERSION)
    {
        int gc_was_enabled;
    #if PY_VERSION_HEX >= 0x030A00b1
        gc_was_enabled = PyGC_Disable();
        (void)__Pyx_PyObject_CallMethod0;
    #else
        PyObject *ret, *py_status;
        PyObject *gc = NULL;
        #if PY_VERSION_HEX >= 0x030700a1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM+0 >= 0x07030400)
        gc = PyImport_GetModule(__pyx_kp_u_gc);
        #endif
        if (unlikely(!gc)) gc = PyImport_Import(__pyx_kp_u_gc);
        if (unlikely(!gc)) return -1;
        py_status = __Pyx_PyObject_CallMethod0(gc, __pyx_kp_u_isenabled);
        if (unlikely(!py_status)) {
            Py_DECREF(gc);
            return -1;
        }
        gc_was_enabled = __Pyx_PyObject_IsTrue(py_status);
        Py_DECREF(py_status);
        if (gc_was_enabled > 0) {
            ret = __Pyx_PyObject_CallMethod0(gc, __pyx_kp_u_disable);
            if (unlikely(!ret)) {
                Py_DECREF(gc);
                return -1;
            }
            Py_DECREF(ret);
        } else if (unlikely(gc_was_enabled == -1)) {
            Py_DECREF(gc);
            return -1;
        }
    #endif
        t->tp_flags |= Py_TPFLAGS_HEAPTYPE;
#if PY_VERSION_HEX >= 0x030A0000
        t->tp_flags |= Py_TPFLAGS_IMMUTABLETYPE;
#endif
#else
        (void)__Pyx_PyObject_CallMethod0;
#endif
    r = PyType_Ready(t);
#if PY_VERSION_HEX >= 0x03050000 && !defined(PYSTON_MAJOR_VERSION)
        t->tp_flags &= ~Py_TPFLAGS_HEAPTYPE;
    #if PY_VERSION_HEX >= 0x030A00b1
        if (gc_was_enabled)
            PyGC_Enable();
    #else
        if (gc_was_enabled) {
            PyObject *tp, *v, *tb;
            PyErr_Fetch(&tp, &v, &tb);
            ret = __Pyx_PyObject_CallMethod0(gc, __pyx_kp_u_enable);
            if (likely(ret || r == -1)) {
                Py_XDECREF(ret);
                PyErr_Restore(tp, v, tb);
            } else {
                Py_XDECREF(tp);
                Py_XDECREF(v);
                Py_XDECREF(tb);
                r = -1;
            }
        }
        Py_DECREF(gc);
    #endif
    }
#endif
    return r;
#endif
}

/* PyObject_GenericGetAttrNoDict */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    __Pyx_TypeName type_name = __Pyx_PyType_GetName(tp);
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'" __Pyx_FMT_TYPENAME "' object has no attribute '%U'",
                 type_name, attr_name);
#else
                 "'" __Pyx_FMT_TYPENAME "' object has no attribute '%.400s'",
                 type_name, PyString_AS_STRING(attr_name));
#endif
    __Pyx_DECREF_TypeName(type_name);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* SetVTable */
static int __Pyx_SetVtable(PyTypeObject *type, void *vtable) {
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
    if (unlikely(!ob))
        goto bad;
#if CYTHON_COMPILING_IN_LIMITED_API
    if (unlikely(PyObject_SetAttr((PyObject *) type, __pyx_n_s_pyx_vtable, ob) < 0))
#else
    if (unlikely(PyDict_SetItem(type->tp_dict, __pyx_n_s_pyx_vtable, ob) < 0))
#endif
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* GetVTable */
static void* __Pyx_GetVtable(PyTypeObject *type) {
    void* ptr;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *ob = PyObject_GetAttr((PyObject *)type, __pyx_n_s_pyx_vtable);
#else
    PyObject *ob = PyObject_GetItem(type->tp_dict, __pyx_n_s_pyx_vtable);
#endif
    if (!ob)
        goto bad;
    ptr = PyCapsule_GetPointer(ob, 0);
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* MergeVTables */
#if !CYTHON_COMPILING_IN_LIMITED_API
static int __Pyx_MergeVtables(PyTypeObject *type) {
    int i;
    void** base_vtables;
    __Pyx_TypeName tp_base_name;
    __Pyx_TypeName base_name;
    void* unknown = (void*)-1;
    PyObject* bases = type->tp_bases;
    int base_depth = 0;
    {
        PyTypeObject* base = type->tp_base;
        while (base) {
            base_depth += 1;
            base = base->tp_base;
        }
    }
    base_vtables = (void**) malloc(sizeof(void*) * (size_t)(base_depth + 1));
    base_vtables[0] = unknown;
    for (i = 1; i < PyTuple_GET_SIZE(bases); i++) {
        void* base_vtable = __Pyx_GetVtable(((PyTypeObject*)PyTuple_GET_ITEM(bases, i)));
        if (base_vtable != NULL) {
            int j;
            PyTypeObject* base = type->tp_base;
            for (j = 0; j < base_depth; j++) {
                if (base_vtables[j] == unknown) {
                    base_vtables[j] = __Pyx_GetVtable(base);
                    base_vtables[j + 1] = unknown;
                }
                if (base_vtables[j] == base_vtable) {
                    break;
                } else if (base_vtables[j] == NULL) {
                    goto bad;
                }
                base = base->tp_base;
            }
        }
    }
    PyErr_Clear();
    free(base_vtables);
    return 0;
bad:
    tp_base_name = __Pyx_PyType_GetName(type->tp_base);
    base_name = __Pyx_PyType_GetName((PyTypeObject*)PyTuple_GET_ITEM(bases, i));
    PyErr_Format(PyExc_TypeError,
        "multiple bases have vtable conflict: '" __Pyx_FMT_TYPENAME "' and '" __Pyx_FMT_TYPENAME "'", tp_base_name, base_name);
    __Pyx_DECREF_TypeName(tp_base_name);
    __Pyx_DECREF_TypeName(base_name);
    free(base_vtables);
    return -1;
}
#endif

/* SetupReduce */
#if !CYTHON_COMPILING_IN_LIMITED_API
static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStrNoError(meth, __pyx_n_s_name);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_getstate = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
    PyObject *getstate = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    getstate = _PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate);
#else
    getstate = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_getstate);
    if (!getstate && PyErr_Occurred()) {
        goto __PYX_BAD;
    }
#endif
    if (getstate) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_getstate = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_getstate);
#else
        object_getstate = __Pyx_PyObject_GetAttrStrNoError((PyObject*)&PyBaseObject_Type, __pyx_n_s_getstate);
        if (!object_getstate && PyErr_Occurred()) {
            goto __PYX_BAD;
        }
#endif
        if (object_getstate != getstate) {
            goto __PYX_GOOD;
        }
    }
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_reduce_cython);
            if (likely(reduce_cython)) {
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
            } else if (reduce == object_reduce || PyErr_Occurred()) {
                goto __PYX_BAD;
            }
            setstate = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_setstate_cython);
                if (likely(setstate_cython)) {
                    ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                    ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                } else if (!setstate || PyErr_Occurred()) {
                    goto __PYX_BAD;
                }
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto __PYX_GOOD;
__PYX_BAD:
    if (!PyErr_Occurred()) {
        __Pyx_TypeName type_obj_name =
            __Pyx_PyType_GetName((PyTypeObject*)type_obj);
        PyErr_Format(PyExc_RuntimeError,
            "Unable to initialize pickling for " __Pyx_FMT_TYPENAME, type_obj_name);
        __Pyx_DECREF_TypeName(type_obj_name);
    }
    ret = -1;
__PYX_GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
    Py_XDECREF(object_getstate);
    Py_XDECREF(getstate);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}
#endif

/* PyObject_GenericGetAttr */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name) {
    if (unlikely(Py_TYPE(obj)->tp_dictoffset)) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    return __Pyx_PyObject_GenericGetAttrNoDict(obj, attr_name);
}
#endif

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType_3_0_11
#define __PYX_HAVE_RT_ImportType_3_0_11
static PyTypeObject *__Pyx_ImportType_3_0_11(PyObject *module, const char *module_name, const char *class_name,
    size_t size, size_t alignment, enum __Pyx_ImportType_CheckSize_3_0_11 check_size)
{
    PyObject *result = 0;
    char warning[200];
    Py_ssize_t basicsize;
    Py_ssize_t itemsize;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *py_basicsize;
    PyObject *py_itemsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#if !CYTHON_COMPILING_IN_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
    itemsize = ((PyTypeObject *)result)->tp_itemsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
    py_itemsize = PyObject_GetAttrString(result, "__itemsize__");
    if (!py_itemsize)
        goto bad;
    itemsize = PyLong_AsSsize_t(py_itemsize);
    Py_DECREF(py_itemsize);
    py_itemsize = 0;
    if (itemsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (itemsize) {
        if (size % alignment) {
            alignment = size % alignment;
        }
        if (itemsize < (Py_ssize_t)alignment)
            itemsize = (Py_ssize_t)alignment;
    }
    if ((size_t)(basicsize + itemsize) < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize+itemsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error_3_0_11 &&
            ((size_t)basicsize > size || (size_t)(basicsize + itemsize) < size)) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd-%zd from PyObject",
            module_name, class_name, size, basicsize, basicsize+itemsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn_3_0_11 && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *module = 0;
    PyObject *empty_dict = 0;
    PyObject *empty_list = 0;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (unlikely(!py_import))
        goto bad;
    if (!from_list) {
        empty_list = PyList_New(0);
        if (unlikely(!empty_list))
            goto bad;
        from_list = empty_list;
    }
    #endif
    empty_dict = PyDict_New();
    if (unlikely(!empty_dict))
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.') != NULL) {
                module = PyImport_ImportModuleLevelObject(
                    name, __pyx_d, empty_dict, from_list, 1);
                if (unlikely(!module)) {
                    if (unlikely(!PyErr_ExceptionMatches(PyExc_ImportError)))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (unlikely(!py_level))
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, __pyx_d, empty_dict, from_list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, __pyx_d, empty_dict, from_list, level);
            #endif
        }
    }
bad:
    Py_XDECREF(empty_dict);
    Py_XDECREF(empty_list);
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    return module;
}

/* ImportDottedModule */
#if PY_MAJOR_VERSION >= 3
static PyObject *__Pyx__ImportDottedModule_Error(PyObject *name, PyObject *parts_tuple, Py_ssize_t count) {
    PyObject *partial_name = NULL, *slice = NULL, *sep = NULL;
    if (unlikely(PyErr_Occurred())) {
        PyErr_Clear();
    }
    if (likely(PyTuple_GET_SIZE(parts_tuple) == count)) {
        partial_name = name;
    } else {
        slice = PySequence_GetSlice(parts_tuple, 0, count);
        if (unlikely(!slice))
            goto bad;
        sep = PyUnicode_FromStringAndSize(".", 1);
        if (unlikely(!sep))
            goto bad;
        partial_name = PyUnicode_Join(sep, slice);
    }
    PyErr_Format(
#if PY_MAJOR_VERSION < 3
        PyExc_ImportError,
        "No module named '%s'", PyString_AS_STRING(partial_name));
#else
#if PY_VERSION_HEX >= 0x030600B1
        PyExc_ModuleNotFoundError,
#else
        PyExc_ImportError,
#endif
        "No module named '%U'", partial_name);
#endif
bad:
    Py_XDECREF(sep);
    Py_XDECREF(slice);
    Py_XDECREF(partial_name);
    return NULL;
}
#endif
#if PY_MAJOR_VERSION >= 3
static PyObject *__Pyx__ImportDottedModule_Lookup(PyObject *name) {
    PyObject *imported_module;
#if PY_VERSION_HEX < 0x030700A1 || (CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM  < 0x07030400)
    PyObject *modules = PyImport_GetModuleDict();
    if (unlikely(!modules))
        return NULL;
    imported_module = __Pyx_PyDict_GetItemStr(modules, name);
    Py_XINCREF(imported_module);
#else
    imported_module = PyImport_GetModule(name);
#endif
    return imported_module;
}
#endif
#if PY_MAJOR_VERSION >= 3
static PyObject *__Pyx_ImportDottedModule_WalkParts(PyObject *module, PyObject *name, PyObject *parts_tuple) {
    Py_ssize_t i, nparts;
    nparts = PyTuple_GET_SIZE(parts_tuple);
    for (i=1; i < nparts && module; i++) {
        PyObject *part, *submodule;
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        part = PyTuple_GET_ITEM(parts_tuple, i);
#else
        part = PySequence_ITEM(parts_tuple, i);
#endif
        submodule = __Pyx_PyObject_GetAttrStrNoError(module, part);
#if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
        Py_DECREF(part);
#endif
        Py_DECREF(module);
        module = submodule;
    }
    if (unlikely(!module)) {
        return __Pyx__ImportDottedModule_Error(name, parts_tuple, i);
    }
    return module;
}
#endif
static PyObject *__Pyx__ImportDottedModule(PyObject *name, PyObject *parts_tuple) {
#if PY_MAJOR_VERSION < 3
    PyObject *module, *from_list, *star = __pyx_n_s__5;
    CYTHON_UNUSED_VAR(parts_tuple);
    from_list = PyList_New(1);
    if (unlikely(!from_list))
        return NULL;
    Py_INCREF(star);
    PyList_SET_ITEM(from_list, 0, star);
    module = __Pyx_Import(name, from_list, 0);
    Py_DECREF(from_list);
    return module;
#else
    PyObject *imported_module;
    PyObject *module = __Pyx_Import(name, NULL, 0);
    if (!parts_tuple || unlikely(!module))
        return module;
    imported_module = __Pyx__ImportDottedModule_Lookup(name);
    if (likely(imported_module)) {
        Py_DECREF(module);
        return imported_module;
    }
    PyErr_Clear();
    return __Pyx_ImportDottedModule_WalkParts(module, name, parts_tuple);
#endif
}
static PyObject *__Pyx_ImportDottedModule(PyObject *name, PyObject *parts_tuple) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030400B1
    PyObject *module = __Pyx__ImportDottedModule_Lookup(name);
    if (likely(module)) {
        PyObject *spec = __Pyx_PyObject_GetAttrStrNoError(module, __pyx_n_s_spec);
        if (likely(spec)) {
            PyObject *unsafe = __Pyx_PyObject_GetAttrStrNoError(spec, __pyx_n_s_initializing);
            if (likely(!unsafe || !__Pyx_PyObject_IsTrue(unsafe))) {
                Py_DECREF(spec);
                spec = NULL;
            }
            Py_XDECREF(unsafe);
        }
        if (likely(!spec)) {
            PyErr_Clear();
            return module;
        }
        Py_DECREF(spec);
        Py_DECREF(module);
    } else if (PyErr_Occurred()) {
        PyErr_Clear();
    }
#endif
    return __Pyx__ImportDottedModule(name, parts_tuple);
}

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        const char* module_name_str = 0;
        PyObject* module_name = 0;
        PyObject* module_dot = 0;
        PyObject* full_name = 0;
        PyErr_Clear();
        module_name_str = PyModule_GetName(module);
        if (unlikely(!module_name_str)) { goto modbad; }
        module_name = PyUnicode_FromString(module_name_str);
        if (unlikely(!module_name)) { goto modbad; }
        module_dot = PyUnicode_Concat(module_name, __pyx_kp_u__6);
        if (unlikely(!module_dot)) { goto modbad; }
        full_name = PyUnicode_Concat(module_dot, name);
        if (unlikely(!full_name)) { goto modbad; }
        #if PY_VERSION_HEX < 0x030700A1 || (CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM  < 0x07030400)
        {
            PyObject *modules = PyImport_GetModuleDict();
            if (unlikely(!modules))
                goto modbad;
            value = PyObject_GetItem(modules, full_name);
        }
        #else
        value = PyImport_GetModule(full_name);
        #endif
      modbad:
        Py_XDECREF(full_name);
        Py_XDECREF(module_dot);
        Py_XDECREF(module_name);
    }
    if (unlikely(!value)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    CYTHON_MAYBE_UNUSED_VAR(tstate);
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStrNoError(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        (void) PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
#if !CYTHON_COMPILING_IN_LIMITED_API
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}
#endif

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
#if PY_VERSION_HEX >= 0x030b00a6 && !CYTHON_COMPILING_IN_LIMITED_API
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_PyCode_Replace_For_AddTraceback(PyObject *code, PyObject *scratch_dict,
                                                       PyObject *firstlineno, PyObject *name) {
    PyObject *replace = NULL;
    if (unlikely(PyDict_SetItemString(scratch_dict, "co_firstlineno", firstlineno))) return NULL;
    if (unlikely(PyDict_SetItemString(scratch_dict, "co_name", name))) return NULL;
    replace = PyObject_GetAttrString(code, "replace");
    if (likely(replace)) {
        PyObject *result;
        result = PyObject_Call(replace, __pyx_empty_tuple, scratch_dict);
        Py_DECREF(replace);
        return result;
    }
    PyErr_Clear();
    #if __PYX_LIMITED_VERSION_HEX < 0x030780000
    {
        PyObject *compiled = NULL, *result = NULL;
        if (unlikely(PyDict_SetItemString(scratch_dict, "code", code))) return NULL;
        if (unlikely(PyDict_SetItemString(scratch_dict, "type", (PyObject*)(&PyType_Type)))) return NULL;
        compiled = Py_CompileString(
            "out = type(code)(\n"
            "  code.co_argcount, code.co_kwonlyargcount, code.co_nlocals, code.co_stacksize,\n"
            "  code.co_flags, code.co_code, code.co_consts, code.co_names,\n"
            "  code.co_varnames, code.co_filename, co_name, co_firstlineno,\n"
            "  code.co_lnotab)\n", "<dummy>", Py_file_input);
        if (!compiled) return NULL;
        result = PyEval_EvalCode(compiled, scratch_dict, scratch_dict);
        Py_DECREF(compiled);
        if (!result) PyErr_Print();
        Py_DECREF(result);
        result = PyDict_GetItemString(scratch_dict, "out");
        if (result) Py_INCREF(result);
        return result;
    }
    #else
    return NULL;
    #endif
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyObject *code_object = NULL, *py_py_line = NULL, *py_funcname = NULL, *dict = NULL;
    PyObject *replace = NULL, *getframe = NULL, *frame = NULL;
    PyObject *exc_type, *exc_value, *exc_traceback;
    int success = 0;
    if (c_line) {
        (void) __pyx_cfilenm;
        (void) __Pyx_CLineForTraceback(__Pyx_PyThreadState_Current, c_line);
    }
    PyErr_Fetch(&exc_type, &exc_value, &exc_traceback);
    code_object = Py_CompileString("_getframe()", filename, Py_eval_input);
    if (unlikely(!code_object)) goto bad;
    py_py_line = PyLong_FromLong(py_line);
    if (unlikely(!py_py_line)) goto bad;
    py_funcname = PyUnicode_FromString(funcname);
    if (unlikely(!py_funcname)) goto bad;
    dict = PyDict_New();
    if (unlikely(!dict)) goto bad;
    {
        PyObject *old_code_object = code_object;
        code_object = __Pyx_PyCode_Replace_For_AddTraceback(code_object, dict, py_py_line, py_funcname);
        Py_DECREF(old_code_object);
    }
    if (unlikely(!code_object)) goto bad;
    getframe = PySys_GetObject("_getframe");
    if (unlikely(!getframe)) goto bad;
    if (unlikely(PyDict_SetItemString(dict, "_getframe", getframe))) goto bad;
    frame = PyEval_EvalCode(code_object, dict, dict);
    if (unlikely(!frame) || frame == Py_None) goto bad;
    success = 1;
  bad:
    PyErr_Restore(exc_type, exc_value, exc_traceback);
    Py_XDECREF(code_object);
    Py_XDECREF(py_py_line);
    Py_XDECREF(py_funcname);
    Py_XDECREF(dict);
    Py_XDECREF(replace);
    if (success) {
        PyTraceBack_Here(
            (struct _frame*)frame);
    }
    Py_XDECREF(frame);
}
#else
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = NULL;
    PyObject *py_funcname = NULL;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_srcfile = NULL;
    py_srcfile = PyString_FromString(filename);
    if (!py_srcfile) goto bad;
    #endif
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        funcname = PyUnicode_AsUTF8(py_funcname);
        if (!funcname) goto bad;
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        if (!py_funcname) goto bad;
        #endif
    }
    #if PY_MAJOR_VERSION < 3
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    #else
    py_code = PyCode_NewEmpty(filename, funcname, py_line);
    #endif
    Py_XDECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_funcname);
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_srcfile);
    #endif
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject *ptype, *pvalue, *ptraceback;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) {
            /* If the code object creation fails, then we should clear the
               fetched exception references and propagate the new exception */
            Py_XDECREF(ptype);
            Py_XDECREF(pvalue);
            Py_XDECREF(ptraceback);
            goto bad;
        }
        __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}
#endif

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if ((sizeof(int) < sizeof(long))) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    }
#endif
    if (unlikely(!PyLong_Check(x))) {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(int, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(int) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 2 * PyLong_SHIFT)) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(int) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 3 * PyLong_SHIFT)) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(int) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 4 * PyLong_SHIFT)) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (int) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(int) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
        } else if ((sizeof(int) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(int, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(int) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(int) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(int) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 4 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(int) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 4 * PyLong_SHIFT)) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(int) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
        } else if ((sizeof(int) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
        }
    }
    {
        int val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (int) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (int) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (int) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (int) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(int) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((int) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(int) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((int) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((int) 1) << (sizeof(int) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (int) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL;
        PyObject *py_bytes = NULL, *arg_tuple = NULL, *kwds = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(int));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        arg_tuple = PyTuple_Pack(2, py_bytes, order_str);
        if (!arg_tuple) goto limited_bad;
        if (!is_unsigned) {
            kwds = PyDict_New();
            if (!kwds) goto limited_bad;
            if (PyDict_SetItemString(kwds, "signed", __Pyx_NewRef(Py_True))) goto limited_bad;
        }
        result = PyObject_Call(from_bytes, arg_tuple, kwds);
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(arg_tuple);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL;
        PyObject *py_bytes = NULL, *arg_tuple = NULL, *kwds = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(long));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        arg_tuple = PyTuple_Pack(2, py_bytes, order_str);
        if (!arg_tuple) goto limited_bad;
        if (!is_unsigned) {
            kwds = PyDict_New();
            if (!kwds) goto limited_bad;
            if (PyDict_SetItemString(kwds, "signed", __Pyx_NewRef(Py_True))) goto limited_bad;
        }
        result = PyObject_Call(from_bytes, arg_tuple, kwds);
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(arg_tuple);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* FormatTypeName */
#if CYTHON_COMPILING_IN_LIMITED_API
static __Pyx_TypeName
__Pyx_PyType_GetName(PyTypeObject* tp)
{
    PyObject *name = __Pyx_PyObject_GetAttrStr((PyObject *)tp,
                                               __pyx_n_s_name);
    if (unlikely(name == NULL) || unlikely(!PyUnicode_Check(name))) {
        PyErr_Clear();
        Py_XDECREF(name);
        name = __Pyx_NewRef(__pyx_n_s__8);
    }
    return name;
}
#endif

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if ((sizeof(long) < sizeof(long))) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    }
#endif
    if (unlikely(!PyLong_Check(x))) {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(long, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(long) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 2 * PyLong_SHIFT)) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(long) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 3 * PyLong_SHIFT)) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(long) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 4 * PyLong_SHIFT)) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (long) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(long) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
        } else if ((sizeof(long) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(long, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(long) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(long) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(long) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 4 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(long) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 4 * PyLong_SHIFT)) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(long) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
        } else if ((sizeof(long) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
        }
    }
    {
        long val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (long) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (long) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (long) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (long) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(long) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((long) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(long) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((long) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((long) 1) << (sizeof(long) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (long) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = __Pyx_PyType_GetSlot(a, tp_base, PyTypeObject*);
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
static CYTHON_INLINE int __Pyx_IsAnySubtype2(PyTypeObject *cls, PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (cls == a || cls == b) return 1;
    mro = cls->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            PyObject *base = PyTuple_GET_ITEM(mro, i);
            if (base == (PyObject *)a || base == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(cls, a) || __Pyx_InBases(cls, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    if (exc_type1) {
        return __Pyx_IsAnySubtype2((PyTypeObject*)err, (PyTypeObject*)exc_type1, (PyTypeObject*)exc_type2);
    } else {
        return __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
static unsigned long __Pyx_get_runtime_version(void) {
#if __PYX_LIMITED_VERSION_HEX >= 0x030B00A4
    return Py_Version & ~0xFFUL;
#else
    const char* rt_version = Py_GetVersion();
    unsigned long version = 0;
    unsigned long factor = 0x01000000UL;
    unsigned int digit = 0;
    int i = 0;
    while (factor) {
        while ('0' <= rt_version[i] && rt_version[i] <= '9') {
            digit = digit * 10 + (unsigned int) (rt_version[i] - '0');
            ++i;
        }
        version += factor * digit;
        if (rt_version[i] != '.')
            break;
        digit = 0;
        factor >>= 8;
        ++i;
    }
    return version;
#endif
}
static int __Pyx_check_binary_version(unsigned long ct_version, unsigned long rt_version, int allow_newer) {
    const unsigned long MAJOR_MINOR = 0xFFFF0000UL;
    if ((rt_version & MAJOR_MINOR) == (ct_version & MAJOR_MINOR))
        return 0;
    if (likely(allow_newer && (rt_version & MAJOR_MINOR) > (ct_version & MAJOR_MINOR)))
        return 1;
    {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compile time Python version %d.%d "
                      "of module '%.100s' "
                      "%s "
                      "runtime version %d.%d",
                       (int) (ct_version >> 24), (int) ((ct_version >> 16) & 0xFF),
                       __Pyx_MODULE_NAME,
                       (allow_newer) ? "was newer than" : "does not match",
                       (int) (rt_version >> 24), (int) ((rt_version >> 16) & 0xFF)
       );
        return PyErr_WarnEx(NULL, message, 1);
    }
}

/* VoidPtrImport */
#ifndef __PYX_HAVE_RT_ImportVoidPtr_3_0_11
#define __PYX_HAVE_RT_ImportVoidPtr_3_0_11
static int __Pyx_ImportVoidPtr_3_0_11(PyObject *module, const char *name, void **p, const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, name);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C variable %.200s",
                PyModule_GetName(module), name);
        goto bad;
    }
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C variable %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), name, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    *p = PyCapsule_GetPointer(cobj, sig);
    if (!(*p))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

/* FunctionImport */
#ifndef __PYX_HAVE_RT_ImportFunction_3_0_11
#define __PYX_HAVE_RT_ImportFunction_3_0_11
static int __Pyx_ImportFunction_3_0_11(PyObject *module, const char *funcname, void (**f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, funcname);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C function %.200s",
                PyModule_GetName(module), funcname);
        goto bad;
    }
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    tmp.p = PyCapsule_GetPointer(cobj, sig);
    *f = tmp.fp;
    if (!(*f))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

/* InitStrings */
#if PY_MAJOR_VERSION >= 3
static int __Pyx_InitString(__Pyx_StringTabEntry t, PyObject **str) {
    if (t.is_unicode | t.is_str) {
        if (t.intern) {
            *str = PyUnicode_InternFromString(t.s);
        } else if (t.encoding) {
            *str = PyUnicode_Decode(t.s, t.n - 1, t.encoding, NULL);
        } else {
            *str = PyUnicode_FromStringAndSize(t.s, t.n - 1);
        }
    } else {
        *str = PyBytes_FromStringAndSize(t.s, t.n - 1);
    }
    if (!*str)
        return -1;
    if (PyObject_Hash(*str) == -1)
        return -1;
    return 0;
}
#endif
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION >= 3
        __Pyx_InitString(*t, t->p);
        #else
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        #endif
        ++t;
    }
    return 0;
}

#include <string.h>
static CYTHON_INLINE Py_ssize_t __Pyx_ssize_strlen(const char *s) {
    size_t len = strlen(s);
    if (unlikely(len > (size_t) PY_SSIZE_T_MAX)) {
        PyErr_SetString(PyExc_OverflowError, "byte string is too long");
        return -1;
    }
    return (Py_ssize_t) len;
}
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    Py_ssize_t len = __Pyx_ssize_strlen(c_str);
    if (unlikely(len < 0)) return NULL;
    return __Pyx_PyUnicode_FromStringAndSize(c_str, len);
}
static CYTHON_INLINE PyObject* __Pyx_PyByteArray_FromString(const char* c_str) {
    Py_ssize_t len = __Pyx_ssize_strlen(c_str);
    if (unlikely(len < 0)) return NULL;
    return PyByteArray_FromStringAndSize(c_str, len);
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY && !CYTHON_COMPILING_IN_LIMITED_API) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
    __Pyx_TypeName result_type_name = __Pyx_PyType_GetName(Py_TYPE(result));
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type " __Pyx_FMT_TYPENAME ").  "
                "The ability to return an instance of a strict subclass of int is deprecated, "
                "and may be removed in a future version of Python.",
                result_type_name)) {
            __Pyx_DECREF_TypeName(result_type_name);
            Py_DECREF(result);
            return NULL;
        }
        __Pyx_DECREF_TypeName(result_type_name);
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type " __Pyx_FMT_TYPENAME ")",
                 type_name, type_name, result_type_name);
    __Pyx_DECREF_TypeName(result_type_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(__Pyx_PyLong_IsCompact(b))) {
        return __Pyx_PyLong_CompactValue(b);
    } else {
      const digit* digits = __Pyx_PyLong_Digits(b);
      const Py_ssize_t size = __Pyx_PyLong_SignedDigitCount(b);
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject* o) {
  if (sizeof(Py_hash_t) == sizeof(Py_ssize_t)) {
    return (Py_hash_t) __Pyx_PyIndex_AsSsize_t(o);
#if PY_MAJOR_VERSION < 3
  } else if (likely(PyInt_CheckExact(o))) {
    return PyInt_AS_LONG(o);
#endif
  } else {
    Py_ssize_t ival;
    PyObject *x;
    x = PyNumber_Index(o);
    if (!x) return -1;
    ival = PyInt_AsLong(x);
    Py_DECREF(x);
    return ival;
  }
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


/* #### Code section: utility_code_pragmas_end ### */
#ifdef _MSC_VER
#pragma warning( pop )
#endif



/* #### Code section: end ### */
#endif /* Py_PYTHON_H */
